package de.cnc;


import javax.servlet.http.*;

import de.cnc.domainparser.util.SourcePosition;
import de.cnc.flowprogram.*;
import de.cnc.util.*;


/**
 * Struktur-Klasse fr einzelne Elemente des {@link FlowStack} . <br/>
 * Enthlt die Return-Adresse und Informationen fr lokale Variable und Parameter. <br/>
 * - Backup-Kopie fr locale Variable und Value-Parameter<br/>
 * - Interne und externe Session-Keys fr Referenz-Parameter<br/>
 * - Externe Session-Keys fr Return-Values<br/>
 *
 * @see FlowStack
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id:$
 */
public final class FlowStackElement {

  /**
   * Return-Adresse
   */
  public int adress;

   /**
    * vorherige Adresse zur Anzeige bei Debug
    */
   public int preAdress;

  /**
   *  HashMap mit lokalen Variablen (Kopien der globalen Variablen zum Wiederherstellen bei return)
   */
  // public HashMap localVarCopyHashMap ; // = new HashMap() ;

  /**
   * FlowProcParamList des Stacks (nicht des Befehls, wichtig zu unterscheiden)
   */
  public FlowProcParamList procParamList;

  /**
   * call-return-Session-Key (fr <call returnSessionKey="...")
   */
  public String callReturnSessionKey;

  /**
   * return-return-Session-Key (fr <return sessionKey="...")
   */
  //public String returnSessionKey;

  /**
   * Konstruktor mit Adresse
   */
  public FlowStackElement( //--
     final int newAdress, //--
     final int newPreAdress) 
  {
    adress = newAdress;
    preAdress =newPreAdress;
  }// end constructor

  /**
   * Sichern Backup fr lokale Variable
   *
   * @param request zum Zugriff auf Session zum Backup fr lokale Variable
   */
  public void addLocalVar(
        HttpServletRequest request, 
        String strPaVarNameWithoutPrefix,
        SourcePosition pSourcePosition) 
  {
    //Log.debug( "addLocalVar" + strPaVarNameWithoutPrefix ) ;
    if (procParamList == null) {
      procParamList = new FlowProcParamList();
    }// end if

    HttpSession session = request.getSession();//true);

	//String sessKey = FlowControlUtil.checkAndCutPrefixSession(strPaVarNameWithoutPrefix, pSourcePosition);
    String sessKey = strPaVarNameWithoutPrefix;

    Object locObj = session.getAttribute( sessKey );

    //new Exception( "remove" ).printStackTrace();

    session.removeAttribute(strPaVarNameWithoutPrefix); // procedure soll leeren session-key vorfinden
    //Log.debug("\naddLocalVar " + strPaVarNameWithoutPrefix + "->" + locObj);

    FlowProcCallParam flowProcedureParameter = new FlowProcCallParam();

    flowProcedureParameter.isRefParam = false;
    flowProcedureParameter.insideKey = strPaVarNameWithoutPrefix;
    flowProcedureParameter.backupValue = locObj;
    flowProcedureParameter.sourcePosition = pSourcePosition;
    procParamList.add(flowProcedureParameter/*, pSourcePosition*/);
  }// end method addLocalVar

  /**
   * Vermerken Parameter
   *
   * @param request zum Zugriff auf Session zum Backup fr lokale Variable
   */
  public void addParam(
        HttpServletRequest request, 
        FlowProcCallParam pFlowProcedureParameter)
        //SourcePosition pSourcePosition) 
  {
    Log.debug("\nFlowStackElement#addParam " , pFlowProcedureParameter);
    if (procParamList == null) {
      procParamList = new FlowProcParamList();
    }// end if


    HttpSession session = request.getSession();//true);
    //String sessKey = FlowControlUtil.checkAndCutPrefixSession(strPaVarNameWithoutPrefix, pSourcePosition);
    Object locObj = session.getAttribute( pFlowProcedureParameter.insideKey );

//hier erst den value wegfangen und dann remove machen
//new Exception( "remove" ).printStackTrace();
//session.removeAttribute(pFlowProcedureParameter.insideKey); // flowcontrol setzen parameter-expressions soll leeren session-key vorfinden

    //Log.debug("\naddParamBackupValue " + pFlowProcedureParameter.insideKey + "->" + locObj);

    pFlowProcedureParameter.backupValue = locObj;

    procParamList.add(pFlowProcedureParameter/*, pSourcePosition*/);
  }// end method addRefParm

  /**
   * Zurck-Kopieren Backups fr lokale Variable
   *
   * @param request zum Zugriff auf Session zum Zurck-Kopieren Backups fr lokale Variable
   */
  public void restore(HttpServletRequest request) {
    //Log.debug("\nFlowStackElement#restore");
    HttpSession session = request.getSession(true);

    // Log.dump( session ) ;

    // Return-value in ref-param umwandeln
//    if (!StringUtil.empty(callReturnSessionKey) && !StringUtil.empty(returnSessionKey)) {
//      if (flowProcedureParameterList == null) {
//        flowProcedureParameterList = new FlowProcParamList();
//      }// end if
//      FlowProcCallParam tmpFlowProcedureParameter = new FlowProcCallParam();
//
//      tmpFlowProcedureParameter.isRefParam = true;
//      tmpFlowProcedureParameter.outsideKey = callReturnSessionKey;
//      tmpFlowProcedureParameter.insideKey = returnSessionKey;
//      flowProcedureParameterList.add(tmpFlowProcedureParameter);
//    }// end if

//System.out.println( "flowProcedureParameterList: " + flowProcedureParameterList );

    if (procParamList != null) {

//      // Sammeln reference parameter
//      for (int i = 0; i < flowProcedureParameterList.size(); i++) {
//        FlowProcCallParam flowProcedureParameter = flowProcedureParameterList.get(i);
//
//        if (flowProcedureParameter.isRefParam) 
//        { 
//          // ref-param
//          flowProcedureParameter.backupValue = session.getAttribute(flowProcedureParameter.insideKey);
//          Log.debug("save ref-param " + flowProcedureParameter.outsideKey + "->" + flowProcedureParameter.insideKey + "->"
//              + flowProcedureParameter.backupValue);
//        }// end if
//      }// end for
//
//      // Lschen interne Variable
//      for (int i = 0; i < flowProcedureParameterList.size(); i++) 
//      {
//        FlowProcCallParam flowProcedureParameter = flowProcedureParameterList.get(i);
//
//        Log.debug("remove local var value " + flowProcedureParameter.insideKey);
//        session.removeAttribute(flowProcedureParameter.insideKey);
//      }// end for

      // Zurckkopieren Werte
      for (int i = 0; i < procParamList.size(); i++) 
      {
        FlowProcCallParam flowProcedureParameter = procParamList.get(i);

        if ( flowProcedureParameter.isRefParam )
        {
//System.out.println( "FlowStackElement#restore refParam");
            //-----------------------------------------------------------------
            //Umkopieren reference-Value von innerem Key auf usseren Key
            //-----------------------------------------------------------------
            
            //vermerken inneren Wert
            Object innerObj = session.getAttribute( flowProcedureParameter.insideKey );

//System.out.println( "FlowStackElement#restore refParam value: " + innerObj );

            
            //Wiederherstellen Originalwert auf innerem Key
            //Log.debug("restore local var value " + flowProcedureParameter.insideKey + "->" + flowProcedureParameter.backupValue);
           CncUtil.setAttribute(session,  flowProcedureParameter.insideKey,
                                  flowProcedureParameter.backupValue);

            //Speichern inneren Wert auf usserem Key
            //Log.debug("move reference param value: insideKey: " + flowProcedureParameter.insideKey + " -> outsideKey: " + flowProcedureParameter.outsideKey + " value: " + innerObj );
           CncUtil.setAttribute(session,  flowProcedureParameter.outsideKey,
                                  innerObj);
        }
        else
        {
            //Wiederherstellen Originalwert auf innerem Key
            //Log.debug("restore local var value " + flowProcedureParameter.insideKey + "->" + flowProcedureParameter.backupValue);
           CncUtil.setAttribute(session,  flowProcedureParameter.insideKey,
                                  flowProcedureParameter.backupValue);
        }
      }// end for
      // flowProcedureParameterArrayList = null ; // flowProcedureParameterArrayList nach restore wegrumen, weil sonst bei nchstem call wieder drin
    }// end if
  }// end method restore

  /**
   * Debug-Ausgabe
   */
  public String toString() {
    StringBuffer sb = new StringBuffer();

    // sb.append( "FlowStackElement Adress: " + adress +"\n" ) ;
    // sb.append( "FlowStackElement callReturnSessionKey: " + callReturnSessionKey +"\n" ) ;
    // sb.append( "FlowStackElement returnSessionKey: " + returnSessionKey +"\n" ) ;
    // sb.append( "FlowStackElement flowProcedureParameterArrayList " + flowProcedureParameterArrayList + "\n" ) ;

    sb.append("FlowStackElement Adress: " + adress + " ");
    sb.append("callReturnSessionKey: " + callReturnSessionKey + " ");
    //sb.append("returnSessionKey: " + returnSessionKey + " ");
    sb.append("flowProcedureParameterArrayList " + procParamList);
    return sb.toString();
  }// end method toString

}// end class FlowStackElement
