package de.cnc;


/**
 * Hier sind globale Request-Keys des Servlet-Request hinterlegt . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner K&uuml;cker</a>
 * <br/>
 *
 * @author Heiner K&uuml;cker
 * @version $Id:$
 */
public interface RequestKeys {

    /**
   * URL-Pattern zum Mapping auf das zentrale CncServlet . <br/>
   * Dieses Pattern *.cnc muss in web.xml eingestellt sein und ist hier fest vorgegeben.
   */
  public final static String CNC_URL_POSTFIX = ".cnc";

  /**
   * Action-Prefix f&uuml;r GOTO-Actions . <br/>
   * Springen ein Label oder eine Prozedur an, ohne dass der Sprung in flow.cnc auscodiert werden muss.<br/>
   * Dienen f&uuml;r Navigationslinks.<br/>
   * Es muss darauf geachtet werden, dass der Prefix nicht in anderen Action-Namen auftaucht.<br/>
   */
  public final static String CNC_GOTO_ACTION_PREFIX = "cnc_goto_";

  /**
   * Request-Key f&uuml;r Abbruch-Buttons in HTML-FORM . <br/>
   * Verhindert das Einparsen des Formulars in {@link CncServlet}
   * @see de.cnc.taglib.FormBreakButtonTag
   */
  public final static String CNC_BUTTON_NO_PARSE = "cnc_button_no_parse";

  /**
   * Request-Key zum Unterdr&uuml;cken des Einparsens der Formular-Daten in {@link CncServlet} . <br/>
   * kann durch einen Button "Abbrechen" gesendet werden.<br/>
   * @see de.cnc.taglib.FormBreakButtonTag
   */
  // public final static String CNC_NO_PARSE = "CNC_NO_PARSE" ;

  /**
   * Parameter-Key zur Adressierung eines Listenelements
   */
  public final static String CNC_LIST_ITEM_ID = "CNC_LIST_ITEM_ID";

  /**
   * Request-Key-Prefix f&uuml;r die Mimik zum Zur&uuml;cksetzen von Checkboxen in Formularen.
   */
  public final static String CNC_CHECKBOX_DEFAULT_PREFIX = "CNC_CHECKBOX_DEFAULT_";

  /**
   * Parameter-Key zum Ermitteln der vorherigen JSP
   */
  public final static String CNC_SRC_JSP = "CNC_SRC_JSP";

  /**
   * Parameter-Key f&uuml;r CNC_STATUS bei F&uuml;hrungsmethode Session-Leading
   */
  public final static String CNC_STATUS = SessionKeys.CNC_STATUS;

  /**
   * Parameter-Key zum Blttern
   */
  // kann wg. implziten Actions weg
  public final static String CNC_FIRST_PAGE_ACTION = "CNC_FIRST_PAGE_ACTION";

  /**
   * Parameter-Key zum Blttern
   */
  public final static String CNC_PREV_PAGE_ACTION = "CNC_PREV_PAGE_ACTION";

  /**
   * Parameter-Key zum Blttern
   */
  public final static String CNC_PREV_POS_ACTION = "CNC_PREV_POS_ACTION";

  /**
   * Parameter-Key zum Blttern
   */
  public final static String CNC_NEXT_POS_ACTION = "CNC_NEXT_POS_ACTION";

  /**
   * Parameter-Key zum Blttern
   */
  public final static String CNC_NEXT_PAGE_ACTION = "CNC_NEXT_PAGE_ACTION";

  /**
   * Parameter-Key zum Blttern
   */
  public final static String CNC_LAST_PAGE_ACTION = "CNC_LAST_PAGE_ACTION";

  /**
   * Parameter-Prefix f&uuml;r implizite Actions
   */
  public final static String CNC_IMPL_ACTION_PREFIX = "CNC_IMPL_ACTION_";

  /**
   * Parameter-Prefix f&uuml;r implizite Session-Values {@
   */
  public final static String CNC_SESSION_VALUE_PREFIX = "CNC_SESSION_VALUE_";

  /**
   * Parameter-Prefix f&uuml;r Formatierungen fr implizite Session-Values
   */
  public final static String CNC_SESSION_VALUEMASK_PREFIX = "CNC_SESSION_VALUEMASK_";

    /** zum Ermitteln, ob JavaScript im Browser des Nutzers eingeschalten ist */
  public static final String CNC_JAVASCRIPT = "CNC_JAVASCRIPT";

} // end Interface RequestKeys
