package de.cnc;




/**
 * Hier sind wichtige Session-Keys der Servlet-Session im CnC-System hinterlegt . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public interface SessionKeys {

  /**
   * Session-Key zum Speichern des Stacks (FlowStack) bei Methode sessionleading
   */
  public final static String CNC_FLOWSTACK_KEY = "CNC_FLOWSTACK_KEY";

  /**
   * Session-Key fr Meldungen auf JSP-Seiten . <br/>
   * @see de.cnc.taglib.MessageTag
   * @see CncUtil#setMessage CncUtil#setMessage
   */
  public final static String CNC_INFO_MESSAGE = "CNC_INFO_MESSAGE";

  /**
   * Session-Key fr Meldungen auf JSP-Seiten . <br/>
   * @see de.cnc.taglib.MessageTag
   * @see CncUtil#setMessage CncUtil#setMessage
   */
  public final static String CNC_WARNING_MESSAGE = "CNC_WARNING_MESSAGE";

  /**
   * Session-Key fr Fehler-Meldungen auf JSP-Seiten . <br/>
   * @see de.cnc.taglib.MessageTag
   * @see CncUtil#setMessage CncUtil#setMessage
   */
  public final static String CNC_ERROR_MESSAGE = "CNC_ERROR_MESSAGE";

  /**
   * Session-Key fr das generelle Blttern-Control
   * @see BrowseControl
   */
  public final static String CNC_BROWSE_CTRL_KEY = "CNC_BROWSE_CTRL_KEY";

  /**
   * Session-Key fr alle FormBeans. <br/>
   * Die Tags <br/>
   * erwarten die FormBeans auf diesem Session-Key. <br/>
   * Im zentralen Servlet {@link CncServlet} werden die Request-Parameter in die Bean auf diesem Key eingeparst. <br/>
   * Die Util-Klasse {@link CncUtil} setzt die Benutzung dieses Keys voraus. <br/>
   */
  public final static String CNC_FORM_KEY = "CNC_FORM_KEY";

  /**
   * Session-Key fr alle Listen. <br/>
   */
  public final static String CNC_LIST_KEY = "CNC_LIST_KEY";

  /**
   * Session-Key fr aktuelle Listen-Nummer. <br/>
   * Die Tags {@link de.cnc.taglib.IterateTag} und
   * {@link de.cnc.taglib.IterateBrowsedTag}
   * legen hier bei jedem Durchlauf die Nummer (index Basis 0) des aktuellen Listenelements ab,
   * damit alle xxxIndexedTags darauf zugreifen knnen.
   */
  public final static String CNC_LIST_ELEMENT_ID_KEY = "CNC_LIST_ELEMENT_ID_KEY";

  /**
   * Session-Key fr das aktuelle Listenelement. <br/>
   * Die Tags {@link de.cnc.taglib.IterateTag} und
   * {@link de.cnc.taglib.IterateBrowsedTag}
   * legen hier bei jedem Durchlauf das aktuelle Listenelement ab,
   * damit alle xxxIndexedTags darauf zugreifen knnen.
   */
  public final static String CNC_LIST_ELEMENT_KEY = "CNC_LIST_ELEMENT_KEY";

  /**
   * Session-Key fr alle Tree's (Tree-Controls). <br/>
   */
  public final static String CNC_TREE_KEY = "CNC_TREE_KEY";

  /**
   * Session-Key zum Speichern der vorherigen JSP
   */
  public final static String CNC_SRC_JSP = "CNC_SRC_JSP";

  /**
   * Session-Key fr CNC_STATUS bei Fhrungsmethode Session-Leading
   */
  public final static String CNC_STATUS = "CNC_STATUS";

  /**
   * Session-Key fr CNC_SINGLE_STEP_DEBUG
   */
  public final static String CNC_SINGLE_STEP_DEBUG = "CNC_SINGLE_STEP_DEBUG";

  /**
   * Session-Key fr CNC_SINGLE_STEP_DEBUG_ACTION . <br/>
   * Zum Vermerken der Original-Action, damit diese beim weiteren Aufruf von singlestepdebug.jsp erhalten bleibt.<br/>
   * Wird durch das Tag debugrqfields weitergeschleust.<br/>
   */
  public final static String CNC_SINGLE_STEP_DEBUG_ACTION = "CNC_SINGLE_STEP_DEBUG_ACTION";
  
  /**
   * Session-Key zum Setzen des Focus auf ein FORM-Element 
   * auf der JSP-Seite ber das &lt;cnc:setfocus&gt;-Tag.
   */  
  public final static String CNC_FOCUS = "CNC_FOCUS";

    /**
     * Fehlermeldungen fr Eingabefelder in Forms
     */
    public static final String CNC_FORM_FIELD_ERROR_MAP = "CNC_FORM_FIELD_ERROR_MAP";

    /**
     * Fehlermeldungen fr Eingabefelder in indizierten Forms, Tabellen
     */
    //public static final String CNC_FORM_FIELD_ERROR_MAP_LIST = "CNC_FORM_FIELD_ERROR_MAP_LIST";

}// end Interface SessionKeys
