package de.cnc.domainparser;


import java.util.*;

import de.cnc.domainparser.util.*;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse reprsentiert eine kommaseparierte Liste,
 * die wiederum aus Element-Listen (WhiteSpace-separiert)
 * besteht. Kommaseparierte Listen sind in runden Klamern
 * {@link ParenthesisArea} und eckigen Klammern
 * {@link BracketArea} erlaubt.
 */
public class CommaSeparatedList extends AbstractNonTerminalElement
{

    /** Liste von ElementListen */
    private ArrayList elementListList = new ArrayList();

    private final String completeSrcStr;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private CommaSeparatedList(SourcePosition pSrcPos, String pSrcStr)
    {
        // Anfangsposition vermerken
        srcPos = pSrcPos.getClone();
        completeSrcStr = pSrcStr;
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static CommaSeparatedList parse(
            final String pStr, 
            final SourcePosition pSrcPos, // Achtung Seiteneffekt
            final DomainParserContext pParserContext)
        throws DomainParseException
    {
        // System.out.println( "CommaSeparatedList parse start" ) ;
        // System.out.println( "CommaSeparatedList#parse pSrcPos: " + pSrcPos );
        // System.out.println( "CommaSeparatedList#parse pStr.charAt( pSrcPos.getPos() ): " + pStr.charAt( pSrcPos.getPos() ) );
        // (new Exception("DEBUG")).printStackTrace();

        CommaSeparatedList commaSepLst = new CommaSeparatedList(pSrcPos, pStr);

        // Vermerken der Anfangsposition
        int anfPos = pSrcPos.getPos();        

        // char runChar = pStr.charAt(pSrcPos.getPos());

        // if ( runChar == ',' )
        // {
        // //gleich am Anfang steht ein Semikolon
        // pSrcPos.addOne(); // Semikolon berspringen
        // return commaSepLst;
        // }

        while (pSrcPos.getPos() < pStr.length())
        {
            // nach dem Element folgende WhiteSpaces berspringen
            DomainParserUtil.parseWhitesAndComments(pStr, pSrcPos);

            ElementList elementList = ElementList.parse(pStr, pSrcPos, pParserContext);

            // System.out.println( "CommaSeparatedList#elementList " + elementList );

            // Element vermerken
            commaSepLst.elementListList.add(elementList);

            // Endposition schon mal vermerken, falls die Liste jetzt endet
            commaSepLst.endSourcePosition = pSrcPos.getClone();

            // Lnge vermerken
            commaSepLst.length = pSrcPos.getPos() - anfPos;

            // nach dem Element folgende WhiteSpaces berspringen
            DomainParserUtil.parseWhitesAndComments(pStr, pSrcPos);

            // if ( element instanceof BracesArea )
            // {
            // //ein BracesArea (geschwungene Klammern) beendet ein Segment
            // break;
            // }

            if (pSrcPos.getPos() < pStr.length())
            {
                char runChar = pStr.charAt(pSrcPos.getPos());

                // System.out.println( "CommaSeparatedList#runChar: " + runChar ) ;

                if (runChar == ',')
                {
                    pSrcPos.addOne(); // Komma berspringen
                    // break;
                }
                else if (runChar == ';' || runChar == '}' || runChar == ')' || runChar == ']')
                {
                    // scheinbar Ende eines Bereiches
                    break;
                }
                else
                {
                    throw new DomainParseException("no element recognized", pSrcPos);
                }
            }
        }

        // Lnge vermerken
        //commaSepLst.length = pSrcPos.getPos() - anfPos - 1;

        //commaSepLst.endSourcePosition = pSrcPos.getClone();

        // System.out.println( "CommaSeparatedList parse end" ) ;
        return commaSepLst;
    }

    /**
     * 
     * @return
     */
    public int elementListCount()
    {
        return elementListList.size();
    }

    /**
     * 
     * @param pI
     * @return
     */
    public ElementList getElementList(int pI)
    {
        return (ElementList) elementListList.get(pI);
    }

    /**
     * 
     */
    public String toString()
    {
        return toString(0);
    }

    /**
     * 
     * @param indent
     * @return
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        for (int i = 0; i < elementListList.size(); i++)
        {
            strBuff.append(((AbstractElement) elementListList.get(i)).toString(indent + 1) + "\n");
        }
        if (elementListList.size() < 1)
        {
            strBuff.append(StringUtil.spaces(indent + 1) + "segment is empty" + "\n");
        }
        return strBuff.toString();
    }

    /**
     * @see de.cnc.domainparser.Element#isEmpty()
     */
    public boolean isEmpty()
    {
       if (elementListList == null || elementListList.isEmpty())
       {
           return true;
       }

       // wenn die Kommaseparierte Liste nur ein Element enthlt und dieses ist auch leer, dann gilt leer==true
       // kann durch StripComments auftreten
       if (this.elementListCount() < 2 
          && this.getElementList( 0 ).isEmpty() )
       {
          return true;
       }
       return false;
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        // throw new UnsupportedOperationException();
        return completeSrcStr.substring(srcPos.getPos(), srcPos.getPos() + length + 2);
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        // throw new UnsupportedOperationException();
        return completeSrcStr.substring(srcPos.getPos() + 1, srcPos.getPos() + length + 1);
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return getContent().indexOf("\n") >= 0;
    }

    /**
     * Lschen Komentare
     */
    public void stripComments(boolean pAllComments)
    {
        for (Iterator iter = elementListList.iterator(); iter.hasNext();)
        {
            Object o = iter.next();

            ((ElementList) o).stripComments(pAllComments);            
        }
    }
}
