package de.cnc.domainparser;


import java.util.*;

import de.cnc.domainparser.util.*;
import de.cnc.util.StringUtil;


/**
 * Eine Liste durch WhiteSpaces separierter Elemente
 */
public class ElementList extends AbstractNonTerminalElement
{

    private ArrayList elementList = new ArrayList();

    private final String completeSrcStr;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private ElementList(SourcePosition pSrcPos, String pSrcStr)
    {
        // Anfangsposition vermerken
        srcPos = pSrcPos.getClone();

        // gesamten Quelltext referenzieren
        completeSrcStr = pSrcStr;
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static ElementList parse(
            final String pStr, 
            final SourcePosition pSrcPos, // Achtung Seiteneffekt
            final DomainParserContext pParserContext)
        throws DomainParseException
    {
        //System.out.println( "Segment#parse pStr.charAt( pSrcPos.getPos() ): " + pStr.charAt( pSrcPos.getPos() ) );
        ElementList elemList = new ElementList(pSrcPos, pStr);

        // Vermerken der Anfangsposition
        int anfPos = pSrcPos.getPos();        

        // char runChar = pStr.charAt(pSrcPos.getPos());

        // if ( runChar == ',' )
        // {
        // //gleich am Anfang steht ein Komma
        // pSrcPos.addOne(); // Komma berspringen
        // return segment;
        // }

        while (pSrcPos.getPos() < pStr.length())
        {
            // nach dem Element folgende WhiteSpaces berspringen
            DomainParserUtil.parseWhitesAndComments(pStr, pSrcPos);

            AbstractElement element = AbstractElement.parseElement(pStr, pSrcPos, pParserContext);

            //System.out.println( "ElementList#element " + element );

            // Element vermerken
            elemList.elementList.add(element);

            // Endposition schon mal vermerken, falls die Liste jetzt ended
            elemList.endSourcePosition = pSrcPos.getClone();

            // Lnge vermerken
            elemList.length = pSrcPos.getPos() - anfPos;

            // nach dem Element folgende WhiteSpaces berspringen
            DomainParserUtil.parseWhitesAndComments(pStr, pSrcPos);

            // if ( element instanceof BracesArea )
            // {
            // //ein BracesArea (geschwungene Klammern) beendet ein Segment
            // break;
            // }

            if (pSrcPos.getPos() < pStr.length())
            {
                char runChar = pStr.charAt(pSrcPos.getPos());

                // System.out.println( "ElementList#runChar: " + runChar ) ;

                if (runChar == ',' || runChar == ';' || runChar == '}' || runChar == ')'
                        || runChar == ']')
                {
                    // scheinbar Ende eines Bereiches
                    break;
                }
            }
        }
        
        //elemList.endSourcePosition = pSrcPos.getClone();
        
        return elemList;
    }

    /**
     * 
     * @return
     */
    public int elementCount()
    {
        return elementList.size();
    }

    /**
     * 
     * @param pI
     * @return
     */
    public AbstractElement getElement(int pI)
    {
        return (AbstractElement) elementList.get(pI);
    }

    /**
     * 
     */
    public String toString()
    {
        return toString(0);
    }

    /**
     * 
     * @param indent
     * @return
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        for (int i = 0; i < elementList.size(); i++)
        {
            strBuff.append(((AbstractElement) elementList.get(i)).toString(indent + 1) + "\n");
        }
        if (elementList.size() < 1)
        {
            strBuff.append(StringUtil.spaces(indent + 1) + "segment is empty" + "\n");
        }
        return strBuff.toString();
    }

    /**
     * @see de.cnc.domainparser.Element#isEmpty()
     */
    public boolean isEmpty()
    {
        return elementList == null || elementList.isEmpty();
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        // throw new UnsupportedOperationException();
        return completeSrcStr.substring(srcPos.getPos(), srcPos.getPos() + length );
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        // throw new UnsupportedOperationException();
        //Content und NettoContent sind identisch, weil es keine umschliessenden Zeichen gibt
        return completeSrcStr.substring(srcPos.getPos(), srcPos.getPos() + length );
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return getContent().indexOf("\n") >= 0;
    }

    /**
     * Lschen Komentare
     */
    public void stripComments(boolean pAllComments)
    {
        for (Iterator iter = elementList.iterator(); iter.hasNext();)
        {
            Object o = iter.next();

            if (o instanceof InlineComment)
            {
                iter.remove();
            }
            else if (o instanceof BracketComment 
                && ( pAllComments || ! ( (BracketComment) o ).isJavadoc() ) )
            {
                iter.remove();
            }
            else if (!((AbstractElement) o).isTerminal())
            {
                ((AbstractElement) o).stripComments(pAllComments);
            }
            
        }
    }
}
