package de.cnc.domainparser;


import de.cnc.domainparser.util.SourcePosition;


/**
 * Diese Klasse reprsentiert ein String-Literal in Apostrophen eingeschlossen.
 * "..."
 */
public class HighCommaStringLiteral extends AbstractTerminalElement
{
    private String content;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private HighCommaStringLiteral(SourcePosition pSrcPos)
    {
        // Anfangsposition vermerken
        srcPos = pSrcPos.getClone();
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static HighCommaStringLiteral parse(
            final String pStr, 
            final SourcePosition pSrcPos, // Achtung Seiteneffekt
            final DomainParserContext pParserContext)
        throws DomainParseException
    {

        if (pParserContext.isJavaStringEncodingEnabled)
        {
            throw new UnsupportedOperationException("noch nicht fertig programmiert");
        }

        if (pStr.charAt(pSrcPos.getPos()) != '\'')
        {
            return null; // oder exception werfen
        }
        
        HighCommaStringLiteral qStrLit = new HighCommaStringLiteral(pSrcPos);

        // den ersten Apostroph berspringen
        pSrcPos.addOne();

        StringBuffer strBuff = new StringBuffer();

        while (pSrcPos.getPos() < pStr.length())
        {
            char runChar = pStr.charAt(pSrcPos.getPos());

            // eventuell noch encodings einbauen optional
            if (runChar == '\'')
            {
                break;
            }
            if (runChar == '\n')
            {
                throw new DomainParseException("end of high comma string literal expected, end of line found",
                        pSrcPos);
            }
            strBuff.append(runChar);
            pSrcPos.addOne();
        }
        // den letzten Apostroph berspringen
        pSrcPos.addOne();

        qStrLit.content = strBuff.toString();        

        return qStrLit;
    }

    /**
     * @return
     */
    public int getLength()
    {
        return content.length();
    }

    public String toString()
    {
        return toString(0);
    }

    /**
     * @see de.cnc.domainparser.Element#toString(int)
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        return strBuff.toString() + " content: " + content;
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        return "'" + content + "'";
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        return content;
    }

    /**
     * @see de.cnc.domainparser.Element#isEmpty()
     */
    public boolean isEmpty()
    {
        return content.trim().length() < 1;
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return false;
    }

}
