package de.cnc.domainparser;


import de.cnc.domainparser.util.SourcePosition;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse reprsentiert einen Inline-Kommentar.
 * 
 * Zweimal Slash // und Text bis zum Zeilenende
 */
public class InlineComment extends AbstractTerminalElement
{
    private String content;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private InlineComment(SourcePosition pSrcPos)
    {
        // Anfangsposition vermerken
        srcPos = pSrcPos.getClone();
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static InlineComment parse(
            final String pStr, 
            final SourcePosition pSrcPos) throws DomainParseException // Achtung Seiteneffekt
    {
        String partStr = pStr.substring(pSrcPos.getPos());

        if (!partStr.startsWith("//"))
        {
            return null; // oder exception werfen
        }
        
        InlineComment inlCmmt = new InlineComment(pSrcPos);

        // die ersten beiden Slash berspringen
        pSrcPos.addOne();
        pSrcPos.addOne();

        StringBuffer strBuff = new StringBuffer();

        while (pSrcPos.getPos() < pStr.length())
        {
            char runChar = pStr.charAt(pSrcPos.getPos());

            if (runChar == '\n')
            {
                break;
            }
            strBuff.append(runChar);
            pSrcPos.addOne();
        }
        inlCmmt.content = strBuff.toString();        
        return inlCmmt;
    }

    /**
     * @return
     */
    public int getLength()
    {
        return content.length();
    }

    public String toString()
    {
        return toString(0);
    }

    /**
     * @see de.cnc.domainparser.Element#toString(int)
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp( indent );

        return strBuff.toString() + StringUtil.spaces(indent) + " content: " + content;
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        return "//" + content + "\n";
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        return content;
    }

    /**
     * @see de.cnc.domainparser.Element#isEmpty()
     */
    public boolean isEmpty()
    {
        return content.trim().length() < 1;
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return false;
    }

}
