package de.cnc.domainparser;


import de.cnc.domainparser.util.SourcePosition;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse reprsentiert eine Zahl mit maximal einem Dezimalpunkt
 */
public class Number extends AbstractTerminalElement
{
    private String content;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private Number(SourcePosition pSrcPos)
    {
        // Anfangsposition vermerken
        srcPos = pSrcPos.getClone();
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static Number parse(
            final String pStr, 
            final SourcePosition pSrcPos) throws DomainParseException // Achtung Seiteneffekt
    {
        if (!Character.isDigit(pStr.charAt(pSrcPos.getPos())))
        {
            return null; // oder exception werfen
        }
        
        Number num = new Number(pSrcPos);

        StringBuffer strBuff = new StringBuffer();

        boolean hasPoint = false;

        while (pSrcPos.getPos() < pStr.length())
        {
            char runChar = pStr.charAt(pSrcPos.getPos());

            if (runChar == '.')
            {
                if (!hasPoint // es ist der erste Punkt
                        && pSrcPos.getPos() < pStr.length() - 1
                        && Character.isDigit(pStr.charAt(pSrcPos.getPos() + 1))) // es kommt noch mindestens eine Ziffer
                {
                    hasPoint = true;
                }
                else
                {
                    throw new DomainParseException("number format not properly", pSrcPos);
                }
            }
            
            else if (!Character.isDigit(runChar))
            {
                break;
            }
            strBuff.append(runChar);
            pSrcPos.addOne();
        }
        num.content = strBuff.toString();        
        return num;
    }

    /**
     * @return
     */
    public int getLength()
    {
        return content.length();
    }

    public String toString()
    {
        return toString(0);
    }

    /**
     * @see de.cnc.domainparser.Element#toString(int)
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        return strBuff.toString() + StringUtil.spaces(indent + 1) + " content: " + content;
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        return content;
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        return content;
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return false;
    }

}
