package de.cnc.domainparser;


import de.cnc.domainparser.util.*;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse reprsentiert den Bereich zwischen
 * ffnender und schliessender runder Klammer ( ... ) .
 */
public class ParenthesisArea extends AbstractNonTerminalElement
{
    private CommaSeparatedList commaSeparatedList;
    
    private final String completeSrcStr;

    /**
     * Konstruktor
     * 
     * @param pSrcPos
     */
    private ParenthesisArea(SourcePosition pSrcPos, String pSrcStr)
    {
        // Anfangsposition vermerken
        srcPos = pSrcPos.getClone();

        // gesamten Quelltext referenzieren
        completeSrcStr = pSrcStr;
    }

    /**
     * @param pStr
     * @param iPos
     * @return
     */
    public static ParenthesisArea parse(
            final String pStr, 
            final SourcePosition pSrcPos, // Achtung Seiteneffekt
            final DomainParserContext pParserContext)
        throws DomainParseException
    {
        if (pStr.charAt(pSrcPos.getPos()) != '(')
        {
            return null; // oder exception werfen
        }

        ParenthesisArea parenArea = new ParenthesisArea(pSrcPos, pStr);

        // Vermerken der Anfangsposition
        int anfPos = pSrcPos.getPos();        

        pSrcPos.addOne(); // ffnende Klammer berspringen

        // System.out.println( "ParenthesisArea#pSrcPos: " + pSrcPos );


        // nach der ffnenden Klammer folgende WhiteSpaces berspringen
        DomainParserUtil.parseWhitesAndComments(pStr, pSrcPos);

        if (pStr.charAt(pSrcPos.getPos()) == ')')
        {
            // Lnge vermerken
            parenArea.length = pSrcPos.getPos() - anfPos;
        
            parenArea.endSourcePosition = pSrcPos.getClone();

            // auf die ffnende Klammer folgt unmittelbar eine schliessende Klammer
            pSrcPos.addOne(); // schliessende Klammer berspringen

            return parenArea;
        }

        // System.out.println( "ParenthesisArea#parenArea.commaSeparatedList: " + parenArea.commaSeparatedList );
        CommaSeparatedList tmpCommaSeparatedList = CommaSeparatedList.parse(pStr, pSrcPos,
                pParserContext);

        parenArea.commaSeparatedList = tmpCommaSeparatedList;

        // nach dem Element folgende WhiteSpaces berspringen
        DomainParserUtil.parseWhitesAndComments(pStr, pSrcPos);

        // char runChar = pStr.charAt(pSrcPos.getPos());
        // System.out.println( "ParenthesisArea#runChar: " + runChar ) ;

        if (pSrcPos.getPos() >= pStr.length())
        {
            // keine schliesende Klammer aufgetreten
            throw new DomainParseException("right parenthesis expected ')', end of source reached",
                    pSrcPos);
        }
        else if (pStr.charAt(pSrcPos.getPos()) != ')')
        {
            // keine schliesende Klammer aufgetreten
            throw new DomainParseException("right parenthesis expected ')'", pSrcPos);
        }
        // runChar = pStr.charAt(pSrcPos.getPos());
        // System.out.println( "ParenthesisArea#runChar: " + runChar ) ;

        // Lnge vermerken
        parenArea.length = pSrcPos.getPos() - anfPos;
        
        parenArea.endSourcePosition = pSrcPos.getClone();

        pSrcPos.addOne(); // schliessende Klammer berspringen

        return parenArea;
    }

    /**
     * 
     * @param pI
     * @return
     */
    public CommaSeparatedList getCommaSeparatedList()
    {
        return commaSeparatedList;
    }

    /**
     * 
     */
    public String toString()
    {
        return toString(0);
    }

    /**
     * 
     * @param indent
     * @return
     */
    public String toString(int indent)
    {
        StringBuffer strBuff = toStringHelp(indent);

        if (commaSeparatedList == null || commaSeparatedList.isEmpty())
        {
            strBuff.append(StringUtil.spaces(indent + 1) + "is empty");
        }
        else
        {
            strBuff.append(commaSeparatedList.toString(indent + 1));
        }
        return strBuff.toString();
    }

    /**
     * @see de.cnc.domainparser.Element#isEmpty()
     */
    public boolean isEmpty()
    {
        if (commaSeparatedList == null || commaSeparatedList.isEmpty())
        {
            return true;
        }

        // wenn die Kommaseparierte Liste nur ein Element enthlt und dieses ist auch leer, dann gilt leer==true
        // kann durch StripComments auftreten
        if (commaSeparatedList.elementListCount() < 2 
           && commaSeparatedList.getElementList( 0 ).isEmpty() )
        {
           return true;
        }
        return false;
    }

    /**
     * @see de.cnc.domainparser.Element#getContent()
     */
    public String getContent()
    {
        // throw new UnsupportedOperationException();
        return completeSrcStr.substring(srcPos.getPos(), srcPos.getPos() + length + 1 );
    }

    /**
     * @see de.cnc.domainparser.Element#getNettoContent()
     */
    public String getNettoContent()
    {
        // throw new UnsupportedOperationException();
        return completeSrcStr.substring(srcPos.getPos() + 1, srcPos.getPos() + length );
    }

    /**
     * @see de.cnc.domainparser.Element#isMultiLined()
     */
    public boolean isMultiLined()
    {
        return getContent().indexOf("\n") >= 0;
    }

    /**
     * Lschen Komentare
     */
    public void stripComments(boolean pAllComments)
    {
        if (commaSeparatedList != null)
        {
            commaSeparatedList.stripComments(pAllComments);
        }
    }
}
