package de.cnc.domainparser;


import java.util.*;

import de.cnc.domainparser.util.*;
import de.cnc.util.StringUtil;


/**
 * Diese Klasse reprsentiert eine eingelesene Datei.
 */
public class Unit
{
    String strSrc;

    int srcPos;
    
    private ArrayList segmentList = new ArrayList();
    
    private final String completeSrcStr;

    /**
     * @param pStr
     */
    private Unit(String pStr, String pSrcStr)
    {
        strSrc = pStr;

        // gesamten Quelltext referenzieren
        completeSrcStr = pSrcStr;
    }

    public static Unit parse( 
            final String pStr,
            final DomainParserContext pParserContext)
        throws DomainParseException
    {
        Unit unit = new Unit(pStr, pStr);

        final SourcePosition srcPos = new SourcePosition();

        // am Anfang stehende WhiteSpaces berspringen
        DomainParserUtil.parseWhitesAndComments(pStr, srcPos);

        while (srcPos.getPos() < pStr.length())
        {
            // Segment parsen
            Segment segment = Segment.parse(pStr, srcPos, pParserContext);

            // System.out.println( "Unit#segment " + segment );

            // Segment vermerken
            unit.segmentList.add(segment);

            // nach dem Segment folgende WhiteSpaces berspringen
            DomainParserUtil.parseWhitesAndComments(pStr, srcPos);
        }
        return unit;
    }

    /**
     * 
     * @return
     */
    public int segmentCount()
    {
        return segmentList.size();
    }

    /**
     * 
     * @param pI
     * @return
     */
    public Segment getSegment(int pI)
    {
        return (Segment) segmentList.get(pI);
    }

    /**
     * 
     */
    public String toString()
    {
        StringBuffer strBuff = new StringBuffer();

        strBuff.append(StringUtil.getShortClassName(this) + "\n");
        for (int i = 0; i < segmentList.size(); i++)
        {
            strBuff.append(((Segment) segmentList.get(i)).toString(1)/* + "\n" */);
        }
        return strBuff.toString();
    }

    public String getContent()
    {
        return completeSrcStr;
    }

    /**
     * Lschen Komentare
     */
    public void stripComments(final boolean pAllComments)
    {
        for (Iterator iter = segmentList.iterator(); iter.hasNext();)
        {
            Object o = iter.next();

            if (o instanceof InlineComment)
            {
                iter.remove();
            }
            else if (o instanceof BracketComment 
                && ( pAllComments || ! ( (BracketComment) o ).isJavadoc() ) )
            {
                iter.remove();
            }
            else if (!((AbstractElement) o).isTerminal())
            {
                ((AbstractElement) o).stripComments(pAllComments);
            }
            
        }
    }

    /**
     * Lschen leere Segmente
     */
    public void stripEmptySegments()
    {
        for (Iterator iter = segmentList.iterator(); iter.hasNext();)
        {
            Segment segment = (Segment) iter.next();

            if ( segment.isEmpty() )
            {
                iter.remove();
            }
            else
            {
                segment.stripEmptySegments();
            }
        }
    }

}
