package de.cnc.domainparser.util;

import de.cnc.util.StringUtil;


/**
 * Position im Source-Code bestehend
 * aus Zeile, Spalte und Absolut-Position
 * 
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 * 
 * @author Heiner Kcker
 * @version $Id:$
 */
public class SourcePosition
{

    /**
     * Absolutposition ohne Beachtung Zeilen und Spalten
     * Basis 0
     */
    private int pos;

    /**
     * Zeilenposition Basis 1
     */
    private int row = 1;

    /**
     * Spaltenposition Basis 1
     */
    private int col = 1;

    /**
     * parameterloser Konstruktor
     */
    public SourcePosition()
    {
        ;
    }

    /**
     * parameterbehafteter Konstruktor
     */
    public SourcePosition(int paPos, int paRow, int paCol)
    {
        if (paPos < 0)
        {
            throw new RuntimeException("pos must >= 0");
        }

        if (paRow < 1)
        {
            throw new RuntimeException("row must > 0");
        }

        if (paCol < 1)
        {
            throw new RuntimeException("col must > 0");
        }

        if (paRow == 0 && paPos != paCol)
        {
            throw new RuntimeException("pos must equals to col if row == 0");
        }

        if (paRow > 1 && paCol < 2)
        {
            throw new RuntimeException("pos + 1 must equals to col if row == 0");
        }

        this.pos = paPos;
        this.row = paRow;
        this.col = paCol;
    }

    /**
     * addiere die bergebene Anzahl Zeichen zur SourcePosition,
     * aber kein Newline.
     * 
     * @param iPaSteps zu addierende Anzahl Zeichen
     */
    public SourcePosition add(int iPaSteps)
    {
        pos += iPaSteps;
        col += iPaSteps;
        return this;
    }

    /**
     * addiere ein Zeichen zur SourcePosition,
     * aber kein Newline.
     */
    public SourcePosition addOne()
    {
        pos++;
        col++;
        return this;
    }

    /**
     * Addiere ein Newline zur SourcePosition,
     * dabei wird die Spalte auf 0 gesetzt und
     * die Absolutposition und die Zeilenposition
     * erhht.
     */
    public SourcePosition addNewLine()
    {
        pos++;
        row++;
        col = 1;
        return this;
    }

    /**
     * gibt einen Clone der SourcePosition zurck.
     * 
     * @return geclonte SourcePosition
     */
    public SourcePosition getClone()
    {
        SourcePosition retSourcePosition = new SourcePosition();

        retSourcePosition.pos = this.pos;
        retSourcePosition.col = this.col;
        retSourcePosition.row = this.row;
        return retSourcePosition;
    }

   /**
    * Setzt die Positionswerte anhand der bergebenen Position
    * 
    * @return geclonte SourcePosition
    */
   public void set(final SourcePosition pPos)
   {
       this.pos = pPos.pos;
       this.col = pPos.col;
       this.row = pPos.row;
   }

    /**
     * Gets the col
     * 
     * @return Returns a int
     */
    public int getCol()
    {
        return col;
    }

    /**
     * Gets the pos
     * 
     * @return Returns a int
     */
    public int getPos()
    {
        return pos;
    }

    /**
     * Gets the row
     * 
     * @return Returns a int
     */
    public int getRow()
    {
        return row;
    }

    /**
     * debug output
     */
    public String toString()
    {
        String strRet = StringUtil.getShortClassName(this) + " pos: " + pos + " row: " + row
                + " col: " + col;

        return strRet;
    }// end method

    /**
     * Prft, ob das bergebene Objekt eine identische SourcePosition ist.
     *
     * @see Object#equals(Object)
     * @param other
     */
    public boolean equals(Object other)
    {
        if (other == null)
        {
            return false;
        }

        if (other.getClass().equals(this.getClass()))
        {
            return this.pos == ((SourcePosition) other).pos
                    && this.row == ((SourcePosition) other).row
                    && this.col == ((SourcePosition) other).col;
        }

        return false;
    }

}

