package de.cnc.expression;

import javax.servlet.ServletContext;
import javax.servlet.http.*;

import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.util.*;

/**
 * This class contents the variables, localization and other runtime informations
 *
 * Hier werden die Variablen fr die im CnC-Framework eingebettet arbeitenden Version
 * der Expression-Library vorgehalten.
 *
 * Die Umgebung des CnC ist der JSP-Adressraum, request,
 * session und application.
 *
 * Mglich ist folgende Schreibweise:
 * session_user, request_form, application_config
 *
 * am 14.09.2004 umgestellt auf beliebig lange Bezeichner
 */
public class JspRuntimeEnvironment
    extends AbstractRuntimeEnvironment
{
    private HttpServletRequest request;

    private HttpServletResponse response;

    private ServletContext application;

    private boolean noValidateBool;

    /** ob der Cnc-Button schon geparst wurde zum Caching des Parsens */
    private boolean isCncButtonParsed = false;

    /** der gecachte Cnc-Button */
    private String cnc_button;

    /** dr gecachte Button-Index */
    private Integer buttInxIntgr;

   /**
    * Konstruktor mit bergabe der erfoderlichen
    * Kontext-Objekte der JSP-Servlet-Applikation.
    *
    * @param pRequest
    * @param pResponse
    * @param pApplication
    *
    * @todo: Umstellen auf CnC-Context statt request, response und application
    */
	public JspRuntimeEnvironment( //-
        final HttpServletRequest pRequest, //-
        final HttpServletResponse pResponse, //-
        final ServletContext pApplication, //--
        final boolean pNoValidateBool)
    {
        this.request = pRequest;
        this.response = pResponse;
        this.application = pApplication;
        this.noValidateBool = pNoValidateBool;
    }


    /**
     * Variable lesen
     *
     * @param pStrName Name der Variable
     * @return Wert der Variable
     */
   public Object getVariable(String pStrName)
        throws ExpressionEvaluationException
   {
       if ( "cnc_link".equals( pStrName ) || "cnc_action".equals( pStrName ) || "cnc_form_action".equals( pStrName ) )
       {
           // Abfrage action="..." der HTML-FORM oder Link HREF
           String requestAction = CncUtil.getAction(request); // util-methode zum lesen action aus request-uri
           Log.debug("requestAction" , requestAction);
           return requestAction;
       }
       if ( "cnc_button".equals( pStrName ) )
       {
           // Abfrage button="..." Submit-Button

            //Laufzeit-Optimierung, nur einmaliges Ermitteln des cnc_buttons
            //Voraussetzung ist das jedesmalige Neuinstanziieren einer JsP-Runtime-Umgebung
            if ( ! isCncButtonParsed)
            {
              cnc_button = CncUtil.getCncButtonName(request);
              isCncButtonParsed = true;
              Log.debug("cncButtonName" , cnc_button);
            }
           return cnc_button;
       }
       if ( "cnc_button_index".equals( pStrName ) )
       {
           // Abfrage index button="..." Submit-Button bei indiziertem Button fr Listenelement
            if ( buttInxIntgr != null )
            {
                return buttInxIntgr;
            }

           buttInxIntgr = new Integer( CncUtil.getCnCButtonIndex(request) );
           //Log.debug("cncButtonIndex" , "" + cncButtonIndex);
           return buttInxIntgr;
       }
       if ( "cnc_no_validate".equals( pStrName ) )
       {
           // Abfrage, ob ein Button mit der Option validate=false" bettigt wurde
           //Log.debug("cncButtonIndex" , "" + cncButtonIndex);

           return ( noValidateBool ? Boolean.TRUE : Boolean.FALSE );
       }
       if (pStrName.startsWith( "session:" ) )
       {
           HttpSession session = request.getSession();
//Log.debug( "session." + StringUtil.cutLeft( pStrName , "session." ) );

            Object obj = session.getAttribute( StringUtil.cutLeft( pStrName , "session:" ) );
//Log.debug( "obj: " +obj );
           return obj;
       }
      if (pStrName.startsWith( "request:param:" ) )
      {
          final String reqKey = StringUtil.cutLeft( pStrName , "request:param:" );
          //Log.debug( "request.param. reqKey" , reqKey );
          final String paramValue = request.getParameter( reqKey );
          //Log.debug( "paramValue" , paramValue );
         return paramValue;
      }
      if (pStrName.startsWith( "request:attr:" ) )
      {
          return request.getAttribute( StringUtil.cutLeft( pStrName , "request:attr:" ) );
      }
       if (pStrName.startsWith( "application:attr:" ) )
       {
           return application.getAttribute( StringUtil.cutLeft( pStrName , "application:attr:" ) );
       }
       if (pStrName.startsWith( "application:init:" ) )
       {
           return application.getInitParameter( StringUtil.cutLeft( pStrName , "application:init:" ) );
       }


       throw new ExpressionEvaluationException( 0, 0, "", "",
            "" + pStrName + " is no valid variable identifier, "
            + " cnc variable 'cnc_link' or 'cnc_action' or 'cnc_form_action' or 'cnc_button' or 'cnc_button_index' or 'cnc_no_validate' or "
            + " prefix 'request:param:', 'request:attr:', 'session:', 'application:attr:' or 'application:init:' expected");
   }

    /**
     * Variable schreiben
     *
     * @param paStrName
     * @param paObjValue
     * @throws ExpressionEvaluationException
     */
   public void setVariable(String pStrName, Object pObjValue)
      throws ExpressionEvaluationException
   {
//      if (!isVariableNameValid(pStrName))
//      {
//         throw new ExpressionEvaluationException(0, 0, "", "",
//               "variable name " + pStrName + " is invalid");
//      }
//      if (ReservedWord.isReservedWord(pStrName))
//      {
//         throw new ExpressionEvaluationException(0, 0, "", "",
//               "variable name " + pStrName + " is an reserved word");
//      }
      if (pStrName.startsWith( "session:" ) )
      {
          HttpSession session = request.getSession();
             // dieser Aufruf ist notwendig weil der Sh...-WebSphere die null nicht richtig zuweist
             // scheinbar eine Frage der Servlet-Spezifikations-Version
         CncUtil.setAttribute(session, StringUtil.cutLeft( pStrName , "session:" ) , pObjValue );
          return;
      }
      if (pStrName.startsWith( "request:attr:" ) )
      {
          request.setAttribute( StringUtil.cutLeft( pStrName , "request:attr:" ) , pObjValue );
          return;
      }
      if (pStrName.startsWith( "application:attr:" ) )
      {
         if (pObjValue == null)
         {
            // dieser Zweig ist notwendig weil der Sh...-WebSphere die null nicht richtig zuweist
            // scheinbar eine Frage der Servlet-Spezifikations-Version
            application.removeAttribute( StringUtil.cutLeft( pStrName , "application:attr:" ) );
         }
         else
         {
            application.setAttribute( StringUtil.cutLeft( pStrName , "application:attr:" ) , pObjValue );
         }
          return;
      }

      throw new ExpressionEvaluationException( 0, 0, "", "",
           "prefix 'request:attr:', 'session:' or 'application:attr:' expected");
   }

    /**
     * Variable lschen
     * @param paStrName
     * @return the removed value
     */
   public Object removeVariable(String pStrName)
    throws ExpressionEvaluationException
   {
       if (pStrName.startsWith( "session:" ) )
       {
           final HttpSession session = this.request.getSession();
           final Object retObj = session.getAttribute( pStrName );
           session.removeAttribute( pStrName );
           return retObj;
       }
       if (pStrName.startsWith( "application:attr:" ) )
       {
           final Object retObj = this.application.getAttribute( pStrName );
           this.application.removeAttribute( pStrName );
           return retObj;
       }
      if (pStrName.startsWith( "request:attr:" ) )
      {
          final Object retObj = this.request.getAttribute( pStrName );
          this.request.removeAttribute( pStrName );
          return retObj;
      }

       throw new ExpressionEvaluationException( 0, 0, "", "",
            "prefix 'request:attr:', 'session:' or 'application:attr:' expected");
   }

    /**
     * Debug-Ausgabe
     */
   public String toString()
   {
      return getClass().getName() + " " + request;
   }

   // first char an letter, rest chars letter or digit
   // maximal length Variable.VAR_NAME_MAX_LEN
   private static boolean isVariableNameValid(String paStrName)
   {
//      if (paStrName.length() > Variable.VAR_NAME_MAX_LEN)
//      {
//         return false;
//      }
      if (paStrName.length() > 0 && Character.isJavaIdentifierStart(paStrName.charAt(0)))
      {
         for (int i = 1; i < paStrName.length(); i++)
         {
            char cTmp = paStrName.charAt(i);
            if ( ! ( Character.isJavaIdentifierPart(cTmp) 
                    || cTmp == ':' // Doppelpunkt im Variablen-Namen erlaubt
                        && i < paStrName.length() - 1
                        && Character.isJavaIdentifierStart(paStrName.charAt( i + 1 ) ) ) ) 
            {
              return false;
            }
         }
         return true;
      }
      return false;
   }// end method

   /**
    * formatierte Ausgabe als HTML
    */
   public String toHtmlString()
   {
return "";

// noch fertig programmieren

//      StringBuffer sb = new StringBuffer();
//      Set keySet = clHashMap.keySet();
//      Iterator it = keySet.iterator();
//
//      while (it.hasNext())
//      {
//         String strKey = (String) it.next();
//
//         sb.append("<b>"+strKey + ": </b>");
//         Object oValue = clHashMap.get(strKey);
//
//         if (oValue != null && oValue.getClass().isArray())
//         {
//            // ein Array
//            sb.append("Array length:" + Array.getLength(oValue) + " [");
//            for (int i = 0; i < Array.getLength(oValue); i++)
//            {
//               sb.append(Array.get(oValue, i));
//               if (i < Array.getLength(oValue)-1)
//               {
//                  sb.append(", ");
//               }
//            }
//            sb.append("]");
//         }
//         else
//         {
//            sb.append(oValue);
//         }
//         sb.append("<br><br>");
//      }
//      return sb.toString();
//
   }// end method toHtmlString

}// end class

