package de.cnc.expression;


import java.lang.reflect.Array;
import java.util.*;

import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.tokensimple.ReservedWord;


/**
 * This class contents the variables, localization and other runtime informations
 * 
 * Hier werden die Variablen fr die allein arbeitenden Version der
 * Expression-Library vorgehalten.
 */
public class StandaloneRuntimeEnvironment
    extends AbstractRuntimeEnvironment
{

   /** HashMap mit den Variablenwerten */
   private HashMap clHashMap = new HashMap();

    /**
     * Variable lesen
     * 
     * @param paStrName Name der Variable
     * @return Wert der Variable
     */
   public Object getVariable(String paStrName)
   {
      return clHashMap.get(paStrName);
   }

    /**
     * Variable schreiben
     * 
     * @param paStrName
     * @param paObjValue
     * @throws ExpressionEvaluationException
     */
   public void setVariable(String paStrName, Object paObjValue)
      throws ExpressionEvaluationException
   {
      if (!isVariableNameValid(paStrName))
      {
         throw new ExpressionEvaluationException(0, 0, "", "",
               "variable name " + paStrName + " is invalid");
      }
      if (ReservedWord.isReservedWord(paStrName))
      {
         throw new ExpressionEvaluationException(0, 0, "", "",
               "variable name " + paStrName + " is an reserved word");
      }
      clHashMap.put(paStrName, paObjValue);
   }

    /**
     * Variable lschen
     * @param paStrName
     * @return the removed value
     */
   public Object removeVariable(String paStrName)
   {
      return this.clHashMap.remove(paStrName);
   }

    /**
     * Debug-Ausgabe
     */
   public String toString()
   {
      return getClass().getName() + " " + clHashMap;
   }

   // first char an letter, rest chars letter or digit
   // maximal length Variable.VAR_NAME_MAX_LEN
   private static boolean isVariableNameValid(String paStrName)
   {
//      if (paStrName.length() > Variable.VAR_NAME_MAX_LEN)
//      {
//         return false;
//      }
      if (paStrName.length() > 0 && Character.isJavaIdentifierStart(paStrName.charAt(0)))
      {
         for (int i = 1; i < paStrName.length(); i++)
         {
            char cTmp = paStrName.charAt(i);
            if ( cTmp == ':' // Doppelpunkt im Variablen-Namen erlaubt
                 /*&& i < Variable.VAR_NAME_MAX_LEN*/
                 && i < paStrName.length() - 1
                 && Character.isJavaIdentifierStart(paStrName.charAt( i + 1 ) ) ) {
            }
            else if (!(Character.isJavaIdentifierPart(cTmp))) {
              return false;
            }
         }
         return true;
      }
      return false;
   }// end method

   /**
    * formatierte Ausgabe als HTML
    */
   public String toHtmlString()
   {
      StringBuffer sb = new StringBuffer();
      Set keySet = clHashMap.keySet();
      Iterator it = keySet.iterator();

      while (it.hasNext())
      {
         String strKey = (String) it.next();

         sb.append("<b>"+strKey + ": </b>");
         Object oValue = clHashMap.get(strKey);

         if (oValue != null && oValue.getClass().isArray())
         {
            // ein Array
            sb.append("Array length:" + Array.getLength(oValue) + " [");
            for (int i = 0; i < Array.getLength(oValue); i++)
            {
               sb.append(Array.get(oValue, i));
               if (i < Array.getLength(oValue)-1)
               {
                  sb.append(", ");
               }
            }
            sb.append("]");
         }
         else
         {
            sb.append(oValue);
         }
         sb.append("<br><br>");
      }
      return sb.toString();

   }// end method toHtmlString

}// end class

