package de.cnc.expression;

import de.cnc.expression.assignmentoperators.AbstractAssignmentOperator;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.functions.AbstractFunction;
import de.cnc.expression.prepostfixoperators.AbstractPrePostfixOperator;
import de.cnc.expression.prepostfixoperators.MinusMinusOperator;
import de.cnc.expression.prepostfixoperators.PlusPlusOperator;
import de.cnc.expression.tokencomplex.AbstractDotOperator;
import de.cnc.expression.tokencomplex.Assignment;
import de.cnc.expression.tokencomplex.IndexOperator;
import de.cnc.expression.tokencomplex.MemberDotOperator;
import de.cnc.expression.tokencomplex.ParenthesisArea;
import de.cnc.expression.tokencomplex.Sequence;
import de.cnc.expression.tokencomplex.Variable;
import de.cnc.expression.tokensimple.NumberLiteral;
import de.cnc.expression.tokensimple.ReservedWord;
import de.cnc.expression.tokensimple.StringLiteral;
import de.cnc.expression.util.ParserUtil;

/**
 * eine Expression vor/zwischen/nach Operatoren mit den
 * dazugehrenden Prefix- und Postfix-Operatoren
 */
public class SubExpression extends Expression // AbstractToken
{

    protected AbstractToken subToken;
    protected AbstractPrePostfixOperator prefixOperator;
    protected AbstractPrePostfixOperator postfixOperator;

    public static SubExpression parse(
        String paStr,
        int paIntLine,
        int paIntCol,
        String paStrOriginalSource)
        throws ExpressionParseException
    {
        //DebugUtil.printMethodInfo();
        SubExpression subExp =
            new SubExpression(paStrOriginalSource, paIntLine, paIntCol);

        // weiss-zeichen berspringen
        int iWhites =
            ParserUtil.parseWhitesAndComments(
                paStr.substring(subExp.iSrcLength));

        subExp.iSrcLength += iWhites;

        // Prefix-Operator !b -i ++i --i
        AbstractPrePostfixOperator prefixOperator =
            AbstractPrePostfixOperator.parsePrefixOperator(
                paStr.substring(subExp.iSrcLength),
                paIntLine,
                paIntCol + subExp.iSrcLength,
                paStrOriginalSource);

        // System.out.println( "prefixOperator:"+prefixOperator ) ;

        if (prefixOperator != null)
        {
            subExp.prefixOperator = prefixOperator;
            subExp.iSrcLength += prefixOperator.getSrcLength();
        }

        // System.out.println("92 subExp.iSrcLength "+subExp.iSrcLength);
        // weiss-zeichen berspringen
        iWhites =
            ParserUtil.parseWhitesAndComments(
                paStr.substring(subExp.iSrcLength));
        subExp.iSrcLength += iWhites;

        // System.out.println("97 subExp.subExp.iSrcLength "+subExp.iSrcLength);

        // subExpression vor/zwischen Operator
        AbstractToken token =
            parseSubExpression(
                paStr.substring(subExp.iSrcLength),
                paIntLine,
                paIntCol + subExp.iSrcLength,
                paStrOriginalSource);

        subExp.subToken = token;
        //System.out.println("57 token:" + token);
        subExp.iSrcLength += token.getSrcLength();
        //System.out.println("59 subExp.iSrcLength:" + subExp.iSrcLength);
        // subExp.iSrcLength+= subExp.leftsubExpression.iSrcLength;
        //System.out.println("61 subExp.iSrcLength:"+subExp.iSrcLength);

        //System.out.println( "63 paStr.substring(subExp.iSrcLength):"+paStr.substring(subExp.iSrcLength));

        // Postfix-Operator i++ i--
        AbstractPrePostfixOperator postfixOperator =
            AbstractPrePostfixOperator.parsePostfixOperator(
                paStr.substring(subExp.iSrcLength),
                paIntLine,
                paIntCol + subExp.iSrcLength,
                paStrOriginalSource);

        if (postfixOperator != null)
        {
            subExp.postfixOperator = postfixOperator;
            subExp.iSrcLength += postfixOperator.getSrcLength();
        }
        // ++ und -- darf nur bei variablen auftreten
        if (prefixOperator instanceof PlusPlusOperator
            || prefixOperator instanceof MinusMinusOperator
            || postfixOperator instanceof PlusPlusOperator
            || postfixOperator instanceof MinusMinusOperator)
        {
            if (!(token instanceof Assignable))
            {
                throw new ExpressionParseException(
                    paIntLine,
                    paIntCol,
                    paStrOriginalSource,
                    "operators ++ and -- only for variables available");
            }
        }
        return subExp;
    } // end method

    /**
     * Parse subExpression between two operators
     */
    public static AbstractToken parseSubExpression(
        String paStr,
        int paIntLine,
        int paIntCol,
        String paStrOriginalSource)
        throws ExpressionParseException
    {
        //DebugUtil.printMethodInfo();
        // System.out.println("parseSubExpression |" + paStr + "|");
        // System.out.println("parseSubExpression:" + paIntCol);
        // System.out.println("parseSubExpression:" + paStrOriginalSource);

        // dirty trick
        // Ausnutzung verkrzte Auflsung logischer Ausdrcke

        AbstractToken retToken = null;

        if (paStr.length() > 0)
        {
                boolean bFake = // ...
//                    ((retToken =
//                        Assignment.parse(paStr, paIntLine, paIntCol, paStrOriginalSource))
//                        != null)
//                        || // ...
        ((retToken =
            ParenthesisArea.parse(
                paStr,
                paIntLine,
                paIntCol,
                paStrOriginalSource))
            != null)
            || // ...
        ((retToken =
            NumberLiteral.parse(
                paStr,
                paIntLine,
                paIntCol,
                paStrOriginalSource))
            != null)
            || // ...
        ((retToken =
            StringLiteral.parse(
                paStr,
                paIntLine,
                paIntCol,
                paStrOriginalSource))
            != null)
            || // ...
        ((retToken =
            ReservedWord.parse(
                paStr,
                paIntLine,
                paIntCol,
                paStrOriginalSource))
            != null)
            || // ...
        ((retToken =
            Sequence.parse(paStr, paIntLine, paIntCol, paStrOriginalSource))
            != null)
            || // ...
        ((retToken =
            AbstractFunction.parse(
                paStr,
                paIntLine,
                paIntCol,
                paStrOriginalSource))
            != null)
            || // ...
    ((retToken =
        Variable.parse(paStr, paIntLine, paIntCol, paStrOriginalSource))
        != null);
            // ...
        }
//System.out.println( "retToken: " + retToken );

        if (retToken != null)
        {

            boolean bCont = true;
            while(bCont) // Schleife durchlaufen bis kein IndexOperator oder DotOperator mehr auftaucht
            {
                bCont = false;
                //Parsen auf eventuelles Index-Element []-Operator
                if ( true 
                    || retToken instanceof Variable
                    || retToken instanceof ParenthesisArea
                    || retToken instanceof Sequence
                    || retToken instanceof AbstractFunction)
                {
                    {
                    IndexOperator inxElem =
                        IndexOperator.parse(
                            paStr.substring(retToken.iSrcLength),
                            paIntLine,
                            paIntCol + retToken.iSrcLength,
                            paStrOriginalSource);
                    if (inxElem != null)
                    {
                        //Index-Operator vor den indizierten Ausdruck setzen (umkapseln)
                        inxElem.setIndexedExpression(retToken);
                        retToken = inxElem;
                        bCont = true;
                        continue;
                    }
                    //System.out.println( "inxElem: " + inxElem );
                    }

                    {
                    //Parser Dot-Operator
                        AbstractDotOperator dotElem =
                            AbstractDotOperator.parse(
                                paStr.substring(retToken.iSrcLength),
                                paIntLine,
                                paIntCol + retToken.iSrcLength,
                                paStrOriginalSource);
//System.out.println( "SubExpression#parseSubExpression dotElem 1: " + dotElem );
                        if (dotElem != null)
                        {
                            //Dot-Operator vor den indizierten Ausdruck setzen (umkapseln)
                            dotElem.setDottedExpression(retToken);
                            retToken = dotElem;

                            bCont = true;
                            continue;
                        }
//System.out.println( "SubExpression#parseSubExpression dotElem 2: " + dotElem );
                    }
                
            }

            }
            
            

            //Abarbeitung Assignment
            
//System.out.println( retToken.getClass().getName() );


            //eventuelle Zuweisung Assignment parsen
            if ( retToken instanceof Assignable)
            {
                int iRun = retToken.getSrcLength();
                // danach null bis beliebig viele weisse zeichen
                int iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
                iRun += iWhites;

                
                AbstractAssignmentOperator assOperator =
                    AbstractAssignmentOperator.parseAssignmentOperator(
                        paStr.substring(iRun),
                        paIntLine,
                        paIntCol + iRun,
                        paStrOriginalSource);

//System.out.println( assOperator );


                if (assOperator != null)
                {
                    iRun += assOperator.getSrcLength();

                    // danach null bis beliebig viele weisse zeichen
                    iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
                    iRun += iWhites;

                    Expression expr =
                        Expression.parseEasy(
                            paStr.substring(iRun),
                            paIntLine,
                            iRun + paIntCol,
                            paStrOriginalSource);

                    iRun += expr.getSrcLength();

                    if (expr == null)
                    {
                        // throw new ExpressionParseException(paIntLine, paIntCol + iRun, paStrOriginalSource, "expression expected");
                        return null;
                    }

                    // danach null bis beliebig viele weisse zeichen
                    if (iRun < paStr.length())
                    {
                        iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
                        iRun += iWhites;
                    }

                    //Assignment vor den indizierten Ausdruck setzen (umkapseln)
                    Assignment assignment = new Assignment(paStrOriginalSource, paIntLine, paIntCol);
                    assignment.destinationSubExpression = retToken;
                    assignment.operator = assOperator;
                    assignment.expression = expr;
                    assignment.strSource = paStr.substring(0, iRun);
                    assignment.iSrcLength = iRun;

                    retToken = assignment;
                }
                
                
            }
            
            return retToken;
        }
        // paStr.length()
        throw new ExpressionParseException(
            paIntLine,
            paIntCol,
            paStrOriginalSource,
            "no subExpression recognized");
    } // end method parseSubsubExpression

    /**
     * constructor
     */
    public SubExpression(
        String paStrOriginalSource,
        int paIntLine,
        int paIntCol)
    {
        super(paStrOriginalSource, paIntLine, paIntCol);
    } // end constructor

    /**
     * process the token
     */
    public Object eval(AbstractRuntimeEnvironment paRunEnv)
        throws ExpressionEvaluationException
    {
        Object retObject = null;

        if (this.prefixOperator != null)
        {
            // !b, -i, --i, ++i, i--, i++
            retObject = this.prefixOperator.eval(paRunEnv, this.subToken);
        }
        else
        {
            retObject = this.subToken.eval(paRunEnv);
        }

        // i++ oder i--
        if (this.postfixOperator != null)
        {
            this.postfixOperator.eval(paRunEnv, this.subToken);
        }

        return retObject;

    } // end method eval

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public int[] getCheckReturnTypes() throws ExpressionParseException
    {

        int[] retTypes = subToken.getCheckReturnTypes();

        if (prefixOperator != null)
        {
            retTypes = prefixOperator.getCheckReturnTypes(retTypes);
        }
        if (postfixOperator != null)
        {
            retTypes = postfixOperator.getCheckReturnTypes(retTypes);
        }

        return retTypes;
    } // end method getCheckReturnTypes

    public AbstractToken getToken()
    {
        return subToken;
    }

    /**
     * rekursives Aufrufen der Klammerung der Ausdrcke
     * entsprechend der Operator-Prioritt.
     * Weiterleiten an den enthaltenen Ausdruck
     */
    public void organizeOperatorPriority()
    {
        subToken.organizeOperatorPriority();
    }

    /**
     *
     */
    public String toString()
    {
        String strRet = getClass().getName();

        if (this.prefixOperator != null)
        {
            strRet += "\n" + this.prefixOperator;
        }
        strRet += "\n" + this.subToken;
        if (this.postfixOperator != null)
        {
            strRet += "\n" + this.postfixOperator;
        }

        return strRet;
    } // end method toString

} // end class SubExpression
