package de.cnc.expression;


/**
 * Erstmal luft die Typ-Prfung ber die hier festgelegten Konstanten
 * Spter sollen bei der Typ-Prfung Java-Klassen beachtet werden.
 * Dazu mssen die numerischen Operatoren aber auf alle numerischen Typen
 * umgestellt werden.
 * Dazu ist eine extra Klasse oder andere Mimik fr unbekannten Rckgabetyp
 * sowie keinen Rckgabe-Wert notwendig.
 */
public class Types {

  /**
   * private Konstruktor
   * instanziieren verboten
   */
  private Types() {}

  /**
   * return type unknown
   */
  public static final int RET_TYPE_UNKNOWN = 0;

  /**
   * return type null
   */
  public static final int RET_TYPE_NULL = 1;

  /**
   * return type boolean
   */
  public static final int RET_TYPE_BOOL = 2;

  /**
   * return type numeric
   */
  public static final int RET_TYPE_NUM = 3;

  /**
   * return type string
   */
  public static final int RET_TYPE_STRING = 4;

  /**
   * return type date
   */
  public static final int RET_TYPE_DATE = 5;

  /**
   * return type expression
   */
  public static final int RET_TYPE_EXPRESSION = 6;

  /**
   * return type variable
   */
  public static final int RET_TYPE_VARIABLE = 7;

  /**
   * return type string array
   * 
   * Eventuell wieder entfernen, da es RET_TYPE_LIST_OR_ARRAY gibt
   */
  public static final int RET_TYPE_STRING_ARRAY = 8;

  /**
   * return type string array
   */
  public static final int RET_TYPE_LIST_OR_ARRAY = 9;

  /**
   * Prfen, ob der Type-Code iPa im Type-Array iArrPa enthalten ist
   */
  public static boolean isIn(int iPa, int[] iArrPa) {
    if (iArrPa != null) {
      for (int i = 0; i < iArrPa.length; i++) {
        if (iArrPa[i] == iPa) {
          return true;
        }
      }
    }
    return false;
  }// end method isIn

  /**
   * Prfen, ob der Type-Code RET_TYPE_UNKNOWN im Type-Array iArrPa enthalten ist
   */
  public static boolean contentsUnknown(int[] iArrPa) {
    if (iArrPa != null) {
      for (int i = 0; i < iArrPa.length; i++) {
        if (iArrPa[i] == RET_TYPE_UNKNOWN) {
          return true;
        }
      }
    }
    return false;
  }// end method contentsUnknown

  /**
   * Prfen, ob der Type-Code iPa oder RET_TYPE_UNKNOWN
   * im Type-Array iArrPa enthalten ist
   */
  public static boolean isInOrUnknown(int iPa, int[] iArrPa) {
    if (iArrPa != null) {
      for (int i = 0; i < iArrPa.length; i++) {
        if (iArrPa[i] == iPa || iArrPa[i] == RET_TYPE_UNKNOWN) {
          return true;
        }
      }
    }
    return false;
  }// end method isInOrUnknown

/**
 * @param iTypesArr
 * @return
 */
public static String arr2str(int[] iTypesArr)
{
    String retStr = "[";
   for (int i = 0; i < iTypesArr.length; i++)
{
    retStr += typ2str(iTypesArr[i]);
    if (i < iTypesArr.length - 1)
    {
        retStr += ", ";
    }
}
    
    return retStr;
}

public static String typ2str(int pTypeInt)
{
    switch (pTypeInt)
    {
        case 0 :
            return "RET_TYPE_UNKNOWN";
            
            //break;

        case 1 :
            return "RET_TYPE_NULL";
            
            //break;

        case 2 :
            return "RET_TYPE_BOOL";
            
            //break;

        case 3 :
            return "RET_TYPE_NUM";
            
            //break;

        case 4 :
            return "RET_TYPE_STRING";
            
            //break;

        case 5 :
            return "RET_TYPE_DATE";
            
            //break;

        case 6 :
            return "RET_TYPE_EXPRESSION";
            
            //break;

        case 7 :
            return "RET_TYPE_VARIABLE";
            
            //break;

        case 8 :
            return "RET_TYPE_STRING_ARRAY";
            
            //break;

        default :
            return "???";
            //break;
    }


}

} // end class

