package de.cnc.expression.assignmentoperators;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;
import de.cnc.expression.tokencomplex.IndexOperator;
import de.cnc.expression.tokencomplex.Variable;


public abstract class AbstractAssignmentOperator {

  protected int iLine;
  protected int iCol;
  protected String strSource;
  protected int iSrcLength = 0;
  protected String strOriginalSource;

  /**
   * protected Konstruktor
   * instanziieren verboten
   */
  protected AbstractAssignmentOperator() {}

  /**
   * Konstruktor
   */
  public AbstractAssignmentOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    // System.out.println("constructor AbstractAssignmentOperator "+ paStrOriginalSource ) ;
    strOriginalSource = paStrOriginalSource;
    iLine = paIntLine;
    iCol = paIntCol;
  }

  /**
   * gets the the source line number
   */
  public int getLine() {
    return iLine;
  }

  /**
   * gets the col position in the source code line
   */
  public int getCol() {
    return iCol;
  }

  public String getSource() {
    return strSource;
  }

  public int getSrcLength() {
    return iSrcLength;
  }

  public String getOriginalSource() {
    return strOriginalSource;
  }

  /**
   * Prfroutine fr Variablen-Namen . <br>
   * @see de.cnc.expression.tokencomplex.Variable#parse
   */
  public static boolean isOperator(String strPa) {
    return
        ":=".equals(strPa) || // :=
        "+=".equals(strPa) || // +=
        "-=".equals(strPa) || // -=
        "*=".equals(strPa) || // *=
        "/=".equals(strPa) || // /=
        ":<=".equals(strPa) || // :<=
        ":>=".equals(strPa) || // :>=
        ":min=".equals(strPa) || // :min=
        ":max=".equals(strPa)  // :max=
        ;
  }// end method isOperator

  /**
   * Parse assignment operators
   *
   * @return geparsten Operator oder null wenn nichts passendes gefunden
   */
  public static AbstractAssignmentOperator parseAssignmentOperator(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException {
//DebugUtil.printMethodInfo();
//System.out.println( "parseAssignmentOperator paStr: >"+paStr+"<");

    // dirty trick
    // Ausnutzung verkrzte Auflsung logischer Ausdrcke

    AbstractAssignmentOperator retOperator = null;
    boolean bFake = // .
        ((retOperator = SetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
        ((retOperator = PlusSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
        ((retOperator = MinusSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
        ((retOperator = TimesSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
        ((retOperator = DivideSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
        ((retOperator = MinSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // .
((retOperator = MaxSetOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) ; //.

    return retOperator;

  }// end method parseInfixOperator

  /**
   * debug output
   */
  public String toString() {
    String strRet = getClass().getName() + " source: " + getSource();

    return strRet;
  }// end method toString

  /**
   * process the operator . <br>
   */
  abstract public Object eval(Assignable pInxOp, Object paObjLeft, AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException ;

  /**
   * rekursives Aufrufen der Klammerung der Ausdrcke
   * entsprechend der Operator-Prioritt.
   * Nichts zu tun.
   */
  void organizeOperatorPriority()
  {
      ;
  }
  
  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  // abstract public int[] getCheckReturnTypes()  throws ExpressionParseException ;
  abstract public int[] getCheckReturnTypes(int[] paIntArrLeft, int[] paIntArrRight)  throws ExpressionParseException;

  /**
   * vorgefertigte Typ-prfung auf Num-Parameter und Num-Rckgabewert
   */
  public static int[] checkNumNumNum(AbstractAssignmentOperator paAssOp, int[] paIntArrLeft, int[] paIntArrRight)
    throws ExpressionParseException {
    int[] iTypesArr = paIntArrLeft;

    if (!Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr)) {
      throw new ExpressionParseException(paAssOp.getLine(), paAssOp.getCol(), paAssOp.getOriginalSource(),
          "numeric type expected");
    }
    iTypesArr = paIntArrRight;
    if (!Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr)) {
      throw new ExpressionParseException(paAssOp.getLine(), paAssOp.getCol(), paAssOp.getOriginalSource(),
          "numeric type expected");
    }

    return new int[] {Types.RET_TYPE_NUM };
  }// end method getCheckReturnTypes

}// end class
