package de.cnc.expression.assignmentoperators;


import java.math.BigDecimal;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.Assignable;
import de.cnc.expression.exceptions.*;
import de.cnc.expression.tokencomplex.IndexOperator;
import de.cnc.expression.tokencomplex.Variable;


/**
 * Times Set Operator <b>*=</b> multiply and set
 */
public class TimesSetOperator extends AbstractAssignmentOperator {

  /**
   * parse
   */
  public static TimesSetOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {

    if ("*=".equals(paStr) || (paStr.length() > 2 && paStr.startsWith("*=") && !Character.isLetterOrDigit(paStr.charAt(2)))) {
      TimesSetOperator tmsSetOperator = new TimesSetOperator(paStrOriginalSource, paIntLine, paIntCol);

      tmsSetOperator.strSource = "*=";
      tmsSetOperator.iSrcLength = 2;
      return tmsSetOperator;
    }
    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no || operator");
  }// end method parse

  /**
   * constructor
   */
  public TimesSetOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator . <br>
   */
  public Object eval(Assignable paVariable, Object paObjRight, AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {

    Object objLeft = paVariable.eval(paRunEnv);

    if (objLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(objLeft instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + objLeft + ":" + objLeft.getClass().getName() + " ; BigDecimal expected");
    }

    if (!(paObjRight instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; BigDecimal expected");
    }
    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }

    String strLeftValue = "" + objLeft;
    String strRightValue = "" + paObjRight;

    // System.out.println("strLeftValue:" + strLeftValue);
    // System.out.println("strRightValue:" + strRightValue);

    Object objResult = (new BigDecimal(strLeftValue)).multiply(new BigDecimal(strRightValue));

    // System.out.println("objResult:" + objResult);

    paVariable.set(paRunEnv, objResult);

    return objResult;
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Times-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes(int[] paIntArrLeft, int[] paIntArrRight)  throws ExpressionParseException {
    return AbstractAssignmentOperator.checkNumNumNum(this,paIntArrLeft, paIntArrRight); // Rckgabe der zu setzenden Typen
  }// end method getCheckReturnTypes

}// end class
