package de.cnc.expression.exceptions;


/** 
 * Diese Klasse dient als Hilfslsung, solange
 * das Mitfhren der Zeilen und Spaltenposition
 * beim Parsen nicht richtig eingebaut ist,
 * um bei den Fehlermeldungen die richtige
 * Zeile und Spalte anzuzeigen.
 * 
 * @since 04.06.2004
 * @author Heiner Kcker
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 */
public class ExpressionExceptionUtil 
{

  /** 
   * Berechnen der korrekten Zeile der Fehlerposition
   * unter Beachtung der Zeilenumbrche.
   * 
   * @param pBaseLine aufzuaddierende Anfangslinie
   * @param pSource mehrzeiliger Quelltext
   * @param pNotLinedCol Zeichen-Position ab Quelltextanfang Basis Null
   * @return berechnete Zeile Basis Null
   */
  public static int computeLine(
      final int pBaseLine,
      final String pSource,
      final int pNotLinedCol
      ) {
    int retLine = pBaseLine;
    int runCol = 0;

    while (runCol < pSource.length() && runCol < pNotLinedCol) {
      char cTmp = pSource.charAt(runCol);

      if (cTmp == '\n') {
        retLine++;
      }
      runCol++;
    }
    return retLine;
  }

  /** 
   * Berechnen der korrekten Spalte der Fehlerposition
   * unter Beachtung der Zeilenumbrche.
   * 
   * @param pBaseLine aufzuaddierende Anfangslinie
   * @param pSource mehrzeiliger Quelltext
   * @param pNotLinedCol Zeichen-Position ab Quelltextanfang Basis Null
   * @return berechnete Spalte Basis Null
   */
  public static int computeCol(
      final int pBaseLine,
      final String pSource,
      final int pNotLinedCol
      ) {
    int retCol = 0;
    int runCol = 0;

    while (runCol < pSource.length() && runCol < pNotLinedCol) {
      char cTmp = pSource.charAt(retCol);

      if (cTmp == '\n') {
        retCol = 0;
      }
      else {
        retCol++;
      }
      runCol++;
    }
    return retCol;
  }

  /** 
   * Zurckgeben einer bestimmten Zeile aus dem Quellcode
   * 
   * @param pSource mehrzeiliger Quelltext
   * @param pLine Zeilen-Nummer auf Basis Null
   * @return gewnschte Zeile aus dem Quellcode
   */
  public static String getLine(
      final String pSource,
      final int pLine
      ) {
    // bis an die gewnschte Zeile ranlaufen
    int runLine = 0;
    int runCol = 0;

    while (runCol < pSource.length() && runLine < pLine) {
      char cTmp = pSource.charAt(runCol);

      if (cTmp == '\n') {
        runLine++;
      }
      runCol++;
    }

    // gewnschte Zeile in StringBuffer bertragen
    StringBuffer retStrBuff = new StringBuffer();

    while (runCol < pSource.length()) {
      char cTmp = pSource.charAt(runCol);

      if (cTmp == '\n') {
        break; // nchste Zeile erreicht
      }
      retStrBuff.append(cTmp);
      runCol++;
    }

    return retStrBuff.toString();
  }

}
