package de.cnc.expression.functions;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;
import de.cnc.expression.util.*;

/**
 * Parent class for all Built-In-Functions . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
abstract public class AbstractFunction extends AbstractToken
{

   /**
    * Lnge maximal 30 Zeichen.
    * umgestellt auf beliebige Lnge am 15.03.2005S
    */
   //final static int FUNC_NAME_MAX_LEN = 30;

   /**
    * name of function
    */
   protected String functionName;

   /**
    * parameter list.
    * TODO mal umstellen auf ExpressionArrayList
    */
   protected AbstractTokenArrayList paramList; // = new AbstractTokenArrayList();

   public static AbstractFunction parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
      throws ExpressionParseException
   { // , NoSuchTokenException {

//      // Function func = new Function(paStrOriginalSource, paIntLine, paIntCol);
//
//      // erste Stelle ein Buchstabe
//      // danach Buchstabe oder Ziffer
//
//      // Prfung auf Buchstabe am Anfang
//      //if (!Character.isLetter(paStr.charAt(0)))
//       if (!Character.isJavaIdentifierStart(paStr.charAt(0)))
//      {
//         return null;
//         // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no letter at start of function name");
//      }
//      int iRun = 1;
//
//      //for (; iRun < FUNC_NAME_MAX_LEN && iRun < paStr.length(); iRun++)
//      for (; iRun < paStr.length(); iRun++)
//      {
//         char cTmp = paStr.charAt(iRun);
//
//         //if (!(Character.isLetterOrDigit(cTmp)))
//         if (!(Character.isJavaIdentifierPart(cTmp)))
//         {
//            // Ende des Variablen-Namens erreicht
//            break;
//         }
//      }
//      // Ende des Funktions-Namens erreicht

       int iRun = ParserUtil.parseIdentifier(paStr);
       if (iRun < 1)
       {
          return null;
          // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no letter at start of function name");
       }
       
       String strFunctionName = paStr.substring(0, iRun);

      // Function-Name lnger als maximum
      //if (iRun >= FUNC_NAME_MAX_LEN)
      //{
      //   throw new ExpressionParseException(paIntLine, paIntCol, paStr, "function name to long");
      //}

      // danach null bis beliebig viele weisse zeichen
      int iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));

      // System.out.println("1iWhites:" + iWhites);
      // System.out.println("2iRun:" + iRun);
      iRun += iWhites;
      // System.out.println("3iRun:" + iRun);
      // System.out.println("4paStr.substring(iRun):" + paStr.substring(iRun));

      // danach eine ffnende Klammer
      if (!(iRun < paStr.length() && paStr.charAt(iRun) == '('))
      {
         return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no function");
      }
      iRun++;
      // danach null bis beliebig viele weisse zeichen
      iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
      // System.out.println("5iWhites:" + iWhites);
      // System.out.println("6iRun:" + iRun);
      iRun += iWhites;
      // System.out.println("7iRun:" + iRun);
      // System.out.println("8paStr.substring(iRun):" + paStr.substring(iRun));
      // System.out.println("9paStr.charAt(iRun):" + paStr.charAt(iRun));

      AbstractTokenArrayList paramListTmp = new AbstractTokenArrayList();

      iRun = parsePrmExprList(paStr, paIntLine, paIntCol, paStrOriginalSource, iRun, paramListTmp);

      String strSourceTmp = paStr.substring(0, iRun);
      int iSrcLengthTmp = iRun;

      AbstractFunction func = null;

      // hier mal eine generische Lsung machen ber Class.forName()
      if ("setVariableForName".equals(strFunctionName) || "setVarForName".equals(strFunctionName))
      {
         func = new SetVariableForNameFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      if ("setVariable".equals(strFunctionName) || "setVar".equals(strFunctionName))
      {
         func = new SetVariableFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      if ("removeVariableForName".equals(strFunctionName) || "removeVarForName".equals(strFunctionName))
      {
         func = new RemoveVariableForNameFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      if ("removeVariable".equals(strFunctionName) || "removeVar".equals(strFunctionName))
      {
         func = new RemoveVariableFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("substring".equals(strFunctionName))
      {
         func = new SubstringFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("startsWith".equals(strFunctionName))
      {
         func = new StartsWithFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("endsWith".equals(strFunctionName))
      {
         func = new EndsWithFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("upper".equals(strFunctionName))
      {
         func = new UpperFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("lower".equals(strFunctionName))
      {
         func = new LowerFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("length".equals(strFunctionName))
      {
         func = new LengthFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("chr".equals(strFunctionName) || // ...
      "char".equals(strFunctionName))
      {
         func = new CharFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("charCode".equals(strFunctionName) || // ...
      "asc".equals(strFunctionName))
      {
         func = new AscFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("left".equals(strFunctionName))
      {
         func = new LeftFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("right".equals(strFunctionName))
      {
         func = new RightFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("contents".equals(strFunctionName))
      {
         func = new ContentsFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("contentsIgnoreCase".equals(strFunctionName))
      {
         func = new ContentsIgnoreCaseFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("pad".equals(strFunctionName) || "padr".equals(strFunctionName))
      {
         func = new PadFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("padl".equals(strFunctionName))
      {
         func = new PadlFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("spaces".equals(strFunctionName))
      {
         func = new SpacesFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("str".equals(strFunctionName))
      {
         func = new StrFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("num".equals(strFunctionName))
      {
         func = new NumFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("sqrt".equals(strFunctionName))
      {
         func = new SqrtFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("max".equals(strFunctionName))
      {
         func = new MaxFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("min".equals(strFunctionName))
      {
         func = new MinFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("reciproc".equals(strFunctionName))
      {
         func = new ReciprocFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("isNull".equals(strFunctionName) || "isnull".equals(strFunctionName))
      {
         func = new IsNullFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if (
         "empty".equals(
            strFunctionName)) // || "isNullOrEmptyString".equals(strFunctionName) || "isnulloremptystring".equals(strFunctionName)) 
      {
         func = new IsEmptyFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("isNullOrSpaceString".equals(strFunctionName) || "isnullorspacestring".equals(strFunctionName))
      {
         func = new IsNullOrSpaceStringFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if (
         "isBool".equals(strFunctionName)
            || "isBoolean".equals(strFunctionName)
            || "isbool".equals(strFunctionName)
            || "isboolean".equals(strFunctionName))
      {
         func = new IsBoolFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if (
         "isNum".equals(strFunctionName)
            || "isNumber".equals(strFunctionName)
            || "isnum".equals(strFunctionName)
            || "isnumber".equals(strFunctionName))
      {
         func = new IsNumberFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if (
         "isStr".equals(strFunctionName)
            || "isString".equals(strFunctionName)
            || "isstr".equals(strFunctionName)
            || "isstring".equals(strFunctionName))
      {
         func = new IsStringFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("isExpression".equals(strFunctionName))
      {
         func = new IsExpressionFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("round".equals(strFunctionName))
      {
         func = new RoundFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("println".equals(strFunctionName) || "outPrintln".equals(strFunctionName))
      {
         func = new PrintlnFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("print".equals(strFunctionName) || "outPrint".equals(strFunctionName))
      {
         func = new PrintFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("if".equals(strFunctionName) || "iif".equals(strFunctionName))
      {
         func = new IfFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("while".equals(strFunctionName) || "iwhile".equals(strFunctionName))
      {
         func = new WhileFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("repeat".equals(strFunctionName) || "irepeat".equals(strFunctionName))
      {
         func = new RepeatFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("for".equals(strFunctionName) || "ifor".equals(strFunctionName))
      {
         func = new ForFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if (
         "breakSequence".equals(strFunctionName)
            || "breakSequ".equals(strFunctionName)
            || "returnSequ".equals(strFunctionName)
            || "returnSequence".equals(strFunctionName))
      {
         func = new BreakSequenceFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("continueSequence".equals(strFunctionName) || "contSequ".equals(strFunctionName))
      {
         func = new ContinueSequenceFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("indexOf".equals(strFunctionName) || "at".equals(strFunctionName) || "pos".equals(strFunctionName))
      {
         func = new IndexOfFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if (
         "lastIndexOf".equals(strFunctionName)
            || "rightAt".equals(strFunctionName)
            || "lastPos".equals(strFunctionName))
      {
         func = new LastIndexOfFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("compile".equals(strFunctionName))
      {
         func = new CompileFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("isValid".equals(strFunctionName) || "isvalid".equals(strFunctionName))
      {
         func = new IsValidFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("eval".equals(strFunctionName))
      {
         func = new EvalFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("newStringArray".equals(strFunctionName))
      {
         func = new NewStringArrayFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("type".equals(strFunctionName))
      {
         func = new TypeFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("isArray".equals(strFunctionName))
      {
         func = new IsArrayFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("trycatch".equals(strFunctionName))
      {
         func = new TryCatchFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("varExists".equals(strFunctionName))
      {
         func = new VariableExistsFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("newObject".equals(strFunctionName))
      {
         func = new NewObjectFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("sin".equals(strFunctionName))
      {
         func = new SinusFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("cos".equals(strFunctionName))
      {
         func = new CosinusFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("abs".equals(strFunctionName))
      {
         func = new AbsolutFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("acos".equals(strFunctionName))
      {
         func = new ArcusCosinusFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("asin".equals(strFunctionName))
      {
         func = new ArcusSinusFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("atan".equals(strFunctionName))
      {
         func = new ArcusTangusFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("ceil".equals(strFunctionName))
      {
         func = new CeilingFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("exp".equals(strFunctionName))
      {
         func = new ExponentialFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("floor".equals(strFunctionName))
      {
         func = new FloorFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("log".equals(strFunctionName))
      {
         func = new LogarithmusFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("pow".equals(strFunctionName))
      {
         func = new PowerFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("random".equals(strFunctionName))
      {
         func = new RandomFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("tan".equals(strFunctionName))
      {
         func = new TangusFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("toDegrees".equals(strFunctionName))
      {
         func = new ToDegreesFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("toRadians".equals(strFunctionName))
      {
         func = new ToRadiansFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("strToDate".equals(strFunctionName))
      {
         func = new StrToDateFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("trim".equals(strFunctionName))
      {
         func = new TrimFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("add".equals(strFunctionName))
      {
         func = new AddFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("remove".equals(strFunctionName))
      {
         func = new RemoveFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("getProperty".equals(strFunctionName))
      {
         func = new GetPropertyFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("setProperty".equals(strFunctionName))
      {
         func = new SetPropertyFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("invokeMethod".equals(strFunctionName))
      {
         func = new InvokeMethodFunction(paStrOriginalSource, paIntLine, paIntCol);
      }
      else if ("compileTemplate".equals(strFunctionName))
      {
         func = new CompileTemplateFunction(paStrOriginalSource, paIntLine, paIntCol);
      }

      if (func == null)
      {
         // // return a whormhole marker object for error message "Unknown Function"
         // func = new UnknownFunction(strFunctionName);
         // //func.functionName = strFunctionName;
         // return func;

         throw new ExpressionParseException(paIntLine, paIntCol, paStr, "unknown function " + strFunctionName);
         // throw new ExpressionParseException(paIntLine, exp.getSrcLength() + iWhites, paStr,
         // "unknown function " + ((AbstractFunction) subExp).getFunctionName());
      }

      func.functionName = strFunctionName;
      func.paramList = paramListTmp;
      func.strSource = strSourceTmp;
      func.iSrcLength = iSrcLengthTmp;
      func.strOriginalSource = paStrOriginalSource;
      return func;

   } // end method parse

   /**
    * @param paStr
    * @param paIntLine
    * @param paIntCol
    * @param paStrOriginalSource
    * @param iRun
    * @param paramListTmp
    * @return
    * @throws ExpressionParseException
    */
   public static int parsePrmExprList(//--
           final String paStr, //--
           final int paIntLine, //--
           final int paIntCol, //-- 
           final String paStrOriginalSource, //-- 
           int iRun, //-- 
           final AbstractTokenArrayList paramListTmp) //--
       throws ExpressionParseException
   {
//System.out.println( "AbstractFunction parsePrmExprList: " + paStr );
       int iWhites;
       if (iRun < paStr.length() && paStr.charAt(iRun) == ')')
         {
            // schliessende Klammer folgt sofort
            iRun++;
         }
         else
         {
            // es gibt Parameter
            // while (iRun < paStr.length() && paStr.charAt(iRun) != ')') {
            while (iRun < paStr.length())
            {
               // System.out.println("10paStr.substring(iRun):" + paStr.substring(iRun));
               paramListTmp.add(
                  Expression.parseEasy(paStr.substring(iRun), paIntLine, paIntCol + iRun, paStrOriginalSource));
               iRun += paramListTmp.tail().getSrcLength();

               // danach null bis beliebig viele weisse zeichen
               iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
               iRun += iWhites;
               // System.out.println("12iWhites:" + iWhites);
               // System.out.println("13iRun:" + iRun);
               // System.out.println("14paStr.charAt(iRun):" + paStr.charAt(iRun));
               // System.out.println("iRun < paStr.length(): " + (iRun < paStr.length()));
               // System.out.println("paStr.charAt(iRun) == ',': " + (paStr.charAt(iRun) == ','));
               // Komma fr nchsten Parameter
               if (iRun < paStr.length() && paStr.charAt(iRun) == ',')
               {
                  iRun++;
               }
               // else if (iRun < paStr.length() && paStr.charAt(iRun) != ')') {
               else if (iRun < paStr.length() && paStr.charAt(iRun) == ')')
               {
                  iRun++;
                  break;
               }
               else
               {
                  throw new ExpressionParseException(
                     paIntLine,
                     paIntCol + iRun,
                     paStrOriginalSource,
                     "right parenthesis ')' or comma ',' expected");
               }

               // danach null bis beliebig viele weisse zeichen
               if (iRun < paStr.length())
               {
                  iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
                  iRun += iWhites;
               }
            } // while
         } // if
       return iRun;
       // schliessende Klammer
       // if (!(iRun < paStr.length() && paStr.charAt(iRun) == ')')) {
       // // keine schliessende Klammer gefunden
       // // throw new ExpressionParseException(paIntLine, paIntCol + iRun, paStr, "')' expected");
       // throw new ExpressionParseException(paIntLine, paIntCol + iRun, paStrOriginalSource, "')' expected");
       // }
       // iRun++;
   }

   /**
    * constructor
    */
   public AbstractFunction(String paStrOriginalSource, int paIntLine, int paIntCol)
   {
      super(paStrOriginalSource, paIntLine, paIntCol);
   } // end constructor

   public String getFunctionName()
   {
      return functionName;
   }

   public String toString()
   {
      String strRet =
         getClass().getName()
            + " srcLength: "
            + getSrcLength()
            + " source: "
            + getSource()
            + " function name: "
            + functionName
            + "\n"
            + "params: "
            + paramList;

      return strRet;
   }

   // --------------------- evaluation section --------------------------------------------------------------

   /**
    * process the token
    */
   abstract public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException;

   /**
    * get and check (for null) an string paramter from the parameter list
    */
   protected String getStringParamNoNull(AbstractRuntimeEnvironment paRunEnv, int paIntPos)
      throws ExpressionEvaluationException
   {
      String strValue = null;

      try
      {
         if (paIntPos < paramList.size())
         {
            strValue = (String) paramList.get(paIntPos).eval(paRunEnv);
         }
      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "" + (paIntPos + 1) + "th parameter is no string " + e);
      }
      if (strValue == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "" + (paIntPos + 1) + "th parameter missed");
      }
      return strValue;
   } // end method

   /**
    * get and check (for null) an number paramter from the parameter list
    */
   protected Number getNumberParamNoNull(AbstractRuntimeEnvironment paRunEnv, int paIntPos)
      throws ExpressionEvaluationException
   {
      Number numValue = null;

      try
      {
         if (paIntPos < paramList.size())
         {
            numValue = (Number) paramList.get(paIntPos).eval(paRunEnv);
         }
      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "" + (paIntPos + 1) + "th parameter is no number " + e);
      }
      if (numValue == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "" + (paIntPos + 1) + "th parameter missed");
      }
      return numValue;
   } // end method

   /**
    * get without check (for null) an number paramter from the parameter list
    */
   protected Number getNumberParamNullable(AbstractRuntimeEnvironment paRunEnv, int paIntPos)
      throws ExpressionEvaluationException
   {
      Number numValue = null;

      try
      {
         if (paIntPos < paramList.size())
         {
            numValue = (Number) paramList.get(paIntPos).eval(paRunEnv);
         }

      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "" + (paIntPos + 1) + "th parameter is no number " + e);
      }
      return numValue;
   } // end method

   /**
    * get and check (for null) an boolean paramter from the parameter list
    */
   protected Boolean getBooleanParamNoNull(AbstractRuntimeEnvironment paRunEnv, int paIntPos)
      throws ExpressionEvaluationException
   {
      Boolean boolValue = null;

      try
      {
         if (paIntPos < paramList.size())
         {
            boolValue = (Boolean) paramList.get(paIntPos).eval(paRunEnv);
         }
      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "" + (paIntPos + 1) + "th parameter is no boolean " + e);
      }
      if (boolValue == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "" + (paIntPos + 1) + "th parameter missed");
      }
      return boolValue;
   } // end method

   /**
    * get and check (for null) an expression paramter from the parameter list
    */
   protected Expression getExpressionParamNoNull(AbstractRuntimeEnvironment paRunEnv, int paIntPos)
      throws ExpressionEvaluationException
   {
      Expression expValue = null;

      try
      {
         if (paIntPos < paramList.size())
         {
            expValue = (Expression) paramList.get(paIntPos).eval(paRunEnv);
         }
      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "" + (paIntPos + 1) + "th parameter is no Expression " + e);
      }
      if (expValue == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "" + (paIntPos + 1) + "th parameter missed");
      }
      return expValue;
   } // end method

   /**
    * get an object paramter from the parameter list
    */
   protected Object getObjectParamNullable(AbstractRuntimeEnvironment paRunEnv, int paIntPos)
      throws ExpressionEvaluationException
   {
      if (paIntPos < paramList.size())
      {
         return paramList.get(paIntPos).eval(paRunEnv);
      }
      return null;
   } // end method

   /**
    * get an object paramter from the parameter list
    */
   protected Object getObjectParamNoNull(AbstractRuntimeEnvironment paRunEnv, int paIntPos)
      throws ExpressionEvaluationException
   {
      Object objValue = null;

      if (paIntPos < paramList.size())
      {
         objValue = paramList.get(paIntPos).eval(paRunEnv);
      }
      if (objValue == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "" + (paIntPos + 1) + "th parameter missed");
      }
      return objValue;
   } // end method

   // --------------------- organizeOperatorPriority section ------------------------------------------------

   /**
    * rekursives Aufrufen der Klammerung der Ausdrcke
    * entsprechend der Operator-Prioritt.
    * Weiterleiten an die Parameter-Liste
    */
   public void organizeOperatorPriority()
   {
       if (this.paramList != null)
       {
           for (int i = 0; i < this.paramList.size(); i++)
           {
              this.paramList.get(i).organizeOperatorPriority();
           }
       }
   }

   // --------------------- type check section --------------------------------------------------------------

   /**
    * get the wahrscheinlichen return typen . <br>
    * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
    * null wenn kein Rckgabe-Wert,
    * mehrere wenn mehrere mglich
    * ( z.B. Plus-Operator kann numerisch oder String liefern)
    */
   abstract public int[] getCheckReturnTypes() throws ExpressionParseException;

   /**
    * check parameters count and throw ExpressionParseException if not properly
    */
   protected void checkParamCount(int paIntCount) throws ExpressionParseException
   {
      if (paramList.size() != paIntCount)
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "" + paIntCount + " parameter" + (paIntCount > 1 ? "s" : "") + " expected");
      }
   } // end method

   /**
    * check parameters count and throw ExpressionParseException if not properly
    */
   protected void checkParamCount(int paIntCountFrom, int paIntCountTo) throws ExpressionParseException
   {
      if (paramList.size() < paIntCountFrom || paramList.size() > paIntCountTo)
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "from " + paIntCountFrom + " to " + paIntCountTo + " parameters expected");
      }
   } // end method

   /**
    * check parameters count minimum and throw ExpressionParseException if not properly
    */
   protected void checkParamCountMinimum(int paIntCount) throws ExpressionParseException
   {
      if (paramList.size() < paIntCount)
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "minimum " + paIntCount + " parameter" + (paIntCount > 1 ? "s" : "") + " expected");
      }
   } // end method

   /**
    * check paramter for string or unknown
    */
   protected void checkParameterForString(int paIntPos) throws ExpressionParseException
   {
      //@todo: hier berall noch die Listen grsse gegen java.lang.IndexOutOfBoundsException absichern
      //und eigene Exception werden
      int[] iTypesArr = paramList.get(paIntPos).getCheckReturnTypes();

      if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr)))
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "string as " + (paIntPos + 1) + "th parameter expected");
      }
   } // end method

   /**
    * check paramter for number or unknown
    */
   protected void checkParameterForNumber(int paIntPos) throws ExpressionParseException
   {
      int[] iTypesArr = paramList.get(paIntPos).getCheckReturnTypes();

      if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr)))
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "number as " + (paIntPos + 1) + "th parameter expected");
      }
   } // end method

   /**
    * check paramter for string or number or unknown
    */
   protected void checkParameterForStringOrNumber(int paIntPos) throws ExpressionParseException
   {
      int[] iTypesArr = paramList.get(paIntPos).getCheckReturnTypes();

      if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr)
         || // ...
      Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr)))
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "string as " + (paIntPos + 1) + "th parameter expected");
      }
   } // end method

   /**
    * check paramter for boolean or unknown
    */
   protected void checkParameterForBoolean(int paIntPos) throws ExpressionParseException
   {
      int[] iTypesArr = paramList.get(paIntPos).getCheckReturnTypes();

      if (!(Types.isInOrUnknown(Types.RET_TYPE_BOOL, iTypesArr)))
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "boolean as " + (paIntPos + 1) + "th parameter expected");
      }
   } // end method

   /**
    * check paramter for expression or unknown
    */
   protected void checkParameterForExpression(int paIntPos) throws ExpressionParseException
   {
      int[] iTypesArr = paramList.get(paIntPos).getCheckReturnTypes();

      if (!(Types.isInOrUnknown(Types.RET_TYPE_EXPRESSION, iTypesArr)))
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "expression as " + (paIntPos + 1) + "th parameter expected");
      }
   } // end method

   /**
    * check paramter for expression or string or unknown
    */
   protected void checkParameterForExpressionOrString(int paIntPos) throws ExpressionParseException
   {
      int[] iTypesArr = paramList.get(paIntPos).getCheckReturnTypes();

      if (!(Types.isInOrUnknown(Types.RET_TYPE_EXPRESSION, iTypesArr)
         || // ...
      Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr)))
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "expression as " + (paIntPos + 1) + "th parameter expected");
      }
   } // end method

   /**
    * check paramter for variable or unknown
    */
   protected void checkParameterForVariable(int paIntPos) throws ExpressionParseException
   {
      int[] iTypesArr = paramList.get(paIntPos).getCheckReturnTypes();

      if (!(Types.isInOrUnknown(Types.RET_TYPE_VARIABLE, iTypesArr)))
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "variable as " + (paIntPos + 1) + "th parameter expected");
      }
   } // end method

   /**
    * check paramter for string or unknown
    */
   protected void checkParameterForListOrArray(int paIntPos) throws ExpressionParseException
   {
      int[] iTypesArr = paramList.get(paIntPos).getCheckReturnTypes();

      if ( ! ( Types.isInOrUnknown(Types.RET_TYPE_STRING_ARRAY , iTypesArr) 
          || Types.isInOrUnknown(Types.RET_TYPE_LIST_OR_ARRAY , iTypesArr) ) )
      {
         throw new ExpressionParseException(
            getLine(),
            getCol(),
            getOriginalSource(),
            "list or array as " + (paIntPos + 1) + "th parameter expected");
      }
   } // end method

} // end class
