package de.cnc.expression.functions;

import java.util.List;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;
import de.cnc.util.StringUtil;

/**
 * add( &lt;List or Array&gt; , &lt;value&gt; [ , &lt;index&gt; ] ) . <br>
 * Anfgen eines Wertes an eine {@link java.util.List} oder eine Array
 */
public class AddFunction extends AbstractFunction
{

   /**
    * constructor
    */
   public AddFunction(String paStrOriginalSource, int paIntLine, int paIntCol)
   {
      super(paStrOriginalSource, paIntLine, paIntCol);
   } // end constructor

   /**
    * process the token
    */
   public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException
   {
      Object oListOrArray = paramList.get(0).eval(paRunEnv);
      Object addValue = paramList.get(1).eval(paRunEnv);
      Number listIndex = getNumberParamNullable(paRunEnv, 2);

      if (oListOrArray == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "first parameter missed");
      }
      if ( oListOrArray instanceof List )
      {
         List oList = (List) oListOrArray;
         if (listIndex == null)
         {
            //kein Index
            oList.add( addValue );
         }
         else
         {
            //Index-Angabe
            oList.add( listIndex.intValue() , addValue );
         }
         
         return oList;
      }
      if ( oListOrArray.getClass().isArray() )
      {
         Object[] oArr = (Object[]) oListOrArray;
         Object[] retArr = StringUtil.arrAdd( oArr , addValue );
         return retArr;
      }
      else
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "list or array expected");
      }

      //try
      //{
      //   Object oValue = paramList.get(1).eval(paRunEnv);

      //   return oValue;
      //}
      //catch (Exception e)
      //{
      //   throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      //}

   } // end method eval

   /**
    * get the wahrscheinlichen return typen . <br>
    * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
    * null wenn kein Rckgabe-Wert,
    * mehrere wenn mehrere mglich
    * ( z.B. Plus-Operator kann numerisch oder String liefern)
    */
   public int[] getCheckReturnTypes() throws ExpressionParseException
   {
      checkParamCount(2 , 3);

      checkParameterForListOrArray(0);

      if (paramList.size() > 2)
      {
         checkParameterForNumber(2);
      }

      return new int[] { Types.RET_TYPE_LIST_OR_ARRAY };
   } // end method getCheckReturnTypes

} // end class
