package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * compileTemplate( &lt;valueString&gt; ) . <br>
 * bersetzungsfunktion-Function fr CnC-Templates.<br>
 * <pre>
 *   &lt;% ...scriptlet code... %&gt;
 *
 *  &lt;%= ...scriptlet expression... %&gt;
 * </pre>
 *
 */
public class CompileTemplateFunction extends AbstractFunction {

  /**
   * constructor
   */
  public CompileTemplateFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
      String strValue = getStringParamNoNull(paRunEnv, 0);

    try {
      return Expression.compileTemplate(strValue);
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      //return "";
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(1);

    checkParameterForString(0);

    return new int[] {Types.RET_TYPE_STRING };
  }// end method getCheckReturnTypes

} // end class
