package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * contentsIgnoreCase( &lt;valueString&gt; , &lt;suchString&gt; [ , &lt;endIndex(base0)&gt; ] ) . <br>
 * contentsIgnoreCase-Function liefert zurck, ob suchString in valueString enthalten ist
 * unter Ignorierung von Unterschieden in der Gross-/Klein-Schreibung
 */
public class ContentsIgnoreCaseFunction extends AbstractFunction {

  /**
   * constructor
   */
  public ContentsIgnoreCaseFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    String strValue = getStringParamNoNull(paRunEnv, 0);
    String strSuch = getStringParamNoNull(paRunEnv, 1);

    if (paramList.size() < 3) {

      try {
        return (strValue.toLowerCase().indexOf(strSuch.toLowerCase()) > -1) ? Boolean.TRUE : Boolean.FALSE;
      } catch (Exception e) {
        // System.out.println( ""+e ) ;
        // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
        return Boolean.FALSE;
      }

    }

    Number numFrom = getNumberParamNoNull(paRunEnv, 2);

    try {
      return (strValue.toLowerCase().indexOf(strSuch.toLowerCase(), numFrom.intValue()) > -1) ? Boolean.TRUE : Boolean.FALSE;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      return Boolean.FALSE;
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(2, 3);
    checkParameterForString(0);
    checkParameterForString(1);
    if (paramList.size() > 2) {
      checkParameterForNumber(2);
    }
    return new int[] {Types.RET_TYPE_BOOL };
  }// end method getCheckReturnTypes

} // end class
