package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * endsWith( &lt;valueString&gt; , &lt;endString&gt; ) . <br>
 * endsWith-Function like java.lang.String.endsWith()
 */
public class EndsWithFunction extends AbstractFunction {

  /**
   * constructor
   */
  public EndsWithFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    String strValue = null;

    try {
      strValue = (String) paramList.get(0).eval(paRunEnv);
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "first parameter is no string " + e);
    }
    if (strValue == null) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "first parameter missed");
    }

    String strEnd = null;

    try {
      strEnd = (String) paramList.get(1).eval(paRunEnv);
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "second parameter is no number " + e);
    }
    if (strEnd == null) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "second parameter missed");
    }

    try {
      return (strValue.endsWith(strEnd)) ? Boolean.TRUE : Boolean.FALSE;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      return Boolean.FALSE;
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(2, 3);

    checkParameterForString(0);

    checkParameterForString(1);

    return new int[] {Types.RET_TYPE_BOOL };
  }// end method getCheckReturnTypes

} // end class
