package de.cnc.expression.functions;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;
import de.cnc.util.BeanUtil;

/**
 * <b>getProperty</b>( &lt;object&gt; , &lt;propertyName (String)&gt; )<br>
 * Abfragen eines Public-Feldes oder Aufruf der entsprechenden Public-getXy()-Methode per Reflection <br>
 */
public class GetPropertyFunction extends AbstractFunction
{

   /**
    * constructor
    */
   public GetPropertyFunction(String paStrOriginalSource, int paIntLine, int paIntCol)
   {
      super(paStrOriginalSource, paIntLine, paIntCol);
   } // end constructor

   /**
    * process the token
    */
   public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException
   {
      Object bean = null;

      try
      {
         bean = paramList.get(0).eval(paRunEnv);
      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "first parameter missed " + e);
      }
      if (bean == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "first parameter missed");
      }
      try
      {
         String propertyName = getStringParamNoNull(paRunEnv, 1);
         return BeanUtil.get(bean, propertyName);
      }
      catch (Exception e)
      {
         throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      }

   } // end method eval

   /**
    * get the wahrscheinlichen return typen . <br>
    * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
    * null wenn kein Rckgabe-Wert,
    * mehrere wenn mehrere mglich
    * ( z.B. Plus-Operator kann numerisch oder String liefern)
    */
   public int[] getCheckReturnTypes() throws ExpressionParseException
   {
      checkParamCount(2);
      checkParameterForVariable(0);
      checkParameterForString(1);

      return new int[] { Types.RET_TYPE_UNKNOWN };
   } // end method getCheckReturnTypes

} // end class
