package de.cnc.expression.functions;


import java.math.BigDecimal;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * indexOf( &lt;valueString&gt; , &lt;searchString&gt; [ , &lt;endIndex(base0)&gt; ] ) . <br>
 * at oder pos oder indexOf
 * indexOf-Function like java.lang.String#indexOf()
 */
public class IndexOfFunction extends AbstractFunction {

  /**
   * constructor
   */
  public IndexOfFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    String strValue = getStringParamNoNull(paRunEnv, 0);

    String strSearch = getStringParamNoNull(paRunEnv, 1);

    if (paramList.size() < 3) {

      try {
        return new BigDecimal(strValue.indexOf(strSearch));
      } catch (Exception e) {
        // System.out.println( ""+e ) ;
        throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
        // return new BigDecimal(-1);
      }
    }

    Number numFrom = getNumberParamNoNull(paRunEnv, 2);

    try {
      return new BigDecimal(strValue.indexOf(strSearch, numFrom.intValue()));
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
      // return new BigDecimal(-1);
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    // if (paramList.size() < 2 || paramList.size() > 3 ) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "2 or 3 parameters expected");
    // }
    checkParamCount(2, 3);

    // int[] iTypesArr = paramList.get(0).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "string as first parameter expected");
    // }

    checkParameterForString(0);

    // iTypesArr = paramList.get(1).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "string as second parameter expected");
    // }
    checkParameterForString(1);

    if (paramList.size() > 2) {

      // iTypesArr = paramList.get(2).getCheckReturnTypes();
      // if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))) {
      // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "number as 3rd parameter expected");
      // }
      checkParameterForNumber(2);

    }
    return new int[] {Types.RET_TYPE_BOOL };
  }// end method getCheckReturnTypes

} // end class
