package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * isArray( &lt;value&gt; ) . <br>
 * ermittelt, ob per Parameter bergebene Ausdrucks ein Array ist<br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class IsArrayFunction extends AbstractFunction {

  /**
   * constructor
   */
  public IsArrayFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    Object objValue = null;

    try {
      objValue = paramList.get(0).eval(paRunEnv);
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "first parameter missed " + e);
    }

    if (objValue == null) {
      return Boolean.FALSE;
    }
    return (objValue.getClass().isArray()) ? Boolean.TRUE : Boolean.FALSE;
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(1);

    return new int[] {Types.RET_TYPE_BOOL };
  }// end method getCheckReturnTypes

} // end class
