package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * isBool( &lt;objectvalue&gt; ) . <br>
 * isBoolean( &lt;objectvalue&gt; ) . <br>
 * isbool( &lt;objectvalue&gt; ) . <br>
 * isboolean( &lt;objectvalue&gt; ) . <br>
 * ermittelt ob der per Parameter bergebene Ausdruck vom Typ Boolean ist
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class IsBoolFunction extends AbstractFunction {

  /**
   * constructor
   */
  public IsBoolFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    Object objValue = getObjectParamNullable(paRunEnv,0);

    return (objValue instanceof Boolean)  ? Boolean.TRUE : Boolean.FALSE;
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(1);

    return new int[] {Types.RET_TYPE_BOOL };
  }// end method getCheckReturnTypes

} // end class
