package de.cnc.expression.functions;

import java.lang.reflect.Array;
import java.math.BigDecimal;
import java.util.Collection;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;

/**
 * length( &lt;valueString&gt; ) . <br>
 * length-Function like java.lang.String.length()
 */
public class LengthFunction extends AbstractFunction
{

   /**
    * constructor
    */
   public LengthFunction(String paStrOriginalSource, int paIntLine, int paIntCol)
   {
      super(paStrOriginalSource, paIntLine, paIntCol);
   } // end constructor

   /**
    * process the token
    */
   public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException
   {
      Object prmObj = paramList.get(0).eval(paRunEnv);
      int length = 0;

      if (prmObj == null)
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "parameter missed");
      }
      if (prmObj instanceof String)
      {
         length = ((String) prmObj).length();
      }
      else if (prmObj instanceof Collection)
      {
         length = ((Collection) prmObj).size();
      }
      else if (prmObj.getClass().isArray())
      {
         length = Array.getLength(prmObj);
      }
      else
      {
         throw new ExpressionEvaluationException(
            getLine(),
            getCol(),
            getSource(),
            getOriginalSource(),
            "string or collection or array expected");
      }

      try
      {
         return new BigDecimal(length);
      }
      catch (Exception e)
      {
         // System.out.println( ""+e ) ;
         // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
         return new BigDecimal(0);
      }

   } // end method eval

   /**
    * get the wahrscheinlichen return typen . <br>
    * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
    * null wenn kein Rckgabe-Wert,
    * mehrere wenn mehrere mglich
    * ( z.B. Plus-Operator kann numerisch oder String liefern)
    */
   public int[] getCheckReturnTypes() throws ExpressionParseException
   {
      checkParamCount(1);

      //checkParameterForString(0);

      return new int[] { Types.RET_TYPE_NUM };
   } // end method getCheckReturnTypes

} // end class
