package de.cnc.expression.functions;


import java.math.BigDecimal;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * max( &lt;valueNumber&gt; , &lt;valueNumber&gt; ) . <br>
 * liefert das Maximum zweier als Parameter bergebenen Werte
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class MaxFunction extends AbstractFunction {

  /**
   * constructor
   */
  public MaxFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    Number numValue1 = getNumberParamNoNull(paRunEnv, 0);
    Number numValue2 = getNumberParamNoNull(paRunEnv, 1);

    try {

      return new BigDecimal(Math.max(numValue1.doubleValue() , numValue2.doubleValue())) ;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      return "";
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(2);

    checkParameterForNumber(0);
    checkParameterForNumber(1);

    return new int[] {Types.RET_TYPE_NUM };
  }// end method getCheckReturnTypes

} // end class
