package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * newStringArray( &lt;numSize&gt; )<br>
 * newStringArray( &lt;str[0]&gt; [ , &lt;str[n]&gt; ] )<br>
 * Erzeugt ein neues String-Array in der verlangten Grsse (numerischer Parameter)
 * oder aus der bergebenen String-Parameter-Liste.<br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class NewStringArrayFunction extends AbstractFunction {

  /**
   * constructor
   */
  public NewStringArrayFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    // String strValue = getStringParamNoNull(paRunEnv, 0);

    Object objValue = getObjectParamNullable(paRunEnv, 0);

    if (objValue instanceof Number) {
      Number numValue = (Number) objValue;

      return new String[ numValue.intValue() ];
    }
    else if (objValue instanceof String) {
      String[] retStrArr = new String[ paramList.size() ];

      for (int i = 0; i < paramList.size(); i++) {
        retStrArr[i] = getStringParamNoNull(paRunEnv, i);
      }
      return retStrArr;
    }
    else {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "number or string parameter expected " + objValue);

    }

    // try {
    // return Expression.parseStrict(strValue, paramList.get(0).getLine(), paramList.get(0).getCol());
    // } catch (ExpressionParseException e) {
    // // System.out.println( ""+e ) ;
    // throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
    // // return "";
    // }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCountMinimum(1);
    if (paramList.size() == 1) {
      checkParameterForStringOrNumber(0);
    }
    else {
      for (int i = 0; i < paramList.size(); i++) {
        checkParameterForString(i);
      }
    }
    return new int[] {Types.RET_TYPE_STRING_ARRAY };
  }// end method getCheckReturnTypes

} // end class
