package de.cnc.expression.functions;


import java.math.BigDecimal;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * reciproc( &lt;valueNumber&gt; ) . <br>
 * liefert das Reziprok (1/x) des bergebenen Parameters
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class ReciprocFunction extends AbstractFunction {

  /**
   * constructor
   */
  public ReciprocFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    Number numValue = getNumberParamNoNull(paRunEnv, 0);

    try {

      return new BigDecimal(1 / numValue.doubleValue()) ;
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      return "";
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(1);

    checkParameterForNumber(0);

    return new int[] {Types.RET_TYPE_NUM };
  }// end method getCheckReturnTypes

} // end class
