package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;
import de.cnc.expression.tokencomplex.Variable;


/**
 * removeVariableForName( variable )                                                <br>
 * removeVarForName( variable )                                                     <br>
 * Lscht eine als literaler Variablen-Name zur Kompilationszeit bekannte Variable  <br>
 */
public class RemoveVariableForNameFunction extends AbstractFunction {

  /**
   * constructor
   */
  public RemoveVariableForNameFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token . <br>
   * set variable
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
      String varName = getStringParamNoNull(paRunEnv, 0);

      try
      {
          final AbstractToken setExpression = SetVariableForNameFunction.parseAssignable(varName, getLine(), getCol(), getOriginalSource());
          
          //varName.set(paRunEnv, paObjValue);
          Assignable assignable = (Assignable) setExpression;

          return assignable.remove(paRunEnv);
      }
      catch (Exception e)
      {
          throw new ExpressionEvaluationException(
              getLine(),
              getCol(),
              getSource(),
              getOriginalSource(),
              "" + e);
      }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    // if (paramList.size() != 2) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "2 parameters expected");
    // }

    checkParamCount(2);

    // int[] iTypesArr = paramList.get(0).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "string as first parameter expected");
    // }

    checkParameterForString(0);

    return new int[] {Types.RET_TYPE_UNKNOWN };
  }// end method getCheckReturnTypes

} // end class
