package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;
import de.cnc.expression.tokencomplex.Variable;


/**
 * removeVariable( variable )                                                         <br>
 * removeVar( variable )                                                    <br>
 * Lscht eine als literaler Variablen-Name zur Kompilationszeit bekannte Variable  <br>
 */
public class RemoveVariableFunction extends AbstractFunction {

  /**
   * constructor
   */
  public RemoveVariableFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token . <br>
   * remove variable
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    Assignable assignable = null;

    try {

      Expression expression = (Expression) paramList.get(0);
      SubExpression subExpression = expression.getFirstSubExpression();

      assignable = (Assignable) subExpression.getToken();

    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "first parameter is no variable " + e);
    }
    if (assignable == null) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "first parameter missed");
    }
    try {
        assignable.remove(paRunEnv);

      return null;
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(1);
    checkParameterForVariable(0);
    return new int[] {Types.RET_TYPE_NULL };
  }// end method getCheckReturnTypes

} // end class
