package de.cnc.expression.functions;


import java.math.BigDecimal;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * round( &lt;valueNumber&gt; ) . <br>
 * Runden mit optionaler Angabe der Stellen hinter dem Komma
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class RoundFunction extends AbstractFunction {

  /**
   * constructor
   */
  public RoundFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    // System.out.println( ""+this ) ;
    Number numValue = getNumberParamNoNull(paRunEnv, 0);
    Number numDigits = getNumberParamNullable(paRunEnv, 1);

    if (numDigits == null) {
      numDigits = new Integer(0);
    }
    // long lRoundFactor = Math.round(Math.pow(10.0, (double) numDigits.intValue()));
    // System.out.println( "lRoundFactor:"+lRoundFactor ) ;
    int iDigits = numDigits.intValue();

    if (iDigits < 0) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "to few digits for round " + iDigits + ", minimum is 0");
    }

    try {
      // return new BigDecimal(Math.round(numValue.doubleValue() * lRoundFactor) / lRoundFactor);
      BigDecimal bigDecValue = new BigDecimal(numValue.toString());

      // Runden durch Division durch eins und Ausnutzung Rundungs- und Skalierungsmechanik des BigDecimal
      return bigDecValue.divide(new BigDecimal(1), iDigits, BigDecimal.ROUND_HALF_EVEN);
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      return null;
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(1, 2);

    checkParameterForNumber(0);

    if (paramList.size() > 1) {
      // optionale Anzahl Dezimalstellen
      checkParameterForNumber(1);
    }
    return new int[] {Types.RET_TYPE_NUM };
  }// end method getCheckReturnTypes

} // end class
