package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;
import de.cnc.expression.tokencomplex.Variable;


/**
 * setVar( variable , &lt;expression&gt; )                                                         <br>
 * setVariable( variable , &lt;expression&gt; )                                                    <br>
 * Setzt eine als literaler Variablen-Name zur Kompilationszeit bekannte Variable  <br>
 */
public class SetVariableFunction extends AbstractFunction {

  /**
   * constructor
   */
  public SetVariableFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token . <br>
   * set variable
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    Assignable variable = null;

    try {

      Expression expression = (Expression) paramList.get(0);
      SubExpression subExpression = expression.getFirstSubExpression();

      variable = (Assignable) subExpression.getToken();

    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "1th parameter is no variable " + e);
    }
    if (variable == null) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "first parameter missed");
    }

    // 2th parameter
    try {
      Object oValue = paramList.get(1).eval(paRunEnv);

      variable.set(paRunEnv, oValue);

      // Rckgabe des gesetzten Wertes
      return oValue;
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(2);
    checkParameterForVariable(0);

    return new int[] {Types.RET_TYPE_UNKNOWN };
  }// end method getCheckReturnTypes

} // end class
