package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * substring( &lt;string value&gt; , &lt;beginIndex(base0)&gt; [ , &lt;endIndex(base0)&gt; ] ) . <br>
 * substring-Function like java.lang.String.substring()
 */
public class SubstringFunction extends AbstractFunction {

  /**
   * constructor
   */
  public SubstringFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    String strValue = null;

    try {
      strValue = (String) paramList.get(0).eval(paRunEnv);
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "first parameter is no string " + e);
    }
    if (strValue == null) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "first parameter missed");
    }

    Number numFrom = null;

    try {
      numFrom = (Number) paramList.get(1).eval(paRunEnv);
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "second parameter is no number " + e);
    }
    if (numFrom == null) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "second parameter missed");
    }

    // System.out.println( "numFrom " + numFrom ) ;

    if (paramList.size() < 3) {
      // string, beginIndex
      try {
        return strValue.substring(numFrom.intValue());
      } catch (Exception e) {
        // System.out.println( ""+e ) ;
        // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
        return "";
      }

    }

    // string, beginIndex, endIndex
    Number numTo = null;

    try {
      numTo = (Number) paramList.get(2).eval(paRunEnv);
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "3rd parameter is no number " + e);
    }
    if (numTo == null) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "3rd parameter missed");
    }
    // System.out.println( "numTo " + numTo ) ;
    try {
      return strValue.substring(numFrom.intValue(), numTo.intValue());
    } catch (Exception e) {
      // System.out.println( ""+e ) ;
      // throw new ExpressionEvaluationException(getLine(), getCol(),getSource() , getOriginalSource(), "" + e);
      return "";
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    // if (paramList.size() < 2 || paramList.size() > 3) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "2 or 3 parameters expected");
    // }
    checkParamCount(2, 3);

    // int[] iTypesArr = paramList.get(0).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_STRING, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "string as first parameter expected");
    // }
    checkParameterForString(0);

    // iTypesArr = paramList.get(1).getCheckReturnTypes();
    // if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))) {
    // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "number as second parameter expected");
    // }
    checkParameterForNumber(1);

    if (paramList.size() > 2) {

      // iTypesArr = paramList.get(1).getCheckReturnTypes();
      // if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))) {
      // throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "number as 3rd parameter expected");
      // }
      checkParameterForNumber(2);
    }

    return new int[] {Types.RET_TYPE_STRING };
  }// end method getCheckReturnTypes

} // end class
