package de.cnc.expression.functions;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * type( &lt;value&gt; ) . <br>
 * ermittelt den Typ des per Parameter bergebene Ausdrucks<br>
 *
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 */
public class TypeFunction extends AbstractFunction {

  /**
   * constructor
   */
  public TypeFunction(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    Object objValue = null;

    try {
      objValue = paramList.get(0).eval(paRunEnv);
    } catch (Exception e) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "first parameter missed " + e);
    }

    if (objValue == null) {
      return "NULL";
    }
    else if (objValue instanceof String || objValue instanceof StringBuffer) {
      return "STRING";
    }
    else if (objValue instanceof Number) {
      return "NUMBER";
    }
    else if (objValue instanceof Boolean) {
      return "BOOLEAN";
    }
    else if (objValue instanceof Expression) {
      return "EXPRESSION";
    }
    // else if ( objValue instanceof Date ){
    // return "DATE";
    // }
    else if (objValue instanceof String[]) {
      return "STRINGARRAY";
    }
    // noch weitere Arrays
    else if (objValue.getClass().isArray()) {
      return "ARRAY";
    }

    return "UNKNOWN";

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    checkParamCount(1);

    return new int[] {Types.RET_TYPE_STRING };
  }// end method getCheckReturnTypes

} // end class
