package de.cnc.expression.infixoperators;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;

public abstract class AbstractInfixOperator
{

   protected int iLine;
   protected int iCol;
   protected String strSource;
   protected int iSrcLength = 0;
   protected String strOriginalSource;

   /**
    * protected Konstruktor
    * instanziieren verboten
    */
   protected AbstractInfixOperator()
   {
   }

   /**
    * Konstruktor
    */
   public AbstractInfixOperator(String paStrOriginalSource, int paIntLine, int paIntCol)
   {
      // System.out.println("constructor AbstractInfixOperator "+ paStrOriginalSource ) ;
      strOriginalSource = paStrOriginalSource;
      iLine = paIntLine;
      iCol = paIntCol;
   }

   /**
    * gets the the source line number
    */
   public int getLine()
   {
      return iLine;
   }

   /**
    * gets the col position in the source code line
    */
   public int getCol()
   {
      return iCol;
   }

   public String getSource()
   {
      return strSource;
   }

   public int getSrcLength()
   {
      return iSrcLength;
   }

   public String getOriginalSource()
   {
      return strOriginalSource;
   }

   /**
    * Prfroutine zum Erkennen von Operatoren. <br>
    * @see de.cnc.expression.tokencomplex.Variable#parse
    */
   public static boolean isOperator(String strPa)
   {
      return "+".equals(strPa) || // +
      "-".equals(strPa) || // -
      "*".equals(strPa) || // *
      "**".equals(strPa) || // **
      "^".equals(strPa) || // ^
      "/".equals(strPa) || // /
      "%".equals(strPa) || // %
      "mod".equals(strPa) || // mod
      "and".equals(strPa) || // and
      "or".equals(strPa) || // or
      "nand".equals(strPa) || // nand
      "nor".equals(strPa) || // nor
      "==".equals(strPa) || // ==
      "<=".equals(strPa) || // <=
      "=<".equals(strPa) || // =<
      ">=".equals(strPa) || // >=
      "=>".equals(strPa) || // =>
      "!=".equals(strPa) || // !=
      "<>".equals(strPa) || // <>
      "&&".equals(strPa) || // &&
      "&".equals(strPa) || // &
      "and".equals(strPa) || // and
      "|".equals(strPa) || // |
      "||".equals(strPa) || // ||
      "or".equals(strPa) || // or
      "nand".equals(strPa) || // nand
      "nor".equals(strPa) || // nor
      "xor".equals(strPa) || // xor

      //Prefix-Operatoren
      "++".equals(strPa) || // ++
      "--".equals(strPa) || // --
      "-".equals(strPa) || // - ( * -1 )
      "!".equals(strPa) || // ! NOT

      //Assignment-Operatoren
      ":=".equals(strPa) || // Zuweisung
      "+=".equals(strPa) || // Zuweisung plus
      "-=".equals(strPa) || // Zuweisung minus
      "*=".equals(strPa) || // Zuweisung mal
      "/=".equals(strPa) || // Zuweisung durch
      ":>=".equals(strPa) || // Zuweisung Minimum
      ":min=".equals(strPa) || // Zuweisung Minimum
      ":<=".equals(strPa) || // Zuweisung Maximum
      ":max=".equals(strPa) // Zuweisung Maximum
      ;

   } // end method isOperator

   /**
    * Parse infix operators like +, -, *, /.
    *
    * @return geparsten Operator oder null wenn nichts passendes gefunden
    */
   public static AbstractInfixOperator parseInfixOperator(
      String paStr,
      int paIntLine,
      int paIntCol,
      String paStrOriginalSource)
      throws ExpressionParseException
   {
      // System.out.println( "parseInfixOperator:"+paStr);

      // dirty trick
      // Ausnutzung verkrzte Auflsung logischer Ausdrcke

      AbstractInfixOperator retOperator = null;
         boolean bFake =
            ((retOperator = PlusOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = MinusOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = PotentizeOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = TimesOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = DivideOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = ModuloOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = EqualOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = NotEqualOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = LessOrEqualOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = LessThanOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
      ((retOperator = GreaterOrEqualOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null)
      || // ...
       ((retOperator = GreaterThanOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = AndAndOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = OrOrOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = AndOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = OrOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = NandOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
       ((retOperator = NorOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
    ((retOperator = XorOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null); // ...

      return retOperator;

   } // end method parseInfixOperator

   /**
    * debug output
    */
   public String toString()
   {
      String strRet = getClass().getName() + " priorityLevel: " + getPriorityLevel() + " source: " + getSource();

      return strRet;
   } // end method toString

   /**
    * process the operator without short circuit evaluation . <br>
    *
    * Diese Methode wird fr die verkrzte Auswertung
    * (short circuit evaluation) berschrieben. Aufpassen, etwas dirty!
    */
   public Object eval(Object paObjLeft, AbstractToken paTokRight, AbstractRuntimeEnvironment paRunEnv)
      throws ExpressionEvaluationException
   {
      return eval(paObjLeft, paTokRight.eval(paRunEnv));
   }

   /**
    * process the operator
    */
   abstract public Object eval(Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException;

   /**
    * get the wahrscheinlichen return typen . <br>
    * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
    * null wenn kein Rckgabe-Wert,
    * mehrere wenn mehrere mglich
    * ( z.B. Plus-Operator kann numerisch oder String liefern)
    */
   // abstract public int[] getCheckReturnTypes()  throws ExpressionParseException ;
   abstract public int[] getCheckReturnTypes(int[] paIntArrLeft, int[] paIntArrRight) throws ExpressionParseException;

   /**
    *
    */
   // abstract public  void checkTypes(AbstractRuntimeEnvironment paRunEnv , AbstractToken paTokLeft, AbstractToken paTokRight) throws ExpressionParseException;

   abstract public int getPriorityLevel();

   /**
    * vorgefertigte Typ-prfung auf Bool-Parameter und Bool-Rckgabewert
    */
   protected static int[] checkBoolBoolBool(AbstractInfixOperator paInfxOp, int[] paIntArrLeft, int[] paIntArrRight)
      throws ExpressionParseException
   {

      int[] iTypesArr = paIntArrLeft;

      if (!Types.isInOrUnknown(Types.RET_TYPE_BOOL, iTypesArr))
      {
         throw new ExpressionParseException(
            paInfxOp.getLine(),
            paInfxOp.getCol(),
            paInfxOp.getOriginalSource(),
            "boolean type expected");
      }
      iTypesArr = paIntArrRight;
      if (!Types.isInOrUnknown(Types.RET_TYPE_BOOL, iTypesArr))
      {
         throw new ExpressionParseException(
            paInfxOp.getLine(),
            paInfxOp.getCol(),
            paInfxOp.getOriginalSource(),
            "boolean type expected");
      }

      return new int[] { Types.RET_TYPE_BOOL };
   } // end method checkBoolBoolBool

   /**
    * vorgefertigte Typ-prfung auf Num-Parameter und Num-Rckgabewert
    */
   public static int[] checkNumNumNum(AbstractInfixOperator paInfxOp, int[] paIntArrLeft, int[] paIntArrRight)
      throws ExpressionParseException
   {
      int[] iTypesArr = paIntArrLeft;

      if (!Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))
      {
         throw new ExpressionParseException(
            paInfxOp.getLine(),
            paInfxOp.getCol(),
            paInfxOp.getOriginalSource(),
            "numeric type expected " + Types.arr2str(iTypesArr));
      }
      iTypesArr = paIntArrRight;
      if (!Types.isInOrUnknown(Types.RET_TYPE_NUM, iTypesArr))
      {
         throw new ExpressionParseException(
            paInfxOp.getLine(),
            paInfxOp.getCol(),
            paInfxOp.getOriginalSource(),
            "numeric type expected");
      }

      return new int[] { Types.RET_TYPE_NUM };
   } // end method getCheckReturnTypes

} // end class
