package de.cnc.expression.infixoperators;


import java.math.BigDecimal;

import de.cnc.expression.Types;
import de.cnc.expression.exceptions.*;
import de.cnc.util.CompareUtil;


public class LessOrEqualOperator extends AbstractInfixOperator {

  public static LessOrEqualOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "LessOrEqualOperator#parse:"+paStr);

    if (paStr.startsWith("<=")) {
      LessOrEqualOperator lessOrEqOp = new LessOrEqualOperator(paStrOriginalSource, paIntLine, paIntCol);

      lessOrEqOp.iSrcLength = 2;
      lessOrEqOp.strSource = "<=";
      return lessOrEqOp;
    }
    else if (paStr.startsWith("=<")) {
      LessOrEqualOperator lessOrEqOp = new LessOrEqualOperator(paStrOriginalSource, paIntLine, paIntCol);

      lessOrEqOp.iSrcLength = 2;
      lessOrEqOp.strSource = "=<";
      return lessOrEqOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no less or equal operator");

  }// end method parse

  /**
   * constructor
   */
  public LessOrEqualOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public Object eval(Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {

    if (paObjLeft == null) {
//      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    else if (paObjLeft instanceof Boolean) {
      // dirty trick, Umwandlung in String
      // da t im Alphabet nach f folgt, ist true > false ( 0 > 1 )
      // klappt nur in englisch
      paObjLeft = "" + paObjLeft;
    }
    else if (paObjLeft instanceof Number) {
      // Normieren numerische Werte auf BigDecimal
      paObjLeft = new BigDecimal("" + paObjLeft);
    }
    else if (!(paObjLeft instanceof Comparable)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; Comparable expected");
    }
    Comparable cmpLeftValue = (Comparable) paObjLeft;

    if (paObjRight == null) {
//      throw new ExpressionEvaluationException(this, "right operand is null");
    }
    else if (paObjRight instanceof Boolean) {
      // dirty trick, Umwandlung in String
      // da t im Alphabet nach f folgt, ist true > false ( 0 > 1 )
      // klappt nur in englisch
      paObjRight = "" + paObjRight;
    }
    else if (paObjRight instanceof Number) {
      // Normieren numerische Werte auf BigDecimal
      paObjRight = new BigDecimal("" + paObjRight);
    }
    else if (!(paObjRight instanceof Comparable)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; Comparable expected");
    }
    Comparable cmpRightValue = (Comparable) paObjRight;

    try {
      return ( CompareUtil.lessEqual( cmpLeftValue , cmpRightValue ) ) ? Boolean.TRUE : Boolean.FALSE;
    } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes(int[] paIntArrLeft, int[] paIntArrRight)  throws ExpressionParseException {
    return new int[] {Types.RET_TYPE_BOOL };
  }// end method getReturnTypes

  public int getPriorityLevel() {
    return 2;
  }// end method getPriorityLevel

}// end class
