package de.cnc.expression.infixoperators;


import java.math.BigDecimal;

import de.cnc.expression.exceptions.*;


public class MinusOperator extends AbstractInfixOperator {

  public static MinusOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "MinusOperator#parse:"+paStr);
    if ("-".equals(paStr) || // ...
        // Schutz gegen Erkennen --
        (paStr.length() > 1 && paStr.startsWith("-") && paStr.charAt(1) != '-')) {

      MinusOperator minusOp = new MinusOperator(paStrOriginalSource, paIntLine, paIntCol);

      minusOp.strSource = "-";
      minusOp.iSrcLength = 1;

      return minusOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no minus operator");

  }// end method parse

  /**
   * constructor
   */
  public MinusOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public Object eval(Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {
    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; BigDecimal expected");
    }

    if (!(paObjRight instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; BigDecimal expected");
    }
    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }

    String strLeftValue = "" + paObjLeft;
    String strRightValue = "" + paObjRight;

    // BigDecimal retBigDec = (new BigDecimal(strLeftValue)).subtract(new BigDecimal(strRightValue));
    // System.out.println( "subtract "+strLeftValue+" and "+strRightValue+" to:"+ retBigDec ) ;
    // return retBigDec ;
    return (new BigDecimal(strLeftValue)).subtract(new BigDecimal(strRightValue));
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes(int[] paIntArrLeft, int[] paIntArrRight)  throws ExpressionParseException {
    return checkNumNumNum(this, paIntArrLeft, paIntArrRight);
  }// end method getCheckReturnTypes

  public int getPriorityLevel() {
    return 3;
  }// end method getPriorityLevel
}// end class
