package de.cnc.expression.infixoperators;


import java.math.BigDecimal;

import de.cnc.expression.exceptions.*;


public class ModuloOperator extends AbstractInfixOperator {

  public static ModuloOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "ModuloOperator#parse:"+paStr);

    if (paStr.startsWith("%")) {
      ModuloOperator moduloOp = new ModuloOperator(paStrOriginalSource, paIntLine, paIntCol);

      moduloOp.strSource = "%";
      moduloOp.iSrcLength = 1;
      return moduloOp;
    }
    else if ("mod".equals(paStr)) {
      ModuloOperator moduloOp = new ModuloOperator(paStrOriginalSource, paIntLine, paIntCol);

      moduloOp.strSource = "mod";
      moduloOp.iSrcLength = 3;
      return moduloOp;
    }
    else if (paStr.length() > 3 && paStr.startsWith("mod") && !Character.isLetterOrDigit(paStr.charAt(3))) {
      ModuloOperator moduloOp = new ModuloOperator(paStrOriginalSource, paIntLine, paIntCol);

      moduloOp.strSource = "mod";
      moduloOp.iSrcLength = 3;
      return moduloOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no modulo operator");

  }// end method parse

  /**
   * constructor
   */
  public ModuloOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public Object eval(Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {

    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; BigDecimal expected");
    }

    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }

    if (!(paObjRight instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; BigDecimal expected");
    }
    String strLeftValue = "" + paObjLeft;
    String strRightValue = "" + paObjRight;

    // Abfangung division durch Null
    if ((new BigDecimal(strRightValue)).compareTo(new BigDecimal(0)) == 0) {
      throw new ExpressionEvaluationException(this, "divide by zero");
    }

    try {
      // System.out.println( "" + (Double.parseDouble(strLeftValue) % Double.parseDouble(strRightValue)) );
      return new BigDecimal(Double.parseDouble(strLeftValue) % Double.parseDouble(strRightValue));
    } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes(int[] paIntArrLeft, int[] paIntArrRight)  throws ExpressionParseException {
    return checkNumNumNum(this, paIntArrLeft, paIntArrRight);
  }// end method getCheckReturnTypes

  public int getPriorityLevel() {
    return 4;
  }// end method getPriorityLevel
}// end class
