package de.cnc.expression.infixoperators;


import java.math.BigDecimal;

import de.cnc.expression.Types;
import de.cnc.expression.exceptions.*;
import de.cnc.util.CompareUtil;


public class NotEqualOperator extends AbstractInfixOperator {

  public static NotEqualOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "NotEqualOperator#parse:"+paStr);

    if (paStr.startsWith("!=")) {
      NotEqualOperator notEqualOp = new NotEqualOperator(paStrOriginalSource, paIntLine, paIntCol);

      notEqualOp.iSrcLength = 2;
      notEqualOp.strSource = "!=";
      return notEqualOp;
    }

    else if (paStr.startsWith("<>")) {
      NotEqualOperator notEqualOp = new NotEqualOperator(paStrOriginalSource, paIntLine, paIntCol);

      notEqualOp.iSrcLength = 2;
      notEqualOp.strSource = "<>";
      return notEqualOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no not equal operator");

  }// end method parse

  /**
   * constructor
   */
  public NotEqualOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public Object eval(Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {

    if (paObjLeft == null || paObjRight == null)
    {
        //Sonderfall null
        if (paObjLeft == null && paObjRight == null)
        {
            return Boolean.FALSE;
        }
        else
        {
            return Boolean.TRUE;
        }
    }

//    if (paObjLeft == null) {
//      //throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "left operand is null");
//    }
    if (paObjLeft instanceof Boolean) {
      // dirty trick, Umwandlung in String
      // da t im Alphabet nach f folgt, ist true > false ( 0 > 1 )
      // klappt nur in englisch
      paObjLeft = "" + paObjLeft;
    }
    else if (paObjLeft instanceof Number) {
      // Normieren numerische Werte auf BigDecimal
      paObjLeft = new BigDecimal("" + paObjLeft);
    }
    else if (!(paObjLeft instanceof Comparable)) {
      throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(),
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; Comparable expected");
    }
    Comparable cmpLeftValue = (Comparable) paObjLeft;

//    if (paObjRight == null) {
//        //throw new ExpressionEvaluationException(this, "right operand is null");
//    }
    if (paObjRight instanceof Boolean) {
      // dirty trick, Umwandlung in String
      // da t im Alphabet nach f folgt, ist true > false ( 0 > 1 )
      // klappt nur in englisch
      paObjRight = "" + paObjRight;
    }
    else if (paObjRight instanceof Number) {
      // Normieren numerische Werte auf BigDecimal
      paObjRight = new BigDecimal("" + paObjRight);
    }
    else if (!(paObjRight instanceof Comparable)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; Comparable expected");
    }
    Comparable cmpRightValue = (Comparable) paObjRight;

    try {
      return ( CompareUtil.notEquals( cmpLeftValue , cmpRightValue ) ) ? Boolean.TRUE : Boolean.FALSE;
    } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }

  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes(int[] paIntArrLeft, int[] paIntArrRight)  throws ExpressionParseException {
    return new int[] {Types.RET_TYPE_BOOL };
  }// end method getReturnTypes

  public int getPriorityLevel() {
    return 2;
  }// end method getPriorityLevel
}// end class
