package de.cnc.expression.infixoperators;


import de.cnc.expression.exceptions.*;


/**
 * | Oder Operator . <br>
 * Oder-Verknpfung ohne verkrzter Auswertung (Short-Circuit-Evaluation)
 */
public class OrOperator extends AbstractInfixOperator {

  public static OrOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "OrOperator#parse:"+paStr);
    if ("|".equals(paStr)) {
      OrOperator orOperator = new OrOperator(paStrOriginalSource, paIntLine, paIntCol);

      orOperator.strSource = "|";
      orOperator.iSrcLength = 1;
      return orOperator;
    }
    // Schutz vor Erkennung ||
    else if (paStr.length() > 1 && paStr.startsWith("|") && paStr.charAt(1) != '|') {
      OrOperator orOperator = new OrOperator(paStrOriginalSource, paIntLine, paIntCol);

      orOperator.strSource = "|";
      orOperator.iSrcLength = 1;
      return orOperator;
    }
    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no || operator");

  }// end method parse

  /**
   * constructor
   */
  public OrOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
    // System.out.println("constructor OrOperator");
    // (new Exception("")).printStackTrace();
  }// end constructor

  /**
   * process the operator
   */
  public Object eval(Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {

    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; Boolean expected");
    }
    Boolean bLeftValue = (Boolean) paObjLeft;

    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }
    if (!(paObjRight instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; Boolean expected");
    }
    Boolean bRightValue = (Boolean) paObjRight;

    // System.out.println("bLeftValue " + bLeftValue);
    // System.out.println("bRightValue " + bRightValue);
    try {
      return (bLeftValue.booleanValue() || bRightValue.booleanValue()) ? Boolean.TRUE : Boolean.FALSE;
    } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes(int[] paIntArrLeft, int[] paIntArrRight)  throws ExpressionParseException {
    return checkBoolBoolBool(this, paIntArrLeft, paIntArrRight);
  }// end method getCheckReturnTypes

  public int getPriorityLevel() {
    return 1;
  }// end method getPriorityLevel

}// end class
