package de.cnc.expression.infixoperators;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * | | Oder Operator . <br>
 * Oder-Verknpfung mit verkrzter Auswertung (Short-Circuit-Evaluation)
 */
public class OrOrOperator extends AbstractInfixOperator {

  public static OrOrOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "OrOrOperator#parse:"+paStr);
    if (paStr.startsWith("||")) {
      OrOrOperator orOrOperator = new OrOrOperator(paStrOriginalSource, paIntLine, paIntCol);

      orOrOperator.strSource = "||";
      orOrOperator.iSrcLength = 2;
      return orOrOperator;
    }

    else if ("or".equals(paStr)) {
      OrOrOperator orOrOperator = new OrOrOperator(paStrOriginalSource, paIntLine, paIntCol);

      orOrOperator.strSource = "or";
      orOrOperator.iSrcLength = 2;
      return orOrOperator;
    }
    else if (paStr.length() > 2 && paStr.startsWith("or") && !Character.isLetterOrDigit(paStr.charAt(2))) {
      OrOrOperator orOrOperator = new OrOrOperator(paStrOriginalSource, paIntLine, paIntCol);

      orOrOperator.strSource = "or";
      orOrOperator.iSrcLength = 2;
      return orOrOperator;
    }
    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no || operator");

  }// end method parse

  /**
   * constructor
   */
  public OrOrOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
    // System.out.println("constructor OrOrOperator");
    // (new Exception("")).printStackTrace();
  }// end constructor

  /**
   * process the operator with short circuit evaluation . <br>
   *
   * Diese Methode wurde fr die verkrzte Auswertung
   * (short circuit evaluation) berschrieben. Aufpassen, etwas dirty!
   */
  public Object eval(Object paObjLeft, AbstractToken paTokRight, AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; Boolean expected");
    }
    Boolean bLeftValue = (Boolean) paObjLeft;

    // verkrzte Auswertung true
    if (bLeftValue.booleanValue()) {
      return bLeftValue;
    }

    return eval(paObjLeft, paTokRight.eval(paRunEnv));
  }

  /**
   * process the operator
   */
  public Object eval(Object paObjLeft, Object paObjRight) throws ExpressionEvaluationException {

    if (paObjLeft == null) {
      throw new ExpressionEvaluationException(this, "left operand is null");
    }
    if (!(paObjLeft instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + paObjLeft + ":" + paObjLeft.getClass().getName() + " ; Boolean expected");
    }
    Boolean bLeftValue = (Boolean) paObjLeft;

    if (paObjRight == null) {
      throw new ExpressionEvaluationException(this, "right operand is null");
    }
    if (!(paObjRight instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "right operand is " + paObjRight + ":" + paObjRight.getClass().getName() + " ; Boolean expected");
    }
    Boolean bRightValue = (Boolean) paObjRight;

    // System.out.println("bLeftValue " + bLeftValue);
    // System.out.println("bRightValue " + bRightValue);
    try {
      return (bLeftValue.booleanValue() || bRightValue.booleanValue()) ? Boolean.TRUE : Boolean.FALSE;
    } catch (Exception e) {
      throw new ExpressionEvaluationException(this, "" + e);
    }
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes(int[] paIntArrLeft, int[] paIntArrRight)  throws ExpressionParseException {
    return checkBoolBoolBool(this, paIntArrLeft, paIntArrRight);
  }// end method getCheckReturnTypes

  public int getPriorityLevel() {
    return 1;
  }// end method getPriorityLevel

}// end class
