package de.cnc.expression.prepostfixoperators;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * Abstrakte Oberklasse aller Prefix- und Postfix-Operatoren.
 * 
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public abstract class AbstractPrePostfixOperator {

  protected int iLine;
  protected int iCol;
  protected String strSource;
  protected int iSrcLength = 0;
  protected String strOriginalSource;

  public AbstractPrePostfixOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    strOriginalSource = paStrOriginalSource;
    iLine = paIntLine;
    iCol = paIntCol;
  }

  /**
   * gets the the source line number
   */
  public int getLine() {
    return iLine;
  }

  /**
   * gets the col position in the source code line
   */
  public int getCol() {
    return iCol;
  }

  public String getSource() {
    return strSource;
  }

  public int getSrcLength() {
    return iSrcLength;
  }

  public String getOriginalSource() {
    return strOriginalSource;
  }

  /**
   * Prfroutine fr Variablen-Namen . <br>
   * @see de.cnc.expression.tokencomplex.Variable#parse
   */
  public static boolean isOperator(String strPa) {
    return
        "!".equals(strPa) //--
        || "-".equals(strPa) //--
        || "++".equals(strPa) ;
  }// end method isOperator

  /**
   * Parse prefix operators like !b, -i, --i and ++i
   *
   * @return geparsten Operator oder null wenn nichts passendes gefunden
   */
  public static AbstractPrePostfixOperator parsePrefixOperator(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException {
    // System.out.println( "parsePrefixOperator:"+paStr);
    // dirty trick
    // Ausnutzung verkrzte Auflsung logischer Ausdrcke
    AbstractPrePostfixOperator retOperator = null;
    boolean bFake = // ...
        ((retOperator = PlusPlusOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
        ((retOperator = MinusMinusOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
        ((retOperator = MinusPrefixOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
        ((retOperator = NotPrefixOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null); // ...

    return retOperator;

  }// end method parsePrefixOperator

  /**
   * Parse postfix operators like i++ and i--
   *
   * @return geparsten Operator oder null wenn nichts passendes gefunden
   */
  public static AbstractPrePostfixOperator parsePostfixOperator(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException {
    // System.out.println( "parsePrefixOperator:"+paStr);
    // dirty trick
    // Ausnutzung verkrzte Auflsung logischer Ausdrcke
    AbstractPrePostfixOperator retOperator = null;
    boolean bFake = // ...
        ((retOperator = PlusPlusOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null) || // ...
        ((retOperator = MinusMinusOperator.parse(paStr, paIntLine, paIntCol, paStrOriginalSource)) != null); // ...

    return retOperator;

  }// end method AbstractPostfixOperator

  /**
   * debug output
   */
  public String toString() {
    String strRet = getClass().getName() + " source: " + getSource();

    return strRet;
  }// end method toString

  /**
   * process the operator
   */
  // public abstract Object eval(Object paValue) throws ExpressionEvaluationException;

  /**
   * process the operator
   */
  public abstract Object eval(AbstractRuntimeEnvironment paRunEnv, AbstractToken   paTok) throws ExpressionEvaluationException;

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  // abstract public int[] getCheckReturnTypes();

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  // abstract public int[] getCheckReturnTypes()  throws ExpressionParseException ;
  abstract public int[] getCheckReturnTypes(int[] paIntArrTypes)  throws ExpressionParseException;

}// end class
