package de.cnc.expression.prepostfixoperators;


import java.math.BigDecimal;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


public class MinusPrefixOperator extends AbstractPrePostfixOperator {

  public static MinusPrefixOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "MinusPrefixOperator#parse:"+paStr);
    if ("-".equals(paStr) || // ...
        // Schutz gegen Erkennen --
        (paStr.length() > 1 && paStr.startsWith("-") && paStr.charAt(1) != '-')) {

      MinusPrefixOperator minusOp = new MinusPrefixOperator(paStrOriginalSource, paIntLine, paIntCol);

      minusOp.strSource = "-";
      minusOp.iSrcLength = 1;

      return minusOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no minus operator");

  }// end method parse

  /**
   * constructor
   */
  public MinusPrefixOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv, AbstractToken   paTok) throws ExpressionEvaluationException {
    return eval(paTok.eval(paRunEnv));
  }// end method

  /**
   * process the operator
   */
  protected Object eval(Object paObjValue) throws ExpressionEvaluationException {
    if (paObjValue == null) {
      throw new ExpressionEvaluationException(this, "operand missed");
    }
    if (!(paObjValue instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "operand is " + paObjValue + ":" + paObjValue.getClass().getName() + " ; numeric expected");
    }

    return new BigDecimal("" + paObjValue).negate();
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes(int[] paIntArrTypes)
    throws ExpressionParseException {

    if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, paIntArrTypes))) {
      throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "numeric type expected");
    }
    return new int[] {Types.RET_TYPE_NUM };
  }// end  method getCheckReturnTypes

}// end class
