package de.cnc.expression.prepostfixoperators;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


public class NotPrefixOperator extends AbstractPrePostfixOperator {

  public static NotPrefixOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "NotPrefixOperator#parse:"+paStr);
    if (paStr.startsWith("!")) {
      NotPrefixOperator notOp = new NotPrefixOperator(paStrOriginalSource, paIntLine, paIntCol);

      notOp.strSource = "!";
      notOp.iSrcLength = 1;

      return notOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no minus operator");

  }// end method parse

  /**
   * constructor
   */
  public NotPrefixOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv, AbstractToken   paTok) throws ExpressionEvaluationException {
    return eval(paTok.eval(paRunEnv));
  }// end method

  /**
   * process the operator
   */
  private Object eval(Object paObjValue) throws ExpressionEvaluationException {
    if (paObjValue == null) {
      throw new ExpressionEvaluationException(this, "operand missed");
    }
    if (!(paObjValue instanceof Boolean)) {
      throw new ExpressionEvaluationException(this,
          "operand is " + paObjValue + ":" + paObjValue.getClass().getName() + " ; Boolean expected");
    }
    return (!((Boolean) paObjValue).booleanValue()) ? Boolean.TRUE : Boolean.FALSE;
  }// end method eval

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes(int[] paIntArrTypes)
    throws ExpressionParseException {

    if (!(Types.isInOrUnknown(Types.RET_TYPE_BOOL, paIntArrTypes))) {
      throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "boolean type expected");
    }
    return new int[] {Types.RET_TYPE_BOOL };
  }// end  method getCheckReturnTypes

}// end class
