package de.cnc.expression.prepostfixoperators;


import java.math.BigDecimal;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;
import de.cnc.expression.tokencomplex.IndexOperator;
import de.cnc.expression.tokencomplex.Variable;


public class PlusPlusOperator extends AbstractPrePostfixOperator {

  public static PlusPlusOperator parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    // System.out.println( "PlusPlusOperator#parse:"+paStr);
    if (paStr.startsWith("++")) {

      PlusPlusOperator pluPlusOp = new PlusPlusOperator(paStrOriginalSource, paIntLine, paIntCol);

      pluPlusOp.strSource = "++";
      pluPlusOp.iSrcLength = 2;

      return pluPlusOp;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no minus operator");

  }// end method parse

  /**
   * constructor
   */
  public PlusPlusOperator(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the operator
   */
  public Object eval(//--
          final AbstractRuntimeEnvironment paRunEnv, //--
          final AbstractToken paTok) //--
      throws ExpressionEvaluationException 
      {
//      if (paTok instanceof IndexOperator) {
//          IndexOperator indexOperator = (IndexOperator) paTok;
//          Object parentObject = indexOperator.getP
//      
//      }
      if (!(paTok instanceof Assignable)) {
      throw new ExpressionEvaluationException(this,
          "operand is " + paTok + ":" + paTok.getClass().getName() + " ; variable expected");
    }

    Object valObject = paTok.eval(paRunEnv);

    if (valObject == null) {
      throw new ExpressionEvaluationException(this, "operand missed");
    }
    if (!(valObject instanceof Number)) {
      throw new ExpressionEvaluationException(this,
          "left operand is " + valObject + ":" + valObject.getClass().getName() + " ; numeric expected");
    }

    valObject = (new BigDecimal("" + valObject)).add(new BigDecimal(1));

    // paRunEnv.setVariable(paTok.getSource(), valObject);
    Assignable varNameTok = (Assignable) paTok;

    varNameTok.set(paRunEnv, valObject);

    // return eval(valObject);
    return valObject;
  }// end method

  /**
   * process the operator
   */

  /*
   public Object eval(Object paObjValue) throws ExpressionEvaluationException {
   if (paObjValue == null) {
   throw new ExpressionEvaluationException(this, "left operand missed");
   }
   if (!(paObjValue instanceof Number)) {
   throw new ExpressionEvaluationException(this,
   "left operand is " + paObjValue + ":" + paObjValue.getClass().getName() + " ; Double expected");
   }

   return new BigDecimal("" + paObjValue).negate();
   }// end method eval
   */

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes(int[] paIntArrTypes)
    throws ExpressionParseException {

    if (!(Types.isInOrUnknown(Types.RET_TYPE_NUM, paIntArrTypes))) {
      throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "numeric type expected");
    }
    return new int[] {Types.RET_TYPE_NUM };
  }// end  method getCheckReturnTypes

}// end class
