package de.cnc.expression.tokencomplex;

import java.lang.reflect.Array;
import java.util.List;
import java.util.Map;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.Assignable;
import de.cnc.expression.Expression;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.util.ExpressionArrayList;
import de.cnc.expression.util.ParserUtil;
import de.cnc.expression.util.ReflectionUtil;

/**
 * Indexoperator <b>[ &lt;index&gt; ]</b>  zur 
 * Adressierung eines Elementes eines Arrays, einer java.util.List
 * oder einer java.util.Map . <br>
 * Notation:<br>
 * [ ... ] [ ... ] ( 1 bis beliebig oft ) <br>
 * oder<br>
 * [ ... , ... , ... ] <br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class IndexOperator extends AbstractToken implements Assignable
{
    /**
     * der Ausdruck, hinter dem ein Index-Operator steht
     */
    AbstractToken indexedExpression;

    /**
     * optionaler Ausdruck fr indiziertes Element varArr[ i ][ j ]
     */
    ExpressionArrayList indexExpressionList;

    /**
     *
     */
    public static IndexOperator parse(
        String paStr,
        int paIntLine,
        int paIntCol,
        String paStrOriginalSource)
        throws ExpressionParseException
    { // , NoSuchTokenException {
//System.out.println(paStr);
        if (!paStr.startsWith("[")) {
            return null;
        }

        IndexOperator inxElem = new IndexOperator(paStrOriginalSource, paIntLine, paIntCol);

        int iRun = 0;

        boolean isKomma = false; // Komma alternativ zu ][ bei mehreren Index-Ausdrcken

        while (iRun < paStr.length()
            && //-
         (isKomma || paStr.charAt(iRun) == '['))
        {
            isKomma = false; // zurcksetzen
//            if (inxElem == null)
//            {
//                inxElem =
//                    new IndexOperator(paStrOriginalSource, paIntLine, paIntCol);
//            }
            if (inxElem.indexExpressionList == null)
            {
                inxElem.indexExpressionList = new ExpressionArrayList();
            }
            // Array-Index
            iRun++;
            // weiss-zeichen berspringen
            iRun += ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
            // Index-Expression parsen
            //varName.indexExpression =
            inxElem.indexExpressionList.add(
                Expression.parseEasy(
                    paStr.substring(iRun),
                    paIntLine,
                    iRun,
                    paStrOriginalSource));
            iRun += inxElem.indexExpressionList.tail().getSrcLength();

            // weiss-zeichen berspringen
            iRun += ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
            if (iRun < paStr.length() && paStr.charAt(iRun) == ',')
            {
                isKomma = true;
            }
            else if (!(iRun < paStr.length() && paStr.charAt(iRun) == ']'))
            {
                throw new ExpressionParseException(
                    paIntLine,
                    iRun,
                    paStrOriginalSource,
                    "] or , expected");
            }
            iRun++;
            inxElem.strSource = paStr.substring(0, iRun);
            inxElem.iSrcLength = iRun;
        }

        return inxElem;

    } // end method parse

    /**
     * constructor
     */
    public IndexOperator(
        String paStrOriginalSource,
        int paIntLine,
        int paIntCol)
    {
        super(paStrOriginalSource, paIntLine, paIntCol);
        // System.out.println( "constructor paStrOriginalSource:"+paStrOriginalSource);
    } // end constructor

    /**
     * set the indexed expression.
     * Setzt den Ausdruck, der am Ende eine Index-Element-Operator []
     * trgt. Somit wird sozusagen der hinter dem indizierten Ausdruck stehende
     * Index-Operator vor den indizierten Ausdruck gesetzt.
     * Demzufolge wird erst der indzierte Ausdruck evaluiert und auf dessen Ergebnis
     * dann der Index-Operator angewendet.
     * 
     * @param token
     */
    public void setIndexedExpression(AbstractToken pToken)
    {
        this.indexedExpression = pToken;
        //aufaddieren der Lnge des indizierten Ausdruckes,
        //da gilt >xxx[ yyy ]< Lnge indizierter Ausdruck + Lnge Index-Operator
        this.iSrcLength += pToken.getSrcLength();
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public int[] getCheckReturnTypes(/*int[] paIntArrTypes*/) 
    throws ExpressionParseException
    {
        // spter noch Erweiterung auf getypte Variable
        // durch Erweiterung in AbstractRuntimeEnvironment
//        if (!(Types.isInOrUnknown(Types.RET_TYPE_LIST_OR_ARRAY, paIntArrTypes))) {
//            throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "List or Array type expected");
//          }

        return new int[] { Types.RET_TYPE_UNKNOWN };
    } // end method getReturnTypes

   /**
     * rekursives Aufrufen der Klammerung der Ausdrcke
     * entsprechend der Operator-Prioritt.
     * Weiterleiten an den Index-Auisdruck ( var[ &lt;exp&gt; ] )
     */
    public void organizeOperatorPriority()
    {
        this.indexedExpression.organizeOperatorPriority();
        if (this.indexExpressionList != null)
        {
            this.indexExpressionList.organizeOperatorPriority();
        }
    }

    /**
     * Es wird erst der indzierte Ausdruck evaluiert und auf dessen Ergebnis
     * dann der Index-Operator angewendet.
     */
    public Object eval(AbstractRuntimeEnvironment paRunEnv)
        throws ExpressionEvaluationException
    {
        //throw new UnsupportedOperationException();
        Object collObj = this.indexedExpression.eval(paRunEnv);
//System.out.println( "eval collObj: " + collObj );
        return getIndexElement(paRunEnv, collObj, this.indexExpressionList, this.getCol());
    }

    /**
     * Es wird erst der indzierte Ausdruck evaluiert und auf dessen Ergebnis
     * dann der Index-Operator angewendet.
     */
    public Object getCollectionForAssignment(AbstractRuntimeEnvironment paRunEnv)
        throws ExpressionEvaluationException
    {
        //throw new UnsupportedOperationException();
        Object collObj = this.indexedExpression.eval(paRunEnv);
        return getParentIndexElement(paRunEnv, collObj, this.indexExpressionList, this.getCol());
    }

    /**
     * get the Array index or Map-Key
     */
    /*package*/
    static Object evalIndex(//--
    Expression indexExpression, //--
    AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException
    {
        Object inxObj = null;

        try
        {
            inxObj = indexExpression.eval(paRunEnv);
        }
        catch (ExpressionEvaluationException e)
        {
            System.err.println(e.getMessage());
            e.printStackTrace();
            throw e; // einfach weiterwerfen
        }
        catch (Exception e)
        {
            System.err.println(e.getMessage());
            e.printStackTrace();
            throw new ExpressionEvaluationException(
                indexExpression.getLine(),
                indexExpression.getCol() + indexExpression.getSrcLength() + 1,
                indexExpression.getSource(),
                indexExpression.getOriginalSource(),
                "index is invalid " + e);
        }
        if (inxObj == null)
        {
            throw new ExpressionEvaluationException(
                indexExpression.getLine(),
                indexExpression.getCol() + indexExpression.getSrcLength() + 1,
                indexExpression.getSource(),
                indexExpression.getOriginalSource(),
                "index value missed");
        }
        //    if (inxObj.intValue() < 0) {
        //      throw new ExpressionEvaluationException(getLine(), getCol() + strVarName.length() + 1, getSource(), getOriginalSource(),
        //          "index value is lesser 0, " + inxObj);
        //    }
        return inxObj; // .intValue();

    } // end method evalIndex

    /**
     * Diese Methode entnimmt dem bergebenen Array,
     * java.util.List oder java.util.Map, auch geschachtelt,
     * das gewnschte Element.
     * 
     * @param paRunEnv
     * @param objValue
     * @param indexExpressionList
     * @return
     * @throws ExpressionEvaluationException
     */
    static Object getIndexElement(//--
            final AbstractRuntimeEnvironment paRunEnv, //--
            final Object objValue, //--
            final ExpressionArrayList indexExpressionList, //--
            final int pBaseCol) //--
        throws ExpressionEvaluationException
    {
        Object retObjValue = objValue;
        for (int i = 0; i < indexExpressionList.size(); i++)
        {
            final Expression runIndexExpr = indexExpressionList.get(i);

//System.out.println( "i: " + i );
//System.out.println( "getIndexElement: retObjValue: " + retObjValue);

            retObjValue = getOneIndexedElement(paRunEnv, pBaseCol, retObjValue, runIndexExpr);

        }
        return retObjValue;
    }

    /**
     * Diese Methode entnimmt dem bergebenen mehrstufigen Array,
     * java.util.List oder java.util.Map, auch geschachtelt,
     * das Elternelement des gewnschten Elementes zur
     * Zuweisung.
     * 
     * @param paRunEnv
     * @param objValue
     * @param indexExpressionList
     * @return
     * @throws ExpressionEvaluationException
     */
    static Object getParentIndexElement(//--
            final AbstractRuntimeEnvironment paRunEnv, //--
            final Object objValue, //--
            final ExpressionArrayList indexExpressionList, //--
            final int pBaseCol) //--
        throws ExpressionEvaluationException
    {
        Object retObjValue = objValue;
        for (int i = 0; i < indexExpressionList.size() - 1; i++)
        {
            final Expression runIndexExpr = indexExpressionList.get(i);
            retObjValue = getOneIndexedElement(paRunEnv, pBaseCol, retObjValue, runIndexExpr);

        }
        return retObjValue;
    }

    /**
     * @param paRunEnv
     * @param pBaseCol
     * @param retObjValue
     * @param runIndexExpr
     * @return
     * @throws ExpressionEvaluationException
     */
    private static Object getOneIndexedElement(//--
            final AbstractRuntimeEnvironment paRunEnv, //--
            final int pBaseCol, //--
            Object retObjValue, //--
            final Expression runIndexExpr) //--
        throws ExpressionEvaluationException
    {
        final Object inxObj = IndexOperator.evalIndex(runIndexExpr, paRunEnv);

        // Index auf Array oder java.util.List
        if (retObjValue != null && retObjValue.getClass().isArray())
        {
            // Array
            int iIndex = ((Number) inxObj).intValue();

            checkArrayIndex(runIndexExpr, retObjValue, iIndex,  pBaseCol);
            retObjValue = java.lang.reflect.Array.get(retObjValue, iIndex);

        }
        else if (retObjValue instanceof java.util.List)
        {
            // java.util.List
            int iIndex = ((Number) inxObj).intValue();

            List listValue = (List) retObjValue;

            checkListIndex(runIndexExpr, listValue, iIndex, pBaseCol);

            retObjValue = listValue.get(iIndex);
        }
        else if (retObjValue instanceof java.util.Map)
        {
            // java.util.Map
            Map mapValue = (Map) retObjValue;

            retObjValue = mapValue.get(inxObj);
        }
        else if (retObjValue == null)
        {
            throw new ExpressionEvaluationException(
                runIndexExpr.getLine(),
                runIndexExpr.getCol() + pBaseCol,
                runIndexExpr.getSource(),
                runIndexExpr.getOriginalSource(),
                "value is null");
        }
        else
        {
            throw new ExpressionEvaluationException(
                runIndexExpr.getLine(),
                runIndexExpr.getCol(),
                runIndexExpr.getSource(),
                runIndexExpr.getOriginalSource(),
                "value is no array or java.util.List or java.util.Map: " + retObjValue.getClass().getName());
        }
        return retObjValue;
    }

    /**
     * Prfen Array-Index auf 0...max und Werfen 
     * der entsprechenden Exceptions bei Nichteinhaltung
     * 
     * @param objValue
     * @param iIndex
     * @throws ExpressionEvaluationException
     */
    static void checkArrayIndex(//--
    Expression runIndexExpr, //--
    Object objValue, //--
    int iIndex, //--
    int pBaseCol) 
    throws ExpressionEvaluationException
    {
        if (iIndex < 0)
        {
            throw new ExpressionEvaluationException(
                runIndexExpr.getLine(),
                runIndexExpr.getCol() + pBaseCol,
                runIndexExpr.getSource(),
                runIndexExpr.getOriginalSource(),
                "index value is lesser 0, " + iIndex);
        }
        if (iIndex >= java.lang.reflect.Array.getLength(objValue))
        {
            // System.out.println( "getSource():"+getSource() ) ;
            // System.out.println( "getOriginalSource():"+getOriginalSource() ) ;
            throw new ExpressionEvaluationException(
                runIndexExpr.getLine(),
                runIndexExpr.getCol() + pBaseCol,
                runIndexExpr.getSource(),
                runIndexExpr.getOriginalSource(),
                "index "
                    + iIndex
                    + " is out of range 0..."
                    + (java.lang.reflect.Array.getLength(objValue) - 1));
            //+ ( objValue.length - 1));
        }
    }

    //     * Prfen List-Index auf 0...max und Werfen 
    //     * der entsprechenden Exceptions bei Nichteinhaltung

    static void checkListIndex(Expression runIndexExpr, //--
    List listValue, //--
    int iIndex,
    int pBaseCol) throws ExpressionEvaluationException
    {
        if (iIndex < 0)
        {
            throw new ExpressionEvaluationException(
                runIndexExpr.getLine(),
                runIndexExpr.getCol(),
                runIndexExpr.getSource(),
                runIndexExpr.getOriginalSource(),
                "index value is lesser 0, " + iIndex);
        }
        if (iIndex >= listValue.size())
        {
            throw new ExpressionEvaluationException(
                runIndexExpr.getLine(),
                runIndexExpr.getCol() + pBaseCol,
                runIndexExpr.getSource(),
                runIndexExpr.getOriginalSource(),
                "index "
                    + iIndex
                    + " is out of range 0..."
                    + (listValue.size() - 1));
        }
    } // end method

    /**
     * debug output
     */
    public String toString()
    {
        String strRet =
            getClass().getName()
                + " srcLength: "
                + getSrcLength()
                + " source: "
                + getSource()
                + " indexed expression: "
                + this.indexedExpression;

        if (this.indexExpressionList != null)
        {
            strRet += "\n" + this.indexExpressionList;
        }
        return strRet;
    }

    /**
     * set value unter beachtung array index
     */
    public void set(//--
            final AbstractRuntimeEnvironment paRunEnv,
            final Object paObjValue) // zu setzender Wert
        throws ExpressionEvaluationException
    {
            
      // Holen Array oder List oder Map
      Object objColl = this.indexedExpression.eval( paRunEnv );

      for (int i = 0; i < this.indexExpressionList.size(); i++)
      {
                // Index auf Array oder java.util.List
                final Expression runIndexExpr = this.indexExpressionList.get( i );
                Object inxObj = IndexOperator.evalIndex( runIndexExpr , paRunEnv );

                if (objColl != null && objColl.getClass().isArray())
                {
                    // Array
                    int iIndex = ((Number) inxObj).intValue();

                    IndexOperator.checkArrayIndex(runIndexExpr, objColl, iIndex, this.getCol());

                    Object objValue = paObjValue;

                    if ( i < this.indexExpressionList.size() - 1 )
                    {
                        //nchster Index
                        objColl = java.lang.reflect.Array.get(objColl, iIndex);
                    }
                    else
                    {
                        // ziemlich instabile Sache
                        // das Konvertieren sollte gecatcht werden
                        if (objColl instanceof boolean[])
                        {
                            objValue = Boolean.valueOf(objValue.toString());
                        }
                        else if (objColl instanceof byte[])
                        {
                            objValue = new Byte(objValue.toString());
                        }
                        else if (objColl instanceof char[])
                        {
                            objValue = new Character(objValue.toString().charAt(0));
                        }
                        else if (objColl instanceof double[])
                        {
                            objValue = new Double(objValue.toString());
                        }
                        else if (objColl instanceof float[])
                        {
                            objValue = new Float(objValue.toString());
                        }
                        else if (objColl instanceof int[])
                        {
                            // hier noch Behandlung von x.0 einbauen
                            objValue = new Integer(objValue.toString());
                        }
                        else if (objColl instanceof long[])
                        {
                            objValue = new Long(objValue.toString());
                        }
                        else if (objColl instanceof short[])
                        {
                            objValue = new Short(objValue.toString());
                        }

                        if ( i < this.indexExpressionList.size() - 1 )
                        {
                            //nchster Index
                            objColl = java.lang.reflect.Array.get(objColl, iIndex);
                        }
                        else
                        {
                            final Object convValue = ReflectionUtil.convertNumberObject( objValue , objColl.getClass().getComponentType() );
                            Array.set(objColl, iIndex, convValue);
                        }
                    }
                }
                else if (objColl instanceof java.util.List)
                {
                    // java.util.List
                    int iIndex = ((Number) inxObj).intValue();
                    List listValue = (List) objColl;

                    IndexOperator.checkListIndex( runIndexExpr, listValue , iIndex, this.getCol());

                    if ( i < this.indexExpressionList.size() - 1 )
                    {
                        //nchster Index
                        objColl = listValue.get(iIndex);
                    }
                    else
                    {
                        listValue.set(iIndex, paObjValue);
                    }
                }
                else if (objColl instanceof java.util.Map)
                {
                    // java.util.Map
                    Map mapValue = (Map) objColl;

                    if ( i < indexExpressionList.size() - 1 )
                    {
                        //nchster Index
                        objColl = mapValue.get(inxObj);
                    }
                    else
                    {
                        mapValue.put(inxObj, paObjValue);
                    }
                }
                else
                {
                    throw new ExpressionEvaluationException(
                        getLine(),
                        getCol(),
                        getSource(),
                        getOriginalSource(),
                        "variable is no array or java.util.List ");
                }
            }
    }

    /**
     * remove variable unter beachtung array index
     */
    public Object remove(AbstractRuntimeEnvironment paRunEnv)
        throws ExpressionEvaluationException
    {

        // Holen Array oder List oder Map
        Object objColl = this.indexedExpression.eval( paRunEnv );

        for (int i = 0; i < this.indexExpressionList.size(); i++)
        {
                // Index auf Array oder java.util.List
                final Expression runIndexExpr = this.indexExpressionList.get( i );
                Object inxObj = IndexOperator.evalIndex( runIndexExpr , paRunEnv );

                if (objColl != null && objColl.getClass().isArray())
                {
                    // Array
                    int iIndex = ((Number) inxObj).intValue();

                    IndexOperator.checkArrayIndex( runIndexExpr, objColl, iIndex, this.getCol());

                    if ( i < this.indexExpressionList.size() - 1 )
                    {
                        //nchster Index
                        objColl = java.lang.reflect.Array.get(objColl, iIndex);
                    }
                    else
                    {
                        final Object retObj = Array.get(objColl, iIndex);
                        Array.set(objColl, iIndex, null);
                        return retObj;
                    }

                }
                else if (objColl instanceof java.util.List)
                {
                    // java.util.List
                    final int iIndex = ((Number) inxObj).intValue();
                    final List listValue = (List) objColl;

                    IndexOperator.checkListIndex(runIndexExpr , listValue, iIndex, this.getCol());

                    if ( i < this.indexExpressionList.size() - 1 )
                    {
                        //nchster Index
                        objColl = listValue.get(iIndex);
                    }
                    else
                    {
                        return listValue.remove(iIndex);
                    }
                }
                else if (objColl instanceof java.util.Map)
                {
                    // java.util.Map
                    Map mapValue = (Map) objColl;

                    if ( i < this.indexExpressionList.size() - 1 )
                    {
                        //nchster Index
                        objColl = mapValue.get(inxObj);
                    }
                    else
                    {
                        return mapValue.remove(inxObj);
                    }
                }
                else
                {
                    throw new ExpressionEvaluationException(
                        getLine(),
                        getCol(),
                        getSource(),
                        getOriginalSource(),
                        "variable is no array or java.util.List or java.util.Map");
                }
            }
  
return null;
        
    } 


}
