package de.cnc.expression.tokencomplex;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.Assignable;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.util.ParserUtil;
import de.cnc.util.BeanUtil;

/**
 * Member-Dot-Operator <b>&lt;index&gt;.member</b>  zur 
 * Adressierung eines Members eines Objekts . <br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class MemberDotOperator extends AbstractDotOperator implements Assignable
{
    /**
     * der Ausdruck, hinter dem ein Dot-Operator steht
     */
    //AbstractToken dottedExpression;
    
    /** Name des adressierten Objekt-Members */
    String memberName;

    /**
     *
     */
    public static MemberDotOperator parseMemberDotOperator(
        String paStr,
        int paIntLine,
        int paIntCol,
        String paStrOriginalSource)
        //throws ExpressionParseException
    { // , NoSuchTokenException {
//System.out.println("MemberDotOperator parseMemberDotOperator: " + paStr);
        if (!paStr.startsWith(".")) {
            return null;
        }

        int iWhites = ParserUtil.parseWhitesAndComments( paStr.substring( 1 ) );

        final int identifierLen = ParserUtil.parseIdentifier( paStr.substring( 1 + iWhites )  );
        if (identifierLen < 1)
        {
            return null;
        }
        
        final MemberDotOperator dotElem = new MemberDotOperator(paStrOriginalSource, paIntLine, paIntCol);
        dotElem.memberName = paStr.substring(iWhites + 1, iWhites + identifierLen + 1);
        int iRun = 1 + iWhites + identifierLen;

        dotElem.strSource = paStr.substring(0, iRun);
        dotElem.iSrcLength = iRun;

        return dotElem;

    } // end method parse

    /**
     * constructor
     */
    public MemberDotOperator(
        String paStrOriginalSource,
        int paIntLine,
        int paIntCol)
    {
        super(paStrOriginalSource, paIntLine, paIntCol);
        // System.out.println( "constructor paStrOriginalSource:"+paStrOriginalSource);
    } // end constructor

    /**
     * set the dotted expression.
     * Setzt den Ausdruck, der am Ende eine Index-Element-Operator []
     * trgt. Somit wird sozusagen der hinter dem indizierten Ausdruck stehende
     * Index-Operator vor den indizierten Ausdruck gesetzt.
     * Demzufolge wird erst der indzierte Ausdruck evaluiert und auf dessen Ergebnis
     * dann der Index-Operator angewendet.
     * 
     * @param token
     */
    public void setDottedExpression(AbstractToken pToken)
    {
        this.dottedExpression = pToken;
        //aufaddieren der Lnge des gedotteten Ausdruckes,
        //da gilt >xxx[ yyy ]< Lnge indizierter Ausdruck + Lnge Index-Operator
        this.iSrcLength += pToken.getSrcLength();
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public int[] getCheckReturnTypes(/*int[] paIntArrTypes*/) 
    throws ExpressionParseException
    {
        // spter noch Erweiterung auf getypte Variable
        // durch Erweiterung in AbstractRuntimeEnvironment
//        if (!(Types.isInOrUnknown(Types.RET_TYPE_LIST_OR_ARRAY, paIntArrTypes))) {
//            throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "List or Array type expected");
//          }

        return new int[] { Types.RET_TYPE_UNKNOWN };
    } // end method getReturnTypes

   /**
     * rekursives Aufrufen der Klammerung der Ausdrcke
     * entsprechend der Operator-Prioritt.
     * Weiterleiten an den Index-Auisdruck ( var[ &lt;exp&gt; ] )
     */
    public void organizeOperatorPriority()
    {
        this.dottedExpression.organizeOperatorPriority();
    }

    /**
     * Es wird erst der gedottete Ausdruck evaluiert und auf dessen Ergebnis
     * dann der Dot-Operator angewendet.
     */
    public Object eval(AbstractRuntimeEnvironment paRunEnv)
        throws ExpressionEvaluationException
    {
        if (! paRunEnv.isMemberEnabled() )
        {
            throw new ExpressionEvaluationException(
                    getLine(),
                    getCol(),
                    getSource(),
                    getOriginalSource(),
                    "member access not enabled");
        }
        Object dottedObj = this.dottedExpression.eval(paRunEnv);
//System.out.println( "eval collObj: " + collObj );
        return  BeanUtil.get(dottedObj, this.memberName );
    }

    /**
     * debug output
     */
    public String toString()
    {
        String strRet =
            getClass().getName()
                + " srcLength: "
                + getSrcLength()
                + " source: "
                + getSource()
                + " dotted expression: "
                + this.dottedExpression;

        return strRet;
    }

    /**
     * set value unter beachtung array index
     */
    public void set(//--
            final AbstractRuntimeEnvironment paRunEnv,
            final Object pObjValue) // zu setzender Wert
        throws ExpressionEvaluationException
    {
        if (! paRunEnv.isMemberEnabled() )
        {
            throw new ExpressionEvaluationException(
                    getLine(),
                    getCol(),
                    getSource(),
                    getOriginalSource(),
                    "member access not enabled");
        }

        // Holen gedottetes Objekt
       Object dottedObj = this.dottedExpression.eval( paRunEnv );
       BeanUtil.set(dottedObj, this.memberName, pObjValue);
    }

    /**
     * remove on object member not supported
     */
    public Object remove(AbstractRuntimeEnvironment paRunEnv)
        throws ExpressionEvaluationException
    {
                    throw new ExpressionEvaluationException(
                        getLine(),
                        getCol(),
                        getSource(),
                        getOriginalSource(),
                        "remove on object member not supported");
    } 


}
