package de.cnc.expression.tokencomplex;

import java.lang.reflect.Method;

import de.cnc.expression.AbstractRuntimeEnvironment;
import de.cnc.expression.AbstractToken;
import de.cnc.expression.Types;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.expression.exceptions.ExpressionParseException;
import de.cnc.expression.functions.AbstractFunction;
import de.cnc.expression.util.AbstractTokenArrayList;
import de.cnc.expression.util.ParserUtil;
import de.cnc.util.BeanUtil;
import de.cnc.util.MethodUtil;

/**
 * Member-Dot-Operator <b>&lt;index&gt;.member</b>  zur 
 * Adressierung eines Members eines Objekts . <br>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class MethodDotOperator extends AbstractDotOperator
{
    /**
     * der Ausdruck, hinter dem ein Dot-Operator steht
     */
    //protected AbstractToken dottedExpression;
    
    /** Name der adressierten Objekt-Methode */
    protected String methodName;

    /**
     * parameter list
     * TODO mal umstellen auf ExpressionArrayList
     */
    protected AbstractTokenArrayList paramList;

    /**
     * @throws ExpressionParseException
     *
     */
    public static MethodDotOperator parseMethodDotOperator(
            final String paStr, //--
            final int paIntLine, //--
            final int paIntCol, //--
            final String paStrOriginalSource) //--
        throws ExpressionParseException
    { // , NoSuchTokenException {
//System.out.println("MethodDotOperator parseMethodDotOperator: " + paStr);
        if (!paStr.startsWith(".")) {
            return null;
        }

        if (paStr.indexOf( "(") < 1 ) {
            return null;
        }

        int iWhites = ParserUtil.parseWhitesAndComments( paStr.substring( 1 ) );

        final int identifierLen = ParserUtil.parseIdentifier( paStr.substring( 1 + iWhites )  );
        if (identifierLen < 1)
        {
            return null;
        }
        
        final MethodDotOperator dotElem = new MethodDotOperator(paStrOriginalSource, paIntLine, paIntCol);
        dotElem.methodName = paStr.substring(iWhites + 1, iWhites + identifierLen + 1);

        int iRun = 1 + iWhites + identifierLen;

        iRun += ParserUtil.parseWhitesAndComments( paStr.substring( iRun ) );

        AbstractTokenArrayList paramListTmp = new AbstractTokenArrayList();

        iRun = AbstractFunction.parsePrmExprList(paStr.substring( 1 ), paIntLine, paIntCol, paStrOriginalSource, iRun, paramListTmp);

        iRun++;

        dotElem.paramList = paramListTmp;
        dotElem.strSource = paStr.substring(0, iRun);
        dotElem.iSrcLength = iRun;

        return dotElem;
    } // end method parse

    /**
     * constructor
     */
    public MethodDotOperator(
        String paStrOriginalSource,
        int paIntLine,
        int paIntCol)
    {
        super(paStrOriginalSource, paIntLine, paIntCol);
        // System.out.println( "constructor paStrOriginalSource:"+paStrOriginalSource);
    } // end constructor

    /**
     * set the dotted expression.
     * Setzt den Ausdruck, der am Ende eine Index-Element-Operator []
     * trgt. Somit wird sozusagen der hinter dem indizierten Ausdruck stehende
     * Index-Operator vor den indizierten Ausdruck gesetzt.
     * Demzufolge wird erst der indzierte Ausdruck evaluiert und auf dessen Ergebnis
     * dann der Index-Operator angewendet.
     * 
     * @param token
     */
    public void setDottedExpression(AbstractToken pToken)
    {
        this.dottedExpression = pToken;
        //aufaddieren der Lnge des gedotteten Ausdruckes,
        //da gilt >xxx[ yyy ]< Lnge indizierter Ausdruck + Lnge Index-Operator
        this.iSrcLength += pToken.getSrcLength();
    }

    /**
     * get the wahrscheinlichen return typen . <br>
     * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
     * null wenn kein Rckgabe-Wert,
     * mehrere wenn mehrere mglich
     * ( z.B. Plus-Operator kann numerisch oder String liefern)
     */
    public int[] getCheckReturnTypes(/*int[] paIntArrTypes*/) 
    throws ExpressionParseException
    {
        // spter noch Erweiterung auf getypte Variable
        // durch Erweiterung in AbstractRuntimeEnvironment
//        if (!(Types.isInOrUnknown(Types.RET_TYPE_LIST_OR_ARRAY, paIntArrTypes))) {
//            throw new ExpressionParseException(getLine(), getCol(), getOriginalSource(), "List or Array type expected");
//          }

        return new int[] { Types.RET_TYPE_UNKNOWN };
    } // end method getReturnTypes

   /**
     * rekursives Aufrufen der Klammerung der Ausdrcke
     * entsprechend der Operator-Prioritt.
     * Weiterleiten an den Index-Auisdruck ( var[ &lt;exp&gt; ] )
     */
    public void organizeOperatorPriority()
    {
        this.dottedExpression.organizeOperatorPriority();
        if (this.paramList != null)
        {
            for (int i = 0; i < this.paramList.size(); i++)
            {
               this.paramList.get(i).organizeOperatorPriority();
            }
        }
    }

    /**
     * Es wird erst der gedottete Ausdruck evaluiert und auf dessen Ergebnis
     * dann der Dot-Operator angewendet.
     */
    public Object eval(AbstractRuntimeEnvironment paRunEnv)
        throws ExpressionEvaluationException
    {
        if (! paRunEnv.isInvokeEnabled() )
        {
            throw new ExpressionEvaluationException(
                    getLine(),
                    getCol(),
                    getSource(),
                    getOriginalSource(),
                    "method invoke not enabled");
        }

//System.out.println( "MethodDotOperator eval this.dottedExpression: " + this.dottedExpression );
        
        Object invokeObj = this.dottedExpression.eval(paRunEnv);
//System.out.println( "eval collObj: " + collObj );
        //return  BeanUtil.get(dottedObj, this.memberName );
        
        
        //Parameter mit optionaler Anzahl auslesen
        final Object[] paramValueArr;
        if (this.paramList == null)
        {
            paramValueArr = new Object[0];
        }
        else
        {
            paramValueArr = new Object[ this.paramList.size() ];
            for (int i = 0; i < this.paramList.size(); i++)
            {
                final AbstractToken runPrmExpr = this.paramList.get(i);
//System.out.println( "i: " + i );
//System.out.println( "runPrmExpr: : " + runPrmExpr);
                paramValueArr[i] = runPrmExpr.eval(paRunEnv);
//System.out.println( "paramValueArr[i]: " + paramValueArr[i]);

            }
        }

           //Erzeugen Class-Array fr Parameter-Klassen
           Class[] paramClassArr = BeanUtil.createParamClassArr(paramValueArr);
           
           //hier funktioniert noch nicht alles sauber, was ist wenn der Parameter null ist
           //bei primitiven Parameter nicht erlaubt, bei nicht primitiven alle erlaubt
           Method method = BeanUtil.getMethod( invokeObj , this.methodName , paramClassArr );
           
           if (method == null)
           {
              String prmClassStr = BeanUtil.createParamClassStr(paramClassArr);
              throw new ExpressionEvaluationException(
                 getLine(),
                 getCol(),
                 getSource(),
                 getOriginalSource(),
                 "method " + invokeObj.getClass().getName() + "#" + this.methodName + "( " + prmClassStr + " )" + " not found");
           }
           
           try
           {
           Object retValue = MethodUtil.invoke( method , invokeObj , paramValueArr );

           return retValue;
        }
        catch (Exception e)
        {
           throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "" + e);
        }



    }

    /**
     * debug output
     */
    public String toString()
    {
        String strRet =
            getClass().getName()
                + " srcLength: "
                + getSrcLength()
                + " source: "
                + getSource()
                + " dotted expression: "
                + this.dottedExpression;

        return strRet;
    }

}
