package de.cnc.expression.tokencomplex;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;
import de.cnc.expression.util.*;


/**
 * Diese Klasse reprsentiert mehrere Expressions / Ausdrcke,
 * die in curly brackets { } eingeschlossen sind, und durch
 * Semikolon abgetrennt werden { ... ; ... ; ... }.
 * Ab 03.06.2004 sind auch leere Anweisungen, dass heisst
 * Semikolon folgt auf Semikolon oder Semikolon steht am Anfang
 * oder Ende, erlaubt.
 * 
 * @author Heiner Kcker
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 */
public class Sequence extends AbstractToken {

  /**
   * expression list
   */
  protected ExpressionArrayList expList = new ExpressionArrayList();

  public static Sequence parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {

    Sequence sequ = new Sequence(paStrOriginalSource, paIntLine, paIntCol);

    int iRun = 0;

    // eine ffnende Klammer
    if (!(iRun < paStr.length() && paStr.charAt(iRun) == '{')) {
      // das hier ist keine Sequence
      return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no function");
    }
    iRun++;
    // danach null bis beliebig viele weisse zeichen
    int iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));

    iRun += iWhites;

    // Befehle Einlesen
    // while (iRun < paStr.length() && paStr.charAt(iRun) != '}') {
    while (iRun < paStr.length()) {
      if (iRun < paStr.length() && paStr.charAt(iRun) == '}') {
        // schliessende Klammer folgt
        iRun++;
        // Abbrechen
        break;
      }
      // sofort auftauchende Semikolon, leere Anweisungen, werden bersprungen
      if (paStr.charAt(iRun) == ';') {
        iRun++;
        // danach Null bis beliebig viele weisse Zeichen
        iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
        iRun += iWhites;
        // neue Runde
        continue;
      }
      sequ.expList.add(Expression.parseEasy(paStr.substring(iRun), paIntLine, iRun + paIntCol, paStrOriginalSource));
      iRun += sequ.expList.tail().getSrcLength();

      // danach null bis beliebig viele weisse zeichen
      iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
      iRun += iWhites;
      // Semikolon fr nchsten Parameter
      if (iRun < paStr.length() && paStr.charAt(iRun) == ';') {
        iRun++;
      }
      else if (iRun < paStr.length() && paStr.charAt(iRun) == '}') {
        iRun++;
        break;
      }
      else {
        throw new ExpressionParseException(paIntLine, paIntCol + iRun, paStrOriginalSource,
            "right curly bracket '}' or semicolon ';' expected");
      }

      if (iRun < paStr.length()) {
        iWhites = ParserUtil.parseWhitesAndComments(paStr.substring(iRun));
        iRun += iWhites;
      }
    }// while
    sequ.strSource = paStr.substring(0, iRun);
    sequ.iSrcLength = iRun;

    return sequ;

  }// end method parse

  /**
   * constructor
   */
  public Sequence(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    // return innerExp.eval(paRunEnv);
    Object retObject = null;

    for (int i = 0; i < expList.size(); i++) {
      if (TimeOut.isTimeOut()) {
        throw new ExpressionEvaluationException(getLine(), getCol(), getSource(), getOriginalSource(), "expression timed out");
      }

      Expression subExp = expList.get(i);

      try {
        retObject = subExp.eval(paRunEnv);
      } catch (BreakSequenceException brSeqExc) {
        // die Sequenz soll mit dem Wert, den die Exception transportiert (cargo), abgebrochen werden
        // System.out.println("BreakSequenceException:" + brSeqExc.getCargoObject());
        retObject = brSeqExc.getCargoObject();
        break;

      } catch (ContinueSequenceException ctSeqExc) {
        // die Sequenz soll von vorn begonnen werden
        // System.out.println("ContinueSequenceException:" + ctSeqExc);
        i = -1;
        continue;
      }
      // System.out.println("sequence eval["+i+"]:"+retObject);
      // if (subExp.getFirstSubExpression().getToken() instanceof BreakSequenceFunction) {
      // // die Sequenz soll abgebrochen werden
      // break;
      // }

      // if (subExp.getFirstSubExpression().getToken() instanceof ContinueSequenceFunction) {
      // // die Sequenz soll von vorn begonnen werden
      // i = -1;
      // continue;
      // }
    }

    return retObject;
  }// end method

  /**
   * debug output
   */
  public String toString() {
    String strRet = getClass().getName() + " srcLength: " + getSrcLength() + " source: " + getSource() + "\n"
        + "expression list: " + expList;

    return strRet;
  }// end method

  /**
   * rekursives Aufrufen der Klammerung der Ausdrcke
   * entsprechend der Operator-Prioritt.
   * Weiterleiten an die Ausdrucks-Liste
   */
  public void organizeOperatorPriority()
  {
      for (int i = 0; i < expList.size(); i++)
      {
          ( (AbstractToken)expList.get(i) ).organizeOperatorPriority();
      }
  }

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()
    throws ExpressionParseException {
    // spter noch Erweiterung auf getypte Variable
    // durch Erweiterung in AbstractRuntimeEnvironment
    return new int[] {Types.RET_TYPE_UNKNOWN };
  }// end method getReturnTypes

}// end class
