package de.cnc.expression.tokensimple;


import java.math.BigDecimal;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


public class NumberLiteral extends AbstractToken {

  // Lnge maximal 12 Zeichen
  final static int NUM_MAX_LEN = 12;
  protected BigDecimal bdValue;

  public static NumberLiteral parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    NumberLiteral numLit = new NumberLiteral(paStrOriginalSource, paIntLine, paIntCol);

    // erste Stelle eine Ziffer oder Minus oder Dezimalpunkt
    // danach Ziffer
    // Dezimalpunkt darf nur einmal auftreten
    boolean hasPoint = false;

    // Prfung auf Ziffer oder Minus oder Dezimalpunkt am Anfang
    if (!(Character.isDigit(paStr.charAt(0)) || (paStr.charAt(0) == '-') || (paStr.charAt(0) == '.'))) {
      return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no digit at start of number");
    }
    if (paStr.charAt(0) == '.') {
      hasPoint = true;
    }
    int iRun = 1;

    for (; iRun < NUM_MAX_LEN && iRun < paStr.length(); iRun++) {
      char cTmp = paStr.charAt(iRun);

      if (!(Character.isDigit(cTmp) //--
              || cTmp == '.' // decimal point
                  && iRun < paStr.length() - 1 //--
                  && Character.isDigit(paStr.charAt( iRun + 1 ) ) ) )
      {
        // Ende der Zahl erreicht
        break;
      }
      if (cTmp == '.') {
        if (hasPoint) {
          throw new ExpressionParseException(paIntLine, iRun, paStr, "no decimal point expected");
        }
        hasPoint = true;
      }

    }
    // Ende des Strings erreicht
    numLit.strSource = paStr.substring(0, iRun);
    numLit.iSrcLength = iRun;
    try {
      numLit.bdValue = new BigDecimal(paStr.substring(0, iRun));
      // System.out.println("NumberLiteral "+ paStr.substring(0, iRun) + " " + numLit.bdValue ) ;
    } catch (NumberFormatException nfex) {
      // throw new RuntimeException("Programmfehler");
      return null; // scheinbar doch keine Zahl
    }

    // Number-String lnger als maximum
    if (iRun >= NUM_MAX_LEN) {
      throw new ExpressionParseException(paIntLine, paIntCol, paStr, "number to long");
    }

    return numLit;

  }// end method parse

  /**
   * constructor
   */
  public NumberLiteral(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    return bdValue;
  }

  public String toString() {
    String strRet = getClass().getName() + " srcLength: " + getSrcLength() + " value: " + bdValue;

    return strRet;
  }

  /**
   * rekursives Aufrufen der Klammerung der Ausdrcke
   * entsprechend der Operator-Prioritt.
   * Nichts zu tun.
   */
  public void organizeOperatorPriority()
  {
      ;
  }

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()throws ExpressionParseException {
    return new int[] {Types.RET_TYPE_NUM };
  }// end method getReturnTypes

}// end class
