package de.cnc.expression.tokensimple;

import java.math.BigDecimal;

import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;

/**
 * Reserved Words true, false, null and so on
 */
public class ReservedWord extends AbstractToken {

  protected Object oValue;
  protected int iRetType;

  public static ReservedWord parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    ReservedWord resWrd = new ReservedWord(paStrOriginalSource, paIntLine, paIntCol);

    if ("true".equals(paStr) || // ...
        ((paStr.length() > 4 && paStr.startsWith("true") && !Character.isLetterOrDigit(paStr.charAt(4))))) {
      resWrd.strSource = "true";
      resWrd.iSrcLength = 4;
      resWrd.oValue = Boolean.TRUE;
      resWrd.iRetType = Types.RET_TYPE_BOOL;
      return resWrd;
    }
    else if ("false".equals(paStr) || // ...
        (paStr.length() > 5 && paStr.startsWith("false") && !Character.isLetterOrDigit(paStr.charAt(5)))) {
      resWrd.strSource = "false";
      resWrd.iSrcLength = 5;
      resWrd.oValue = Boolean.FALSE;
      resWrd.iRetType = Types.RET_TYPE_BOOL;
      return resWrd;
    }
    if ("null".equals(paStr) || // ...
        (paStr.length() > 4 && paStr.startsWith("null") && !Character.isLetterOrDigit(paStr.charAt(4)))) {
      resWrd.strSource = "null";
      resWrd.iSrcLength = 4;
      resWrd.oValue = null;
      resWrd.iRetType = Types.RET_TYPE_NULL;
      return resWrd;
    }
    if ("apostroph".equals(paStr) || // ...
        (paStr.length() > 9 && paStr.startsWith("apostroph") && !Character.isLetterOrDigit(paStr.charAt(9)))) {
      resWrd.strSource = "apostroph";
      resWrd.iSrcLength = 9;
      resWrd.oValue = "\"";
      resWrd.iRetType = Types.RET_TYPE_STRING;
      return resWrd;
    }
    if ("quote".equals(paStr) || // ...
        (paStr.length() > 5 && paStr.startsWith("quote") && !Character.isLetterOrDigit(paStr.charAt(5)))) {
      resWrd.strSource = "quote";
      resWrd.iSrcLength = 5;
      resWrd.oValue = "\"";
      resWrd.iRetType = Types.RET_TYPE_STRING;
      return resWrd;
    }
    if ("highcomma".equals(paStr) || // ...
        (paStr.length() > 9 && paStr.startsWith("highcomma") && !Character.isLetterOrDigit(paStr.charAt(9)))) {
      resWrd.strSource = "highcomma";
      resWrd.iSrcLength = 9;
      resWrd.oValue = "'";
      resWrd.iRetType = Types.RET_TYPE_STRING;
      return resWrd;
    }
    if ("highComma".equals(paStr) || // ...
        (paStr.length() > 9 && paStr.startsWith("highComma") && !Character.isLetterOrDigit(paStr.charAt(9)))) {
      resWrd.strSource = "highComma";
      resWrd.iSrcLength = 9;
      resWrd.oValue = "'";
      resWrd.iRetType = Types.RET_TYPE_STRING;
      return resWrd;
    }
    if ("newline".equals(paStr) || // ...
        (paStr.length() > 7 && paStr.startsWith("newline") && !Character.isLetterOrDigit(paStr.charAt(7)))) {
      resWrd.strSource = "highcomma";
      resWrd.iSrcLength = 7;
      resWrd.oValue = "\n";
      resWrd.iRetType = Types.RET_TYPE_STRING;
      return resWrd;
    }
    if ("newLine".equals(paStr) || // ...
        (paStr.length() > 7 && paStr.startsWith("newLine") && !Character.isLetterOrDigit(paStr.charAt(7)))) {
      resWrd.strSource = "highComma";
      resWrd.iSrcLength = 7;
      resWrd.oValue = "\n";
      resWrd.iRetType = Types.RET_TYPE_STRING;
      return resWrd;
    }
    if ("pi".equals(paStr) || // ...
        (paStr.length() > 2 && paStr.startsWith("pi") && !Character.isLetterOrDigit(paStr.charAt(2)))) {
      resWrd.strSource = "pi";
      resWrd.iSrcLength = 2;
      resWrd.oValue = new BigDecimal( Math.PI ) ;
      resWrd.iRetType = Types.RET_TYPE_NUM;
      return resWrd;
    }
    if ("PI".equals(paStr) || // ...
        (paStr.length() > 2 && paStr.startsWith("PI") && !Character.isLetterOrDigit(paStr.charAt(2)))) {
      resWrd.strSource = "PI";
      resWrd.iSrcLength = 2;
      resWrd.oValue = new BigDecimal( Math.PI ) ;
      resWrd.iRetType = Types.RET_TYPE_NUM;
      return resWrd;
    }
    if ("euler".equals(paStr) || // ...
        (paStr.length() > 5 && paStr.startsWith("euler") && !Character.isLetterOrDigit(paStr.charAt(5)))) {
      resWrd.strSource = "euler";
      resWrd.iSrcLength = 5;
      resWrd.oValue = new BigDecimal( Math.E ) ;
      resWrd.iRetType = Types.RET_TYPE_NUM;
      return resWrd;
    }
    if ("Euler".equals(paStr) || // ...
        (paStr.length() > 5 && paStr.startsWith("Euler") && !Character.isLetterOrDigit(paStr.charAt(5)))) {
      resWrd.strSource = "Euler";
      resWrd.iSrcLength = 5;
      resWrd.oValue = new BigDecimal( Math.E ) ;
      resWrd.iRetType = Types.RET_TYPE_NUM;
      return resWrd;
    }
    if ("EULER".equals(paStr) || // ...
        (paStr.length() > 5 && paStr.startsWith("EULER") && !Character.isLetterOrDigit(paStr.charAt(5)))) {
      resWrd.strSource = "EULER";
      resWrd.iSrcLength = 5;
      resWrd.oValue = new BigDecimal( Math.E ) ;
      resWrd.iRetType = Types.RET_TYPE_NUM;
      return resWrd;
    }

    return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no digit at start of number");
  }// end method parse

  /**
   * constructor
   */
  public ReservedWord(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    return oValue;
  }

  /**
   * Prfroutine fr Variablen-Namen . <br>
   * @see de.cnc.expression.tokencomplex.Variable#parse
   */
  public static boolean isReservedWord(String strPa) {
    return // ...
        "true".equals(strPa) || // ...
        "false".equals(strPa) || // ...
        "null".equals(strPa) ||// ...
        "new".equals(strPa) ||// ...
        "apostroph".equals(strPa) || // ...
        "quote".equals(strPa) || // ...
        "highcomma".equals(strPa) || // ...
        "highComma".equals(strPa) || // ...
        "newline".equals(strPa) || // ...
        "newLine".equals(strPa) ||  // ...
        "pi".equals(strPa) ||  // ...
        "PI".equals(strPa) ||  // ...
        "euler".equals(strPa) ||  // ...
        "Euler".equals(strPa) ||  // ...
        "EULER".equals(strPa)  // ...
        ;
  }

  public String toString() {
    String strRet = getClass().getName() + " srcLength: " + getSrcLength() + " value: " + oValue;

    return strRet;
  }

  /**
   * rekursives Aufrufen der Klammerung der Ausdrcke
   * entsprechend der Operator-Prioritt.
   * Nichts zu tun.
   */
  public void organizeOperatorPriority()
  {
      ;
  }

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()throws ExpressionParseException {
    return new int[] {iRetType };
  }// end method getReturnTypes

}// end class
