package de.cnc.expression.tokensimple;


import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;


/**
 * Stringliteral . <br>
 * "abc" oder 'abc' sind mglich
 */
public class StringLiteral extends AbstractToken {

  protected String strContent;

  public static StringLiteral parse(String paStr, int paIntLine, int paIntCol, String paStrOriginalSource)
    throws  ExpressionParseException { // , NoSuchTokenException {
    boolean bUseHighComma = false;

    if (paStr.charAt(0) == '"') {
      // Apostroph am Anfang
      bUseHighComma = false;
    }
    else if (paStr.startsWith("'")) {
      // Hochkomma am Anfagn
      bUseHighComma = true;
    }
    else {
      // weder Apostrph noch Hochkomma am Anfang
      return null; // throw new NoSuchTokenException(paIntLine, paIntCol, paStr, "no apostroph at start of string");
    }
    int iRun = 1;
    StringLiteral strLit = new StringLiteral(paStrOriginalSource, paIntLine, paIntCol);

    for (; iRun < paStr.length(); iRun++) {

      char cTmp = paStr.charAt(iRun);

      if (cTmp == '"' && iRun > 1 && paStr.charAt(iRun - 1) == '\\') {
        // encodestes Apostroph
        continue;
      }

      // Prfung auf endendes Apostroph oder Hochkomma (chr 39)
      if (((!bUseHighComma) && cTmp == '"') || (bUseHighComma && cTmp == (char) 39)) {
        // Ende des Strings erreicht
        strLit.strSource = paStr.substring(0, iRun + 1);
        strLit.iSrcLength = iRun + 1;
        // kan man optimieren, indem man durchlaufen und decodieren in einem durchgang macht
        strLit.strContent = decodeString(paStr.substring(1, iRun));
        // System.out.println("StringLiteral#parse: "+  strLit ) ;
        return strLit;
      }
    }
    // throw new ExpressionParseException(paIntLine, paIntCol, paStr, "no apostroph at end of string");
    throw new ExpressionParseException(paIntLine, paIntCol, paStrOriginalSource,
        "no " + (bUseHighComma ? "highcomma '" : "apostroph \"") + " at end of string");
  }// end method parse

  private static String decodeString(String paStr)
    throws  ExpressionParseException {
    StringBuffer sb = new StringBuffer();

    for (int i = 0; i < paStr.length(); i++) {
      char cTmp = paStr.charAt(i);

      // System.out.println(cTmp);
      if (cTmp != '\\') {
        sb.append(cTmp);
      }
      else {
        i++;
        if (i >= paStr.length()) {
          throw new ExpressionParseException(0, i, paStr, "escape character expected");
        }

        char cEsc = paStr.charAt(i);

        switch (cEsc) {
        case '\\':
          sb.append('\\');
          break;

        case '"':
          sb.append('"');
          break;

        case 'b':
          sb.append('\b');
          break;

        case 'n':
          sb.append('\n');
          break;

        case 'r':
          sb.append('\r');
          break;

        case 't':
          sb.append('\t');
          break;

        default:
          throw new ExpressionParseException(0, i, paStr, "escape character unknown");
        }
      }

    }// for
    return sb.toString();
  }// end method decodeString

  /**
   * constructor
   */
  public StringLiteral(String paStrOriginalSource, int paIntLine, int paIntCol) {
    super(paStrOriginalSource, paIntLine, paIntCol);
  }// end constructor

  /**
   * process the token
   */
  public Object eval(AbstractRuntimeEnvironment paRunEnv) throws ExpressionEvaluationException {
    return strContent;
  }

  public String toString() {
    String strRet = getClass().getName() + " srcLength: " + getSrcLength() + " content: " + strContent;

    return strRet;
  }

  /**
   * rekursives Aufrufen der Klammerung der Ausdrcke
   * entsprechend der Operator-Prioritt.
   * Nichts zu tun.
   */
  public void organizeOperatorPriority()
  {
      ;
  }

  /**
   * get the wahrscheinlichen return typen . <br>
   * Wahrscheinlicher Return Type zum vorbeugenden Prfen der Funktionsparameter
   * null wenn kein Rckgabe-Wert,
   * mehrere wenn mehrere mglich
   * ( z.B. Plus-Operator kann numerisch oder String liefern)
   */
  public int[] getCheckReturnTypes()throws ExpressionParseException {
    return new int[] {Types.RET_TYPE_STRING };
  }// end method getReturnTypes

}// end class
