package de.cnc.expression.util;


import java.io.*;
import java.util.StringTokenizer;

import de.cnc.util.Log;


/**
 * Stellt Hilfsmethoden zum Debuggen zur Verfgung
 *
 * @author Heiner Kcker
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 */
public class DebugUtil {

  /**
   * Verhindert das Instanziieren der Klasse
   */
  private DebugUtil() {}

  /**
   * Ausgeben einer zur aufrufenden Methode passenden
   * StackTrace-Zeile . <br>
   * Ist analog zu Aufruf methodInfo(0).<br>
   * Verwendung zum Beispiel: System.out.println( DebugUtils.methodInfo() );
   */
  public static void printMethodInfo() 
  {
    Log.debug(methodInfo(1));
  }// end method printlnMethodInfo

  /**
   * Zurckgeben einer zur aufrufenden Methode passenden
   * StackTrace-Zeile . <br>
   * Ist analog zu Aufruf methodInfo(0).<br>
   * Verwendung zum Beispiel: System.out.println( DebugUtils.methodInfo() );
   */
  public static String methodInfo() {
    return methodInfo(1);
  }// end method methodInfo

  /**
   * Zurckgeben einer zur aufrufenden Methode passenden
   * StackTrace-Zeile . <br>
   * Verwendung zum Beispiel: System.out.println( DebugUtils.methodInfo() );
   *
   * @param paIntStackHeigth Anzahl Stack-Stufen, die zurckgegangen werden muss
   */
  public static String methodInfo(int paIntStackHeigth) {
    // Erzeugen einer Exception und Heraus-Schreiben des StackTrace
    Exception exc = new Exception();
    ByteArrayOutputStream bao = new ByteArrayOutputStream();
    PrintWriter pw = new PrintWriter(bao);

    exc.printStackTrace(pw);
    pw.close();
    String strStackTrace = bao.toString();
    // System.out.println(strStackTrace) ;

    // zeilenweise aufteilen
    StringTokenizer tokenizer = new StringTokenizer(strStackTrace, "\n");

    // erste Zeile berspringen
    tokenizer.nextToken();
    String retStr = "???";

    // n-te Zeile verwenden
    for (int i = 0; i <= paIntStackHeigth && tokenizer.hasMoreTokens(); i++) {
      tokenizer.nextToken();
    }
    if (tokenizer.hasMoreTokens()) {
      retStr = tokenizer.nextToken();
    }

    // Abschneiden fhrendes "at "
    retStr = retStr.trim();
    if (retStr.startsWith("at ")) {
      retStr = retStr.substring(3);
    }

    return retStr;
  }// end method methodInfo

  /**
   * Ausgabe StackTrace um zu erfahren, von wo diese Methode gerufen wurde.
   *
   * @author Heiner Kcker
   */
  public static void printStackTrace() {
    (new Exception("debug")).printStackTrace();
    System.err.println("");
  }// end method printStackTrace

}// end class

