package de.cnc.flowcompiler;

import de.cnc.FlowControlUtil;
import de.cnc.domainparser.*;

/**
 * Util-Klasse
 */
public class FlowControlCompilerUtil {

	/**
	 * Instanziieren verboten
	 */
	private FlowControlCompilerUtil() {
		super();
	}

    /**
     * Element auf Identifier prfen und casten
     * 
     * @param pElement
     */
    public static Identifier checkForIdentifier(AbstractElement pElement)
    {
        if (!(pElement instanceof Identifier))
        {
            FlowControlUtil.complain("identifier expected", pElement.getSourcePos());
        }
        return (Identifier) pElement;
    }

    /**
     * Element auf Identifier prfen und Namen (Inhalt) des
     * Identifiers zurckgeben.
     * 
     * @param pElement
     */
    public static String getIdentifierName(AbstractElement pElement)
    {
        Identifier identifier = checkForIdentifier(pElement);
        return identifier.getNettoContent();
    }

    /**
     * Element auf ParenthesisArea prfen und casten
     * 
     * @param pElement
     */
    public static ParenthesisArea checkForParenthesisArea(AbstractElement pElement)
    {
        if (!(pElement instanceof ParenthesisArea))
        {
            FlowControlUtil.complain("parenthesis area expected", pElement.getSourcePos());
        }
        return (ParenthesisArea) pElement;
    }

    /**
     * Element auf BracesArea prfen und casten
     * 
     * @param pElement
     */
    public static BracesArea checkForBracesArea(AbstractElement pElement)
    {
        if (!(pElement instanceof BracesArea))
        {
            FlowControlUtil.complain("braces area expected", pElement.getSourcePos());
        }
        return (BracesArea) pElement;
    }

    public static ElementList checkForOneElementList(ParenthesisArea pParenthesisArea)
    {
        CommaSeparatedList commaSeparatedList = pParenthesisArea.getCommaSeparatedList();

        if (commaSeparatedList.elementListCount() < 1)
        {
            FlowControlUtil.complain("expression expected", commaSeparatedList.getSourcePos());
        }
        if (commaSeparatedList.elementListCount() > 1)
        {
            FlowControlUtil.complain("exactly one expression expected, comma found",
                    commaSeparatedList.getElementList(1).getSourcePos());
        }
        return (ElementList) commaSeparatedList.getElementList(0);
    }

    /**
     * Segment auf genau einen Identifier prfen.
     * 
     * @param pElement
     */
    public static void checkForOneIdentifier(Segment pSegment)
    {
        if ( pSegment.elementCount() != 1 )
        {
            FlowControlUtil.complain( 
                 "exactly one Identifier expected" , 
                 pSegment.getSourcePos() );
        }
        checkForIdentifier( pSegment.getElement( 0 ) );
    }

    /**
     * Segment auf genau zwei Identifier prfen.
     * 
     * @param pElement
     */
    public static void checkForTwoIdentifier(Segment pSegment)
    {
        if ( pSegment.elementCount() != 2 )
        {
            FlowControlUtil.complain( 
                 "exactly two Identifier expected" , 
                 pSegment.getSourcePos() );
        }
        checkForIdentifier( pSegment.getElement( 0 ) );
        checkForIdentifier( pSegment.getElement( 1 ) );
    }

    /**
     * Segment auf Identifier, ParenthesisArea und BracesArea,
     * beispielsweise 'if ( ... ) { ... }' prfen.
     * 
     * @param pElement
     */
    public static void checkForIdentifierParenthesisBraces(Segment pSegment)
    {
        if ( pSegment.elementCount() != 3 )
        {
            FlowControlUtil.complain( 
                 "Identifier, ParenthesisArea and BracesArea expected, example: 'if ( ... ) { ... }'" , 
                 pSegment.getSourcePos() );
        }
        checkForIdentifier( pSegment.getElement( 0 ) );
        checkForParenthesisArea( pSegment.getElement( 1 ) );
        checkForBracesArea( pSegment.getElement( 2 ) );
    }

    /**
     * Segment auf Identifier, Identifier, ParenthesisArea und BracesArea,
     * beispielsweise 'else if ( ... ) { ... }' prfen.
     * 
     * @param pElement
     */
    public static void checkForIdentifierIdentifierParenthesisBraces(Segment pSegment)
    {
        if ( pSegment.elementCount() != 4 )
        {
            FlowControlUtil.complain( 
                 "Identifier, Identifier, ParenthesisArea and BracesArea expected, example: 'else if ( ... ) { ... }'" , 
                 pSegment.getSourcePos() );
        }
        checkForIdentifier( pSegment.getElement( 0 ) );
        checkForIdentifier( pSegment.getElement( 1 ) );
        checkForParenthesisArea( pSegment.getElement( 2 ) );
        checkForBracesArea( pSegment.getElement( 3 ) );
    }

    /**
     * Segment auf Identifier und ParenthesisArea,
     * beispielsweise 'jsp ( ... )' prfen.
     * 
     * @param pElement
     */
    public static void checkForIdentifierParenthesis(Segment pSegment)
    {
        if ( pSegment.elementCount() != 2 )
        {
            FlowControlUtil.complain( 
                 "Identifier and ParenthesisArea expected, example: 'jsp ( ... )'" , 
                 pSegment.getSourcePos() );
        }
        checkForIdentifier( pSegment.getElement( 0 ) );
        checkForParenthesisArea( pSegment.getElement( 1 ) );
    }

    /**
     * Segment auf Identifier, Identifier und ParenthesisArea,
     * beispielsweise 'jsp ( ... )' prfen.
     * 
     * @param pElement
     */
    public static void checkForIdentifierIdentifierParenthesis(Segment pSegment)
    {
        if ( pSegment.elementCount() != 3 )
        {
            FlowControlUtil.complain( 
                 "Identifier and ParenthesisArea expected, example: 'call name ( ... )'" , 
                 pSegment.getSourcePos() );
        }
        checkForIdentifier( pSegment.getElement( 0 ) );
        checkForIdentifier( pSegment.getElement( 1 ) );
        checkForParenthesisArea( pSegment.getElement( 2 ) );
    }

    /**
     * Segment auf Identifier und Braces,
     * beispielsweise 'else { ... }' prfen.
     * 
     * @param pElement
     */
    public static void checkForIdentifierBraces(Segment pSegment)
    {
        if ( pSegment.elementCount() != 2 )
        {
            FlowControlUtil.complain( 
                 "Identifier and Braces expected, example: 'else { ... }'" , 
                 pSegment.getSourcePos() );
        }
        checkForIdentifier( pSegment.getElement( 0 ) );
        checkForBracesArea( pSegment.getElement( 1 ) );
    }

}
