package de.cnc.flowprogram;




/**
 * Klasse fr auszufhrende Operation in {@link FlowControlInstruction} . <br/>
 * notwendig wegen fehlendem Aufzhlungstyp in Java. <br/>
 * Realsiert entsprechend Konzept in EnumTyp.txt . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id:$
 */
public final class ControlCode {

  /**
   * Steuercode fr nichts tun ( No Operation)
   */
  //public static final ControlCode NOP = new ControlCode("NOP");
  public static final int NOP = 0;

  /**
   * Steuercode fr Aufruf Action-Methode
   */
  //public static final ControlCode ACTION = new ControlCode("ACTION");
  public static final int ACTION = 1;

  /**
   * Steuercode fr Aufruf JSP
   */
  //public static final ControlCode JSP = new ControlCode("JSP");
  public static final int JSP = 2;

  /**
   * Steuercode fr JUMP ohne Bedingung
   */
  //public static final ControlCode JUMP = new ControlCode("JUMP");
  public static final int JUMP = 3;

  /**
   * Steuercode fr JUMP mit Bedingung
   */
  //public static final ControlCode JUMP_IF = new ControlCode("JUMP_IF");
  public static final int JUMP_IF = 4;

  /**
   * Steuercode fr JUMP mit Bedingung negiert
   */
  //public static final ControlCode JUMP_IF_NOT = new ControlCode("JUMP_IF_NOT");
  public static final int JUMP_IF_NOT = 5;

  /**
   * Steuercode fr den Beginn einer Procedure. <br/>
   * Fhrt keinen Befehl aus (NOP), dient aber als Merker
   * zum Auflsen der CALLs
   */
  //public static final ControlCode START_PROC = new ControlCode("START_PROC");
  public static final int PROC_START = 6;

  /**
   * Steuercode fr den Beginn einer Function. <br/>
   * Fhrt keinen Befehl aus (NOP), dient aber als Merker
   * zum Auflsen der CALLs
   */
  //public static final ControlCode START_PROC = new ControlCode("START_PROC");
  public static final int FUNC_START = 7;

  /**
   * Steuercode fr CALL
   */
  //public static final ControlCode CALL = new ControlCode("CALL");
  public static final int CALL = 8;

  /**
   * Steuercode fr PROC_RETURN
   */
  //public static final ControlCode PROC_RETURN = new ControlCode("PROC_RETURN");
  public static final int PROC_RETURN = 9;

  /**
   * Steuercode fr FUNC_RETURN
   */
  //public static final ControlCode FUNC_RETURN = new ControlCode("FUNC_RETURN");
  public static final int FUNC_RETURN = 10;

  /**
   * Steuercode fr eine Sprung-Marke. <br/>
   * Sprungmarken drfen nur in der procedure main liegen
   */
  //public static final ControlCode LABEL_FOR_GOTO = new ControlCode("LABEL_FOR_GOTO");
  public static final int LABEL_FOR_GOTO = 11;

  /**
   * Steuercode fr ein GOTO, GOTOs drfen Labels oder Prozeduren anspringen
   */
  //public static final ControlCode GOTO = new ControlCode("GOTO");
  public static final int GOTO = 12;

  /**
   * Steuercode fr das Setzen eines Session-Wertes. <br/>
   * Dabei wird der Session-Key und der Session-Wert jeweils als
   * String bergeben.
   */
  //public static final ControlCode SET = new ControlCode("SET");

  /**
   * Steuercode fr das Rcksetzen eines Session-Wertes. <br/>
   * Dabei wird der Session-Key als String bergeben.
   */
  //public static final ControlCode RESET = new ControlCode("RESET");
  public static final int RESET = 13;

  /**
   * Steuercode fr das Abarbeiten einer Expression
   */
  //public static final ControlCode EVAL_EXPRESSSION = new ControlCode("EVAL_EXPRESSSION");
  public static final int EVAL_EXPRESSSION = 14;

  /**
   * Steuercode fr den Beginn einer WHILE-Schleife. <br/>
   * Fhrt JUMP_IF_NOT aus,
   * Dient als Merker zum Auflsen der CONTINUEs
   */
  //public static final ControlCode START_WHILE = new ControlCode("START_WHILE");
  public static final int START_WHILE = 15;

  /**
   * Steuercode fr das Ende einer WHILE-Schleife. <br/>
   * Fhrt JUMP aus.
   * Dient als Merker zum Auflsen der BREAKs.
   */
  //public static final ControlCode END_WHILE = new ControlCode("END_WHILE");
  public static final int END_WHILE = 16;

  /**
   * Steuercode fr den Beginn einer DO-Schleife. <br/>
   * Fhrt keinen Befehl aus (NOP), dient aber als Merker
   * zum Auflsen der CONTINUEs
   */
  //public static final ControlCode START_DO = new ControlCode("START_DO");
  public static final int START_DO = 17;

  /**
   * Steuercode fr den Ende einer DO-Schleife. <br/>
   * Fhrt JUMP_IF aus.
   * Dient als Merker zum Auflsen der BREAKs
   */
  //public static final ControlCode END_DO = new ControlCode("END_DO");
  public static final int END_DO = 18;

  /**
   * Steuercode fr das Fortsetzen einer Schleife. <br/>
   */
  //public static final ControlCode LOOP_CONTINUE = new ControlCode("LOOP_CONTINUE");
  public static final int LOOP_CONTINUE = 19;

  /**
   * Steuercode fr das Abbrechen einer Schleife. <br/>
   */
  //public static final ControlCode LOOP_BREAK = new ControlCode("LOOP_BREAK");
  public static final int LOOP_BREAK = 20;

  /**
   * Steuercode fr Session-invalidate . <br/>
   */
  //public static final ControlCode SESSION_INVALIDATE = new ControlCode("SESSION_INVALIDATE");
  public static final int SESSION_INVALIDATE = 21;

  /**
   * Steuercode fr locale Variable . <br/>
   */
  //public static final ControlCode LOCAL_VAR = new ControlCode("LOCAL_VAR");
  public static final int LOCAL_VAR = 22;

  /**
   * Name des Steuercodes
   */
  //private String name;

  /**
   * Konstruktor, von aussen nicht aufrufbar
   */
//  private ControlCode(String name) {
//    this.name = name;
//    // this.ord = obereGrenze++ ;
//  }

  /**
   * Debug-Ausgabe Name
   */
//  public String toString() {
//    return this.name;
//  }

  /**
   * auszufhrende Operation. <br/>
   * auf {@link #ACTION} oder {@link #JSP}
   * oder {@link #JUMP} oder {@link #JUMP_IF} gesetzt.
   */
  // private int opCode ;

  /**
   * Konstruktor ohne Parameter
   */

  /*
   public ControlCode() {
   opCode = NOP ;
   }//end constructor
   */

  /**
   * Konstruktor mit Parameter
   */

  /*
   public ControlCode( int newOpCode ) {
   this.opCode = newOpCode ;
   }//end constructor
   */

  /**
   * get opCode
   *
   * @return opCode
   */

  /*
   public int getOpCode() {
   return opCode;
   }//end method getOpCode
   */

  /**
   * set opCode
   *
   * @param opCode new Value
   */

  /*
   public void setOpCode( int newOpCode ) {
   this.opCode = newOpCode ;
   }//end method setOpCode

   /**
   * Debug-Ausgabe
   */

  /*
   public String toString() {
   int iWidth = 12 ;
   switch ( opCode ) {
   case NOP :
   return StringUtil.pad( "NOP" , iWidth ) ;
   case ACTION :
   return StringUtil.pad( "ACTION" , iWidth ) ;
   case JSP :
   return StringUtil.pad( "JSP" , iWidth ) ;
   case JUMP :
   return StringUtil.pad( "JUMP" , iWidth ) ;
   case JUMP_IF :
   return StringUtil.pad( "JUMP_IF" , iWidth ) ;
   case JUMP_IF_NOT :
   return StringUtil.pad( "JUMP_IF_NOT" , iWidth ) ;
   case START_PROC :
   return StringUtil.pad( "START_PROC" , iWidth ) ;
   case CALL :
   return StringUtil.pad( "CALL" , iWidth ) ;
   case PROC_RETURN :
   return StringUtil.pad( "PROC_RETURN" , iWidth ) ;
   case GOTO_LABEL :
   return StringUtil.pad( "GOTO_LABEL" , iWidth ) ;
   case GOTO :
   return StringUtil.pad( "GOTO" , iWidth ) ;
   case SET :
   return StringUtil.pad( "SET" , iWidth ) ;
   case RESET :
   return StringUtil.pad( "RESET" , iWidth ) ;
   default:
   return "??? " + opCode ;
   }//end switch
   }//end method toString
   */

  /**
   * Debug-Ausgabe
   */

  /*
   public String toString() {
   switch ( opCode ) {
   case NOP :
   return "NOP" ;
   case ACTION :
   return "ACTION" ;
   case JSP :
   return "JSP" ;
   case JUMP :
   return "JUMP" ;
   case JUMP_IF :
   return "JUMP_IF" ;
   case JUMP_IF_NOT :
   return "JUMP_IF_NOT" ;
   case START_PROC :
   return "START_PROC" ;
   case CALL :
   return "CALL" ;
   case PROC_RETURN :
   return "PROC_RETURN" ;
   case LABEL_FOR_GOTO :
   return "LABEL_FOR_GOTO" ;
   case GOTO :
   return "GOTO" ;
   case SET :
   return "SET" ;
   case RESET :
   return "RESET" ;
   case START_WHILE :
   return "START_WHILE" ;
   case END_WHILE :
   return "END_WHILE" ;
   case START_DO :
   return "START_DO" ;
   case END_DO :
   return "END_DO" ;
   case LOOP_CONTINUE :
   return "LOOP_CONTINUE" ;
   case LOOP_BREAK :
   return "LOOP_BREAK" ;
   default:
   return "??? " + opCode ;
   }//end switch
   }//end method toString
   */

    /** Array mit den namen der Control-Codes */
   static final String[] OP_CODE_NAMES = new String[]{
   "NOP" , 
   "ACTION" , 
   "JSP" , 
   "JUMP" , 
   "JUMP_IF" , 
   "JUMP_IF_NOT" , 
   "PROC_START" , 
   "FUNC_START" , 
   "CALL" , 
   "PROC_RETURN" , 
   "FUNC_RETURN" , 
   "LABEL_FOR_GOTO" , 
   "GOTO" , 
   "RESET" , 
   "EVAL_EXPRESSSION" , 
   "START_WHILE" , 
   "END_WHILE" , 
   "START_DO" , 
   "END_DO" , 
   "LOOP_CONTINUE" , 
   "LOOP_BREAK" , 
   "SESSION_INVALIDATE" , 
   "LOCAL_VAR" };

   /**
    * @param controlCode
    * @return
    */
   public static String toText(int opCode)
   {
       if ( opCode >= 0 && opCode < OP_CODE_NAMES.length )
       {
           return OP_CODE_NAMES[ opCode ];
       }
       return "??? " + opCode ;
   }

}// end class ControlCode
