package de.cnc.flowprogram;


import de.cnc.FlowControlUtil;
import de.cnc.domainparser.*;
import de.cnc.domainparser.util.SourcePosition;
import de.cnc.expression.*;
import de.cnc.expression.exceptions.*;
import de.cnc.flowcompiler.FlowControlCompilerUtil;
import de.cnc.util.*;


/**
 * Reprsentation eines Low-Level-Befehls des Steuer-Programmes. <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class FlowControlInstruction
{

    // *** Membervariable ***

    /**
     * Zeilen-Nummer im XML-Dokument fr Debugging
     */
    public SourcePosition sourcePosition;

    /**
     * auszufhrende Operation. <br/>
     */
    //private ControlCode controlCode;
   public int controlCode;

    /**
     * anzuspringende Adresse
     */
    public int jumpDestination;

    /**
     * String mit dem Namen der aufzuspringenden Prozedur oder Label (Sprung-Marke)
     */
    public String procOrLabelNameToCall;

    /**
     * Expression mit dem Namen der aufzurufenden Action oder JSP
     */
    private Expression nameExpression;

    /**
     * Expression fr bedingte Ausfhrung von JUMP oder CALL oder ...
     */
    private Expression conditionExpression;

    /**
     * auszufhrende Expression, die Zeile (das DomainParser-Segment)
     * besteht nur aus der Expression
     */
    private Expression evalExpression;

    /**
     * Debug-Info (zugehriger Strukturbefehl)
     */
    public String debugInfo = "";

    /**
     * Schalter, ob fr diesen Befehl keine User-Validierung durchgefhrt werden soll . <br/>
     * Ist notwendig fr Befehle im Abschnitt Logon (&lt;logon&gt; ... &lt;/logon&gt;), <br/>
     * damit Einloggen gelingt. <br/>
     * Default ist false (also prfen).
     */
    public boolean noUserValidation = false;

    /**
     * Name der laufenden Procedure
     */
    public String procName;

    /**
     * Merker fr Schachtelungstiefe . <br/>
     * Wird in {@link FlowControlCompiler#resolveLabels} verwendet . <br/>
     * hier etwas schlampig als public-variable , weil sowieso ein redesign ansteht
     */
    public int iLoopLevel;

    /**
     * Parameterliste fr call, hier etwas schlampig als public-variable , weil sowieso ein redesign ansteht
     */
    public FlowProcParamList callParamArrList;

    /**
     * Parameterliste fr procedure-begin, hier etwas schlampig als public-variable , weil sowieso ein redesign ansteht
     */
    public FlowProcDeclParamList procParamList;

    /**
     * Return-session-Key (Aufruf-Seite)
     */
    public String returnSessionKey;

    /**
     * Return-Expression (aufgerufene Seite)
     */
    public Expression returnExpression;

    /**
     * Name einer lokalen Variable
     * (Session-Key)
     */
    public String localVarName;

    /**
     * Konstruktor parameterlos
     */
    public FlowControlInstruction()
    {
        ;
    }

    /**
     * Konstruktor parametrisiert
     */
    public FlowControlInstruction(          // -
            SourcePosition  newSourcePosition,  // -
            int             newControlCode,     // -
            int             jumpDestination,    // -
            String          pProcOrLabelName,   // -
            Expression      newExpression,      // -
            String          newDebugInfo,       // -
            String          procName)
    {

        this.sourcePosition = newSourcePosition;
        this.controlCode = newControlCode;
        this.jumpDestination = jumpDestination;
        this.procOrLabelNameToCall = pProcOrLabelName;
        this.conditionExpression = newExpression;
        this.debugInfo = newDebugInfo;
        this.procName = procName;
    }// end constructor

    /**
     * get controlCode
     *
     * @return controlCode
     */
//    public ControlCode getControlCode()
//    {
//        return controlCode;
//    }// end method getControlCode

    /**
     * set controlCode
     *
     * @param controlCode new Value
     */
//    public void setControlCode(ControlCode newControlCode)
//    {
//        this.controlCode = newControlCode;
//    }// end method setControlCode

    /**
     * get name
     *
     * @return name
     */
//    public String getProcOrLabelNameToCall()
//    {
//        return procOrLabelNameToCall;
//    }// end method getName

    /**
     * set name der aufzurufenden Prozedur oder des anzuspringenden Labels
     *
     * @param name new Value
     */
//    public void setProcOrLabelNameToCall(String newName)
//    {
//        this.procOrLabelNameToCall = newName;
//    }// end method setName

    /**
     * get jumpDestination
     *
     * @return jumpDestination
     */
//    public int getJumpDestination()
//    {
//        return jumpDestination;
//    }

    /**
     * set jumpDestination
     *
     * @param jumpDestination new Value
     */
//    public void setJumpDestination(int newJumpDestination)
//    {
//        this.jumpDestination = newJumpDestination;
//    }

    /**
     * get xmlLineNumber
     *
     * @return xmlLineNumber
     */
//    public SourcePosition getSourcePosition()
//    {
//        return sourcePosition;
//    }

    /**
     * set xmlLineNumber
     *
     * @param xmlLineNumber new Value
     */
//    public void setSourcePosition(SourcePosition newSourcePosition)
//    {
//        this.sourcePosition = newSourcePosition;
//    }

    /**
     * get debugInfo
     *
     * @return debugInfo
     */
//    public String getDebugInfo()
//    {
//        return debugInfo;
//    }// end method getDebugInfo

    /**
     * set debugInfo
     *
     * @param debugInfo new Value
     */
//    public void setDebugInfo(String newDebugInfo)
//    {
//        this.debugInfo = newDebugInfo;
//    }// end method setDebugInfo

    /**
     * get procName
     *
     * @return procName
     */
//    public String getProcName()
//    {
//        return procName;
//    }// end method getProcName

    /**
     * set procName
     *
     * @param procName new Value
     */
//    public void setProcName(String newProcName)
//    {
//        this.procName = newProcName;
//    }// end method setProcName

    /**
     * get noUserValidation
     *
     * @return noUserValidation
     */
//    public boolean getNoUserValidation()
//    {
//        return noUserValidation;
//    }// end method getNoUserValidation

    /**
     * set noUserValidation
     *
     * @param noUserValidation new Value
     */
//    public void setNoUserValidation(boolean newNoUserValidation)
//    {
//        this.noUserValidation = newNoUserValidation;
//    }// end method setNoUserValidation

    /**
     * Auslesen der erforderlichen Parameter fr die Sprungbedingungen aus
     * den Attributen des Tags. <br/>
     *
     * @param parAttributeReader Attribute des Tags, geliefert von {@link FlowControlCompiler#readStartTag}
     */
    public void setJumpConditionen(
            AbstractElement pElement) // muss ein ParenthesisArea sein
        throws DomainParseException
    {
        conditionExpression = parseExpressionFromParenthesisArea(pElement, Types.RET_TYPE_BOOL);
    }// end method setJumpConditionen

    /**
     * Expression mit Namen fr JSP oder ACTION parsen
     * 
     * @param expression
     */
    public void setNameExpression(
            AbstractElement pElement) // muss ein ParenthesisArea sein
        throws DomainParseException
    {
        nameExpression = parseExpressionFromParenthesisArea(pElement, Types.RET_TYPE_STRING);
    }

    /**
     * Expression return-Value parsen
     * 
     * @param expression
     */
    public void setReturnExpression(
            AbstractElement pElement) // muss ein ParenthesisArea sein
        throws DomainParseException
    {
        returnExpression = parseExpressionFromParenthesisArea(pElement);
    }

    /**
     * auszufhrende Expression parsen
     * 
     * @param expression
     */
    public void setEvalExpression(
            Segment pSegment)
        throws DomainParseException
    {
        String pEvalExpressionStr = pSegment.getNettoContent();

        //System.out.println("pEvalExpressionStr: " + pEvalExpressionStr);
        try 
        {
            evalExpression = Expression.parseStrict(pEvalExpressionStr, 0, 0);

            SourcePosition srcPos = pSegment.getSourcePos();
            evalExpression.setLine( srcPos.getRow() );
            evalExpression.setCol( srcPos.getCol() );
        }
        catch (ExpressionParseException e) 
        {
           //hier gebe ich nicht die Original-Exception aus, weil man dann zum sehen der Fehlerposition nach unten scrollen muss
            //System.err.println(e.getMessage());
            //e.printStackTrace();
            throw new DomainParseException(e.getMessage(), pSegment.getSourcePos());
        }
    }

    /**
     * Abfragen Konditionen fr bedingte Ausfhrung
     */
    public boolean executeCondition(
            JspRuntimeEnvironment pJspRuntimeEnvironment) 
    {
        if (conditionExpression == null) 
        {
            return true; // unbedingte Ausfhrung, voraussetzung ist, das in setJumpConditionen equalOperation richtig gesetzt wurde
        }
        Object evalObj;

        try
        {
            evalObj = conditionExpression.eval( pJspRuntimeEnvironment );
        }
        catch (ExpressionEvaluationException e)
        {
           Log.fatal( e );
           throw new RuntimeException(e.getMessage());
        }
        // hier noch Fehlerbehandlung einbauen
        Boolean boolObj = (Boolean) evalObj;

        return boolObj.booleanValue();
    }// end method excuteCondition

    /**
     * Abfragen, ob die Instruction ein NOP(No-Operation)-Befehl ist
     */
    public boolean isNop()
    {
        if (controlCode == ControlCode.NOP //--
        || controlCode == ControlCode.PROC_START //--
                || controlCode == ControlCode.LABEL_FOR_GOTO //--
                || controlCode == ControlCode.START_DO)
        {
            return true;
        }// end if
        return false;
    }// end method isNop

    /**
     * Zeilen-Nummer in Flow-Script-Datei
     * 
     * @return
     */
    public int getLineNumber()
    {
        return sourcePosition.getRow();
    }

    /**
     * Einlesen einer Expresion vom bergebenen Parenthesis-Area
     * 
     * @param pElement
     * @return
     * @throws DomainParseException
     */
    private static Expression parseExpressionFromParenthesisArea( // -
            AbstractElement pElement)
        throws DomainParseException 
    {
        ParenthesisArea parenthesisArea = FlowControlCompilerUtil.checkForParenthesisArea(pElement);
        ElementList elementList = FlowControlCompilerUtil.checkForOneElementList(parenthesisArea);
    
        Expression tmpExpression = null;

        try 
        {
            final String exprsStr = elementList.getContent();

            //System.out.println("exprsStr: " + exprsStr);
            tmpExpression = Expression.parseStrict(exprsStr, 0, 0);
        }
        catch (ExpressionParseException e) 
        {
            Log.fatal( e );
            throw new RuntimeException(e.getMessage());
        }
        return tmpExpression;
    }

    /**
     * Einlesen einer Expresion vom bergebenen Parenthesis-Area
     * 
     * @param pElement
     * @return
     * @throws DomainParseException
     */
    private static Expression parseExpressionFromParenthesisArea( // -
            AbstractElement pElement,
            int pType)
        throws DomainParseException 
    {
        ParenthesisArea parenthesisArea = FlowControlCompilerUtil.checkForParenthesisArea(pElement);
        ElementList elementList = FlowControlCompilerUtil.checkForOneElementList(parenthesisArea);
    
        Expression tmpExpression = null;

        try 
        {
            final String exprsStr = elementList.getContent();

            //System.out.println("exprsStr: " + exprsStr);
            tmpExpression = Expression.parseStrict(exprsStr, 0, 0);
            int[] iTypesArr = tmpExpression.getCheckReturnTypes();

            if (!(Types.isInOrUnknown(pType, iTypesArr))) 
            {
                throw new DomainParseException("boolean expression expected",
                        elementList.getSourcePos());
            }
        }
        catch (ExpressionParseException e) 
        {
            Log.fatal( e );
            throw new RuntimeException(e.getMessage());
        }
        return tmpExpression;
    }

    /**
     * Kopf fr die mit toString() aufgebaute Tabelle
     */
    public static String toStringHeader()
    {
        return "nuv mean no user validation\n"
                + "Adr|xmlLine| nuv |   procedure   | controlCode  |jumpDst|        name       |     sessionKey    |     requestKey    |fromAction|      action       |    buttonName     | buttonIndexedName |     equOp    |       value      |   debugInfo       | returnSessionKey";
    }// end method toStringHeader

    /**
     * Debug-Info in Tabellenform. <br/>
     * den Kopf liefert die Methode {@link #toStringHeader}
     *
     * @param iPaNr auszugebendes Tabellen-Element
     */
    public String toString(int  iPaNr)
    {
        StringBuffer sb = new StringBuffer(1024);

        sb.append(StringUtil.leftPad("" + iPaNr, 3) + "|" //--
                + StringUtil.leftPad("" + sourcePosition.getRow() , 7) + "|" //--
                + StringUtil.pad("" + noUserValidation, 5) + "|" //--
                + StringUtil.pad("" + procName, 15) + "|" //--
                + StringUtil.pad("" + ControlCode.toText( controlCode ) , 18) + "|" //--
                + StringUtil.leftPad("" + jumpDestination, 7) + "|"
                + StringUtil.pad("" + procOrLabelNameToCall, 19) + "|" // + StringUtil.pad("" + sessionKey, 19) + "|"
                // + StringUtil.pad("" + requestKey, 19) + "|" 
                // + StringUtil.pad("" + fromAction, 10) + "|"
                // + StringUtil.pad("" + action, 19) + "|" 
                // + StringUtil.pad("" + buttonName, 19) + "|"
                // + StringUtil.pad("" + buttonIndexedName, 19) + "|" 
                // + StringUtil.leftPad("" + equalOperation, 14) + "|"
                // + StringUtil.pad("" + value, 18) + "|" 
                + StringUtil.pad(debugInfo, 20) + "|" // + StringUtil.pad(returnSessionKey, 20)) + "|"
                + "" + conditionExpression);

        if (callParamArrList != null)
        {
            sb.append("|" + callParamArrList);
        }// end if
        if (procParamList != null)
        {
            sb.append("|" + procParamList);
        }// end if
        return sb.toString();
    }

   public String toString()
   {
      return toString( 0 );
   }

    /**
     * Jsp- oder Action-Name dynamisch zur Laufzeit
     * durch Expression-Auswertung feststellen.
     * 
     * @return
     */
    public String getJspOrActionName( //--
      JspRuntimeEnvironment pJspRuntimeEnvironment)
    {
        try
        {
            //hier noch Fehlerbehandlung fr ClassCast zu String einbauen
            return ( String ) nameExpression.eval( pJspRuntimeEnvironment );
        }
        catch (ExpressionEvaluationException e)
        {
            Log.fatal( e );
            throw new RuntimeException(e.getMessage());
        }
    }

    /**
     * Expresssion bei ControlCode.EVAL_EXPRESSSION
     * ausfhren.
     */
    public void evalExpression( //--
         JspRuntimeEnvironment pJspRuntimeEnvironment)
    {
        try
        {
            evalExpression.eval( pJspRuntimeEnvironment );
        }
        catch (ExpressionEvaluationException e)
        {
            //System.err.println(e.getMessage());
            //e.printStackTrace();
            Log.fatal( e , this );
            throw new RuntimeException(e.getMessage());
        }
    }

    /**
     * Expresssion bei ControlCode.PROC_RETURN
     * ausfhren.
     */
    public Object evalReturnExpression( //--
      JspRuntimeEnvironment pJspRuntimeEnvironment) 
    {
        Object retObj = null;
        try
        {
            retObj = returnExpression.eval( pJspRuntimeEnvironment );
        }
        catch (ExpressionEvaluationException e)
        {
            Log.fatal( e , this );
            throw new RuntimeException(e.getMessage());
        }
        return retObj;
    }

    /**
     * @return
     */
    public String getLocalVarNameWithoutPrefix()
    {
        //Abschneiden des Pflichtprefix 'session.', weil der aufrufende Code den zurckgegebenen
        //String als Session-Key verwendet, unter Umgehung des JspRuntimeEnvironment
        //return StringUtil.cutLeft( localVarName , "session." );
        return FlowControlUtil.checkAndCutPrefixSession( localVarName , sourcePosition );
    }// end method toString

}// end class FlowControlInstruction

