package de.cnc.flowprogram;


import java.util.ArrayList;

import de.cnc.util.Log;


/**
 * Eigentliches Low-Level-Programm (Steuertabelle) fr CnC. <br/>
 * Sie enthlt die einzelnen Befehle {@link FlowControlInstruction}. <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id:$
 */
public class FlowControlTable extends ArrayList {

  /**
   * get spezifiziertes Objekt ohne notwendiges Type-Casting wie bei ArrayList.get()
   *
   * @param Index des gewnschten Elementes (Basis 0)
   *
   * @return gewnschtes Element
   */
  public FlowControlInstruction getFlowElement(int iPaPos) {
    // Abfangung gegen falsche Nummer
    if (iPaPos < 0 || iPaPos >= size()) {

      //Sprung auf Adresse 0, wenn die angegebene Adresse nicht existiert
      Log.debug( "FlowControlTable: adress " + iPaPos + " dont exist, reset to adress 0." );
      iPaPos = 0;

      if (size() < 1)
      {
         throw new RuntimeException( "FlowControlTable is empty" ) ;
      }
    }// end if

    return (FlowControlInstruction) super.get(iPaPos);
  }// end method get

  /**
   * Zurckgeben einer passenden Adresse fr eine als Paramter bergeben XML-Line . <br/>
   * Weil eine XML-Line in mehrere Befehle aufgeteilt sein kann, ist die Adresse nicht eindeutig.<br/>
   * Es wird die erste passende Adresse zurckgegeben.<br/>
   * Wenn keine passende Adresse gefunden wird, wird -1 zurckgegeben.
   *
   * @param iPaXmlLine XML-Zeilen-Nummer
   * @return Steuertabellen-Adresse zur XML-Zeile oder -1 bei Fehler
   */
  public int getAdrForSrcLine(int iPaXmlLine) {
    for (int i = 0; i < size(); i++) {
      FlowControlInstruction elem = (FlowControlInstruction) this.get(i);

      if (elem.getLineNumber() == iPaXmlLine) {
        return i; // gefunden
      }// end if
    }// end for

    return -1; // Default-Wert wenn nichts gefunden
  }// end method getAdrForXmlLine

  /**
   * Zurckgeben des Prozedur-Namens fr eine als Paramter bergeben XML-Line . <br/>
   * Wenn keine passende Prozedur gefunden wird, wird null zurckgegeben.
   *
   * @see   #getAdrForXmlLine
   * @param iPaXmlLine Nummer der Zeile in <b>flow.cnc</b> (Basis 0)
   * @return Name der entsprechenden <b>procedure</b> (CnC-Language-procedure)
   */
  public String getProcNameForXmlLine(int iPaXmlLine) {
    String retStr = null;
    int iAdr = getAdrForSrcLine(iPaXmlLine);

    if (iAdr >= 0) {
      FlowControlInstruction elem = (FlowControlInstruction) this.get(iAdr);

      retStr = elem.procName;
    }// end if
    return retStr;
  }// end method getProcNameForXmlLine

  /**
   * Debug-Ausgabe iteriert ber alle Elemente der FlowControlTable
   *
   * @return umgebrochener String mit allen {@link FlowControlInstruction}
   */
  public String toString() {
    StringBuffer sb = new StringBuffer(1024 * 4);

    sb.append("FlowControlTable\n");
    sb.append(FlowControlInstruction.toStringHeader() + "\n");
    for (int i = 0; i < this.size(); i++) {
      // sb.append( StringUtil.leftPad( "" + i , 4 ) + table.get( i ) + "\n" ) ;
      sb.append(((FlowControlInstruction) this.get(i)).toString(i) + "\n");
    }// end for
    return sb.toString();
  }// end method toString

}// end class FlowControlTable
