package de.cnc.flowprogram;

import de.cnc.domainparser.util.SourcePosition;
import de.cnc.expression.*;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.util.Log;




/**
 * Struktur-Klasse fr Value-Parameter und Reference-Parameter . <br/>
 * Dieser Parameter wird von der Call-Seite aus gesehen und enthlt
 * auch die Laufzeit-Informationen fr den FlowStack(-Element).
 * Dazu wird er jeweils beim CALL geclont.
 * 
 * @see FlowControlCompiler
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id:$
 */
public final class FlowProcCallParam {

  /**
   * Schalter Reference-Parameter (alternativ Value-Parameter)
   */
  public boolean isRefParam = false;

  /**
   * Session-Key ausserhalb der procedure
   */
  public String outsideKey;

  /**
   * Backup-Value fr Value-Parameter (werden bei return zurckkopiert)
   */
  public Object backupValue;

  /**
   * Session-Key innerhalb der procedure fr Reference-Parameter (werden bei return von diesem Session-Key zurckkopiert)
   */
  public String insideKey;

  /**
   * ubergebene Expression
   */
  public Expression paramExpression;


  public SourcePosition sourcePosition;

  /**
   * Expresssion bei ControlCode.EVAL_EXPRESSSION
   * ausfhren.
   */
  public Object evalParamExpression( //--
      JspRuntimeEnvironment pJspRuntimeEnvironment)
  {
      try
      {
          return paramExpression.eval( pJspRuntimeEnvironment );
      }
      catch (ExpressionEvaluationException e)
      {
          Log.fatal( e , this );
          throw new RuntimeException(e.getMessage() + sourcePosition );
      }
  }

  /**
   * clone-Methode vom Parameter der Prozedur (in der Befehlstabelle) zum Parameter zur Laufzeit (im FlowStack)
   */
  public FlowProcCallParam getClone() {
    FlowProcCallParam newFlowProcedureParameter = new FlowProcCallParam();

    newFlowProcedureParameter.isRefParam = this.isRefParam;
    newFlowProcedureParameter.outsideKey = this.outsideKey;
    newFlowProcedureParameter.backupValue = this.backupValue;
    newFlowProcedureParameter.insideKey = this.insideKey;
    newFlowProcedureParameter.paramExpression = this.paramExpression;
    newFlowProcedureParameter.sourcePosition = this.sourcePosition;
    return newFlowProcedureParameter;
  }// end method getClone

  /**
   * Debug-Ausgabe
   */
  public String toString() {
    return
        "isRefParam: " + isRefParam
      + " paramExpression: " + paramExpression
      + " outsideKey: " + outsideKey
      + " insideKey: " + insideKey
      + " backupValue: " + Log.objToStrListCompact( backupValue )
      + " sourcePosition: " + sourcePosition;
  }

//  /**
//   * @return
//   */
//  public String getInsideKeyWithoutPrefix()
//  {
//      return FlowControlUtil.checkAndCutPrefixSession( 
//              insideKey , 
//              sourcePosition );
//  }// end method toString

}// end class FlowProcCallParam
