package de.cnc.persistens;

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.cnc.CncContext;
import de.cnc.util.ClassBuilder;
import de.cnc.util.Log;
import de.cnc.util.StringUtil;

/**
 * Klasse zum Ausfhren der SQL-Statements auf der Datenbank . <br/>
 * Untersttzung Transaktions-Steuerung (aber nicht unter DB2 getestet) . <br/>
 * Aus anderem Projekt bernommen und angepat. <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: DatabaseStore.java,v 1.2 2004/10/05 22:50:19 aex116e Exp $
 */
public class DatabaseStore
{
   /**
    * Merker fr die zuletzt beim SQL-Statement aufgetretene SQL-Exception zur weiteren Auswertung im rufenden Programm
    */
   private SQLException mySqlException = null;

   /**
    * Connection zur Klammerung mehrerer Aufrufe innerhalb einer Transaktion
    */
   private java.sql.Connection connection = null;

   /**
    * Fehler-Status (-1)
    */
   public final static int FAILED = -1;

   /**
    * Constructor declaration
    */
   public DatabaseStore()
   {
   }

   /**
    * Einen Datensatz von der Datenbank holen
    *
    * @param pSqlStr SQL-Statement
    * @param pClass {@link IStorable}
    * @return Bean
    */
   synchronized public IStorable retrieve(String pSqlStr, Class pClass)
   {
      // Rcksetzen Fehlermerker
      this.mySqlException = null;

      java.sql.Connection con = null;

      try
      {
         IStorable dbo = (IStorable) ClassBuilder.build(pClass);

         String dbName = PersistensHelper.getDbName(dbo);
         // Vorgehensweise Tomcat: con = DatabaseConnectionFactory.getConnection( dbName );
         con = DbConnectionFactory.makeDBConnection(dbName);

         Log.debug("retrieve(String pSqlStr, Class pClass)" , pSqlStr);
         //PreparedStatement myPreparedStatement = con.prepareStatement(pSqlStr);
         Statement stmt = con.createStatement();
         //ResultSet resultSet = myPreparedStatement.executeQuery();
         ResultSet resultSet = stmt.executeQuery(pSqlStr);

         if (resultSet != null && resultSet.next())
         {
            PersistensHelper.readResultSet(dbo, resultSet);

            // Ablegen im Cache
            if (PersistensHelper.cacheUse(dbo))
            {
               String primaryKeyFieldName = PersistensHelper.getPrimaryKeyFieldName(dbo);
               if (PersistensHelper.cacheUse(pClass) && primaryKeyFieldName != null)
               {
                  DatabaseCache.put(dbo);
               }
            }

            return dbo;
         }
      return null;
    }
    catch (Exception e)
    {
      setSQLException(e);

      return null;
    }
    finally
    {
      if (con != null)
      {
        close(con);
      }
    }
  }

   /**
    * Holen eines bestimmten Datensatzes anhand des Primery-Key
    *
    * @param objectId Primery-Key des Datensatzes (BIGINT)
    * @param pClass {@link IStorable}
    * @return Bean, die der gewnschten Klasse entspricht
    */
   synchronized public IStorable retrieve(long pObjectId, Class pClass)
   {
      IStorable retStoreable = null;

      //Lesen vom Cache
      boolean cacheUse = PersistensHelper.cacheUse(pClass);
      if (cacheUse)
      {
         retStoreable = DatabaseCache.getForPrimeryKey(pClass, new Long(pObjectId));
         if (retStoreable != null)
         {
            return retStoreable;
         }
      }

      //try
      //{
      IStorable emptyObject = (IStorable) ClassBuilder.build(pClass);

      emptyObject.setObjectId(pObjectId);

      String whereString = PersistensHelper.getSelectIdWhereKlausel(emptyObject);
      List selectList = getListWhere(pClass, whereString);

      if (selectList != null && selectList.size() > 0)
      {
         // Problem: was ist wenn mehrere Treffer
         retStoreable = (IStorable) selectList.get(0);
      }
      //}
      //catch (Exception e)
      //{
      //  merkeSQLException(e);
      //}

      // im Cache vermerken @todo: wird das Objekt nicht schon in getListWhere() oder weiter aufgerufenen Methoden gecacht?
      if (cacheUse && retStoreable != null)
      {
         DatabaseCache.put(retStoreable);
      }
      return retStoreable;
   }

   /**
    * Holen eines bestimmten Datensatzes anhand des Primery-Key
    *
    * @param objectId Primery-Key des Datensatzes (BIGINT)
    * @param pClass {@link IStorable}
    * @return Bean, die der gewnschten Klasse entspricht
    */
   synchronized public IStorable[] retrieveForForeignKey( //--
      final long pObjectId, //--
      final String pPropertyName, //--
      final String pFieldName, //--
      final Class pClass)
   {
      IStorable[] retStoreableArr = null;

      //Lesen vom Cache
      boolean cacheUse = PersistensHelper.cacheUse(pClass);
      if (cacheUse)
      {
         retStoreableArr = DatabaseCache.getForForeignKey(pClass, pPropertyName , new Long(pObjectId));
         if (retStoreableArr != null)
         {
            return retStoreableArr;
         }
      }

      //try
      //{
      IStorable emptyObject = (IStorable) ClassBuilder.build(pClass);

      emptyObject.setObjectId(pObjectId);

      String whereString = pFieldName + "=" + pObjectId;
      List selectList = getListWhere(pClass, whereString);

      if (selectList != null && selectList.size() > 0)
      {
         // Problem: was ist wenn mehrere Treffer
         retStoreableArr = (IStorable[]) selectList.toArray( new IStorable[ selectList.size() ] );
      }
      //}
      //catch (Exception e)
      //{
      //  merkeSQLException(e);
      //}

      // im Cache vermerken @todo: wird das Objekt nicht schon in getListWhere() oder weiter aufgerufenen Methoden gecacht?
      if (cacheUse && retStoreableArr != null)
      {
         for (int i = 0; i < retStoreableArr.length; i++)
         {
            DatabaseCache.put(retStoreableArr[i]);
         }
      }
      return retStoreableArr;
   }

   /**
    * Holen eines Datensatzes von der Datenbank entsprechend der Bean-Klasse und WHERE-Bedingung . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param whereString String mit WHERE-Bedingung, aber ohne WHERE
    * @param pClass Bean-Klasse des zu holenden Datensatzes {@link IStorable}
    * @return Bean, die der gewnschten Klasse entspricht
    */
   synchronized public IStorable retrieveWhere(String pWhereStr, Class pClass)
   {
      IStorable retStoreable = null;

      List selectList = getListWhere(pClass, pWhereStr);

      if (selectList != null && selectList.size() > 0)
      {
         // Problem: was ist wenn mehrere Treffer
         retStoreable = (IStorable) selectList.get(0);
      }

      return retStoreable;
   } //end method retrieveDatabaseObjectWhere

   /**
    * Holen eines Datensatzes von der Datenbank entsprechend der Bean-Klasse und WHERE-Bedingung . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param whereString String mit WHERE-Bedingung, aber ohne WHERE
    * @param pClass Bean-Klasse des zu holenden Datensatzes {@link IStorable}
    * @return Bean, die der gewnschten Klasse entspricht
    */
   synchronized public IStorable retrieveWhereCached(String pWhereStr, Class pClass)
   {
      //Lesen vom Cache
      IStorable retStoreable = null;

      List selectList = getListWhereCached(pClass, pWhereStr);

      if (selectList != null && selectList.size() > 0)
      {
         // Problem: was ist wenn mehrere Treffer
         retStoreable = (IStorable) selectList.get(0);
      }

      return retStoreable;
   } //end method

   /**
    * Holen einer Liste Datenstze von der Datenbank entsprechend der Bean-Klasse und dem Select-Statement . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern
    * ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param pSqlStr String mit SQL-Statement
    * @param pClass Bean-Klasse des zu holenden Datensatzes {@link IStorable}
    * @return ArrayList mit Beans {@link IStorable}
    */
   public synchronized List retrieveList(String pSqlStr, Class pClass)
   {
      boolean cacheUse = PersistensHelper.cacheUse(pClass);
      
      // Rcksetzen Fehlermerker
      this.mySqlException = null;

      java.sql.Connection con = null;
      List list = new ArrayList();

      try
      {
         IStorable dbo = (IStorable) ClassBuilder.build(pClass);
         String dbName = PersistensHelper.getDbName(dbo);
         // Vorgehensweise Tomcat: con = DatabaseConnectionFactory.getConnection( dbName );
         con = DbConnectionFactory.makeDBConnection(dbName);

         Log.debug("retrieveList(String pSqlStr, Class pClass)" , pSqlStr);
         //PreparedStatement myPreparedStatement = con.prepareStatement(pSqlStr);
         Statement stmt = con.createStatement();
         //ResultSet resultSet = myPreparedStatement.executeQuery();
         ResultSet resultSet = stmt.executeQuery(pSqlStr);

         if (resultSet != null)
         {
            while (resultSet.next())
            {
               //Log.debug( "retrieveDatabaseObjectList" );

               dbo = (IStorable) ClassBuilder.build(pClass);

               //if (dbo != null)
               //{
               PersistensHelper.readResultSet(dbo, resultSet);

               list.add(dbo);

               // Ablegen im Cache
               if (cacheUse)
               {
                  //String primaryKeyFieldName = PersistensHelper.getPrimaryKeyFieldName(dbo);
                  //if (primaryKeyFieldName != null)
                  //{
                     DatabaseCache.put(dbo);
                  //}
               }

               //}
               //else
               //{
               //  return null;
               //}
            }
         }
         return null;
       }
       catch (Exception e)
       {
         setSQLException(e);

         return null;
       }
       finally
       {
         if (con != null)
         {
           close(con);
         }
       }
     }//end method retrieveDatabaseObjectList

   /**
    * Holen einer Liste Datenstze von der Datenbank entsprechend der Bean-Klasse und dem Select-Statement . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern
    * ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param pSqlStr String mit SQL-Statement
    * @param pClass Bean-Klasse des zu holenden Datensatzes {@link IStorable}
    * @return ArrayList mit Beans {@link IStorable}
    */
   public synchronized List retrieveListCached(String pSqlStr, Class pClass)
   {
      //Cache Abfragen
      List cachedList = DatabaseCache.getForWhereStr(/*pClass,*/ pSqlStr);
      if (cachedList != null && cachedList.size() > 0)
      {
         return cachedList;
      }
      cachedList = retrieveList(pSqlStr, pClass);
      if (cachedList != null && cachedList.size() > 0)
      {
         DatabaseCache.putForWhereStr(cachedList, pSqlStr);
      }
      return cachedList;
   }

   /**
    * Abspeichern einer Bean in der Datenbank . <br/>
    * Anhand der Property <b>objecktId</b> wird der entsprechende Datensatz (ID-Feld) festgestellt oder
    * bei <b>objecktId</b>=={@link de.cnc.server.IStorable#NOT_STORED NOT_STORED} ein INSERT ausgefhrt . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern
    * ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param pStorable abzuspeichernde Bean {@link IStorable}
    * @return ob erfolgreich
    */
   synchronized public boolean store(IStorable pStorable, CncContext pCncContext)
   {
      boolean cacheUse = PersistensHelper.cacheUse(pStorable);

      // Ablegen im Cache
      if (cacheUse)
      {
         DatabaseCache.put( pStorable );
      }

      boolean isInsert = false;

      java.sql.Connection con = null;

      // Rcksetzen Fehlermerker
      this.mySqlException = null;

      try
      {
         if (this.connection == null)
         {
            String dbName = PersistensHelper.getDbName(pStorable);
            // Vorgehensweise Tomcat: con = DatabaseConnectionFactory.getConnection( dbName );
            con = DbConnectionFactory.makeDBConnection(dbName);
         }
         else
         {
            // globale Transaction
            con = this.connection;
         }

         if (this.connection == null)
         {
            //nicht commiten wegen websphere transaction: 
            //con.setTransactionIsolation(java.sql.Connection.TRANSACTION_SERIALIZABLE);
            //con.setAutoCommit(false);
         }

         String sql;

         // if ! must Insert
         if (pStorable.getObjectId() != IStorable.NOT_STORED)
         {
            // lock the rows before the update
            sql = PersistensHelper.getSelectForUpdateSQLStatement(pStorable);
            PreparedStatement myPreparedStatement = con.prepareStatement(sql);
            myPreparedStatement.setLong(1, pStorable.getObjectId()); // Setzen ?-Parameter
            myPreparedStatement.execute();
            try
            {
               myPreparedStatement.close();
            }
            catch (Exception e)
            {
            }

            // Timestamp-Locking nicht realisiert
            // if (!checkTimestamp (aDBObject, con))
            // {
            // throw new Exception( "Timestamp in Database is newer than in Bean" ) ;
            // return false;
            // }
            sql = PersistensHelper.getUpdateSQLStatement(pStorable, pCncContext);
         }
         else
         {
            sql = PersistensHelper.getInsertSQLStatement(pStorable, pCncContext);
            isInsert = true;
         }

         Log.debug("SQL Statement" , sql);
         //PreparedStatement myPreparedStatement = con.prepareStatement(sql);
         Statement stmt = con.createStatement();
         //boolean stored = myPreparedStatement.executeUpdate() == 1;
         boolean stored = stmt.executeUpdate(sql) == 1;

         // reset objectid if insert failed
         if (isInsert &&!stored)
         {
             pStorable.setObjectId(IStorable.NOT_STORED);
         }

         if (this.connection != null)
         {
           // globale Transaction
           return stored;
         }

         if (stored)
         {
           con.commit();

           return true;
         }
         else
         {
           con.rollback();

           return false;
         }
       }
       catch (Exception e)
       {
         setSQLException(e);

         // reset objectid if insert failed
         if (isInsert)
         {
           pStorable.setObjectId(IStorable.NOT_STORED);
         }

         try
         {
           if (this.connection == null)
           {
             con.rollback();
           }
         }
         catch (Exception ex)
         {
           ;
         }

         Log.fatal(e);

         return false;
       }
       finally
       {
         if (this.connection == null)
         {
           if (con != null)
           {
             close(con);
           }
         }
       }
     }//end method store


   /**
    * Speichern einer Liste Beans im Rahmen einer Transaktion . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern
    * ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param list
    * @return ob erfolgreich
    */
   synchronized public boolean storeList(List list, CncContext cncContext)
   {
      if (list.isEmpty())
      {
         return true; // false;
      }

      boolean cacheUse = PersistensHelper.cacheUse((IStorable) list.get(0));
      IStorable dbo = null;
      boolean isInsert = false;

      java.sql.Connection con = null;
      boolean stored = false;

      // Rcksetzen Fehlermerker
      mySqlException = null;

      try
      {
         String dbName = null;
         if (connection == null)
         {
            dbName = PersistensHelper.getDbName((IStorable) list.get(0));
            // Vorgehensweise Tomcat: con = DatabaseConnectionFactory.getConnection( dbName );
            con = DbConnectionFactory.makeDBConnection(dbName);
         }
         else
         {
            // globale Transaction
            con = connection;
         }

         if (connection == null)
         {
            //nicht committen wegen websphere transaction: 
            //Log.debug("zusammengefasste Transaktion hier auskommentiert wegen Speicherplatzproblem auf Rechner im Nebenraum");
            //        con.setTransactionIsolation(java.sql.Connection.TRANSACTION_SERIALIZABLE);
            //        con.setAutoCommit(false);
         }

         //Zhler fr sie auf der Connection ausgefhrten Statements
         int partUpdateCounter = 0;

         for (Iterator i = list.iterator(); i.hasNext();)
         {
            dbo = (IStorable) i.next();

            // if (dbo.isChanged () || dbo.mustInsert ())
            if (dbo.getChanged() || dbo.getObjectId() == IStorable.NOT_STORED)
            {

               // Log.debugOut( getClass().getName() + " dbo.isChanged (): " + dbo.isChanged () ) ;
               // Ausfhren bei gendertem Datensatz ( isChanged )
               // bei neuen Datenstzen wird isChanged ignoriert
               String sqlStr;

               // Abfrage auf Insert oder Update
               if (dbo.getObjectId() != IStorable.NOT_STORED)
               {
                  // Log.debugOut ("!dbo.mustInsert");
                  // lock the rows before the update
                  sqlStr = PersistensHelper.getSelectForUpdateSQLStatement(dbo);
                  PreparedStatement myPreparedStatement = con.prepareStatement(sqlStr);
                  myPreparedStatement.setLong(1, dbo.getObjectId()); // Setzen ?-Parameter
                  myPreparedStatement.execute();
                  try
                  {
                     myPreparedStatement.close();
                  }
                  catch (Exception e)
                  {
                  }

                  sqlStr = PersistensHelper.getUpdateSQLStatement(dbo, cncContext);
               }
               else
               {
                  // Log.debugOut ("dbo.mustInsert");
                  sqlStr = PersistensHelper.getInsertSQLStatement(dbo, cncContext);
                  isInsert = true;
               }

               Log.debug("SQL Statement(LIST)" , sqlStr);
               //PreparedStatement myPreparedStatement = con.prepareStatement(sqlStr);
               Statement stmt = con.createStatement();
               //myPreparedStatement.executeUpdate(sqlStr);
               stmt.executeUpdate( sqlStr );

               try
               {
                  //myPreparedStatement.close();
                  stmt.close();
               }
               catch (Exception e)
               {
               }

            } // end if isChanged || mustInsert

            dbo.setChanged(false);

            // Ablegen im Cache
            if (cacheUse)
            {
               DatabaseCache.put(dbo);
            }

            partUpdateCounter++;

//            // neue Connection besorgen wegen  COM.ibm.db2.jdbc.DB2Exception: [IBM][CLI Driver][DB2/NT] SQL0954C  Not enough storage is available in the application heap to process the statement.  SQLSTATE=57011
//            // @todo: Anzahl Statements je Connection noch konfigurierbar machen, eventuell in Connectionpooling mit einbauen
//            if (connection == null // keine globale Transaktion
//               && partUpdateCounter >= 16) // maximale Anzahl Statements auf dieser Verbindung
//            {
//               try
//               {
//                  // nicht committen wegen websphere transaction: 
//                  //con.commit();
//               }
//               catch (Exception e)
//               {
//               }
//               try
//               {
//                  con.close();
//               }
//               catch (Exception e)
//               {
//               }
//               // Vorgehensweise Tomcat: con = DatabaseConnectionFactory.getConnection( dbName );
//               con = DbConnectionFactory.makeDBConnection(dbName);
//               partUpdateCounter = 0;
//            }

         }
         stored = true;

         if (connection != null)
         {
           // globale Transaction
           return stored;
         }

         if (stored)
         {
           con.commit();

           return true;
         }
         else
         {
           con.rollback();

           return false;
         }
       }
       catch (Exception e)
       {
         setSQLException(e);

         // reset objectid if insert failed
         if (isInsert && dbo != null)
         {
           dbo.setChanged(false);
         }

         try
         {
           con.rollback();
         }
         catch (Exception ex)
         {
             Log.fatal(ex);
         }

         Log.fatal(e);

         return false;
       }
       finally
       {
         if (this.connection == null)
         {
           if (con != null)
           {
             close(con);
           }
         }
       }
   } //end method storeDatabaseObjectList

   /**
    * Lschen eines Datensatzes anhand <b>objectId</b> der Bean . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern
    * ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param pStorable
    * @return ob erfolgreich
    */
   synchronized public boolean delete(IStorable pStorable)
   {
      java.sql.Connection con = null;

      // Rcksetzen Fehlermerker
      this.mySqlException = null;

      try
      {
         if (this.connection == null)
         {
            String dbName = PersistensHelper.getDbName(pStorable);
            // Vorgehensweise Tomcat: con = DatabaseConnectionFactory.getConnection( dbName );
            con = DbConnectionFactory.makeDBConnection(dbName);
         }
         else
         {
            // globale Transaction
            con = this.connection;
         }

         if (this.connection == null)
         {
            con.setTransactionIsolation(java.sql.Connection.TRANSACTION_SERIALIZABLE);
            con.setAutoCommit(false);
         }

         String sql = PersistensHelper.getDeleteSQLStatement(pStorable);

         Log.debug("SQL Delete Statement" , sql );
         //PreparedStatement myPreparedStatement = con.prepareStatement(sql);
         Statement stmt = con.createStatement();
         //boolean deleted = myPreparedStatement.executeUpdate(sql) == 1;
         boolean deleted = stmt.executeUpdate(sql) == 1;
         
         if (this.connection != null)
         {
           // globale Transaction
           return deleted;
         }

         if (deleted)
         {
           con.commit();

           return true;
         }
         else
         {
           con.rollback();

           return false;
         }
       }
       catch (Exception e)
       {
         setSQLException(e);

         try
         {
           if (this.connection == null)
           {
             con.rollback();
           }
         }
         catch (Exception ex)
         {
           Log.fatal(ex);
         }

         return false;
       }
       finally
       {
         if (this.connection == null)
         {
           if (con != null)
           {
             close(con);
           }
         }
       }
   } //end method delete

   /**
    * Lschen aller Datenstze einer Bean . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern
    * ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param pClass Bean-Klasse {@link IStorable}
    * @return ob erfolgreich
    */
   synchronized public boolean deleteAll(Class pClass)
   {
      java.sql.Connection con = null;

      // Rcksetzen Fehlermerker
      this.mySqlException = null;

      try
      {
         IStorable dbo = (IStorable) ClassBuilder.build(pClass);

         if (this.connection == null)
         {
            String dbName = PersistensHelper.getDbName(dbo);
            // Vorgehensweise Tomcat: con = DatabaseConnectionFactory.getConnection( dbName );
            con = DbConnectionFactory.makeDBConnection(dbName);
         }
         else
         {
            // globale Transaction
            con = this.connection;
         }

         if (this.connection == null)
         {
//          nicht commiten wegen websphere transaction: 
            //con.setTransactionIsolation(java.sql.Connection.TRANSACTION_SERIALIZABLE);
            //con.setAutoCommit(false);
         }

         String sql = PersistensHelper.getDeleteAllSQLStatement(dbo);

         // Log.addLog ("SQL Delete All Statement: " + sql, Log.DEBUG);
         //PreparedStatement myPreparedStatement = con.prepareStatement(sql);
         Statement stmt = con.createStatement();
         //myPreparedStatement.executeUpdate(sql);
         stmt.executeUpdate( sql );

         if (connection != null)
         {
           return true;
         }

         con.commit();

         return true;
       }
       catch (Exception e)
       {
         setSQLException(e);

         try
         {
           if (this.connection == null)
           {
             con.rollback();
           }
         }
         catch (Exception ex)
         {
           Log.fatal(ex);
         }

         return false;
       }
       finally
       {
         if (this.connection == null)
         {
           if (con != null)
           {
             close(con);
           }
         }
       }
     } // end method deleteAll

   /**
    * Ausfhren SQL-Statement in Transaktionsrahmen
    *
    * @param sql
    * @return JDBC-Return-Code (xx Records Selected aus SQL)
    */
   //  synchronized public int executeUpdate(String sql)
   //  {
   //    // Rcksetzen Fehlermerker
   //    mySQLException = null;
   //
   //    java.sql.Connection con = null;
   //
   //    try
   //    {
   //      if (connection == null)
   //      {
   //        con = DatabaseConnectionFactory.getConnection();
   //      }
   //      else
   //      {
   //        // globale Transaction
   //        con = connection;
   //      }
   //
   //      // Log.addLog ("SQL Statement: " + sql, Log.DEBUG);
   //      PreparedStatement myPreparedStatement = con.prepareStatement(sql);
   //
   //      return myPreparedStatement.executeUpdate();
   //    }
   //    catch (Exception e)
   //    {
   //      merkeSQLException(e);
   //
   //      //return FAILED;
   //      throw new RuntimeException( e.getMessage() );
   //    }
   //    finally
   //    {
   //      if (connection == null)
   //      {
   //          close(con);
   //      }
   //    }
   //  }//end method executeUpdate

   /**
    * Start globale Transaction
    *
    * @throws Exception wenn keine Databaseconnection vorbereitet
    */
   //  synchronized public void beginTransaction()
   //    throws Exception
   //  {
   //    if (connection == null)
   //    {
   //      connection = DatabaseConnectionFactory.getConnection();
   //
   //      connection.setTransactionIsolation(java.sql.Connection.TRANSACTION_SERIALIZABLE);
   //      connection.setAutoCommit(false);
   //    }
   //    else
   //    {
   //      throw new DatabaseStoreException(DatabaseStoreException.InTransaction);
   //    }
   //  }

   /**
    * Rollback globale Transaction
    */
//   synchronized public void rollback()
//   {
//      // Rcksetzen Fehlermerker
//      mySqlException = null;
//
//      try
//      {
//         if (connection != null)
//         {
//            // globale Transaction
//            connection.rollback();
//            connection.close();
//
//            connection = null;
//         }
//      }
//      catch (Exception e)
//      {
//         merkeSQLException(e);
//      }
//   }

   /**
    * Commit globale Transaction
    */
//   synchronized public void commit()
//   {
//      // Rcksetzen Fehlermerker
//      mySqlException = null;
//
//      try
//      {
//         if (connection != null)
//         {
//            // globale Transaction
//            connection.commit();
//            connection.close();
//
//            connection = null;
//         }
//      }
//      catch (Exception e)
//      {
//         merkeSQLException(e);
//      }
//   }

   /**
    * Rckgabe der letzten vermerkten SQL-Exception zur Auswertung im bergeordneten Programm
    *
    * @return die Exception
    */
   public SQLException getSQLException()
   {
      return this.mySqlException;
   }

   /**
    * Methode zum Abfangen der SqlException beim Schliessen der java.sql.Connection
    *
    * @param con zu schliessende Connection
    */
   private void close(java.sql.Connection con)
   {
      if (con != null)
      {
         //      try
         //      {
         //        con.close();
         //      }
         //      catch (Exception e)
         //      {
         //        //merkeSQLException(e);
         //      }
         DbConnectionFactory.close(con);
      }
   }

   // Vermerken SQLException zum weiteren Auswerten in aufrufender Routine

   /**
    * Vermerken aufgetretener SQL-Exceptions zur Auswertung im bergeordnetem Programm und
    * Loggen der Exception fr Applikationsberwachung (Verschicken Exception-Meldung per e-Mail)
    *
    * @param e
    */
   private void setSQLException(Exception e)
   {
      Log.fatal(e);
      if (e instanceof SQLException)
      {
         // wenn die geworfenene Exception eine SQLException war
         this.mySqlException = (SQLException) e;
      }
      else
      {
         // erzeugen einer gefakten SQLException, damit auswertende Routinen nicht noch extra auf instanceof oder null-Pointer prfen mssen
         this.mySqlException = new SQLException(e.getMessage(), e.getMessage());
      }
   }

   /**
    * Holen aller Datenstze fr eine Bean-Klasse . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern
    * ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param pClass Bean-Klasse {@link IStorable}
    * @return die Eergebnisliste
    */
   public final List getList(Class pClass)
   {
      List list = null;

      //try
      //{
      DatabaseStore db = new DatabaseStore();
      IStorable databaseBean = (IStorable) ClassBuilder.build(pClass);
      StringBuffer sb = new StringBuffer();

      sb.append(
         "SELECT "
            + PersistensHelper.getSelectFieldsStr(databaseBean)
            + " FROM "
            + PersistensHelper.getTableName(databaseBean));

      // Log.debugOut (sb.toString ());
      list = db.retrieveList(sb.toString(), pClass);

      //hier cache einbauen
      //}
      //catch (Exception e)
      //{
      //merkeSQLException(e);
      //}

      return list;
   } // end method getList

   /**
    * Holen alle Datenstze mit ORDER-BY-Option . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern
    * ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param aClass {@link IStorable}
    * @param orderString
    * @return Treffer-Liste oder null, wenn Fehler oder keine Treffer
    */
   //  public final List getListOrder(Class aClass, String orderString)
   //  {
   //    List  aList = null;
   //
   //    //try
   //    //{
   //      DatabaseStore       db = new DatabaseStore();
   //      IStorable  databaseBean = (IStorable) ClassBuilder.build(aClass);
   //      StringBuffer        sb = new StringBuffer();
   //
   //      sb.append("SELECT " + PersistensHelper.getSelectFieldsStr(databaseBean) + " FROM " + PersistensHelper.getTableName(databaseBean));
   //      sb.append(" ORDER BY " + orderString);
   //
   //      // Log.debugOut (sb.toString ());
   //      aList = db.retrieveDatabaseObjectList(sb.toString(), aClass);
   //    //}
   //    //catch (Exception e)
   //    //{
   //    //  merkeSQLException(e);
   //    //}
   //
   //    return aList;
   //  } // end method getList

   /**
    * Holen Datenstze laut WHERE-Bedingung . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern
    * ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param pClass {@link IStorable}
    * @param whereString
    * @return Treffer-Liste oder null, wenn Fehler oder keine Treffer
    */
   public final List getListWhere(Class pClass, String pWhereStr)
   {
      List list = null;

      //try
      //{
      DatabaseStore db = new DatabaseStore();
      IStorable databaseBean = (IStorable) ClassBuilder.build(pClass);
      StringBuffer sb = new StringBuffer();

      sb.append(
         "SELECT "
            + PersistensHelper.getSelectFieldsStr(databaseBean)
            + " FROM "
            + PersistensHelper.getTableName(databaseBean));
      if (!StringUtil.empty(pWhereStr))
      {
         sb.append(" WHERE " + pWhereStr);
      }

      Log.debug("" , sb );
      list = db.retrieveList(sb.toString(), pClass);

      //hier knnte man den Cache einbauen, wenn der Cache je WHERE-String eine Liste speichern wrde
      //wegen Riesen-Speicherbedarf erst mal nicht gemacht

      //}
      //catch (Exception e)
      //{
      //  merkeSQLException(e);
      //}

      return list;
   } // end method getList

   /**
    * Holen Datenstze laut WHERE-Bedingung . <br/>
    * Das Mapping der Bean-Klasse zur Datenbanktabelle und von den Bean-Properties zu den Datenbankfeldern
    * ist in der Datei <default-app-props>.properties hinterlegt . <br/>
    *
    * @param pClass {@link IStorable}
    * @param whereString
    * @return Treffer-Liste oder null, wenn Fehler oder keine Treffer
    */
   public final List getListWhereCached(Class pClass, String pWhereStr)
   {
      List list = null;

      //try
      //{
      DatabaseStore db = new DatabaseStore();
      IStorable databaseBean = (IStorable) ClassBuilder.build(pClass);
      StringBuffer sb = new StringBuffer( 256 );

      sb.append(
         "SELECT "
            + PersistensHelper.getSelectFieldsStr(databaseBean)
            + " FROM "
            + PersistensHelper.getTableName(databaseBean));
      if (!StringUtil.empty(pWhereStr))
      {
         sb.append(" WHERE " + pWhereStr);
      }

      Log.debug("" , sb );
      list = db.retrieveListCached(sb.toString(), pClass);

      //hier knnte man den Cache einbauen, wenn der Cache je WHERE-String eine Liste speichern wrde
      //wegen Riesen-Speicherbedarf erst mal nicht gemacht

      //}
      //catch (Exception e)
      //{
      //  merkeSQLException(e);
      //}

      return list;
   } // end method getList

} //end class DatabaseStore
