package de.cnc.persistens;


/**
 * Interface fr alle Beans, die einen Datensatz reprsentieren. <br/>
 * Die Methoden diese Interfaces liefern alle notwendigen Informationen fr <br/>
 * {@link de.cnc.server.DatabaseStore} und <br/>
 * {@link de.cnc.server.PersistensHelper}.<br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: IStorable.java,v 1.2 2004/10/05 22:50:19 aex116e Exp $
 */
public interface IStorable
{

  /**
   * es wird impliziert, da bei Vorliegen einer ObjectId == -1 ein INSERT statt eines UPDATE erfolgen mu
   */
  final static int  NOT_STORED = -1;

  /**
   * get database object id, Wert Primery-Key
   *
   * @return ID-Number
   */
  public long getObjectId();

  /**
   * set database object id, Wert Primery-Key
   */
  public void setObjectId(long pId);

  /**
   * Feststellen, ob der Datensatz gendert wurde und deshalb ein SQL-UPDATE notwendig ist. <br/>
   * Wenn die Bean keine Change/Update-Mimik untersttzt, einfach immer true zurckgeben. <br/>
   */
  public boolean getChanged();

  /**
   * Setzen, ob der Datensatz gendert wurde und deshalb ein SQL-UPDATE notwendig ist. <br/>
   * Wenn die Bean keine Change/Update-Mimik untersttzt, einfach leer implementieren. <br/>
   */
  public void setChanged(boolean pChg);

  /**
   * Maximale Treffer-Anzahl bei SELECT.
   * Wird realisiert ber "FETCH FIRST &gt;num&lt; ONLY".
   * 0 oder kleiner bedeutet keine Angabe.
   *
   * @return maximale Trefferanzahl
   */
  //public int getMaxFetch();

} // end interface IStorable
