package de.cnc.persistens;

import de.cnc.util.StringUtil;

/**
 * Hilfsmethoden zum Aufbauen von SQL-Statements . <br/>
 * Verwendet in {@link de.cnc.server.PersistensHelper PersistensHelper}. <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: SqlUtil.java,v 1.1 2004/09/30 12:32:02 aex116e Exp $
 */
public class SqlUtil
{
  /**
   * Constructor declaration. <br/>
   * Instanziieren nicht erlaubt
   */
  private SqlUtil()
  {
    ;
  } // end constructor

  /**
   * Aufbau Where-Klausel mit AND-Verknpfung.             <br/>
   * Anhngen an angefangene Where-Klausel                 <br/>
   * Beachtung ob searchValue empty                        <br/>
   * Beachtung exact/like und caseSensitive/ignoreCase     <br/>
   *
   * @param preWhereKlausel
   * @param fieldName Name Datenbankfeld
   * @param searchValue
   * @param exact
   * @param caseSensitive
   * @return Rckgabe erweiterte Klausel
   */
  public static StringBuffer appendAndWhereKlausel(
      StringBuffer preWhereKlausel,
      final String fieldName,
      final String searchValue,
      final boolean exact,
      final boolean caseSensitive,
      final boolean preJoker,
      final boolean postJoker)
  {
     return appendWhereKlausel(
         "AND" ,
         preWhereKlausel,
         fieldName,
         searchValue,
         exact,
         caseSensitive,
         preJoker,
         postJoker);
  } // end method appendAndWhereKlausel

  /**
   * Aufbau Where-Klausel mit AND-Verknpfung.             <br/>
   * Anhngen an angefangene Where-Klausel                 <br/>
   * Beachtung ob searchValue null                         <br/>
   *
   * @param preWhereKlausel
   * @param fieldName Name Datenbankfeld
   * @param searchValue
   * @return Rckgabe erweiterte Klausel
   */
  public static StringBuffer appendAndWhereKlausel(
      final StringBuffer preWhereKlausel,
      final String fieldName,
      final Number searchValue)
  {
     return appendWhereKlausel(
         "AND",
         preWhereKlausel,
         fieldName,
         searchValue);
  } // end method appendAndWhereKlausel

  /**
   * Aufbau Where-Klausel mit AND-Verknpfung.             <br/>
   * Anhngen an angefangene Where-Klausel                 <br/>
   * Beachtung ob searchValue empty                        <br/>
   * Beachtung exact/like und caseSensitive/ignoreCase     <br/>
   *
   * @param preWhereKlausel
   * @param fieldName
   * @param strOperator
   * @param iValue
   * @return erweiterte WHERE-Klausel
   */
  public static StringBuffer appendAndWhereKlausel(
      final StringBuffer preWhereKlausel,
      final String fieldName,
      final String strOperator,
      final int iValue)
  {
    return appendWhereKlausel(
         "AND",
         preWhereKlausel,
         fieldName,
         strOperator,
         iValue);
  } // end method appendAndWhereKlausel

  // *************************************************************************************************

  /**
   * Aufbau Where-Klausel mit OR-Verknpfung.             <br/>
   * Anhngen an angefangene Where-Klausel                 <br/>
   * Beachtung ob searchValue empty                        <br/>
   * Beachtung exact/like und caseSensitive/ignoreCase     <br/>
   *
   * @param preWhereKlausel
   * @param fieldName Name Datenbankfeld
   * @param searchValue
   * @param exact
   * @param caseSensitive
   * @return Rckgabe erweiterte Klausel
   */
  public static StringBuffer appendOrWhereKlausel(
      StringBuffer preWhereKlausel,
      final String fieldName,
      final String searchValue,
      final boolean exact,
      final boolean caseSensitive,
      final boolean preJoker,
      final boolean postJoker)
  {
     return appendWhereKlausel(
         "OR" ,
         preWhereKlausel,
         fieldName,
         searchValue,
         exact,
         caseSensitive,
         preJoker,
         postJoker);
  } // end method appendAndWhereKlausel

  /**
   * Aufbau Where-Klausel mit OR-Verknpfung.             <br/>
   * Anhngen an angefangene Where-Klausel                 <br/>
   * Beachtung ob searchValue null                         <br/>
   *
   * @param preWhereKlausel
   * @param fieldName Name Datenbankfeld
   * @param searchValue
   * @return Rckgabe erweiterte Klausel
   */
  public static StringBuffer appendOrWhereKlausel(
      final StringBuffer preWhereKlausel,
      final String fieldName,
      final Number searchValue)
  {
     return appendWhereKlausel(
         "OR",
         preWhereKlausel,
         fieldName,
         searchValue);
  } // end method appendAndWhereKlausel

  /**
   * Aufbau Where-Klausel mit OR-Verknpfung.             <br/>
   * Anhngen an angefangene Where-Klausel                 <br/>
   * Beachtung ob searchValue empty                        <br/>
   * Beachtung exact/like und caseSensitive/ignoreCase     <br/>
   *
   * @param preWhereKlausel
   * @param fieldName
   * @param strOperator
   * @param iValue
   * @return erweiterte WHERE-Klausel
   */
  public static StringBuffer appendOrWhereKlausel(
      final StringBuffer preWhereKlausel,
      final String fieldName,
      final String strOperator,
      final int iValue)
  {
    return appendWhereKlausel(
         "OR",
         preWhereKlausel,
         fieldName,
         strOperator,
         iValue);
  } // end method appendAndWhereKlausel

  // *************************************************************************************************

  /**
   * Aufbau Where-Klausel mit AND oder OR-Verknpfung.             <br/>
   * Anhngen an angefangene Where-Klausel                 <br/>
   * Beachtung ob searchValue empty                        <br/>
   * Beachtung exact/like und caseSensitive/ignoreCase     <br/>
   *
   * @param preWhereKlausel
   * @param fieldName Name Datenbankfeld
   * @param searchValue
   * @param exact
   * @param caseSensitive
   * @return Rckgabe erweiterte Klausel
   */
  public static StringBuffer appendWhereKlausel(
      String pStrOperator,
      StringBuffer preWhereKlausel,
      final String fieldName,
      final String searchValue,
      final boolean exact,
      final boolean caseSensitive,
      final boolean preJoker,
      final boolean postJoker)
  {
    // nur dann die WHERE-Klausel erweitern, wenn searchValue nicht empty
    if (!StringUtil.empty(searchValue))
    {
      // AND zum Verknpfen der Terms, aber nicht beim ersten
      preWhereKlausel = concatWithOperator(preWhereKlausel, pStrOperator);

      if (exact)
      {
        if (caseSensitive)
        {
          preWhereKlausel.append(fieldName + "='");
          preWhereKlausel.append(StringUtil.stripHochKomma(searchValue));
          preWhereKlausel.append("'");
        }
        else
        {
          // Umwandlung LOWER weil bei UPPER  in SS umgewandelt wird und Suche fehlschlgt
          preWhereKlausel.append("LOWER(" + fieldName + ")='");
          preWhereKlausel.append((StringUtil.stripHochKomma(searchValue)).toLowerCase());
          preWhereKlausel.append("'");
        }
      }
      else // nicht exact sondern like
      {
        if (caseSensitive)
        {
          preWhereKlausel.append(fieldName + " LIKE '");
          if (preJoker)
          {
             preWhereKlausel.append("%");
          }
          preWhereKlausel.append(StringUtil.stripHochKomma(searchValue));
        }
        else
        {
          // Umwandlung LOWER weil bei UPPER  in SS umgewandelt wird und Suche fehlschlgt
          preWhereKlausel.append("LOWER(" + fieldName + ") LIKE '");
          if (preJoker)
          {
             preWhereKlausel.append("%");
          }
          preWhereKlausel.append((StringUtil.stripHochKomma(searchValue)).toLowerCase());
        }
        if (postJoker)
        {
           preWhereKlausel.append("%");
        }
        preWhereKlausel.append("'");
      }
    }

    return preWhereKlausel;
  } // end method appendAndWhereKlausel

  /**
   * Aufbau Where-Klausel mit AND-Verknpfung.             <br/>
   * Anhngen an angefangene Where-Klausel                 <br/>
   * Beachtung ob searchValue null                         <br/>
   *
   * @param preWhereKlausel
   * @param fieldName Name Datenbankfeld
   * @param searchValue
   * @return Rckgabe erweiterte Klausel
   */
  public static StringBuffer appendWhereKlausel(
     String pStrOperator,
     StringBuffer preWhereKlausel,
     String fieldName,
     Number searchValue)
  {
    // nur dann die WHERE-Klausel erweitern, wenn searchValue nicht empty
    if ( searchValue != null )
    {
      // AND zum Verknpfen der Terms, aber nicht beim ersten
      preWhereKlausel = concatWithOperator(preWhereKlausel, pStrOperator);

      preWhereKlausel.append(fieldName + "=");
      preWhereKlausel.append(searchValue);
    }
    return preWhereKlausel;
  } // end method appendAndWhereKlausel

  /**
   * Aufbau Where-Klausel mit AND-Verknpfung.             <br/>
   * Anhngen an angefangene Where-Klausel                 <br/>
   * Beachtung ob searchValue empty                        <br/>
   * Beachtung exact/like und caseSensitive/ignoreCase     <br/>
   *
   * @param preWhereKlausel
   * @param fieldName
   * @param strOperator
   * @param iValue
   * @return erweiterte WHERE-Klausel
   */
  public static StringBuffer appendWhereKlausel(
      String pStrOperator,
      StringBuffer preWhereKlausel,
      String fieldName,
      String strOperator,
      int iValue)
  {
    // AND zum Verknpfen der Terms, aber nicht beim ersten
    preWhereKlausel = concatWithOperator(preWhereKlausel, pStrOperator);

    preWhereKlausel.append(fieldName);
    preWhereKlausel.append(strOperator);
    preWhereKlausel.append("" + iValue);

    return preWhereKlausel;
  } // end method appendAndWhereKlausel

  /**
   * Anfgen eines Operators (per Parameter bergeben) an eine Klausel,
   * wenn nicht leer mit Abfangung Null.
   *
   * @param preWhereKlausel
   * @param pStrOperator
   */
  public static StringBuffer concatWithOperator(StringBuffer preWhereKlausel, String pStrOperator)
  {
     if (preWhereKlausel == null)  // Abfangen Nullpointer
     {
       preWhereKlausel = new StringBuffer();
     }

     if (!StringUtil.empty(preWhereKlausel))
     {
        preWhereKlausel.append(" " + pStrOperator + " ");
     }
     return preWhereKlausel;
  }

  // *************************************************************************************************

  /**
   * Aufbau SET-Klausel fr UPDATE mit Komma-Verknpfung.  <br/>
   * Anhngen an angefangene SET-Klausel                   <br/>
   *
   * @param preUpdateSetKlausel
   * @param fieldName Name Datenbankfeld
   * @param newValue
   * @return Rckgabe erweiterte Klausel
   */
  public static StringBuffer appendUpdateSetKlausel(StringBuffer preUpdateSetKlausel, String fieldName, String newValue)
  {
    if (newValue == null) // Abfangen Nullpointer
    {
      newValue = "";
    }

    if (preUpdateSetKlausel == null)  // Abfangen Nullpointer
    {
      preUpdateSetKlausel = new StringBuffer();
    }

    // Komma zum Verknpfen der Terms, aber nicht beim ersten
    if (!StringUtil.empty(preUpdateSetKlausel))
    {
      preUpdateSetKlausel.append(", ");
    }

    preUpdateSetKlausel.append(fieldName);
    preUpdateSetKlausel.append("=");
    preUpdateSetKlausel.append("'");
    preUpdateSetKlausel.append(StringUtil.stripHochKomma(newValue));
    preUpdateSetKlausel.append("'");

    return preUpdateSetKlausel;
  } // end method appendUpdateSetKlausel

  /**
   * Aufbau SET-Klausel fr UPDATE mit Komma-Verknpfung.  <br/>
   * Anhngen an angefangene SET-Klausel                   <br/>
   * Dabei wird der Parameter newValue als generisches Objekt bergeben.<br/>
   * In der Methode {@link #object2String object2String} wird das Objekt in einen SQL-entsprechenden String<br/>
   * (String in Hochkomma, Boolean als SMALLINT, Datumsformat usw.) umgewandelt.<br/>
   * Dadurch ist es nicht ntig, fr jede Parameter-Klasse eine eigene Methode zu schreiben<br/>
   * und es ist mglich, die von den get-Methoden in {@link de.cnc.server.PersistensHelper PersistensHelper}
   * gelieferten Objekte ohne Type-Casting (if-else-Verteiler) in SQL-Statements zu packen.<br/>
   *
   * @param preUpdateSetKlausel bereits angefangene Klausel oder leer oder null
   * @param fieldName Name Datenbankfeld
   * @param newValue generisches Objekt
   * @return Rckgabe erweiterte Klausel
   */
  public static StringBuffer appendUpdateSetKlausel(StringBuffer preUpdateSetKlausel, String fieldName, Object newValue)
  {
    if (preUpdateSetKlausel == null)  // Abfangen Nullpointer
    {
      preUpdateSetKlausel = new StringBuffer();
    }

    // Komma zum Verknpfen der Terms, aber nicht beim ersten
    if (!StringUtil.empty(preUpdateSetKlausel))
    {
      preUpdateSetKlausel.append(", ");
    }

    preUpdateSetKlausel.append(fieldName);
    preUpdateSetKlausel.append("=");

    String  newValueString = objectToSqlString(newValue);

    preUpdateSetKlausel.append(newValueString);

    return preUpdateSetKlausel;
  } // end method appendUpdateSetKlausel

  // ***************************************************************************************************************

  /**
   * Aufbau INSERT-Klausel mit Komma-Verknpfung.  <br/>
   * Anhngen an angefangene INSERT-Klausel.<br/><br/>
   * Aufgrund der Notation des INSERT-Statements:<br/>
   * INSERT INTO ... (feldnamen) VALUES (werte)<br/>
   * werden zwei StringBuffer in einem Array,<br/>
   * jeweils Feldnamen und Feldwerte,<br/>
   * getrennt aufgebaut und schlielich in<br/>
   * {@link #completeInsertKlausel completeInsertKlausel} zusammengefhrt.<br/>
   *
   * @param preInsertKlausel bereits angefangene Klausel oder leer oder null
   * @param fieldName Name Datenbankfeld
   * @param newValue generisches Objekt
   * @return Rckgabe erweiterte Klausel
   */
//  public static StringBuffer[] appendInsertKlausel(StringBuffer[] preInsertKlausel, String fieldName, String newValue)
//  {
//     Log.debug( "appendInsertKlausel: " + fieldName + ": " + newValue );
//    if (preInsertKlausel == null) // Abfangen Nullpointer
//    {
//      preInsertKlausel = new StringBuffer[2];
//      preInsertKlausel[0] = new StringBuffer();
//      preInsertKlausel[1] = new StringBuffer();
//    }
//
//    if (preInsertKlausel.length != 2) // Abfangen Dimensionsfehler
//    {
//      preInsertKlausel = new StringBuffer[2];
//      preInsertKlausel[0] = new StringBuffer();
//      preInsertKlausel[1] = new StringBuffer();
//    }
//
//    if (preInsertKlausel[0] == null)  // Abfangen Nullpointer
//    {
//      preInsertKlausel[0] = new StringBuffer();
//    }
//
//    if (preInsertKlausel[1] == null)  // Abfangen Nullpointer
//    {
//      preInsertKlausel[1] = new StringBuffer();
//    }
//
//    // Komma zum Verknpfen der Terms, aber nicht beim ersten
//    if (!StringUtil.empty(preInsertKlausel[0]))
//    {
//      preInsertKlausel[0].append(", ");  // Feldname
//    }
//
//    preInsertKlausel[0].append(fieldName);  // Feldname
//
//    // Komma zum Verknpfen der Terms, aber nicht beim ersten
//    if (!StringUtil.empty(preInsertKlausel[1]))
//    {
//      preInsertKlausel[1].append(", ");  // Value
//    }
//
//    if (newValue == null) // Abfangen Nullpointer
//    {
//      newValue = "";
//    }
//
//    // Value
//    preInsertKlausel[1].append("'");
//    preInsertKlausel[1].append(StringUtil.stripHochKomma(newValue));
//    preInsertKlausel[1].append("'");
//
//    return preInsertKlausel;
//  }

  /**
   * Aufbau INSERT-Klausel mit Komma-Verknpfung.  <br/>
   * Anhngen an angefangene INSERT-Klausel.<br/><br/>
   * Aufgrund der Notation des INSERT-Statements:<br/>
   * INSERT INTO ... (feldnamen) VALUES (werte)<br/>
   * werden zwei StringBuffer in einem Array,<br/>
   * jeweils Feldnamen und Feldwerte,<br/>
   * getrennt aufgebaut und schlielich in<br/>
   * {@link #completeInsertKlausel completeInsertKlausel} zusammengefhrt.<br/><br/>
   * Hier wird der Parameter newValue als generisches Objekt bergeben.<br/>
   * In der Methode {@link #object2String object2String} wird das Objekt in einen SQL-entsprechenden String<br/>
   * (String in Hochkomma, Boolean als SMALLINT, Datumsformat usw.) umgewandelt.<br/>
   * Dadurch ist es nicht ntig, fr jede Parameter-Klasse eine eigene Methode zu schreiben<br/>
   * und es ist mglich, die von den get-Methoden in {@link de.cnc.server.PersistensHelper PersistensHelper}
   * gelieferten Objekte<br/> ohne Type-Casting (if-else-Verteiler) in SQL-Statements zu packen.<br/>
   *
   * @param preInsertKlausel bereits angefangene Klausel oder leer oder null
   * @param fieldName Name Datenbankfeld
   * @param newValue generisches Objekt
   * @return Rckgabe erweiterte Klausel
   */
  public static StringBuffer[] appendInsertKlausel(StringBuffer[] preInsertKlausel, String fieldName, Object newValue)
  {
    //Log.debug( "appendInsertKlausel: " + fieldName + ": " + newValue );
    if (preInsertKlausel == null) // Abfangen Nullpointer
    {
      preInsertKlausel = new StringBuffer[2];
      preInsertKlausel[0] = new StringBuffer();
      preInsertKlausel[1] = new StringBuffer();
    }

    if (preInsertKlausel.length != 2) // Abfangen Dimensionsfehler
    {
      preInsertKlausel = new StringBuffer[2];
      preInsertKlausel[0] = new StringBuffer();
      preInsertKlausel[1] = new StringBuffer();
    }

    if (preInsertKlausel[0] == null)  // Abfangen Nullpointer
    {
      preInsertKlausel[0] = new StringBuffer();
    }

    if (preInsertKlausel[1] == null)  // Abfangen Nullpointer
    {
      preInsertKlausel[1] = new StringBuffer();
    }

    // Komma zum Verknpfen der Terms, aber nicht beim ersten
    if (!StringUtil.empty(preInsertKlausel[0]))
    {
      preInsertKlausel[0].append(", ");  // Feldname
    }

    preInsertKlausel[0].append(fieldName);  // Feldname

    // Komma zum Verknpfen der Terms, aber nicht beim ersten
    if (!StringUtil.empty(preInsertKlausel[1]))
    {
      preInsertKlausel[1].append(", ");  // Value
    }

    // Value
    String  newValueString = objectToSqlString(newValue);

    preInsertKlausel[1].append(newValueString);

    return preInsertKlausel;
  }

  /**
   * endgltiges Zusammenfgen der mit
   * {@link #appendInsertKlausel(StringBuffer[],String,String) appendInsertKlausel(String)}
   * oder
   * {@link #appendInsertKlausel(StringBuffer[],String,Object) appendInsertKlausel(Object)}
   * aufgebauten INSERT-VALUE-Klausel.
   *
   * @param preInsertKlausel bereits angefangene Klausel
   * @return fertige Klausel aber ohne "INSERT INTO tabellenname "
   */
  public static StringBuffer completeInsertKlausel(StringBuffer[] preInsertKlausel)
  {
    if (preInsertKlausel == null) // Abfangen Nullpointer
    {
      preInsertKlausel = new StringBuffer[2];
      preInsertKlausel[0] = new StringBuffer();
      preInsertKlausel[1] = new StringBuffer();
    }

    if (preInsertKlausel.length != 2) // Abfangen Dimensionsfehler
    {
      preInsertKlausel = new StringBuffer[2];
      preInsertKlausel[0] = new StringBuffer();
      preInsertKlausel[1] = new StringBuffer();
    }

    if (preInsertKlausel[0] == null)  // Abfangen Nullpointer
    {
      preInsertKlausel[0] = new StringBuffer();
    }

    if (preInsertKlausel[1] == null)  // Abfangen Nullpointer
    {
      preInsertKlausel[1] = new StringBuffer();
    }

    StringBuffer  retStrBuff = new StringBuffer();

    retStrBuff.append(" (");
    retStrBuff.append(preInsertKlausel[0]);
    retStrBuff.append(") VALUES (");
    retStrBuff.append(preInsertKlausel[1]);
    retStrBuff.append(")");

    return retStrBuff;
  }

  // *******************************************************************************************************************

  /**
   * Konvertiert ein beliebiges Objekt in einen String fr SQL-Statements . <br/>
   * Hier wird das Parameter-Objekt in einen SQL-entsprechenden String<br/>
   * (String in Hochkomma, Boolean als SMALLINT, Datumsformat usw.) umgewandelt.<br/>
   * Dadurch ist es nicht ntig, fr jede Parameter-Klasse eine eigene Methode zu schreiben<br/>
   * und es ist mglich, die von den get-Methoden in {@link de.cnc.server.PersistensHelper PersistensHelper}
   * gelieferten Objekte<br/> ohne Type-Casting (if-else-Verteiler) in SQL-Statements zu packen.<br/>
   *
   * @param valueObject generisches Objekt
   * @return String fr SQL-Statement
   */
  private static String objectToSqlString(Object valueObject)
  {
    String  retString = null;

    if (valueObject instanceof String)
    {
      // String in Hochkomma einschliessen
      retString = "'" + StringUtil.stripHochKomma((String) valueObject) + "'";
    }
    else if (valueObject instanceof java.sql.Timestamp)
    {
      // Methode Timestamp.toString liefert einen String im JDBC-Timestamp-Format
      // String in Hochkomma einschliessen
      //retString = "'" + StringUtil.stripHochKomma(valueObject.toString()) + "'";
      retString = "'" + StringUtil.toDB2TimestampStr( (java.sql.Timestamp) valueObject ) + "'";
    }
    else if (valueObject instanceof java.util.Date)
    {
      // Konvertieren in java.sql.Date
      java.sql.Date sqlDate = new java.sql.Date(((java.util.Date) valueObject).getTime());
      String        datString = sqlDate.toString(); // Format jjjj-MM-dd

      retString = "'" + StringUtil.stripHochKomma(datString) + "'";
    }
    else if (valueObject instanceof java.lang.Boolean)
    {
      boolean bValue = ((java.lang.Boolean) valueObject).booleanValue();

      retString = (bValue ? "1" : "-1"); // 1 - true, -1 - false
    }
//    else if (valueObject instanceof Number)
//    {
//       //extra Logik fr Host-DB2
//       retString = "" + valueObject ;
//       if (retString.indexOf( '.' ) < 0)
//       {
//          //die Sch.-Host DB2 braucht auch bei INTEGER eine x.0
//          retString += ".0";
//       }
//    }
    else
    {
      retString = "" + valueObject ;
    }

    return retString;
  }
  
  /**
   * Anhngen Klausel " FETCH FIRST ROW ONLY"
   * mit Seiteneffekt auf bergebenen StringBuffer.
   * 
   * @param pStrBuff
   * @return
   */
  public static StringBuffer appendSingleRowClause(StringBuffer pStrBuff)
  {
     return pStrBuff.append( " FETCH FIRST ROW ONLY" );
  }

   /**
    * einzige Instance der Plathalterklasse {@link CurrentTimeStampJoker}
    */
   public static final CurrentTimeStampJoker CURRENT_TIMESTAMP_JOKER = new CurrentTimeStampJoker();

   /**
    * Diese Klasse ist ein Platzhalter fr "CURRENT TIMESTAMP".
    * In der Methode {@link SqlUtil#objectToSqlString(Object)}
    * wird sie per toString() in den literalen Text
    * "CURRENT TIMESTAMP" umgewandelt.
    *
    * @author Heiner Kcker
    * @since 09.09.2004
    */
   public static class CurrentTimeStampJoker
   {
      public String toString()
      {
         return "CURRENT TIMESTAMP";
      }
   }

}//end class SqlUtil
