package de.cnc.taglib;


import java.io.IOException;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.util.*;


/**
 * Tag zum Anzeigen der Seitennummer, Seitenanzahl, Treffernummer von bis und Trefferanzahl in Listen beim Blttern. <body><br/><br/>
 * Die Informationen werden vom {@link BrowseControl} ber die Methoden<br/>
 * {@link BrowseControl#staticGetCurrentPage}<br/>
 * {@link BrowseControl#staticGetPageCount}<br/>
 * {@link BrowseControl#staticElementAb}<br/>
 * {@link BrowseControl#staticElementBis}<br/>
 * {@link BrowseControl#staticElementAnz}<br/>
 * ermittelt. <br/><br/>
 * <pre>
 * Verwendung:
 *
 * Ablegen Liste von Datenbank in {@link de.cncdemo.actions} auf festem SessionKey {@link SessionKeys#CNC_LIST_KEY}
 * <b>
 * JSP
 * Kopf: (&lt;cnc:browseifo/&gt;)
 * &lt;cnc:iteratepaged&gt;
 *   ... Liste
 * &lt;/cnc:iteratepaged&gt;
 * </b>
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @see IterateBrowsedTag
 * @see BrowseControl
 * @author Heiner Kcker
 * @version $Id: $
 */
public class BrowseIfoTag extends TagSupport {

  /**
   * Erzeugen HTML-Code
   *
   * @return Code fr JSP-Seite weiter abarbeiten
   *
   * @exception JspException if a JSP exception has occurred
   */
  public int doStartTag()
      throws JspException {
    HttpSession   session = pageContext.getSession();

    // Vorbereiten Ausgabe

    StringBuffer  outStrBuff = new StringBuffer(1024);

    outStrBuff.append(StringUtil.cutRight("Seite " + BrowseControl.staticGetCurrentPage(session), ".0"));
    outStrBuff.append(" von " + BrowseControl.staticGetPageCount(session));
    outStrBuff.append(" , Position " + BrowseControl.staticElementAb(session));
    outStrBuff.append(" bis " + BrowseControl.staticElementBis(session));
    outStrBuff.append(" von " + BrowseControl.staticElementAnz(session));

    JspWriter writer = pageContext.getOut();

    try {
      writer.print(outStrBuff.toString());
    } catch (IOException e) {
      throw new JspException(e.toString());
    }
    return (EVAL_PAGE);
  } // end method doStartTag

  /**
   * Tut nichts.<br/>
   *
   * @return Code fr Weiterverarbeitung JSP
   *
   * @throws JspException
   */
  public int doEndTag()
      throws JspException {
    return EVAL_PAGE;
  } // end method doEndTag

}// end class BrowseIfoTag
