package de.cnc.taglib;


import java.io.IOException;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.SessionKeys;
import de.cnc.util.StringUtil;


/**
 * Erzeugen eines Buttons-<b>&lt;input name=&quot;...&quot; value=&quot;...&quot;&gt;</b>-Tags . <br/><br/>
 * <pre>
 * Verwendung:
 * <b>&lt;cnc:button name="..." value="..."/&gt;</b>
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id:$
 */
public class ButtonTag extends TagSupport {

  /**
   * Dieser String wird nach Button-Prefix im name="..."-Parameter
   * des Button-Tags eingebaut und kann spter in flow.cnc mit &lt;if button="name">
   * abgefragt werden.
   */
  private String name;

  /**
   * Dieser Text wird als value= in das erzeugte Tag eingefgt und ist der Text auf dem Button
   */
  private String value;

  /**
   * ob indiziert, fr Tabellenzeile
   */
  private String indexed;

  /**
   * ob validieren, default is true
   */
  private String validate = "true";

  /**
   * get name
   *
   * @return name
   */
//  public String getName() {
//    return name;
//  }// end method getName

  /**
   * set name
   *
   * @param name new Value
   */
  public void setName(String newName) {
    this.name = newName;
  }// end method setName

  /**
   * get value
   *
   * @return value
   */
//  public String getValue() {
//    return value;
//  }// end method getValue

  /**
   * set value
   *
   * @param value new Value
   */
  public void setValue(String newValue) {
    this.value = newValue;
  }// end method setValue

  /**
   * @param string
   */
  public void setIndexed(String string)
  {
      indexed = string;
  }

  /**
   * @param string
   */
  public void setValidate(String string)
  {
      validate = string;
  }

  /**
   * Erzeugung HTML-Code
   *
   * @return Code fr Weiterbearbeiten Seite oder Abbruch
   *
   * @exception JspException if a JSP exception has occurred
   */
  public int doStartTag()
      throws JspException {

          HttpSession session = pageContext.getSession();

          String indexStr = "";

          final boolean isIndexed = StringUtil.str2bool(indexed);
          if (isIndexed)
          { // indiziert
              // Ermitteln laufende Nummer
              // index wurde vom cnc:iterate/iteratepaged-Tag anhand der Option indexId als Integer-Objekt gesetzt
              // Index an Namen anhngen
              indexStr ="_cnc_index_" + 
                  ((Integer) session
                      .getAttribute(SessionKeys.CNC_LIST_ELEMENT_ID_KEY))
                      .toString();// NullPointerException provozieren

          } // end if indiziert

String noValidateStr = "";

if ( ! StringUtil.str2bool(validate) )
{
    noValidateStr = "_cnc_no_validate_"; //@todo: konstante machen
}

    // <input type="submit" name="cnc_button_add" value="Add"/>

    // fester Prefix cnc_button_
    // noch Konstante fr cnc_button_ machen
    String  outString = "<input type=\"submit\" value=\"" + value 
    + "\" name=\"cnc_button_" 
    + noValidateStr
    + name + indexStr + "\">";

    JspWriter writer = pageContext.getOut();

    try {
      writer.print(outString);
    } catch (IOException e) {
      throw new JspException(e.toString());
    }

    return (EVAL_PAGE);
  } // end method doStartTag

  /**
   * nichts zu tun
   *
   * @return Code fr Weiterverarbeitung JSP
   *
   * @throws JspException
   */
  public int doEndTag()
      throws JspException {
    return (EVAL_PAGE);
  }// end method doStartTag

}// end class ButtonTag
