package de.cnc.taglib;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.SessionKeys;

/**
 * ExistInSession-Tag . <br/>
 * Abarbeitung Tag-Body-Content abhngig von einem boolean-Property einer Session-Bean.<br/><br/>
 *
 * <pre>
 * Verwendung:
 * <b>&lt;cnc:if  name=&quot;SessionKey&quot; property=&quot;boolean-Property&quot;&gt;</b>
 *  ... Tag-Body-Content
 * <b>&lt;/cnc:if&gt;</b>
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id:$
 */
public class ExistInSessionTag extends TagSupport
{

   /**
    * Session-Key der abzufragenden Bean
    */
   private String sessionkey = SessionKeys.CNC_FORM_KEY;

   /**
    * get sessionkey
    *
    * @return sessionkey
    */
   public String getSessionkey()
   {
      return sessionkey;
   } // end method getsessionkey

   /**
    * set sessionkey
    *
    * @param sessionkey new Value
    */
   public void setSessionkey(String newSessionkey)
   {
      this.sessionkey = newSessionkey;
   } // end method setSessionkey

   /**
    * Entscheidung, ob Body abgearbeitet werden soll
    *
    * @return Code, ob Body abgearbeitet werden soll
    *
    * @throws JspException
    */
   public int doStartTag() throws JspException
   {
      // Log.debug( "doStartTag " + getClass().getName() ) ;
      int returnInt = SKIP_BODY;
      //boolean bOk = false;

      HttpSession session = pageContext.getSession();

      //Log.debug( "doStartTag " + getClass().getName() + " sessionkey " + sessionkey ) ;
      Object sessionObj = session.getAttribute(sessionkey);

      if (sessionObj != null)
      {
         returnInt = EVAL_PAGE;
      }
      return returnInt;
   } // end method doStartTag

   /**
    * nix zu tun
    *
    * @return Code fr Weiterverarbeitung
    *
    * @throws JspException
    */
   public int doEndTag() throws JspException
   {
      // Log.debug("doEndTag " + getClass().getName());
      return EVAL_PAGE;
   } // end method doEndTag

} // end class IfTag
