package de.cnc.taglib;

import java.io.IOException;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.SessionKeys;
import de.cnc.util.StringUtil;
import de.cnc.validator.*;

/**
 * Ausgabe Fehler-Text fr ein invalides Firm-Feld auf JSP-Seite . <br/>
 * Verwendung: <b>&lt;cnc:???/&gt;</b><br/><br/>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class FieldInvalidMessageTag extends TagSupport
{

    /**
     * anzuzeigendes Property der Bean auf dem angegeben SessionKey
     */
    protected String property = "";

    /**
     * ob die Fehler-Meldung fr eine Tabellenzeile ausgegeben werden soll.
     */
    protected String indexed;

    /**
     * set property
     *
     * @param property new Value
     */
    public void setProperty(String newProperty)
    {
        this.property = newProperty;
    }

    /**
     * @param string
     */
    public void setIndexed(String string)
    {
        indexed = string;
    }

    /**
     * Defer our checking until the end of this tag is encountered.
     *
     * @exception JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException
    {
        return (EVAL_PAGE);
    } // end method doStartTag

    /**
     * Ausgabe Fehlertext . <br/>
     * Nach der Ausgabe wird der Fehlertext aus der Session gelscht.<br/>
     *
     * @return Code fr Weiterverarbeitung JSP
     */
    public int doEndTag() throws JspException
    {
        HttpSession session = pageContext.getSession();

        // Form-Name des angesprochenenen Form-Feldes
        String propIndexed = property;

        final boolean isIndexed = StringUtil.str2bool(indexed);
        int indexInt = -1;
        if (isIndexed)
        { // indiziert
            // Ermitteln laufende Nummer
            // index wurde vom cnc:iterate/iteratepaged-Tag anhand der Option indexId als Integer-Objekt gesetzt
            indexInt =
                ((Integer) session
                    .getAttribute(SessionKeys.CNC_LIST_ELEMENT_ID_KEY))
                    .intValue();

            // input-name ist property-name des Elements + [index]
            // Index an Namen anhngen
            propIndexed += "[" + indexInt + "]";
        } // end if indiziert

        String outString = "";

        FormFieldErrorMap formFieldErrorMap =
            (FormFieldErrorMap) session.getAttribute(
                SessionKeys.CNC_FORM_FIELD_ERROR_MAP);
        //System.out.println( "formFieldErrorMap: " + formFieldErrorMap );
        if (formFieldErrorMap != null)
        {
            final FormFieldError formFieldError =
                formFieldErrorMap.get(propIndexed);
            if (formFieldError != null)
            {
                outString += formFieldError.getErrMsg();
            }
            //System.out.println( "outString: " + outString );
        }

        JspWriter writer = pageContext.getOut();

        try
        {
            writer.print(outString);
        }
        catch (IOException e)
        {
            throw new JspException(e.toString());
        }

        return EVAL_PAGE;
    } // end method doEndTag

} // end class WriteTag
