package de.cnc.taglib;


import java.io.IOException;
import java.net.URLEncoder;

import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.*;
import de.cnc.util.*;


/**
 * Erzeugen eines <b>&lt;form action=&quot;...&quot;&gt;</b>-Tags mit URL-Encoding und Hidden-Field fr die Source-JSP . <br/>
 * Das URL-Encoding ist zur Weitergabe der Session auch ohne Cookies notwendig.<br/>
 * Die Weitergabe der Source-JSP im Hidden-Field srcjsp ist z.B. fr die Bltter-Actions (kehren zur srcjsp zurck) notwendig.<br/>
 * Methode POST ist fest codiert.<br/>
 * <pre>
 * Verwendung:
 * <b>&lt;cnc:form action="von Bedingung auszuwertende Action"/&gt;</b>
 *  ...
 * &lt;/form&gt;
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @see CncServlet
 * @author Heiner Kcker
 * @version $Id:$
 */
public class FormTag extends TagSupport {

  /**
   * Die anzuspringende Action . <br/>
   * In web.xml ist die URI *.cnc auf die applikations-spezifische Ableitung von {@link CncServlet} gemappt.<br/>
   * {@link CncServlet} nimmt den Fluss wieder auf.<br/>
   * Die Auswertung der <b>action</b> ist in <b>flow.cnc</b> ber <b>&lt;if action="..."</b> mglich.
   */
  protected String action = "";

  /**
   * Option enctype (z.B.  ENCTYPE="multipart/form-data")
   */
  protected String enctype= null;

  /**
   * set action, wird von der JSP-Engine zum Einschreiben des action-Parameter benutzt
   *
   * @param action new Value
   */
  public void setAction(String newAction) {
    this.action = newAction;
  }

  /**
   * @param pString
   */
  public void setEnctype(String pString)
  {
     enctype = pString;
  }

  /**
   * Erzeugen HTML-Code fr Form-Tag
   *
   * @return Code fr Weiterverarbeitung JSP
   *
   * @throws JspException
   */
  public int doStartTag()
      throws JspException {
Log.debug( this.getClass().getName() + "#doStartTag()" );

    HttpServletRequest  request = (HttpServletRequest) pageContext.getRequest();
    HttpSession session = request.getSession();
//Log.dump( session );

    // Anhngen Postfix fr URL-Pattern *.cnc
    action += CncServlet.actionPostfix;

    // Cnc-Status
    final String strCncStatus = (String) session.getAttribute(SessionKeys.CNC_STATUS);
//Log.debug( "FormTag strCncStatus: " + strCncStatus );

    // RequestURI (aktueller JSP-Name)
    final String srcJsp = CncUtil.getCurrentJspName(request);

    // URL-Encoding
    StringBuffer  outStrBuff = new StringBuffer(300);
    outStrBuff.append("<form action=\"" + ((HttpServletResponse) pageContext.getResponse()).encodeURL(action) );

    final boolean isEnctype = ! StringUtil.empty( enctype );

    if ( isEnctype )
    {
       // bei enctype mulitpart funktionieren die hidden fields nicht
       outStrBuff.append("?" + SessionKeys.CNC_STATUS + "=" + URLEncoder.encode( strCncStatus )
                          + "&" + RequestKeys.CNC_SRC_JSP + "=" + URLEncoder.encode( srcJsp ) );
    }

    outStrBuff.append( "\"" );

    // outStrBuff.append( "\" method=\"get\">" ) ; nur fr Debug

    if ( isEnctype )
    {
       outStrBuff.append(" ENCTYPE=\"" + enctype + "\"");
    }

    outStrBuff.append(" method=\"post\"");
    outStrBuff.append(">");

    // bergabe CnC-Status in einem Hidden-Field
   outStrBuff.append("<input type=\"hidden\" name=\"" + RequestKeys.CNC_STATUS + "\" value=\""
        + strCncStatus + "\"/>");

    // bergabe RequestURI (aktueller JSP-Name) im Hidden-Field srcjsp
    outStrBuff.append("<input type=\"hidden\" name=\"" + RequestKeys.CNC_SRC_JSP + "\" value=\"" + srcJsp + "\"/>");

    //Feststellen, ob JavaScript eingeschalten ist
    if ( true ) // spter hier einen Schalter einbauen, ob abgefragt werden soll, ob JavaScript eingeschalten ist
    {
        outStrBuff.append( "<script language=\"JavaScript\" type=\"text/javascript\">\n" );
        outStrBuff.append( "<!--\n" );
        outStrBuff.append( "  document.write( \"<input type='hidden' name='CNC_JAVASCRIPT' value='on'/>\");\n" );
        outStrBuff.append( "//-->\n" );
        outStrBuff.append( "</script>\n" );
    }

    JspWriter writer = pageContext.getOut();

    try {
      writer.print(outStrBuff.toString());
    } catch (IOException e) {
        Log.fatal(e);
      throw new JspException(e.toString());
    }

    return (EVAL_PAGE);
  } // end method doStartTag

  /**
   * nichts zu tun
   *
   * @return Code fr Weiterverarbeitung JSP
   *
   * @throws JspException
   */
  public int doEndTag()
      throws JspException {
          Log.debug( this.getClass().getName() + "#doEndTag()" );
    return (EVAL_PAGE);
  }// end method doEndTag

}// end class FormTag
