package de.cnc.taglib;


import java.io.IOException;
import java.net.URLEncoder;

import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.*;
import de.cnc.util.*;


/**
 * Erzeugen eines <b>&lt;frame src=&quot;...&quot;&gt;</b>-Tags mit URL-Encoding und Hidden-Field fr die Source-JSP . <br/>
 * Das URL-Encoding ist zur Weitergabe der Session auch ohne Cookies notwendig.<br/>
 * Die Weitergabe der Source-JSP im Hidden-Field srcjsp ist z.B. fr die Bltter-Actions (kehren zur srcjsp zurck) notwendig.<br/>
 * Methode POST ist fest codiert.<br/>
 * <pre>
 * Verwendung:
 * <b>&lt;cnc:frame srcProc="..." name="..."/&gt;</b>
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @see CncServlet
 * @author Heiner Kcker
 * @version $Id:$
 */
public class FrameTag extends TagSupport {

  /**
   * Die aufzurufende Prozedur aus der
   * Fluss-Steuerdatei flow.cnc . <br/>
   */
  protected String srcProc;

  /**
   * Name des geffneten Frames.
   */
  protected String name;

  /**
   * @param srcProc The srcProc to set.
   */
  public void setSrcProc(String srcProc)
  {
      this.srcProc = srcProc;
  }

  /**
   * @param name The name to set.
   */
  public void setName(String name)
  {
      this.name = name;
  }


  /**
   * Erzeugen HTML-Code fr Form-Tag
   *
   * @return Code fr Weiterverarbeitung JSP
   *
   * @throws JspException
   */
  public int doStartTag()
      throws JspException {
Log.debug( this.getClass().getName() , "#doStartTag()" );

    HttpServletRequest  request = (HttpServletRequest) this.pageContext.getRequest();
    HttpSession session = request.getSession();
//Log.dump( session );

    // Anhngen Postfix fr URL-Pattern *.cnc
    String action = RequestKeys.CNC_GOTO_ACTION_PREFIX + this.srcProc + CncServlet.actionPostfix;

    // RequestURI (aktueller JSP-Name)
    final String srcJsp = CncUtil.getCurrentJspName(request);

    // URL-Encoding
    StringBuffer  outStrBuff = new StringBuffer(300);
    outStrBuff.append("<frame src=\"" + ((HttpServletResponse) this.pageContext.getResponse()).encodeURL(action) );

    // Cnc-Status
    //final String strCncStatus = (String) session.getAttribute(SessionKeys.CNC_STATUS);
//Log.debug( "FormTag strCncStatus: " + strCncStatus );

    outStrBuff.append("?" + SessionKeys.CNC_STATUS + "=" 
            // + URLEncoder.encode( strCncStatus )
            + "cncStatusNoCheck" // Prfen Session-Status (Synchronisationstoken) unterdrcken
                          + "&" + RequestKeys.CNC_SRC_JSP + "=" + URLEncoder.encode( srcJsp ) );

    outStrBuff.append( "\"" );

    // outStrBuff.append( "\" method=\"get\">" ) ; nur fr Debug

    outStrBuff.append(" name=\""+this.name+"\"");
    outStrBuff.append(">");

//    //Feststellen, ob JavaScript eingeschalten ist
//    if ( true ) // spter hier einen Schalter einbauen, ob abgefragt werden soll, ob JavaScript eingeschalten ist
//    {
//        outStrBuff.append( "<script language=\"JavaScript\" type=\"text/javascript\">\n" );
//        outStrBuff.append( "<!--\n" );
//        outStrBuff.append( "  document.write( \"<input type='hidden' name='CNC_JAVASCRIPT' value='on'/>\");\n" );
//        outStrBuff.append( "//-->\n" );
//        outStrBuff.append( "</script>\n" );
//    }

    JspWriter writer = this.pageContext.getOut();

    try {
Log.debug( this.getClass().getName() + "\n" , outStrBuff );
      writer.print(outStrBuff.toString());
    } catch (IOException e) {
        Log.fatal(e);
      throw new JspException(e.toString());
    }

    return (EVAL_PAGE);
  } // end method doStartTag

  /**
   * nichts zu tun
   *
   * @return Code fr Weiterverarbeitung JSP
   *
   * @throws JspException
   */
  public int doEndTag()
      throws JspException {
          Log.debug( this.getClass().getName() , "#doEndTag()" );
    return (EVAL_PAGE);
  }// end method doEndTag

}// end class FormTag
