package de.cnc.taglib;


import java.io.IOException;

import javax.servlet.http.*;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.*;


/**
 * Erzeugen eines Link-<b>&lt;a href=&quot;...&quot;&gt;</b>-Tags mit URL-Encoding . <br/><br/>
 * Das URL-Encoding ist zur Weitergabe der Session auch ohne Cookies notwendig.<br/>
 * <pre>
 * Verwendung:
 * <b>&lt;cnc:href href="..."/&gt;</b>
 *  ...
 * &lt;/a&gt;
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class HrefTag extends TagSupport {

  /**
   * Die anzuspringende Action oder JSP-URI. <br/>
   * In web.xml ist die URI *.cnc auf {@link CncServlet} gemappt.<br/>
   * {@link CncServlet} ruft die Action-Methode ActionBereich.Action.cnc auf.
   */
  protected String href;

    /** Der Link-Text */
    protected String txt;

  /**
   * get href
   *
   * @return href
   */
//  public String getHref() {
//    return href;
//  }

  /**
   * set href
   *
   * @param href new Value
   */
  public void setHref(String newHref) {
    this.href = newHref;
  }

  /**
   * @param string
   */
  public void setTxt(String string)
  {
      txt = string;
  }

  /**
   * target-Option fr Arbeit mit Frames oder Sub-Fenstern. <br/>
   */
  protected String target;

  /**
   * get target
   *
   * @return target
   */
//  public String getTarget() {
//    return target;
//  }

  /**
   * set target
   *
   * @param target new Value
   */
  public void setTarget(String newTarget) {
    this.target = newTarget;
  }

  /**
   * Erzeugung HTML-Code
   *
   * @return Code fr Weiterbearbeiten Seite oder Abbruch
   *
   * @exception JspException if a JSP exception has occurred
   */
  public int doStartTag()
      throws JspException {

    HttpSession session = pageContext.getSession();

    // Anhngen Postfix fr URL-Pattern *.cnc
    href += CncServlet.actionPostfix;

    StringBuffer hrefStrBuff = new StringBuffer(300);

    // URL-Encoding
    //outStrBuff.append("<a href='" + ((HttpServletResponse) pageContext.getResponse()).encodeURL(href));
    hrefStrBuff.append("href='" + ((HttpServletResponse) pageContext.getResponse()).encodeURL(href));

    hrefStrBuff.append("?" + RequestKeys.CNC_STATUS + "=" + (String) session.getAttribute(SessionKeys.CNC_STATUS));

    // bergabe der aktuellen JSP in als URL-Parameter
    String  srcJsp = ((HttpServletRequest) pageContext.getRequest()).getRequestURI();

    // Prefix /cnc entfernen, weil sonst doppelt in der Ziel-URI
    // evtl. konfigurierbare Lsung machen
    srcJsp = srcJsp.substring(1);                   // beginnendes / entfernen
    srcJsp = srcJsp.substring(srcJsp.indexOf("/")); // alles ab erstem / bernehmen

    // bergabe RequestURI (JSP-Name) als Parameter srcjsp
    hrefStrBuff.append("&" + RequestKeys.CNC_SRC_JSP + "=" + srcJsp);

    //outStrBuff.append( "\"" );

    //Target-Option
    String targetStr = "";
    if (target != null)
    {
       targetStr = " target='" + target + "'";
    }

    //outStrBuff.append( ">" );

    //StringBuffer outStrBuff = null;

    //Feststellen, ob JavaScript eingeschalten ist
    //if ( true ) // spter hier einen Schalter einbauen, ob abgefragt werden soll, ob JavaScript eingeschalten ist
    //{
    StringBuffer outStrBuff = new StringBuffer( 
          "<script language=\"JavaScript\" type=\"text/javascript\">\n"
        + "<!--\n"
        + "  document.write( \"<a " + hrefStrBuff + "&"+RequestKeys.CNC_JAVASCRIPT+"=on'" + targetStr + ">" + txt + "</a>\" );\n"
        + "//-->\n"
        + "</script><noscript><a " + hrefStrBuff + "'" + targetStr + ">" + txt + "</a></noscript>" );
    //}

    JspWriter writer = pageContext.getOut();

    try {
      writer.print(outStrBuff.toString());
    } catch (IOException e) {
      throw new JspException(e.toString());
    }

    return (EVAL_PAGE);
  } // end method doStartTag

  /**
   * nichts zu tun
   *
   * @return Code fr Weiterverarbeitung JSP
   *
   * @throws JspException
   */
  public int doEndTag()
      throws JspException {
    return (EVAL_PAGE);
  }// end method doStartTag

}// end class HrefTag
