package de.cnc.taglib;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.SessionKeys;
import de.cnc.util.StringUtil;
import de.cnc.validator.*;

/**
 * IF-Form-Field-Invalid-Tag . <br/>
 * Abarbeitung Tag-Body-Content abhngig von einem boolean-Property einer Session-Bean.<br/><br/>
 *
 * <pre>
 * Verwendung:
 * <b>&lt;cnc:if  name=&quot;SessionKey&quot; property=&quot;boolean-Property&quot;&gt;</b>
 *  ... Tag-Body-Content
 * <b>&lt;/cnc:if&gt;</b>
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class IfFieldInvalidTag extends TagSupport
{
    /**
     * anzuzeigendes Property der Bean auf dem angegeben SessionKey
     */
    protected String property = "";

    /**
     * ob die Fehler-Meldung fr eine Tabellenzeile ausgegeben werden soll.
     */
    protected String indexed;

    /**
     * set property
     *
     * @param property new Value
     */
    public void setProperty(String newProperty)
    {
        this.property = newProperty;
    }

    /**
     * @param string
     */
    public void setIndexed(String string)
    {
        indexed = string;
    }

   /**
    * Entscheidung, ob Body abgearbeitet werden soll
    *
    * @return Code, ob Body abgearbeitet werden soll
    *
    * @throws JspException
    */
   public int doStartTag() throws JspException
   {
      // Log.debug( "doStartTag " + getClass().getName() ) ;
      boolean bIsInvalid = false;

      HttpSession session = pageContext.getSession();

      // Form-Name des angesprochenenen Form-Feldes
      String propIndexed = property;

      final boolean isIndexed = StringUtil.str2bool(indexed);
      int indexInt = -1;
      if (isIndexed)
      { // indiziert
          // Ermitteln laufende Nummer
          // index wurde vom cnc:iterate/iteratepaged-Tag anhand der Option indexId als Integer-Objekt gesetzt
          indexInt =
              ((Integer) session
                  .getAttribute(SessionKeys.CNC_LIST_ELEMENT_ID_KEY))
                  .intValue();

          // input-name ist property-name des Elements + [index]
          // Index an Namen anhngen
          propIndexed += "[" + indexInt + "]";
      } // end if indiziert

      FormFieldErrorMap formFieldErrorMap =
          (FormFieldErrorMap) session.getAttribute(
              SessionKeys.CNC_FORM_FIELD_ERROR_MAP);
      //System.out.println( "formFieldErrorMap: " + formFieldErrorMap );
      if (formFieldErrorMap != null)
      {
          final FormFieldError formFieldError =
              formFieldErrorMap.get(propIndexed);
          if (formFieldError != null)
          {
              bIsInvalid = true;
          }
      }

      if (bIsInvalid)
      {
         return EVAL_PAGE;
      }
      return  SKIP_BODY;
   } // end method doStartTag

   /**
    * nix zu tun
    *
    * @return Code fr Weiterverarbeitung
    *
    * @throws JspException
    */
   public int doEndTag() throws JspException
   {
      // Log.debug("doEndTag " + getClass().getName());
      return EVAL_PAGE;
   } // end method doEndTag

} // end class IfTag
