package de.cnc.taglib;

import javax.servlet.ServletContext;
import javax.servlet.http.*;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.*;
import de.cnc.util.*;

/**
 * IF-Tag . <br/>
 * Abarbeitung Tag-Body-Content abhngig von einem boolean-Property einer Session-Bean.<br/><br/>
 *
 * <pre>
 * Verwendung:
 * <b>&lt;cnc:if  name=&quot;SessionKey&quot; property=&quot;boolean-Property&quot;&gt;</b>
 *  ... Tag-Body-Content
 * <b>&lt;/cnc:if&gt;</b>
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class IfTag extends TagSupport
{

   /**
    * Session-Key der abzufragenden Bean
    */
   private String sessionkey = SessionKeys.CNC_FORM_KEY;

   /**
    * abzufragende Property der Bean
    */
   protected String property = null;

   /**
    * aufzurufende Action-Methode zur Abfrage Condition
    */
   protected String actionmethod = null;

   /**
    * get sessionkey
    *
    * @return sessionkey
    */
//   public String getSessionkey()
//   {
//      return sessionkey;
//   } // end method getsessionkey

   /**
    * set sessionkey
    *
    * @param sessionkey new Value
    */
   public void setSessionkey(String newSessionkey)
   {
      this.sessionkey = newSessionkey;
   } // end method setSessionkey

   /**
    * get property
    *
    * @return property
    */
//   public String getProperty()
//   {
//      return property;
//   }

   /**
    * set property
    *
    * @param property new Value
    */
   public void setProperty(String newProperty)
   {
      this.property = newProperty;
   }

   /**
    * get actionmethod
    *
    * @return actionmethod
    */
   public String getActionmethod()
   {
      return actionmethod;
   } // end method getActionmethod

   /**
    * set actionmethod
    *
    * @param newActionmethod actionmethod new Value
    */
   public void setActionmethod(String newActionmethod)
   {
      this.actionmethod = newActionmethod;
   } // end method setActionmethod

   /**
    * Entscheidung, ob Body abgearbeitet werden soll
    *
    * @return Code, ob Body abgearbeitet werden soll
    *
    * @throws JspException
    */
   public int doStartTag() throws JspException
   {
      // Log.debug( "doStartTag " + getClass().getName() ) ;
      boolean bOk = false;

      HttpServletRequest request = (HttpServletRequest) pageContext.getRequest();
      HttpServletResponse response = (HttpServletResponse) pageContext.getResponse();
      HttpSession session = pageContext.getSession();
      ServletContext application = pageContext.getServletContext();
      
      CncContext cncContext = new CncContext( request, response, session, application, null , false);

      if (!StringUtil.empty(actionmethod))
      {
         // Log.debug( "actionmethod " + actionmethod ) ;
         Object actionRetObj =
            ActionMethodClassCache.invokeActionMethod(
               actionmethod,
               -1,
               request,
               (HttpServletResponse) pageContext.getResponse(),
               cncContext);

         // Auswerten Rckgabe-String der Action-Methode als boolean
         bOk = StringUtil.str2bool( "" + actionRetObj);
      } // end if actionmethod
      // Bitte Beachten, da sessionkey mit SessionKeys.CNC_FORM_KEY vorbelegt wird
      else if (!StringUtil.empty(sessionkey))
      {
         //Log.debug( "doStartTag " + getClass().getName() + " sessionkey " + sessionkey ) ;
         Object bean = session.getAttribute(sessionkey);

         if (bean == null)
         {
            //Log.debug( "doStartTag " + getClass().getName() + " bean " + sessionkey + " undefined" ) ;
         }
         else
         {
            if (StringUtil.empty(property))
            {
               // Log.debug("doStartTag " + getClass().getName() + " property undefined" ) ;
               // Log.debug( "bean " + bean ) ;
               bOk = StringUtil.str2bool("" + bean);
               // Log.debug( "bOk " + bOk ) ;
            }
            else
            {
               Object value = BeanUtil.getIndexed(bean, property);

               bOk = StringUtil.str2bool("" + value);
               // Log.debug( "bOk " + bOk ) ;
            }
         }
      } // end if sessionkey
      else
      { // irgendwas nicht in Ordnung
         throw new JspException("Tag cnc;if cnc:ifnot Attribute sessionkey or actionmethod mandantory");
      } // end if

      if (bOk)
      {
         return EVAL_PAGE;
      }
      return  SKIP_BODY;
   } // end method doStartTag

   /**
    * nix zu tun
    *
    * @return Code fr Weiterverarbeitung
    *
    * @throws JspException
    */
   public int doEndTag() throws JspException
   {
      // Log.debug("doEndTag " + getClass().getName());
      return EVAL_PAGE;
   } // end method doEndTag

} // end class IfTag
