package de.cnc.taglib;


import java.io.IOException;

import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.util.StringUtil;


/**
 * Tag fr Image-Button zum Ansprechen eines Listenelements . <br/>
 * <pre>
 * Verwendung:
 *
 * <b>&lt;cnc:imagebuttongoto parametername=&quot;MvDn&quot; src=&quot;images/arr_dn.gif&quot; alt=&quot;Datensatz nach unten verschieben&quot; border=&quot;0&quot;/&gt;</b>
 *
 * Erzeugt:
 *
 * <b>&lt;input type=&quot;image&quot; name=&quot;MvUp0&quot; src=&quot;images/arr_up.gif&quot; alt=&quot;Datensatz nach oben verschieben&quot; border=&quot;0&quot;/&gt;</b>
 * </pre>
 * <br/>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class ImageButtonGotoTag extends TagSupport {

  /**
   * Dieser String wird mit Goto-Prefix und im name="..."-Parameter des Button-Tags eingebaut.
   */
  private String name = "";

  /**
   * Dieser Text wird als value= in das erzeugte Tag eingefgt und ist der Text auf dem Button
   */
  private String value = "";

  /**
   * src= Name der Grafikdatei
   */
  private String    src = "";

  /**
   * Tooltip-Text
   */
  private String    alt = "";

  /**
   * width-Option
   */
  private String    width = "";

  /**
   * height-Option
   */
  private String    height = "";

  /**
   * border-Option
   */
  private String    border = "";

  /**
   * get name
   *
   * @return name
   */
  public String getName() {
    return name;
  }// end method getName

  /**
   * set name
   *
   * @param name new Value
   */
  public void setName(String newName) {
    this.name = newName;
  }// end method setName

  /**
   * get value
   *
   * @return value
   */
  public String getValue() {
    return value;
  }// end method getValue

  /**
   * set value
   *
   * @param value new Value
   */
  public void setValue(String newValue) {
    this.value = newValue;
  }// end method setValue

  /**
   * get src
   *
   * @return src
   */
  public String getSrc() {
    return src;
  }

  /**
   * set src
   *
   * @param src new Value
   */
  public void setSrc(String newSrc) {
    this.src = newSrc;
  }

  /**
   * get alt
   *
   * @return alt
   */
  public String getAlt() {
    return alt;
  }

  /**
   * set alt
   *
   * @param alt new Value
   */
  public void setAlt(String newAlt) {
    this.alt = newAlt;
  }

  /**
   * get width
   *
   * @return width
   */
  public String getWidth() {
    return width;
  }

  /**
   * set width
   *
   * @param width new Value
   */
  public void setWidth(String newWidth) {
    this.width = newWidth;
  }

  /**
   * get height
   *
   * @return height
   */
  public String getHeight() {
    return height;
  }

  /**
   * set height
   *
   * @param height new Value
   */
  public void setHeight(String newHeight) {
    this.height = newHeight;
  }

  /**
   * get border
   *
   * @return border
   */
  public String getBorder() {
    return border;
  }

  /**
   * set border
   *
   * @param border new Value
   */
  public void setBorder(String newBorder) {
    this.border = newBorder;
  }

  /**
   * Erzeugen HTML-Code
   *
   * @return Code fr JSP-Seite weiter abarbeiten
   *
   * @throws JspException
   */
  public int doStartTag()
      throws JspException {
    String  outString = "<input type=\"image\" value=\"" + value + "\" name=\"cnc_goto_" + name + "\"";

    // src-Option
    if (!StringUtil.empty(src)) {
      outString += " src=\"" + src + "\"";
    }

    // alt-Option
    if (!StringUtil.empty(alt)) {
      outString += " alt=\"" + alt + "\"";
    }

    // height-Option
    if (!StringUtil.empty(height)) {
      outString += " height=\"" + height + "\"";
    }

    // width-Option
    if (!StringUtil.empty(width)) {
      outString += " width=\"" + width + "\"";
    }

    // border-Option
    if (!StringUtil.empty(border)) {
      outString += " border=\"" + border + "\"";
    }

    // abschliessendes "/>"
    outString += "/>";

    JspWriter writer = pageContext.getOut();

    try {
      writer.print(outString);
    } catch (IOException e) {
      throw new JspException(e.toString());
    }

    // return retInt;
    return EVAL_PAGE;
  }// end method doStartTag

  /**
   * nichts zu tun
   *
   * @return Code fr Weiterverarbeitung JSP
   *
   * @throws JspException
   */
  public int doEndTag()
      throws JspException {
    return EVAL_PAGE;
  }// end method doEndTag

}// end class ImageButtonGotoTag
