package de.cnc.taglib;


import java.io.IOException;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.SessionKeys;
import de.cnc.util.StringUtil;


/**
 * Tag fr Image-Button . <br/>
 * <pre>
 * Verwendung:
 *
 * <b>&lt;cnc:imagebutton parametername=&quot;MvDn&quot; src=&quot;images/arr_dn.gif&quot; alt=&quot;Datensatz nach unten verschieben&quot; border=&quot;0&quot;/&gt;</b>
 *
 * Erzeugt:
 *
 * <b>&lt;input type=&quot;image&quot; name=&quot;MvUp0&quot; src=&quot;images/arr_up.gif&quot; alt=&quot;Datensatz nach oben verschieben&quot; border=&quot;0&quot;/&gt;</b>
 * </pre>
 * <br/>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id:$
 */
public class ImageButtonTag extends TagSupport {

  /**
   * Dieser String wird zwischen Button-Prefix und Index-Interfix im name="..."-Parameter
   * des Button-Tags eingebaut und kann spter in flow.cnc mit &lt;if buttonindexed="name">
   * abgefragt werden.
   */
  private String name = "";

  /**
   * Dieser Text wird als value= in das erzeugte Tag eingefgt und ist der Text auf dem Button
   */
  private String value = "";

  /**
   * src= Name der Grafikdatei
   */
  private String    src = "";

  /**
   * Tooltip-Text
   */
  private String    alt = "";

  /**
   * width-Option
   */
  private String    width = "";

  /**
   * height-Option
   */
  private String    height = "";

  /**
   * border-Option
   */
  private String    border = "";

  /**
   * ob indiziert, fr Tabellenzeile
   */
  private String indexed;

  /**
   * get name
   *
   * @return name
   */
  public String getName() {
    return name;
  }// end method getName

  /**
   * set name
   *
   * @param name new Value
   */
  public void setName(String newName) {
    this.name = newName;
  }// end method setName

  /**
   * get value
   *
   * @return value
   */
  public String getValue() {
    return value;
  }// end method getValue

  /**
   * set value
   *
   * @param value new Value
   */
  public void setValue(String newValue) {
    this.value = newValue;
  }// end method setValue

  /**
   * get src
   *
   * @return src
   */
  public String getSrc() {
    return src;
  }

  /**
   * set src
   *
   * @param src new Value
   */
  public void setSrc(String newSrc) {
    this.src = newSrc;
  }

  /**
   * get alt
   *
   * @return alt
   */
  public String getAlt() {
    return alt;
  }

  /**
   * set alt
   *
   * @param alt new Value
   */
  public void setAlt(String newAlt) {
    this.alt = newAlt;
  }

  /**
   * get width
   *
   * @return width
   */
  public String getWidth() {
    return width;
  }

  /**
   * set width
   *
   * @param width new Value
   */
  public void setWidth(String newWidth) {
    this.width = newWidth;
  }

  /**
   * get height
   *
   * @return height
   */
  public String getHeight() {
    return height;
  }

  /**
   * set height
   *
   * @param height new Value
   */
  public void setHeight(String newHeight) {
    this.height = newHeight;
  }

  /**
   * get border
   *
   * @return border
   */
  public String getBorder() {
    return border;
  }

  /**
   * set border
   *
   * @param border new Value
   */
  public void setBorder(String newBorder) {
    this.border = newBorder;
  }

  /**
   * @param string
   */
  public void setIndexed(String string)
  {
      indexed = string;
  }

  /**
   * Erzeugen HTML-Code
   *
   * @return Code fr JSP-Seite weiter abarbeiten
   *
   * @throws JspException
   */
  public int doStartTag()
      throws JspException {
    // String  outString = "<input type=\"image\"";
    // outString += " name=\"";
    // //parametername wird als name= in das erzeugte Tag eingefgt und ist nach dem Absenden des Fornulars der Parametername im Request
    // outString += parametername;
    //
    // //Anhngen Element-Nummer an Parameter-Name
    // //deshalb in Action scannen mit CncUtil#scanForNameStartsWith
    // Integer indexInteger = (Integer) pageContext.findAttribute(SessionKeys.CNC_LIST_ELEMENT_ID_KEY);
    // outString += "" + indexInteger + "\"";

    HttpSession session = pageContext.getSession();

    String indexStr = "";

    final boolean isIndexed = StringUtil.str2bool(indexed);
    if (isIndexed)
    { // indiziert
        // Ermitteln laufende Nummer
        // index wurde vom cnc:iterate/iteratepaged-Tag anhand der Option indexId als Integer-Objekt gesetzt
        // Index an Namen anhngen
        indexStr ="_cnc_index_" + 
            ((Integer) session
                .getAttribute(SessionKeys.CNC_LIST_ELEMENT_ID_KEY))
                .toString();// NullPointerException provozieren

    } // end if indiziert

//@todo: stringbuffer
    String  outString = "<input type=\"image\" value=\"" + value + "\" name=\"cnc_button_";

    outString += name;
    outString += indexStr;
    
    outString += "\"";

    // src-Option
    if (!StringUtil.empty(src)) {
      outString += " src=\"" + src + "\"";
    }

    // alt-Option
    if (!StringUtil.empty(alt)) {
      outString += " alt=\"" + alt + "\"";
    }

    // height-Option
    if (!StringUtil.empty(height)) {
      outString += " height=\"" + height + "\"";
    }

    // width-Option
    if (!StringUtil.empty(width)) {
      outString += " width=\"" + width + "\"";
    }

    // border-Option
    if (!StringUtil.empty(border)) {
      outString += " border=\"" + border + "\"";
    }

    // abschliessendes "/>"
    outString += "/>";

    JspWriter writer = pageContext.getOut();

    try {
      writer.print(outString);
    } catch (IOException e) {
      throw new JspException(e.toString());
    }

    // return retInt;
    return EVAL_PAGE;
  }// end method doStartTag

  /**
   * nichts zu tun
   *
   * @return Code fr Weiterverarbeitung JSP
   *
   * @throws JspException
   */
  public int doEndTag()
      throws JspException {
    return EVAL_PAGE;
  }// end method doEndTag

}// end class ImageButtonIndexedTag
