package de.cnc.taglib;

import java.util.List;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.JspException;

import de.cnc.SessionKeys;
import de.cnc.util.*;

/**
 * Iterate-Browsd-Tag zum seitenweisen Anzeigen. <body><br/><br/>
 * Using:<br/><br/>
 * &lt;logic:iterate id="element" name="&lt;%= SessionKeys.BERATER_NEU_LISTE %&gt;"&gt; <br/>
 * &nbsp;&nbsp;&nbsp;&lt;cnc:linkindexed href="beraterforumNeuEditAction.do" name="element" property="itemNr"/&gt;Link-Text&lt;/a&gt; <br/>
 * &lt;/cnc:iteratebrowsed&gt; <br/><br/>
 * Vorgehensweise: <br/>
 *
 * Alles komplett berarbeiten !!!<br/>
 * In der Session gibt es zu jeder seitenweise anzuzeigenden Collection (meist ArrayList)
 * noch ein Objekt {@link de.cnc.util.BrowseControl} zum Steuern des Bltterns.<br/>
 * Vor jedem Anzeigen {@link IterateTag#doStartTag} wird die {@link BrowseControl#validate}-Methode
 * des {@link BrowseControl}-Objektes aufgerufen, um zu sichern,
 * da die anzuzeigende Seite im darstellbaren Bereich liegt.
 *
 * </body>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @see IterateTag
 * @author Heiner Kcker
 * @version $Id: $
 */
public class IterateBrowsedTag extends IterateTag
{

    /**
     *
     * @exception JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException
    {

        //Log.debug(this.getClass().getName() , "doStartTag()");

        HttpSession session = pageContext.getSession();

        // BrowseControl aus der Session holen
        BrowseControl iterateBrowseControl =
            (BrowseControl) session.getAttribute(
                SessionKeys.CNC_BROWSE_CTRL_KEY);
        // mit festem Session-Key fr den Browse-Control

        if (iterateBrowseControl == null)
        {
            iterateBrowseControl = new BrowseControl();
            CncUtil.setAttribute(
                session,
                SessionKeys.CNC_BROWSE_CTRL_KEY,
                iterateBrowseControl);
            // mit festem Session-Key fr den Browse-Control
        } // end if

        // Liste aus der Session holen
        // list ist Memeber der Elternklasse
        list = (List) session.getAttribute(SessionKeys.CNC_LIST_KEY);

//Log.debug( "list" , list );

        if (list == null)
        {
            throw new JspException("ArrayList on Session-Key CNC_LIST_KEY not found");
        } // end if

        // sichern, da die anzuzeigende Seite im darstellbaren Bereich liegt
        // Variable arrayList kommt von der Elternklasse
        iterateBrowseControl.validate(list.size());

        // Offset fr Elternklasse (iterate-Tag) setzen
        elementPos = iterateBrowseControl.getOffset();

        // Anzahl anzuzeigender Positionen setzen
        endPos = elementPos + iterateBrowseControl.getItemsPerPage() /*- 1*/;

        if (list.size() > 0)
        {
            // Element in Session ablegen
            Object element = list.get(elementPos);

            CncUtil.setAttribute(
                session,
                SessionKeys.CNC_LIST_ELEMENT_KEY,
                element);
            // fester SessionKey
            // Elementnummer in Session ablegen
            CncUtil.setAttribute(
                session,
                SessionKeys.CNC_LIST_ELEMENT_ID_KEY,
                new Integer(elementPos));
            // fester SessionKey
        }
        else
        {
            return SKIP_BODY;
        } // end if

        return EVAL_PAGE;
    } // end method doStartTag

    /**
     * Endposition Seite
     */
    protected int endPos = 0;

    /**
     *
     *
     * @throws JspException
     */
    public void doInitBody() throws JspException
    { // Log.debug( "doInitBody " + getClass().getName() ) ;
    }

    /**
     *
     * @return Steuerwert fr Iteration fortsetzen/abbrechen
     *
     * @throws JspException
     */
    public int doAfterBody() throws JspException
    {
        // Log.debug( getClass().getName() , "doAfterBody");

        if ( elementPos >= endPos )
        {
            return SKIP_BODY;
        }

        return super.doAfterBody();
        // Achtung elementPos wird in der Methode der Elternklasse inkrementiert
    } // end method doAfterBody

} // end class IterateBrowsedTag
