package de.cnc.taglib;


import java.io.IOException;
import java.util.*;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.BodyTagSupport;

import de.cnc.SessionKeys;
import de.cnc.util.*;


/**
 * Iterate-Tag zum Anzeigen von Listen. <body><br/><br/>
 * Die Tags {@link IterateTag} und
 * {@link IterateBrowsedTag}<br/>
 * legen bei jedem Durchlauf
 * die Nummer (index Basis 0) des aktuellen Listenelements auf dem festen SessionKey {@link SessionKeys#CNC_LIST_ELEMENT_ID_KEY} und<br/>
 * das aktuelle Listenelement auf dem festen SessionKey {@link SessionKeys#CNC_LIST_ELEMENT_KEY} ab,<br/>
 * wodurch die laufende Nummer und das aktuelle Element ermittelt werden knnen.<br/>
 * <pre>
 * Verwendung:
 *
 * Ablegen Liste von Datenbank auf festem SessionKey {@link SessionKeys#CNC_LIST_KEY}
 * <b>
 * JSP
 * &lt;cnc:iterate&gt;
 *
 *  ... BodyContent
 *
 * &lt;/cnc:iterate&gt;
 * </b>
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @see IterateBrowsedTag
 * @author Heiner Kcker
 * @version $Id: $
 */
public class IterateTag extends BodyTagSupport {

  /**
   * laufende Nummer (Basis 0)
   */
  protected int elementPos = 0;

  /**
   * Referenz auf die Liste ber welche iteriert wird {@link SessionKeys#CNC_LIST_KEY}
   */
  protected List list = null;

  /**
   * Vorbereitung Abarbeitung
   *
   * @return Code, ob Body abgearbeitet werden soll
   *
   * @throws JspException
   */
  public int doStartTag()
      throws JspException {
    //Log.debug("doStartTag " + getClass().getName());

    elementPos = 0;

    HttpSession session = pageContext.getSession();

    // Liste von festem Sessionkey holen
    list = (ArrayList) session.getAttribute(SessionKeys.CNC_LIST_KEY);

//Log.debug( "list" +  list );

    if (list != null && list.size() > 0) 
    {
      // Element in Session ablegen
      Object element = list.get(elementPos);

      CncUtil.setAttribute(session, SessionKeys.CNC_LIST_ELEMENT_KEY, element); // fester SessionKey

      // Elementnummer in Session ablegen
      CncUtil.setAttribute(session, SessionKeys.CNC_LIST_ELEMENT_ID_KEY, new Integer(elementPos)); // fester SessionKey
//Log.debug( getClass().getName()+ " CNC_LIST_ELEMENT_ID_KEY " + session.getAttribute( SessionKeys.CNC_LIST_ELEMENT_ID_KEY ) ) ;

      return EVAL_PAGE;
    } 
    else 
    {
      // keine Elemente vorhanden
      return SKIP_BODY;
    }
  }// end method doStartTag

  /**
   * nichts zu tun
   *
   * @throws JspException
   */
  public void doInitBody()
      throws JspException {// Log.debug("doInitBody " + getClass().getName());
      //Log.debug("doInitBody " + getClass().getName());
  }

  /**
   * nchsten Durchlauf vorbereiten
   * und Ausgabe Body-Content
   *
   * @return Code fr Weiterverarbeitung
   *
   * @throws JspException
   */
  public int doAfterBody()
      throws JspException {
    //Log.debug("doAfterBody " + getClass().getName());

    //  notwendiger Block fr BodyTagSupport
    try
    {
       JspWriter out = bodyContent.getEnclosingWriter();
       out.print( bodyContent.getString() );
       bodyContent.clearBody();
    }
    catch (IOException ioe)
    {
       Log.fatal( ioe );
    }

    elementPos++;

    if (elementPos >= list.size()) 
    {
       //Speicher freigeben
       HttpSession session = pageContext.getSession();
       session.removeAttribute(SessionKeys.CNC_LIST_ELEMENT_KEY);
       session.removeAttribute(SessionKeys.CNC_LIST_ELEMENT_ID_KEY);

       // Iterieren Beenden
       return SKIP_BODY;
    }
    HttpSession session = pageContext.getSession();

    // Element in Session ablegen
    Object element = list.get(elementPos);

    CncUtil.setAttribute(session, SessionKeys.CNC_LIST_ELEMENT_KEY, element); // fester SessionKey

    // Elementnummer in Session ablegen
    CncUtil.setAttribute(session, SessionKeys.CNC_LIST_ELEMENT_ID_KEY, new Integer(elementPos)); // fester SessionKey
//Log.debug( getClass().getName()+ " CNC_LIST_ELEMENT_ID_KEY " + session.getAttribute( SessionKeys.CNC_LIST_ELEMENT_ID_KEY ) ) ;

    // Iterieren Fortsetzen
    // return EVAL_BODY_TAG ;
    // return IterationTag.EVAL_BODY_AGAIN;
    // return EVAL_BODY_TAG ; // fr BodyTagSupport
    // return EVAL_BODY_INCLUDE;
    return EVAL_BODY_BUFFERED;
  } // end method doAfterBody

  /**
   * endgltige Ausgabe Body-Content
   *
   * @return Code fr Weiterverarbeitung
   *
   * @throws JspException
   */
  public int doEndTag()
      throws JspException {
    // Log.debug("doEndTag " + getClass().getName());
    return EVAL_PAGE;
  }// end method doEndTag

}// end class IterateTag
