package de.cnc.taglib;

import java.io.IOException;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.SessionKeys;

/**
 * Kursorhnliche Ausgabe fr eine Liste auf JSP-Seite . <br/>
 * Verwendung: <b>&lt;cnc:listcursor cursorKey=&quot;...&quot; yesOut=&quot;...&quot; noOut=&quot;...&quot;/&gt;</b><br/><br/>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class ListCursorTag extends TagSupport
{
   /**
    * Session-Key mit einer Integer-Variable mit der Positionsnummer
    * (Basis 0) zum Hervorheben des positionierten Tabellenelements
    */
   protected String cursorKey;
   
   /**
    * Ausgabe, wenn es sich um das positionierte Tabellenelement handelt
    */
   protected String yesOut;
   
   /**
    * Ausgabe, wenn es sich nicht um das positionierte Tabellenelement handelt
    */
   protected String noOut;   

   /**
    * @param pString
    */
   public void setCursorKey(String pString)
   {
      cursorKey = pString;
   }

   /**
    * @param pString
    */
   public void setNoOut(String pString)
   {
      noOut = pString;
   }

   /**
    * @param pString
    */
   public void setYesOut(String pString)
   {
      yesOut = pString;
   }

   /**
    * Defer our checking until the end of this tag is encountered.
    *
    * @exception JspException if a JSP exception has occurred
    */
   public int doStartTag() throws JspException
   {
      return (EVAL_PAGE);
   } // end method doStartTag

   /**
    * Ausgabe Text . <br/>
    * Nach der Ausgabe wird der Text aus der Session gelscht.<br/>
    *
    * @return Code fr Weiterverarbeitung JSP
    */
   public int doEndTag() throws JspException
   {
      final HttpSession session = pageContext.getSession();
      final Integer lastZuordIndexInteger = (Integer) session.getAttribute( cursorKey );
      final String outStr;

      if ( lastZuordIndexInteger != null
         && lastZuordIndexInteger.equals( (Integer) session.getAttribute( SessionKeys.CNC_LIST_ELEMENT_ID_KEY ) ) )
      {
         outStr = yesOut;
      }
      else
      {
         outStr = noOut;
      }

      final JspWriter writer = pageContext.getOut();

      try
      {
         writer.print(outStr);
      }
      catch (IOException e)
      {
         throw new JspException(e.toString());
      }

      return EVAL_PAGE;
   } // end method doEndTag

} // end class MessageTag
