package de.cnc.taglib;

import java.io.IOException;
import java.util.List;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.SessionKeys;
import de.cnc.util.*;

/**
 * Tag zur Ausgabe in Listen. <br/><br/>
 * Die Tags {@link IterateTag} und
 * {@link IterateBrowsedTag}<br/>
 * legen bei jedem Durchlauf
 * die Nummer (index Basis 0) des aktuellen Listenelements auf dem festen SessionKey {@link SessionKeys#CNC_LIST_ELEMENT_ID_KEY} und<br/>
 * das aktuelle Listenelement auf dem festen SessionKey {@link SessionKeys#CNC_LIST_ELEMENT_KEY} ab,<br/>
 * wodurch die laufende Nummer und das aktuelle Element ermittelt werden knnen.<br/>
 * <pre>
 * Verwendung:
 *
 * Ablegen Liste von Datenbank auf festem SessionKey {@link SessionKeys#CNC_LIST_KEY}
 * <b>
 * JSP
 * &lt;cnc:iterate&gt;
 *
 *    &lt;cnc:listwrite property="..."/&gt;
 *
 *    oder
 *
 *    &lt;cnc:listwrite property="..."/&gt;
 *
 * &lt;/cnc:iterate&gt;
 * </b>
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class ListWriteTag extends TagSupport
{

   /**
    * anzuzeigendes Property der Bean auf dem festen SessionKey {@link SessionKeys#CNC_LIST_ELEMENT_KEY}
    */
   protected String property = "";

   /**
    * Lnge des auszugebenden Strings (optional)
    */
   protected String length = "";

   /**
    * Summieren aller Werte dieses Property (optional)
    */
   protected String sum;

   /**
    * get property
    *
    * @return property
    */
   public String getProperty()
   {
      return property;
   }

   /**
    * set property
    *
    * @param property new Value
    */
   public void setProperty(String newProperty)
   {
      this.property = newProperty;
   }

   /**
    * get length
    *
    * @return length
    */
//   public String getLength()
//   {
//      return length;
//   }

   /**
    * set length
    *
    * @param length new Value
    */
   public void setLength(String newLength)
   {
      this.length = newLength;
   }

   /**
    * set sum mode
    *
    * @param length new Value
    */
   public void setSum(String newSum)
   {
      this.sum = newSum;
   }

   /**
    * Erzeugen HTML-Code
    *
    * @return Code fr JSP-Seite weiter abarbeiten
    *
    * @exception JspException if a JSP exception has occurred
    */
   public int doStartTag() throws JspException
   {
      HttpSession session = pageContext.getSession();
      String outString = "";
      if ( StringUtil.str2bool( sum ) )
      {
          // Summier-Modus
          List sumList = (List) session.getAttribute(SessionKeys.CNC_LIST_KEY);
          double dblSum = 0;
          for (int i = 0; i < sumList.size(); i++)
          {
            // Zugriff auf Property
            Object propValue = BeanUtil.get(sumList.get( i ), property);

            dblSum += ( (Number) propValue ).doubleValue() ;
          }
          outString = "" + dblSum;
      }
      else
      {
          Object  bean = session.getAttribute(SessionKeys.CNC_LIST_ELEMENT_KEY);

          if (bean != null)
          {
             if (StringUtil.empty(property))
             {
                // Zugriff auf Objekt ingesamt
                outString = "" + bean;
             }
             else
             {
                // Zugriff auf Property
                Object propValue = BeanUtil.get(bean, property);

                if (propValue instanceof Boolean)
                {
                   //Landes-spezifische Ausgabe, hier nur provisorisch realisisert
                   if (((Boolean) propValue).booleanValue())
                   {
                      outString = "ja";
                   }
                   else
                   {
                      outString = "nein";
                   }
                }
                else if (propValue != null)
                {
                   outString = propValue.toString();
                } // end if
             } // end if
          } // end if

          // auszugebenden String auf gewnschte Lnge abschneiden
          if (!StringUtil.empty(length))
          {
             int iLength = StringUtil.str2int(length);

             if (iLength > 0)
             {
                outString = StringUtil.left(outString, iLength);
             }
          }

      }


      // bei Leerstring ein &nbsp; senden, damit die Tabellenzellen korrekt dargestellt werden, Rnder fehlen sonst
      if (StringUtil.empty(outString))
      {
         outString = "&nbsp;";
      }

      JspWriter writer = pageContext.getOut();

      try
      {
         writer.print(outString);
      }
      catch (IOException e)
      {
         throw new JspException(e.toString());
      }

      return (EVAL_PAGE);
   } // end method doStartTag

   /**
    * nichts zu tun
    *
    * @return Code fr Weiterverarbeitung JSP
    *
    * @throws JspException
    */
   public int doEndTag() throws JspException
   {
      // Log.fatal("CheckLogonTag#doStartTag", Log.DEBUG);
      return (EVAL_PAGE);
   } // end method doStartTag

} // end class ListWriteTag
