package de.cnc.taglib;

import java.io.IOException;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.SessionKeys;

/**
 * Ausgabe Meldung auf JSP-Seite . <br/>
 * Nach der Ausgabe wird der Text aus der Session gelscht.<br/>
 * Verwendung: <b>&lt;cnc:message/&gt;</b><br/><br/>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class MessageTag extends TagSupport
{

   /**
    * Defer our checking until the end of this tag is encountered.
    *
    * @exception JspException if a JSP exception has occurred
    */
   public int doStartTag() throws JspException
   {
      return (EVAL_PAGE);
   } // end method doStartTag

   /**
    * Ausgabe Text . <br/>
    * Nach der Ausgabe wird der Text aus der Session gelscht.<br/>
    *
    * @return Code fr Weiterverarbeitung JSP
    */
   public int doEndTag() throws JspException
   {
      HttpSession session = pageContext.getSession();

      StringBuffer outStrBuff = new StringBuffer();

      //Fehler Meldung
      {
          String sessionErrMsgStr = (String) session.getAttribute(SessionKeys.CNC_ERROR_MESSAGE);
          if (sessionErrMsgStr != null)
          {
             session.removeAttribute(SessionKeys.CNC_ERROR_MESSAGE); // message consumed
             outStrBuff.append("<span id=\"CNC_ERROR_MESSAGE\">" + sessionErrMsgStr + "</span>");
          }
      }

      //Warn Meldung
      {
          String sessionWarnMsgStr = (String) session.getAttribute(SessionKeys.CNC_WARNING_MESSAGE);
          if (sessionWarnMsgStr != null)
          {
             session.removeAttribute(SessionKeys.CNC_WARNING_MESSAGE); // message consumed
             if ( outStrBuff.length() > 0 )
             {
                outStrBuff.append("<br>\n");
             }
             outStrBuff.append("<span id=\"CNC_WARNING_MESSAGE\">" + sessionWarnMsgStr + "</span>");
          }
      }

      //Info Meldung
      {
          String sessionInfoMsgStr = (String) session.getAttribute(SessionKeys.CNC_INFO_MESSAGE);
          if (sessionInfoMsgStr != null)
          {
             session.removeAttribute(SessionKeys.CNC_INFO_MESSAGE); // message consumed
             if ( outStrBuff.length() > 0 )
             {
                outStrBuff.append("<br>\n");
             }
             outStrBuff.append("<span id=\"CNC_INFO_MESSAGE\">" + sessionInfoMsgStr + "</span>");
          }
      }

      if ( outStrBuff.length() < 1 )
      {
         outStrBuff.append("&nbsp;");
      }

      JspWriter writer = pageContext.getOut();

      try
      {
         writer.print(outStrBuff.toString());
      }
      catch (IOException e)
      {
         throw new JspException(e.toString());
      }

      return EVAL_PAGE;
   } // end method doEndTag

} // end class MessageTag
