package de.cnc.taglib;

import java.io.IOException;

import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.SessionKeys;
import de.cnc.util.StringUtil;

/**
 * Setzen des Focus auf ein FORM-Element auf JSP-Seite . <br/>
 * Verwendung: <b>&lt;cnc:setfocus/&gt; nach &lt;/form&gt;</b><br/>
 * Das Attribut <b>elemName</b> ist fr die Angabe des zu fokussierenden Elementes
 * auf der JSP-Seite gedacht. Alternativ ist es mglich ber den 
 * {@link de.cnc.SessionKeys#CNC_FOCUS}-SessionKey den Focus zu setzen.
 * <br/>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class SetFocusTag extends TagSupport
{
    private String elemName;

    /**
     * Setzen des Namens des FORM-Eelementes auf welches
     * der Focus zu setzen ist.
     * 
     * @param string
     */
    public void setElemName(String pStr)
    {
        elemName = pStr;
    }

    /**
     * Defer our checking until the end of this tag is encountered.
     *
     * @exception JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException
    {
        return (EVAL_PAGE);
    } // end method doStartTag

    /**
     * Ausgabe Fehlertext . <br/>
     * Nach der Ausgabe wird der Fehlertext aus der Session gelscht.<br/>
     *
     * @return Code fr Weiterverarbeitung JSP
     */
    public int doEndTag() throws JspException
    {
        HttpSession session = pageContext.getSession();

        String outString;

        String focusElemFromSession = (String) session.getAttribute(SessionKeys.CNC_FOCUS);

        if ( focusElemFromSession != null )
        {
            // ein ber die Session gesetztes zu fokussierendes Form-Feld hat 
            // Vorrang gegenber der Angabe im Tag auf der JSP-Seite
            outString = focusElemFromSession;
        }
        else if (!StringUtil.empty(elemName))
        {
            outString = elemName;
            //message consumed
            session.removeAttribute(SessionKeys.CNC_FOCUS);
        }
        else
        {
            return EVAL_PAGE;
        }

        outString =
            "<script language=\"JavaScript\" type=\"text/javascript\">\n" //--
            +"  <!--\n" //-
            +"    document.forms[0].elements['" + outString + "'].focus();\n" //--
            +"  // -->\n" //--
            +"</script>";

        JspWriter writer = pageContext.getOut();

        try
        {
            writer.print(outString);
        }
        catch (IOException e)
        {
            throw new JspException(e.toString());
        }

        return EVAL_PAGE;
    } // end method doEndTag

}
