package de.cnc.taglib;

import java.io.IOException;
import java.net.URLEncoder;

import javax.servlet.http.*;
import javax.servlet.http.HttpSession;
import javax.servlet.jsp.*;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.*;
import de.cnc.treecontrol.TreeNode;
import de.cnc.util.CncUtil;
import de.cnc.util.StringUtil;

/**
 * Tag fr das Tree-Control. <br/><br/>
 * <pre>
 * Verwendung:
 *
 * <b>
 * JSP
 * &lt;cnc:tree property="..."/&gt;
 *
 * oder
 *
 * &lt;cnc:tree sessionkey="..."/&gt;
 * </b>
 * </pre>
 * siehe Datei <b>cnc.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class TreeTag extends TagSupport
{
    /** 
     * Session-Key des Tree-Control . <br>
     * Default ist {@link SessionKeys#CNC_TREE_KEY}.
     */
    private String sessionkey = SessionKeys.CNC_TREE_KEY;

    private String nameAsLink;
    
    private String gotoProc;
    
    private String target;
    
    
    /**
     * @param pStr
     */
    public void setSessionkey(String pStr)
    {
        sessionkey = pStr;
    }

    /**
     * @param pStr The nameAsLink to set.
     */
    public void setNameAsLink(String pStr)
    {
        this.nameAsLink = pStr;
    }
    /**
     * @param pStr The gotoProc to set.
     */
    public void setGotoProc(String pStr)
    {
        this.gotoProc = pStr;
    }
    /**
     * @param pStr The target to set.
     */
    public void setTarget(String pStr)
    {
        this.target = pStr;
    }
    /**
     * Erzeugen HTML-Code
     *
     * @return Code fr JSP-Seite weiter abarbeiten
     *
     * @exception JspException if a JSP exception has occurred
     */
    public int doStartTag() throws JspException
    {
        HttpSession session = pageContext.getSession();
        StringBuffer outStrBuff = new StringBuffer();

        TreeNode node = (TreeNode) session.getAttribute( sessionkey );

        outStrBuff.append(render(node, "", this.pageContext, StringUtil.str2bool( this.nameAsLink ), this.gotoProc, this.target) );

        JspWriter writer = pageContext.getOut();

        try
        {
            writer.print(outStrBuff);
        }
        catch (IOException e)
        {
            throw new JspException(e.toString());
        }

        return (EVAL_PAGE);
    } // end method doStartTag

    /**
     * nichts zu tun
     *
     * @return Code fr Weiterverarbeitung JSP
     *
     * @throws JspException
     */
    public int doEndTag() throws JspException
    {
        return (EVAL_PAGE);
    }

    private static String render(//--
    final TreeNode pNode, //--
    final String pPathString, //--
    final PageContext pPageContext, //--
    final boolean pNameAsLink, //--
    final String pGotoProc, //--
    final String pTarget)
    {
        StringBuffer outStrBuff = new StringBuffer();

        outStrBuff.append("<table border=\"0\"  cellspacing=\"0\" cellpadding=\"0\">\n<tr>\n");

        //erste Tabellen-Zeile
        //outStrBuff.append("<td colspan=\"0\" align=\"left\">\n");
        outStrBuff.append("<td align=\"left\">\n");
        
//        if (pPathString.length() > 0 )
//        {
//            //nicht die root
//            outStrBuff.append("&nbsp;" );
//        }

        if (pNode.nodeCount() > 0 && !pNode.isExpanded)
        {
            outStrBuff.append("<a href=\"" + "impl_action.cnc"
            // Hinterlegt in web.xml servlet-mapping
            +"?" 
                + URLEncoder.encode( RequestKeys.CNC_IMPL_ACTION_PREFIX
                + "TreeActions#expand" )
                + "=Expand"
                + "&"
                + "elemPath=" // @todo: Konstante
                + pPathString
                + "&"
            // RequestURI (aktueller JSP-Name)
            +RequestKeys.CNC_SRC_JSP
                + "="
                + URLEncoder.encode(
                    CncUtil.getCurrentJspName((HttpServletRequest) pPageContext.getRequest()))
                + "&"

            // Cnc-Status
            +SessionKeys.CNC_STATUS
                + "="
                + URLEncoder.encode(
                    (String) pPageContext.getSession().getAttribute(SessionKeys.CNC_STATUS))
                + "\">+</a>");
        }
        else if (pNode.nodeCount() > 0 && pNode.isExpanded)
        {
            outStrBuff.append("<a href=\"" + "impl_action.cnc"
            // Hinterlegt in web.xml servlet-mapping
            +"?" 
                + URLEncoder.encode( RequestKeys.CNC_IMPL_ACTION_PREFIX
                + "TreeActions#collapse" )
                + "=Collapse"
                + "&"
                + "elemPath=" // @todo: Konstante
                + pPathString
                + "&"
            // RequestURI (aktueller JSP-Name)
            +RequestKeys.CNC_SRC_JSP
                + "="
                + URLEncoder.encode(
                    CncUtil.getCurrentJspName((HttpServletRequest) pPageContext.getRequest()))
                + "&"

            // Cnc-Status
            +SessionKeys.CNC_STATUS
                + "="
                + URLEncoder.encode(
                    (String) pPageContext.getSession().getAttribute(SessionKeys.CNC_STATUS))
                + "\">-</a>");
        }
        else
        {
            outStrBuff.append("+");
        }

        outStrBuff.append("</td>\n");

        outStrBuff.append("<td colspan=\"2\" align=\"left\">\n");

        if ( pNameAsLink )
        {
            outStrBuff.append("&nbsp;<a href=\"cnc_goto_" + pGotoProc + ".cnc"
                    // Hinterlegt in web.xml servlet-mapping
                    +"?" 
                        //+ URLEncoder.encode( RequestKeys.CNC_IMPL_ACTION_PREFIX
                        //+ "TreeActions#collapse" )
                        //+ "=Collapse"
                        //+ "&"
                        + "elemPath=" // @todo: Konstante
                        + pPathString
                        + "&"
                    // RequestURI (aktueller JSP-Name)
                    +RequestKeys.CNC_SRC_JSP
                        + "="
                        + URLEncoder.encode(
                            CncUtil.getCurrentJspName((HttpServletRequest) pPageContext.getRequest()))
                        + "&"

                    // Cnc-Status
                    +SessionKeys.CNC_STATUS
                        + "=cncStatusNoCheck\"" );
            if (pTarget != null)
            {
                outStrBuff.append( " target=\"" + pTarget + "\"" );
            }
            outStrBuff.append( ">"
            + pNode.name + "</a>");
        }
        else
        {
            outStrBuff.append(pNode.name);
        }

        outStrBuff.append("</td>\n");

        if (pNode.nodeCount() > 0 && pNode.isExpanded)
        {
            outStrBuff.append("</tr>\n");

            //zweite Tabellen-Zeile
            outStrBuff.append("<tr>\n");

            outStrBuff.append("<td>\n");

            outStrBuff.append("&nbsp;\n");

//            if (pPathString.length() > 0 )
//            {
//                //nicht die root
//                outStrBuff.append("&nbsp;" );
//            }

            outStrBuff.append("</td>\n");

            outStrBuff.append("<td>\n");

            outStrBuff.append("&nbsp;\n");

            outStrBuff.append("</td>\n");

            outStrBuff.append("<td align=\"left\">\n");

            for (int i = 0; i < pNode.nodeCount(); i++)
            {
                outStrBuff.append(render(pNode.getNode(i), ( pPathString.length() > 0 ? pPathString + "_" : pPathString ) + i, pPageContext, pNameAsLink, pGotoProc, pTarget ) );
            }

            outStrBuff.append("</td>\n");

        }

        outStrBuff.append("</tr>\n</table>\n");

        return outStrBuff.toString();
    }

} // end class ListWriteTag
