package de.cnc.util;

import java.lang.reflect.*;
import java.util.*;

/**
 * Sammlung verschiedener Methoden zum Umgang mit Beans . <br/>
 * Java-Beans sind Klassen mit set-get-Methoden fr die Membervariablen (Properties). <br/>
 * Es gibt Methoden zum Aufruf von Settern und Gettern mit und ohne Index. <br/>
 * Der Name der Property wird als String nicht kapitalisiert (kleiner Anfangsbuchstabe) bergeben. <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public final class BeanUtil
{

//@todo erledigt : _public Members set und get
//@todo: aufruf anderer Methoden wie add

   private static final Object[] EMPTY_OBJ_ARR = new Object[0];

   /**
    * Constructor declaration<br/>
    * Instanziieren nicht erlaubt
    */
   private BeanUtil(){}

   /**
    * Public-Feld setzen oder die entsprechende
    * set-Methode mit einfachem Parameter aufrufen. <br/>
    * setXy( wert-Objekt )
    *
    * @param bean      die Bean
    * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
    * @param value     zu setzender Wert als Objekt, Elementartypen mssen in die entsprechenden Klassen umgewandelt werden (Integer, Double, Boolean usw.)
    */
   public static void set(Object bean, String property, Object value)
   {
      // was ist wenn value == null ( noch unterscheiden: Fehler oder beabsichtigtes Rcksetzen )
      /*
       if ( property.indexOf( "." ) >= 0 ) { // if  ein Punkt enthalten, also Sondermimik fr flexcolumns (DynaBeans)
       // Aufruf fr geholtes Element ohne Index, aber mit flexcolumn-name
       set( bean , StringUtil.prePoint( property ) , StringUtil.postPoint( property ) , value ) ;
       }//end if Bindestrich
       */

      //Versuch ber public-Field
      Field field = null;
      try
      {
         //@todo: field cache aufbauen
         field = bean.getClass().getField( property );
      }
      catch (Exception e)
      {
         //wird nicht behandelt
      }
      if (field != null && Modifier.isPublic( field.getModifiers() ) )
      {
         try
         {
            if ( type2Class( field.getType() ).equals( type2Class( value.getClass() ) ) ) // ob Klasse passt
            {
               field.set( bean , value );
               return; // Auftrag erfllt
            }
            else
            {
               // alternativen setter probieren fr String-Value in typisiertes Property
               try
               {
                  Object obj = convertObjectToClass(value, field.getType() );
                  field.set( bean , obj );
                  return; // Auftrag erfllt
               }
               catch (Exception e)
               {
                  // wird nicht behandelt, ist eben keine Zuweisung des Wertes zum Property mglich
                  Log.fatal(e);
               }
            }
         }
         catch (Exception e)
         {
            Log.fatal(e);
            throw new RuntimeException( "error on set access on field " + field + " with value: " + bean + "(" + (bean != null ? bean.getClass().getName() : "" ) + ")" );
         }
      }
      
      
      
      //Versuch ber set-Methode
      String setProp = "set" + StringUtil.upper1(property);
      try
      {
         Method[] methodArray = bean.getClass().getMethods();

         for (int i = 0; i < methodArray.length; i++)
         {
            if (setProp.equals(methodArray[i].getName()))
            { // Namensgleichheit
               // Prfen ob Paramterliste ok
               Class[] paramClassArray = methodArray[i].getParameterTypes();

               if (paramClassArray.length == 1)
               { // ein Parameter
                  //if (paramClassArray[0].isInstance(value)) // ob Klasse passt
                  if ( type2Class( paramClassArray[0] ).equals( type2Class( value.getClass() ) ) ) // ob Klasse passt
                  {
                     Object[] paramArr = { value };

                     methodArray[i].invoke(bean, paramArr);
                     return; // Auftrag erfllt
                  }
                  else
                  {
                     // alternativen setter probieren fr String-Value in typisiertes Property
                     try
                     {
                        Object obj = convertObjectToClass(value, paramClassArray[0]);
                        Object[] paramArr = { obj };

                        methodArray[i].invoke(bean, paramArr);
                        return; // Auftrag erfllt
                     }
                     catch (Exception e)
                     {
                        // wird nicht behandelt, ist eben keine Zuweisung des Wertes zum Property mglich
                        Log.fatal( "error on invoke " + methodArray[i] + " on object: " + bean + "(" + (bean != null ? bean.getClass().getName() : "" ) + ")" );
                        Log.fatal(e);
                     }
                  }
               }
            }
         }
      }
      catch (Exception e)
      {
         // wird nicht behandelt, ist eben keine Zuweisung des Wertes zum Property mglich
         Log.fatal( "error on invoke " + setProp + " on object: " + bean + "(" + (bean != null ? bean.getClass().getName() : "" ) + ")" );
         Log.fatal(e);
      }

//      Method method = null;
//      try
//      {
//         String methodName = "set" + StringUtil.upper1(property);
//
//         //Achtung: Funktioniert nicht korrekt, wenn es mehrere Methoden mit dem gleichen Namen gibt
//         method = MethodCache.getMethod(bean, methodName);
//
//         Class[] paramClassArray = method.getParameterTypes();
//
//         if (paramClassArray.length == 1)
//         { // ein Parameter
//            if (paramClassArray[0].isInstance(value)) // ob Klasse passt
//            {
//               method.invoke(bean, new Object[] { value } );
//               return; // Auftrag erfllt
//            }
//            else
//            {
//               // alternativen setter probieren fr String
//               try
//               {
//                  Object obj = convertToClass(value, paramClassArray[0]);
//                  Object[] paramArr = { obj };
//
//                  method.invoke(bean, paramArr);
//                  return; // Auftrag erfllt
//               }
//               catch (Exception e)
//               {
//                  // wird nicht behandelt, ist eben keine Zuweisung des Wertes zum Property mglich
//                  Log.fatal(e);
//               }
//            }
//         }
//      }
//      catch (Exception e)
//      {
//         Log.fatal(e);
//         throw new RuntimeException( "error on invoke " + method + " on object: " + bean + " with value null " );
//      }
   } // end method set

   /**
    * Public-Feld abfragen oder die entsprechende
    * get-Methode ohne Parameter aufrufen. <br/>
    * getXy()
    *
    * @param bean      die Bean
    * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
    * @return          abgefragter Wert als Objekt, Elementartypen werden in die entsprechenden Klassen umgewandelt (Integer, Double, Boolean usw.)
    */
   public static Object get(Object bean, String propertyName)
   {
//System.out.println( "propertyName: " + propertyName);
      /*
       if ( property.indexOf( "." ) >= 0 ) { // if  ein Punkt enthalten, also Sondermimik fr flexcolumns (DynaBeans)
       // Aufruf fr geholtes Element ohne Index, aber mit flexcolumn-name
       return get( bean , StringUtil.prePoint( property ) , StringUtil.postPoint( property ) ) ;
       }//end if Bindestrich
       */
      //    try {
      //      String    methodName = "get" + StringUtil.upper1(property);
      //      Method[]  methodArray = bean.getClass().getMethods();
      //
      //      for (int i = 0; i < methodArray.length; i++) {
      //        if (methodName.equals(methodArray[i].getName())) // Namensgleichheit
      //        {
      //          // Prfen ob Paramterliste ok
      //          Class[] paramClassArray = methodArray[i].getParameterTypes();
      //
      //          if (paramClassArray.length == 0)            // kein Parameter
      //          {
      //            return methodArray[i].invoke(bean, null); // Auftrag erfllt
      //          }
      //        }
      //      }
      //    } catch (Exception e) {
      //      Log.fatal(e);
      //    }

      //Versuch ber public-Field
      Field field = null;
      try
      {
         //@todo: field cache aufbauen
         field = bean.getClass().getField( propertyName );
      }
      catch (Exception e)
      {
         //wird nicht behandelt
          //Log.fatal(e);
      }
      if (field != null && Modifier.isPublic( field.getModifiers() ) )
      {
         try
         {
            return field.get( bean );
         }
         catch (Exception e)
         {
            Log.fatal(e);
            throw new RuntimeException( "error on get access on field " + field + " on object: " + bean );
         }
      }
      
      
      
      //Versuch ber get-Methode
      Method method = null;
      try
      {
         String methodName = "get" + StringUtil.upper1(propertyName);

         //Achtung: Funktioniert nicht korrekt, wenn es mehrere Methoden mit dem gleichen Namen gibt
         //method = MethodCache.getMethod(bean, methodName);
         method = bean.getClass().getMethod( methodName , new Class[ 0 ] );

         return method.invoke(bean, EMPTY_OBJ_ARR );
      }
      catch (Exception e)
      {
         Log.fatal(e);
         throw new RuntimeException( "error on invoke " + method + " on object: " + bean + " with value null " );
      }
      //return null;
   } // end method get

   /**
    * set-Methode mit indiziertem Parameter aufrufen. <br/>
    * setXy( int-index , wert-Objekt )
    *
    * @param bean      die Bean
    * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
    * @param index     laufende Nummer
    * @param value     zu setzender Wert als Objekt, Elementartypen mssen in die entsprechenden Klassen umgewandelt werden (Integer, Double, Boolean usw.)
    */
   public static void set(Object bean, String property, int index, Object value)
   {
//      try
//      {
//         String setProp = "set" + StringUtil.upper1(property);
//         Method[] methodArray = bean.getClass().getMethods();
//
//         for (int i = 0; i < methodArray.length; i++)
//         {
//            if (setProp.equals(methodArray[i].getName())) // Namensgleichheit
//            {
//               // Prfen ob Paramterliste ok
//               Class[] paramClassArray = methodArray[i].getParameterTypes();
//
//               if (paramClassArray.length == 2) // zwei Parameter
//               {
//                  if ("int".equals(paramClassArray[0].getName())) // erster Parameter ist Integer
//                  {
//                     if (paramClassArray[1].isInstance(value)) // ob Klasse passt
//                     {
//                        Object[] paramArr = new Object[2];
//
//                        paramArr[0] = new Integer(index);
//                        paramArr[1] = value;
//
//                        methodArray[i].invoke(bean, paramArr);
//
//                        return; // Auftrag erfllt
//                     }
//                     else
//                     {
//                        // alternativen setter probieren fr String
//                        try
//                        {
//                           Object obj = convertToClass(value, paramClassArray[0]);
//                           Object[] paramArr = new Object[2];
//
//                           paramArr[0] = new Integer(index);
//                           paramArr[1] = obj;
//
//                           methodArray[i].invoke(bean, paramArr);
//
//                           return; // Auftrag erfllt
//                        }
//                        catch (Exception e)
//                        {
//                           // wird nicht behandelt, ist eben keine Zuweisung des Wertes zum Property mglich
//                           Log.fatal(e);
//                        }
//                     }
//                  }
//               }
//            }
//         }
//      }
//      catch (Exception e)
//      {
//         Log.fatal(e);
//      }

      Method method = null;
      try
      {
         String methodName = "set" + StringUtil.upper1(property);

         //Achtung: Funktioniert nicht korrekt, wenn es mehrere Methoden mit dem gleichen Namen gibt
         method = MethodCache.getMethod(bean, methodName);

         // Prfen ob Paramterliste ok
         Class[] paramClassArray = method.getParameterTypes();

         if (paramClassArray.length == 2) // zwei Parameter
         {
            if ("int".equals(paramClassArray[0].getName())) // erster Parameter ist Integer
            {
               if (paramClassArray[1].isInstance(value)) // ob Klasse passt
               {
                  Object[] paramArr = new Object[2];

                  paramArr[0] = new Integer(index);
                  paramArr[1] = value;

                  method.invoke(bean, paramArr);

                  return; // Auftrag erfllt
               }
               else
               {
                  // alternativen setter probieren fr String
                  try
                  {
                     Object obj = convertObjectToClass(value, paramClassArray[0]);
                     Object[] paramArr = new Object[2];

                     paramArr[0] = new Integer(index);
                     paramArr[1] = obj;

                     method.invoke(bean, paramArr);

                     return; // Auftrag erfllt
                  }
                  catch (Exception e)
                  {
                     // wird nicht behandelt, ist eben keine Zuweisung des Wertes zum Property mglich
                     Log.fatal(e);
                  }
               }
            }
         }
      }
      catch (Exception e)
      {
         Log.fatal(e);
         throw new RuntimeException( "error on invoke " + method + " on object: " + bean + " with value null " );
      }


   } // end method set

   /**
    * get-Methode mit Index aufrufen. <br/>
    * getXy( int-index )
    *
    * @param bean      die Bean
    * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
    * @param index     laufende Nummer
    * @return          abgefragter Wert als Objekt, Elementartypen werden in die entsprechenden Klassen umgewandelt (Integer, Double, Boolean usw.)
    */
   public static Object get(Object bean, String property, int index)
   {
//      try
//      {
//         String methodName = "get" + StringUtil.upper1(property);
//         Method[] methodArray = bean.getClass().getMethods();
//
//         for (int i = 0; i < methodArray.length; i++)
//         {
//            if (methodName.equals(methodArray[i].getName())) // Namensgleichheit
//            {
//               // Prfen ob Paramterliste ok
//               Class[] paramClassArray = methodArray[i].getParameterTypes();
//
//               if (paramClassArray.length == 1) // ein Parameter
//               {
//                  Object[] paramArr = { new Integer(index)};
//
//                  return methodArray[i].invoke(bean, paramArr); // Auftrag erfllt
//               }
//            }
//         }
//      }
//      catch (Exception e)
//      {
//         Log.fatal(e);
//      }

      Method method = null;
      try
      {
         String methodName = "get" + StringUtil.upper1(property);

         //Achtung: Funktioniert nicht korrekt, wenn es mehrere Methoden mit dem gleichen Namen gibt
         method = bean.getClass().getMethod( methodName, null ); // MethodCache.getMethod( bean , methodName );

         Object[] paramArr = { new Integer(index)};

         return method.invoke(bean, paramArr); // Auftrag erfllt
      }
      catch (Exception e)
      {
         Log.fatal(e);
         throw new RuntimeException( "error on invoke " + method + " on object: " + bean + " with value null " );
      }
      //return null;
   } // end method get

   /**
    * set-Methode mit indiziertem Parameter aufrufen. <br/>
    * setXy( int-index , wert-Objekt )
    *
    * @param bean      die Bean
    * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
    * @param strKey    Name der Spalte fr Flexcolumn (DynaBeans)
    * @param value     zu setzender Wert als Objekt, Elementartypen mssen in die entsprechenden Klassen umgewandelt werden (Integer, Double, Boolean usw.)
    */

   /*
    public static void set(Object bean, String property, String strKey , Object value) {
    //Log.debug( "set mit flex " + property + " " + strKey ) ;
    try
    {
    String    setProp = "set" + StringUtil.upper1(property);
    Method[]  methodArray = bean.getClass().getMethods();
   
    for (int i = 0; i < methodArray.length; i++)
    {
    if (setProp.equals(methodArray[i].getName()))       // Namensgleichheit
    {
    // Prfen ob Paramterliste ok
    Class[] paramClassArray = methodArray[i].getParameterTypes();
   
    if (paramClassArray.length == 2)                  // zwei Parameter
    {
    if (paramClassArray[0].isInstance(value))       // ob Klasse passt
    {
    if (paramClassArray[1].isInstance(value))     // ob Klasse passt
    {
    Object[]  paramArr = new Object[2];
   
    paramArr[0] = strKey ;
    paramArr[1] = value ;
   
    methodArray[i].invoke( bean , paramArr ) ;
   
    //Log.debug( "set erfolgreich" );
    return; // Auftrag erfllt
    }
    }
    }
    }
    }
    }
    catch (Exception e)
    {
    Log.fatal(e);
    }
    }// end method set
    */

   /**
    * get-Methode mit Index aufrufen. <br/>
    * getXy( int-index )
    *
    * @param bean      die Bean
    * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
    * @param strKey    Name der Spalte fr Flexcolumn (DynaBeans)
    * @return          abgefragter Wert als Objekt, Elementartypen werden in die entsprechenden Klassen umgewandelt (Integer, Double, Boolean usw.)
    */

   /*
    public static Object get(Object bean, String property , String strKey )
    {
    //Log.debug( "get mit flex " + property + " " + strKey ) ;
    try
    {
    String    getProp = "get" + StringUtil.upper1(property);
    Method[]  methodArray = bean.getClass().getMethods();
   
    for (int i = 0; i < methodArray.length; i++)
    {
    if (getProp.equals(methodArray[i].getName()))     // Namensgleichheit
    {
    // Prfen ob Paramterliste ok
    Class[] paramClassArray = methodArray[i].getParameterTypes();
   
    if (paramClassArray.length == 1)                // ein Parameter
    {
    Object[]  paramArr = { strKey };
   
    //Log.debug( "get erfolgreich" );
    return methodArray[i].invoke(bean, paramArr); // Auftrag erfllt
    }
    }
    }
    }
    catch (Exception e)
    {
    Log.fatal(e);
    }
   
    return null;
    } // end method get
    */

   /**
    * set-Methode mit indiziertem Parameter als "parameter[inx]" (z.B. name[3]) aufrufen. <br/>
    * die Methode setIndexed stellt automatisch fest, ob ein Index im Property-Namen enthalten ist. <br/>
    * Der Aufruf ohne Index ([nr]) ist also auch mglich. <br/>
    * Untersttzt in der neuen Version auch List of Beans. <br/>
    * Fr flexmodule (DynaBeans) einfhrung eines zustzlichen Column-Parameters (String) . <br/>
    *
    * @param bean      die Bean
    * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
    * @param value     zu setzender Wert als Objekt, Elementartypen mssen in die entsprechenden Klassen umgewandelt werden (Integer, Double, Boolean usw.)
    */
   public static void setIndexed(Object bean, String property, Object value)
   {
      // Log.debug( "setIndexed: " + property ) ;
      try
      {
         if (property.indexOf('[') > -1 && property.indexOf(']') > -1)
         {
            String propStr = property.substring(0, property.indexOf('['));
            String nrStr = property.substring(property.indexOf('[') + 1);

            nrStr = nrStr.substring(0, nrStr.indexOf(']'));

            int index = (new Integer(nrStr)).intValue();

            if (bean instanceof List) // List wird extra behandelt
            {
               // Element aus List holen
               Object elementBean = ((List) bean).get(index);

               // Aufruf fr geholtes Element ohne Index
               set(elementBean, propStr, value);
            }
            else
            {
               // keine List, Aufruf mit Index
               set(bean, propStr, index, value);
            }
         }
         else
         {
            set(bean, property, value);
         }
      }
      catch (Exception e)
      {
         Log.fatal(e);
      }
   } // end method setIndexed

   /**
    * get-Methode mit indiziertem Parameter als "parameter[inx]" (z.B. name[3]) aufrufen. <br/>
    * die Methode getIndexed stellt automatisch fest, ob ein Index im Property-Namen enthalten ist. <br/>
    * Der Aufruf ohne Index ([nr]) ist also auch mglich. <br/>
    * Untersttzt in der neuen Version auch List of Beans. <br/>
    * Fr flexmodule einfhrung eines zustzlichen Column-Parameters (String) . <br/>
    *
    * @param bean      die Bean
    * @param property  Name der Property als String nicht kapitalisiert (kleiner Anfangsbuchstabe)
    * @return          abgefragter Wert als Objekt, Elementartypen werden in die entsprechenden Klassen umgewandelt (Integer, Double, Boolean usw.)
    */
   public static Object getIndexed(Object bean, String property)
   {
      // Log.debug( "getIndexed: " + property ) ;
      try
      {
         if (property.indexOf('[') > -1 && property.indexOf(']') > -1)
         {
            String propStr = property.substring(0, property.indexOf('['));
            String nrStr = property.substring(property.indexOf('[') + 1);

            nrStr = nrStr.substring(0, nrStr.indexOf(']'));

            int index = (new Integer(nrStr)).intValue();

            if (bean instanceof List) // List wird extra behandelt
            {
               // Element aus List holen
               Object elementBean = ((List) bean).get(index);

               // Aufruf fr geholtes Element ohne Index
               return get(elementBean, propStr);

            }
            else
            {
               // keine List, Aufruf mit Index
               return get(bean, propStr, index);
            }
         }
         else
         {
            return get(bean, property);
         }
      }
      catch (Exception e)
      {
         Log.fatal(e);
      }

      return null;
   } // end method getIndexed

   /**
    * Zurckgeben Methode fr Parameter-Bean entsprechend bergebenem Namen und Parameterliste
    *
    * @param bean            die Bean
    * @param methodName      Name der Methode
    * @param paramClassArray Array mit den Klassen (Class-Objekte) der Parameter
    * @return die Methode oder null, wenn keine passende Methode gefunden
    */
   public static Method getMethod(Object bean, String methodName, Class[] paramClassArray) 
       //throws Exception
   {
      Class beanClass = bean.getClass();
      return getMethod( beanClass, methodName, paramClassArray );
   }

   /**
    * Zurckgeben Methode fr Parameter-Bean entsprechend bergebenem Namen und Parameterliste
    *
    * @param bean            die Bean
    * @param methodName      Name der Methode
    * @param paramClassArray Array mit den Klassen (Class-Objekte) der Parameter
    * @return die Methode oder null, wenn keine passende Methode gefunden
    */
   public static Method getMethod(Class beanClass, String methodName, Class[] paramClassArray)
   {
          Method[] methodArray = beanClass.getMethods();
    
          for (int i = 0; i < methodArray.length; i++)
          {
             if (methodName.equals(methodArray[i].getName())) // Namensgleichheit
             {
                // Prfen ob Paramterliste ok
                int paramAnzahl = 0;
    
                if (paramClassArray != null)
                {
                   paramAnzahl = paramClassArray.length;
                }
    
                boolean paramOk = true;
                Class[] methodParamClassArray = typeArr2ClassArr( methodArray[i].getParameterTypes() );
    
                if (methodParamClassArray.length != paramAnzahl)
                {
                    
                   paramOk = false;
                   continue;
                }
    
                if (paramOk)
                {
                   for (int j = 0; j < paramAnzahl; j++)
                   {
    //                  if (!typeName2ClassName(paramClassArray[j].getName())
    //                     .equals(typeName2ClassName(methodParamClassArray[j].getName())))
                      // hier knnte man noch auf Zuweisungkompatibilitt prfen isInstance(value)
                      // funktioniert nicht: if ( ! type2Class( methodParamClassArray[j] ).isInstance( type2Class( paramClassArray[j] ) ) )
    
    //System.out.println( "methodParamClassArray[j]: " + methodParamClassArray[j] );
    //System.out.println( "paramClassArray[j]: " + paramClassArray[j] );
                       
                       if ( paramClassArray[j] != null
                               && ( ! methodParamClassArray[j].isAssignableFrom( paramClassArray[j] ) ) )
                      {
                         paramOk = false;
    
                         break;
                      }
                   }
                }
    
                if (paramOk)
                {
                   return methodArray[i]; // Auftrag erfllt
                }
             }
          }
          return null;
   } // end method getMethod

   /**
    * Zurckgeben Wrapper-Klasse fr Elementar-Typ fr das Package java.lang.* . <br/>
    * Verwendet in {@link #getMethod}.<br/>
    *
    * @param classPa Elementartyp oder andere Klasse
    * @return Wrapper-Klasse des Elementar-Types oder Paramter classPa, wenn kein Elementar-Typ
    */
   private static Class type2Class(Class classPa)
   {
      // auf HashTable umstellen
      if ( ! classPa.isPrimitive() )
      {
         //kein Primitivtyp
         return classPa;
      }

      String typeName = classPa.getName();

      // mal direkt auf class prfen
      if ("int".equals(typeName))
      {
         return java.lang.Integer.class;
      }
      else if ("long".equals(typeName))
      {
         return java.lang.Long.class;
      }
      else if ("double".equals(typeName))
      {
         return java.lang.Double.class;
      }
      else if ("float".equals(typeName))
      {
         return java.lang.Float.class;
      }
      else if ("byte".equals(typeName))
      {
         return java.lang.Byte.class;
      }
      else if ("char".equals(typeName))
      {
         return java.lang.Character.class;
      }
      else if ("short".equals(typeName))
      {
         return java.lang.Short.class;
      }
      else if ("boolean".equals(typeName))
      {
         return java.lang.Boolean.class;
      }

      return classPa;
   } // end method type2Class

   /**
    * @param parameterTypes
    * @return
    */
   private static Class[] typeArr2ClassArr(Class[] parameterTypes)
   {
       if (parameterTypes == null)
       {
           return null;
       }
       Class[] retClassArr = new Class[ parameterTypes.length ];
       for (int i = 0; i < retClassArr.length; i++)
       {
           retClassArr[i] = type2Class( parameterTypes[i] );
       }
       return retClassArr;
   }

   /**
    * Zurckgeben qualifizierter Name fr einfachen Klassen-Namen fr das Package java.lang.* . <br/>
    * Verwendet in {@link #getMethod}.<br/>
    *
    * @param typeName nicht qualifizierter Name (Klasse ohne Package)
    * @return qualifizierte Name (Package.Klasse)
    */
   private static String typeName2ClassName(String typeName)
   {
      // auf HashTable umstellen
      if ("int".equals(typeName))
      {
         return "java.lang.Integer";
      }
      else if ("long".equals(typeName))
      {
         return "java.lang.Long";
      }
      else if ("double".equals(typeName))
      {
         return "java.lang.Double";
      }
      else if ("float".equals(typeName))
      {
         return "java.lang.Float";
      }
      else if ("byte".equals(typeName))
      {
         return "java.lang.Byte";
      }
      else if ("char".equals(typeName))
      {
         return "java.lang.Character";
      }
      else if ("short".equals(typeName))
      {
         return "java.lang.Short";
      }
      else if ("boolean".equals(typeName))
      {
         return "java.lang.Boolean";
      }

      return typeName;
   } // end method typeName2ClassName

   /**
    * Zurckgeben Get-Methode fr Parameter-Bean entsprechend bergebenem Property-Namen und Parameterliste
    *
    * @param bean            die Bean
    * @param property        Name der Property (Methoden-Name: getProperty)
    * @param paramClassArray Array mit den Klassen (Class-Objekte) der Parameter
    * @return die Methode oder null, wenn keine passende Methode gefunden
    */
   public static Method getGetMethod(Object bean, String property, Class[] paramClassArray) throws Exception
   {
      String getProp = "get" + StringUtil.upper1(property);

      return getMethod(bean, getProp, paramClassArray);
   } // end method getGetMethod

   /**
    * Konvertieren Parameter in gewnschte Zielklasse,
    * weil vom Browser nur Strings geliefert werden.
    *
    * @param objPa zu konvertierendes Objekt
    * @param claPa Zielklasse
    *
    * @return konvertiertes Objekt
    */
   public static Object convertObjectToClass(Object pObj, Class pClass)
   {
      //Log.debug( "convertToClass" , "" + pObj + " " + ( pObj != null ? pObj.getClass().getName() : "" ) + " -> " + pClass );
      
      //@todo: super schlechte Lsung, die Konvertierung vom String aus dem HTML-Request in die korrekte Klasse
      //hat hier eigentlich nichts zu suchen
      //ausserdem fehlt die Internationalisierung
      if ( pObj instanceof String)
      {
          if (pClass == Boolean.class || pClass == boolean.class)
          {
             //return StringUtil.str2boolObject((String) pObj);
             return StringUtil.str2boolObject("" + pObj);
          }
          //@todo: Datum lokalisiert
          if (pClass == Date.class)
          {
              return StringUtil.str2date("" + pObj);
          }
      }

      try
      {
         Class paramClass = type2Class(pClass);
         Class[] argsClassArray = { pObj.getClass()};
         Constructor constr = paramClass.getConstructor(argsClassArray);
         Object[] argsArray = { pObj };
         Object obj = constr.newInstance(argsArray);

         return obj;
      }
      catch (Exception e)
      {
          Log.fatal(e);
         return null;
      }
   } // end method

   /**
    * Aus dem bergebenen Object-Array mit Werten
    * ein Array mit den jeweiligen Klassen
    * aufbauen.
    * 
    * @param pParamValueArr
    * @return
    */
   public static Class[] createParamClassArr(Object[] pParamValueArr)
   {
       Class[] paramClassArr = new Class[ pParamValueArr.length ];
        for (int i = 0; i < paramClassArr.length; i++)
        {
           if (pParamValueArr[ i ] != null)
           {
              paramClassArr[ i ] = pParamValueArr[ i ].getClass();
           }
        }
       return paramClassArr;
   }

   /**
    * Erzeugen eines Strings aus dem
    * bergebenen Klassen-Array fr
    * eine Fehlermeldung
    * 'Mezthode nicht gefunden':
    * 
    * @param pParamClassArr
    * @return
    */
   public static String createParamClassStr(Class[] pParamClassArr)
   {
       final StringBuffer prmClassStrBuff = new StringBuffer();
       for (int i = 0; i < pParamClassArr.length; i++)
       {
          prmClassStrBuff.append( pParamClassArr[ i ].getName() );
          if ( i < pParamClassArr.length - 1 )
          {
              prmClassStrBuff.append( ", " );
          }
       }
       return prmClassStrBuff.toString();
   }

} // end class BeanUtil
