package de.cnc.util;


/**
 * Methoden zum Erzeugen von Objekten ber den ClassLoader 
 * und weitere Hilfsmethoden fr Reflection . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public final class ClassBuilder
{
  /**
   * Constructor declaration<br/>
   * Instanziieren nicht erlaubt
   */
  private ClassBuilder()
  {
    ;
  } // end constructor

  /**
   * Erzeugen neues Objekt der Parameter-Klasse ohne Abfangen von Exceptions
   *
   * @param c Klasse des gewnschten Objekts
   * @return  neues Objekt
   *
   * @throws Exception weiterwerfen der Exceptions vom ClassLoader
   */
  public static Object build(Class c)
  {
    try
   {
      return c.newInstance();
   }
   catch (Exception e)
   {
      throw new RuntimeException( e.getMessage() );
   }
  }

  /**
   * Erzeugen neues Objekt der Parameter-Klasse mit Abfangen von Exceptions . <br/>
   * Aufgetretene Exceptions werden in das Logging des CnC eingespeist. <br/>
   *
   * @param c Klasse des gewnschten Objekts
   * @return neues Objekt oder null, wenn Fehler
   */
  public static Object buildCatched(Class c)
  {
    Object  retObject = null;

    try
    {
      retObject = build(c);
    }
    catch (Exception e)
    {
      Log.fatal(e);
    }
    return retObject;
  }

  /**
   * Erzeugen neues Objekt anhand des als String bergebenen Klassen-Namens ohne Abfangen von Exceptions
   *
   * @param classname Klassen-Name (voll qualifiziert Package.Class) des gewnschten Objekts
   * @return  neues Objekt
   *
   * @throws Exception weiterwerfen der Exceptions vom ClassLoader
   */
  public static Object build(String classname)
    throws Exception
  {
    return Class.forName(classname).newInstance();
  }

  /**
   * Erzeugen neues Objekt anhand des als String bergebenen Klassen-Namens fr die Verwendung in {@link de.cnc.server.PersistensHelper PersistensHelper} . <br/>
   * Speziell angepasst auf JDBC-ResultSet.<br/>
   * ohne Abfangen von Exceptions.<br/>
   *
   * @param classname Klassen-Name qualifiziert (Package.Class) oder nicht qualifiziert (nur Class)
   * @return Objekt der gewnschten Klasse
   *
   * @throws Exception weiterwerfen der Exceptions vom ClassLoader
   */
  public static Object buildSqlDefaultValue(String classname)
    throws Exception
  {
    if ("Boolean".equals(classname) || "java.lang.Boolean".equals(classname) || "boolean".equals(classname))
    {
      return Boolean.FALSE;
    }
    else if ("Integer".equals(classname) || "java.lang.Integer".equals(classname) || "int".equals(classname))
    {
      return new Integer(0);
    }
    else if ("Long".equals(classname) || "java.lang.Long".equals(classname) || "long".equals(classname))
    {
      return new Long(0);
    }
    else if ("Double".equals(classname) || "java.lang.Double".equals(classname) || "double".equals(classname))
    {
      return new Double(0);
    }
    else if ("String".equals(classname) || "java.lang.String".equals(classname))
    {
      return "";
    }
    else if ("Date".equals(classname) || "java.sql.Date".equals(classname))
    {
      return new java.sql.Date(0);
    }
    else if ("java.util.Date".equals(classname))
    {
      return new java.util.Date(0);
    }
    else if ("Timestamp".equals(classname))
    {
      return new java.sql.Timestamp(0);
    }

    return Class.forName(classname).newInstance();
  }

}//end class ClassBuilder
