package de.cnc.util;

import java.io.*;
import java.math.BigDecimal;
import java.util.*;

import javax.servlet.http.*;

import de.cnc.*;
import de.cnc.implicit.MarkableInterface;

/**
 * Sammlung statischer Methoden f&uuml;r Operationen auf Session und Request . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner K&uuml;cker</a>
 * <br/>
 *
 * @author Heiner K&uuml;cker
 * @version $Id: $
 */
public final class CncUtil
{

    /**
     * Constructor declaration<br/>
     * Instanziieren nicht erlaubt
     */
    private CncUtil()
    {
        ;
    }

    /**
     * Parsen des Requests und Setzen der entsprechenden Bean-Properties
     *
     * @param request Request vom Servlet/JSP
     * @param beanObject in diese Bean werden die Daten aus dem Request eingeparst
     */
    public static void parseAndSet(
        HttpServletRequest request,
        Object beanObject)
    {
        // throws ServletException {
        Enumeration en = request.getParameterNames();

        while (en.hasMoreElements())
        {
            String s = (String) en.nextElement();
            Object o = request.getParameter(s);

            BeanUtil.setIndexed(beanObject, s, o);

            // Log.debug("parseAndSet " + s);
        }
    } // end method parseAndSet

    /**
     * Einlesen implziter Session-Values,
     * das heisst Resquest-Keys mit dem Prefix RequestKeys.CNC_SESSION_VALUE_PREFIX.
     * Der Prefix wird abgeschnitten und der Value unter dem Rest-Key in der Session gespeichert.
     * Das cnc:input-Tag setzt den Prefix bei gesetztem Attribut sessionvalue="true"
     * vor den Request-Key.
     *
     * !!! Noch Lsung fr Checkboxen machen !!!
     *
     * @param request Request vom Servlet/JSP
     */
    public static void parseAndSetImplSessionValues(HttpServletRequest request)
    {
        // throws ServletException {
        HttpSession session = request.getSession(true);

        //Einlesen der Werte vom Request
        Enumeration en = request.getParameterNames();
        while (en.hasMoreElements())
        {
            String s = (String) en.nextElement();
            if (s.startsWith(RequestKeys.CNC_SESSION_VALUE_PREFIX))
            {
                Object o = request.getParameter(s);
                setAttribute(
                    session,
                    StringUtil.cutLeft(s, RequestKeys.CNC_SESSION_VALUE_PREFIX),
                    o);
            }

            // Log.debug("parseAndSetImplSessionValues " + s);
        }

        // Abarbeiten eventueller Maskierungen, Formatierungen
        en = request.getParameterNames();
        while (en.hasMoreElements())
        {
            String reqKey = (String) en.nextElement();
            if (reqKey.startsWith(RequestKeys.CNC_SESSION_VALUEMASK_PREFIX))
            {
                String maskStr = request.getParameter(reqKey);

                String sessionKey =
                    StringUtil.cutLeft(
                        reqKey,
                        RequestKeys.CNC_SESSION_VALUEMASK_PREFIX);
                Object umaskedObj = session.getAttribute(sessionKey);
                Object maskedObj = StringUtil.mask(umaskedObj, maskStr);
                CncUtil.setAttribute(session, sessionKey, maskedObj);
            }

            // Log.debug("parseAndSetImplSessionValues " + s);
        }
    } // end method parseAndSetImplSessionValues

    /**
     * Suche nach einem Parameternamen, der mit dem Such-String beginnt. <br/>
     * Wird bentigt f&uuml;r Gif-Buttons, da dort statt dem gew&uuml;nschten value die Pixelposition beim Klick auf das Gif geliefert wird. <br/>
     * HTML: <b>&lt;input type="image" name="name" value="value"&gt;</b><br>
     * Request: <b>value.x=3</b><br/>
     * Request: <b>value.y=4</b><br/>
     *
     * @param request Request vom Servlet/JSP
     * @param searchValue
     * @return entsprechender Name oder null wenn nichts gefunden
     */
    public static String scanForNameStartsWith(//--
    HttpServletRequest request, //--
    String searchValue)
    {
        String strReturn = null;
        Enumeration en = request.getParameterNames();

        while (en.hasMoreElements())
        {
            String theName = (String) en.nextElement();

            if (theName.startsWith(searchValue))
            {
                strReturn = theName;
                break;
            }
        }

        return strReturn;
    } // end method scanForNameStartsWith

    /**
     * Suche nach einem Parameternamen, der mit dem Such-String beginnt. <br/>
     * Wird bentigt f&uuml;r Gif-Buttons, da dort statt dem gew&uuml;nschten value die Pixelposition beim Klick auf das Gif geliefert wird. <br/>
     * HTML: <b>&lt;input type="image" name="name" value="value"&gt;</b><br>
     * Request: <b>value.x=3</b><br/>
     * Request: <b>value.y=4</b><br/>
     *
     * @param request Request vom Servlet/JSP
     * @param searchValue
     * @return entsprechender Name oder null wenn nichts gefunden
     */
    public static String scanForNameContents(
        HttpServletRequest request,
        String searchValue)
    {
        String strReturn = null;
        Enumeration en = request.getParameterNames();

        while (en.hasMoreElements())
        {
            String theName = (String) en.nextElement();

            if (theName.indexOf(searchValue) > -1)
            {
                strReturn = theName;
                break;
            }
        }

        return strReturn;
    } // end method scanForNameStartsWith

    /**
     * Ermitteln des gewhlten Listen-Elements anhand des festen Parameters {@link RequestKeys#CNC_LIST_ITEM_ID}
     *
     * @param request Request vom Servlet/JSP ServletRequest
     * @return Nummer des Listenelements (Basis 0)
     */
    public static int getListItemId(HttpServletRequest request)
    {
        String strItemId =
            (String) request.getParameter(RequestKeys.CNC_LIST_ITEM_ID);

        return StringUtil.str2int(strItemId); // String -> int
    } // end method

    /**
     * Ermitteln des aktuell durchlaufenen Listen-Elements im Iterate-tag anhand des festen Session-Keys {@link RequestKeys#CNC_LIST_ITEM_ID}
     *
     * @param request Request vom Servlet/JSP ServletRequest
     * @return Nummer des Listenelements (Basis 0)
     */
    public static int getIterateNumber(HttpServletRequest request) // throws ServletException
    {
        HttpSession session = request.getSession(true);

        // Anhngen Element-Nummer an Parameter-Name
        // deshalb in Action scannen mit CncUtil#scanForNameStartsWith
        Integer indexInteger =
            (Integer) session.getAttribute(SessionKeys.CNC_LIST_ELEMENT_ID_KEY);

        if (indexInteger == null)
        {
            throw new RuntimeException(
                "IterateNumber on SessionKey "
                    + SessionKeys.CNC_LIST_ELEMENT_ID_KEY
                    + " not found");
        } // end if
        return indexInteger.intValue();

        // String  strItemId = (String) request.getParameter(RequestKeys.CNC_LIST_ELEMENT_ID_KEY ) ;
        // return StringUtil.str2int(strItemId); // String -> int
    } // end method

    /**
     * Entnehmen eines Listenelements aus der aktuellen Liste (fester SessionKey {@link SessionKeys#CNC_LIST_KEY})
     * und Ablegen des Elements auf dem festen SessionKey {@link SessionKeys#CNC_FORM_KEY SessionKeys#CNC_FORM_KEY}
     * anhand des festen Request-Parameters {@link RequestKeys#CNC_LIST_ITEM_ID}.
     *
     * @param request Request vom Servlet/JSP ServletRequest
     */
    public static void setListElementToForm(HttpServletRequest request)
    {
        // HttpSession session = request.getSession(true);

        // Abfragen, welches Element zu editieren ist
        // Log.dump(request);
        int itemId = getListItemId(request);

        // zu editierendes Element in Session legen
        setListElementToForm(request, itemId);
    } // end method

    /**
     * Entnehmen eines Listenelements aus der aktuellen Liste (fester SessionKey {@link SessionKeys#CNC_LIST_KEY SessionKeys#CNC_LIST_KEY})
     * und Ablegen des Elements auf dem festen SessionKey {@link SessionKeys#CNC_FORM_KEY}
     * anhand des int-Parameters.
     *
     * @param request Request vom Servlet/JSP ServletRequest
     * @param elementNr Nummer des Listenelements (Basis 0)
     */
    public static void setListElementToForm(
        HttpServletRequest request,
        int elementNr)
    {
        HttpSession session = request.getSession(true);

        // zu editierendes Element in Session legen
        ArrayList sessionArrayList =
            (ArrayList) session.getAttribute(SessionKeys.CNC_LIST_KEY);
        // fester Sessionkey
        Object formObject = sessionArrayList.get(elementNr);

        CncUtil.setAttribute(session, SessionKeys.CNC_FORM_KEY, formObject);
    } // end method setListElementToForm

    /**
     * Anf&uuml;gen der aktuellen Form vom festen SessionKey {@link SessionKeys#CNC_FORM_KEY}
     * an die aktuelle Liste auf dem festen SessionKey {@link SessionKeys#CNC_LIST_KEY}
     *
     * @param request Request vom Servlet/JSP ServletRequest
     */
    public static void addFormToList(HttpServletRequest request)
    {
        HttpSession session = request.getSession(true);

        Object formObject = session.getAttribute(SessionKeys.CNC_FORM_KEY);

        if (formObject != null)
        {
            ArrayList sessionArrayList =
                (ArrayList) session.getAttribute(SessionKeys.CNC_LIST_KEY);

            if (sessionArrayList != null)
            {
                sessionArrayList.add(formObject);
            } // end if
        } // end if
    } // end method addFormToList

    /**
     * Hinzuf&uuml;gen eines Fehlertextes zu den evtl. schon gesetzten Fehlertexten.<br/>
     * Die Ausgabe auf der JSP-Seite erfolgt mit dem <b>&lt;cnc:message/&gt;</b>-Tag
     * {@link de.cnc.taglib.MessageTag}<br/>
     *
     * @param request Request vom Servlet/JSP
     * @param errStr Fehlertext eine Zeile
     */
    public static void setInfoMessage(
        HttpServletRequest request,
        String errStr)
    {
        setInfoMessage(request.getSession(true), errStr);
    } // end method setMessage

    /**
     * Hinzuf&uuml;gen eines Fehlertextes zu den evtl. schon gesetzten Fehlertexten.<br/>
     * Die Ausgabe auf der JSP-Seite erfolgt mit dem <b>&lt;cnc:message/&gt;</b>-Tag
     * {@link de.cnc.taglib.MessageTag}<br/>
     *
     * @param session ServletSession
     * @param errStr Fehlertext eine Zeile
     */
    public static void setInfoMessage(HttpSession session, String errStr)
    {
        Log.debug("CncUtil.setMessage", errStr);
        String altErrStr =
            (String) session.getAttribute(SessionKeys.CNC_INFO_MESSAGE);

        if (altErrStr != null)
        {
            // neue Zeile fuer naechste Fehlermeldung
            errStr = altErrStr + "<br/>" + errStr;
        }
        CncUtil.setAttribute(session, SessionKeys.CNC_INFO_MESSAGE, errStr);
    } // end method setMessage

    /**
     * Hinzuf&uuml;gen eines Fehlertextes zu den evtl. schon gesetzten Fehlertexten.<br/>
     * Die Ausgabe auf der JSP-Seite erfolgt mit dem <b>&lt;cnc:message/&gt;</b>-Tag
     * {@link de.cnc.taglib.MessageTag}<br/>
     *
     * @param request Request vom Servlet/JSP
     * @param errStr Fehlertext eine Zeile
     */
    public static void setErrMessage(HttpServletRequest request, String errStr)
    {
        setErrMessage(request.getSession(), errStr);
    } // end method setMessage

    /**
     * Hinzuf&uuml;gen eines Fehlertextes zu den evtl. schon gesetzten Fehlertexten.<br/>
     * Die Ausgabe auf der JSP-Seite erfolgt mit dem <b>&lt;cnc:message/&gt;</b>-Tag
     * {@link de.cnc.taglib.MessageTag}<br/>
     *
     * @param session ServletSession
     * @param errStr Fehlertext eine Zeile
     */
    public static void setErrMessage(HttpSession session, String errStr)
    {
        Log.debug("CncUtil.setErrMessage", errStr);
        String altErrStr =
            (String) session.getAttribute(SessionKeys.CNC_ERROR_MESSAGE);

        if (altErrStr != null)
        {
            // neue Zeile fuer naechste Fehlermeldung
            errStr = altErrStr + "<br/>" + errStr;
        }
        CncUtil.setAttribute(session, SessionKeys.CNC_ERROR_MESSAGE, errStr);
    } // end method setMessage

    /**
     * Hinzuf&uuml;gen eines Fehlertextes zu den evtl. schon gesetzten Fehlertexten.<br/>
     * Die Ausgabe auf der JSP-Seite erfolgt mit dem <b>&lt;cnc:message/&gt;</b>-Tag
     * {@link de.cnc.taglib.MessageTag}<br/>
     *
     * @param request Request vom Servlet/JSP
     * @param errStr Fehlertext eine Zeile
     */
    public static void setWarnMessage(
        HttpServletRequest request,
        String errStr)
    {
        setWarnMessage(request.getSession(), errStr);
    } // end method setMessage

    /**
     * Hinzuf&uuml;gen eines Fehlertextes zu den evtl. schon gesetzten Fehlertexten.<br/>
     * Die Ausgabe auf der JSP-Seite erfolgt mit dem <b>&lt;cnc:message/&gt;</b>-Tag
     * {@link de.cnc.taglib.MessageTag}<br/>
     *
     * @param session ServletSession
     * @param errStr Fehlertext eine Zeile
     */
    public static void setWarnMessage(HttpSession session, String errStr)
    {
        Log.debug("CncUtil.setWarnMessage", errStr);
        String altErrStr =
            (String) session.getAttribute(SessionKeys.CNC_ERROR_MESSAGE);

        if (altErrStr != null)
        {
            // neue Zeile fuer naechste Fehlermeldung
            errStr = altErrStr + "<br/>" + errStr;
        }
        CncUtil.setAttribute(session, SessionKeys.CNC_WARNING_MESSAGE, errStr);
    } // end method

    /**
     * Erzeugt eine Bean mit dem paramterlosen Konstruktor und
     * legt diese auf dem festen SessionKey {@link de.cnc.SessionKeys#CNC_FORM_KEY} ab.
     *
     * @param session ServletSession
     * @param beanClass gew&uuml;nschte Klasse
     */
    public static final void resetForm(HttpSession session, Class beanClass)
    {
        Object beanObject = ClassBuilder.buildCatched(beanClass);

        CncUtil.setAttribute(session, SessionKeys.CNC_FORM_KEY, beanObject);
    } // end method resetForm

    /**
     * Setzt Objekt auf festen SessionKey {@link de.cnc.SessionKeys#CNC_FORM_KEY}
     *
     * @param session ServletSession
     * @param formObject zu setzendes Objekt (Bean)
     */
    public static final void setForm(HttpSession session, Object formObject)
    {
        CncUtil.setAttribute(session, SessionKeys.CNC_FORM_KEY, formObject);
    } // end method setForm

    /**
     * Holt eine Bean vom festen SessionKey {@link de.cnc.SessionKeys#CNC_FORM_KEY}
     * und pr&uuml;ft die Klasse der Bean. <br/>
     * Wenn die Klasse nicht mit der gew&uuml;nschten Klasse &uuml;bereinstimmt, wird null zur&uuml;ckgegeben. <br/>
     * Verhindert so ClassCastExceptions.
     *
     * @param session ServletSession
     * @param beanClass
     * @return die Bean von CNC_FORM_KEY
     */
    public static final Object getForm(HttpSession session, Class beanClass)
    {
        return getAttribute(session, SessionKeys.CNC_FORM_KEY, beanClass);
    } // end method getForm

    /**
     * Holt eine Bean vom festen SessionKey {@link de.cnc.SessionKeys#CNC_FORM_KEY}
     * und pr&uuml;ft die Klasse der Bean. <br/>
     * Wenn die Klasse nicht mit der gew&uuml;nschten Klasse &uuml;bereinstimmt, wird null zur&uuml;ckgegeben. <br/>
     * Verhindert so ClassCastExceptions.
     *
     * @param request HttpServletRequest
     * @param beanClass
     * @return die Bean von CNC_FORM_KEY
     */
    public static final Object getForm(
        HttpServletRequest request,
        Class beanClass)
    {
        HttpSession session = request.getSession(true);

        return getForm(session, beanClass);
    } // end method getForm

    /**
     * Holt eine Bean vom als Parameter &uuml;bergebenen SessionKey
     * und pr&uuml;ft die Klasse der Bean. <br/>
     * Wenn die Klasse nicht mit der gew&uuml;nschten Klasse &uuml;bereinstimmt, wird null zur&uuml;ckgegeben. <br/>
     * Verhindert so ClassCastExceptions.
     *
     * @param session ServletSession
     * @param sessionKey abgefragter Session-Key
     * @param beanClass gew&uuml;nschte Klasse
     * @return Session-Object oder null, wenn nicht vorhanden oder falsche Klasse
     */
    public static final Object getAttribute(
        HttpSession session,
        String sessionKey,
        Class beanClass)
    {
        Object retObject = session.getAttribute(sessionKey);

        if (retObject != null)
        {
            if (!(beanClass.isInstance(retObject)))
            {
                setErrMessage(
                    session,
                    "Programmfehler: "
                        + sessionKey
                        + " ist nicht Klasse "
                        + beanClass);
                setErrMessage(
                    session,
                    "sondern " + retObject.getClass().getName());
                setErrMessage(
                    session,
                    "\n"
                        + getMessageAndStackTrace(
                            new Exception("getAttribute wrong class")));

                retObject = null;
            }
        }
        return retObject;
    }

    /**
     * Pr&uuml;fen ob ActionFormBean der gew&uuml;nschten Klasse entspricht und wenn nicht, dann neu initialisieren
     *
     * @param session ServletSession
     * @param beanClass gew&uuml;nschte Klasse
     */
    public static final void testAndResetForm(
        HttpSession session,
        Class beanClass)
    {
        Object beanObject = getForm(session, beanClass);

        if (beanObject == null)
        {
            resetForm(session, beanClass);
        } // end if
    } // end method testAndResetForm

    /**
     * Entnehmen der aktuellen Liste (fester SessionKey {@link SessionKeys#CNC_LIST_KEY SessionKeys#CNC_LIST_KEY})
     * aus der Session.
     *
     * @param session Session von Servlet/JSP
     * @param elementNr Nummer des Listenelements (Basis 0)
     */
    public static Object getList(HttpSession session, Class listClass)
    {
        // Liste holen
        Object listObj = session.getAttribute(SessionKeys.CNC_LIST_KEY);
        // fester Sessionkey

        // Log.debug( "" + listObj ) ;
        // Klasse pruefen
        if (listObj != null)
        {
            if (!(listClass.isInstance(listObj)))
            {
                // bei falscher Klasse null setzen
                listObj = null;
            } // end if
        } // end if

        return listObj;
    } // end method getList

    /**
     * Entnehmen der aktuellen Liste (fester SessionKey {@link SessionKeys#CNC_LIST_KEY SessionKeys#CNC_LIST_KEY})
     * aus der Session.
     *
     * @param request Request vom Servlet/JSP ServletRequest
     * @param elementNr Nummer des Listenelements (Basis 0)
     */
    public static Object getList(HttpServletRequest request, Class listClass)
    {
        HttpSession session = request.getSession(true);

        return getList(session, listClass);
    } // end method getList

    /**
     * Vermerken der vorherigen JSP-Seite (srcjsp) auf dem Session-Merker {@link SessionKeys#CNC_SRC_JSP}
     *
     * @param request Request vom Servlet/JSP
     */
    public static final void setSrcJsp(HttpServletRequest request)
    {
        // Ermitteln der vorherigen JSP
        // hidden-Field srcjsp= ... wird von FormTag.java, GoNextButtonTag.java und GoPrevButtonTag.java mitgegeben
        String strSrcjsp = null;

        strSrcjsp = (String) request.getParameter(RequestKeys.CNC_SRC_JSP);

        if (strSrcjsp == null)
        {
            strSrcjsp = "";
        } // end if
        // Log.debug("srcjsp " + strSrcjsp);
        HttpSession session = request.getSession(true);

        CncUtil.setAttribute(session, SessionKeys.CNC_SRC_JSP, strSrcjsp);
    } // end method setSrcJsp

    /**
     * Abfragen der vorherigen JSP-Seite (srcjsp) aus dem Session-Merker {@link SessionKeys#CNC_SRC_JSP}
     *
     * @param session ServletSession
     * @return Request-URI der vorherigen JSP-Seite
     */
    public static final String getSrcJsp(HttpSession session)
    {
        String retStr =
            (String) getAttribute(session,
                SessionKeys.CNC_SRC_JSP,
                String.class);

        if (retStr == null)
        {
            retStr = "";
        }

        return retStr;
    } // end method

    /**
     * Ermitteln des Namens (mit Pfad) der gerade aufgerufenen JSP . <br/>
     *
     * @param request Request vom Servlet/JSP ServletRequest
     *
     * @return Namen (mit Pfad) der gerade aufgerufenen JSP
     */
    public static final String getCurrentJspName(HttpServletRequest request)
    {
        String srcJsp = request.getRequestURI();

        // Prefix der Applikation entfernen, weil sonst doppelt in der Ziel-URI
        // evtl. konfigurierbare Lsung machen
        srcJsp = srcJsp.substring(1); // beginnendes / entfernen
        srcJsp = srcJsp.substring(srcJsp.indexOf("/"));
        // alles ab erstem / &uuml;bernehmen
        return srcJsp;
    } // end method getCurrentJspName

    /**
     * Zur&uuml;ckgeben ob die Liste auf dem festen SessionKey {@link SessionKeys#CNC_LIST_KEY} leer ist . <br/>
     *
     * @param session ServletSession
     * @return ob Liste null oder keine Elemente
     */
    public static final boolean isListeEmpty(HttpSession session)
    {
        ArrayList sessionArrayList =
            (ArrayList) session.getAttribute(SessionKeys.CNC_LIST_KEY);
        // fester Sessionkey

        if (sessionArrayList == null || sessionArrayList.isEmpty())
        {
            return true;
        }
        return false;
    }

    /**
     * Abfragen der Action des aktuellen Requests mit Beachtung Single-Step-Debug . <br/>
     * Wird aus der Request-URI durch Abschneiden des Namens der Webapplikation (Prefix)<br/>
     * und des URL-Pattern des CnC-Servlets (Postfix) ermittelt.
     *
     * @param session ServletSession
     * @return Request-URI der vorherigen JSP-Seite
     */
    public static final String getAction(HttpServletRequest request)
    {
        HttpSession session = request.getSession(true);

        if (session.getAttribute(SessionKeys.CNC_SINGLE_STEP_DEBUG) != null)
        {
            // Einzelschritt-Debug
            return (String) session.getAttribute(
                SessionKeys.CNC_SINGLE_STEP_DEBUG_ACTION);
        } // end if

        return getTrueAction(request);
    } // end method getAction

    /**
     * Abfragen der Action des aktuellen Requests ohne Beachtung Single-Step-Debug . <br/>
     * Wird aus der Request-URI durch Abschneiden des Namens der Webapplikation (Prefix)<br/>
     * und des URL-Pattern des CnC-Servlets (Postfix) ermittelt.
     *
     * @param session ServletSession
     * @return Request-URI der vorherigen JSP-Seite
     */
    public static final String getTrueAction(HttpServletRequest request)
    {
        // Abfragen requestUri
        String actionStr = request.getRequestURI();

        // Abfragen Name der Web-Applikation
        String webappPrefix = request.getContextPath() + "/";

        // Abschneiden des Prefixes /app-name/ von der requestUri weil sonst weitere Verarbeitung nicht klappt
        actionStr = StringUtil.cutLeft(actionStr, webappPrefix);
        // webappPrefix abschneiden

        // actionPostfix "*.cnc" abschneiden
        actionStr = StringUtil.cutRight(actionStr, CncServlet.actionPostfix);

        return actionStr;
    } // end method getAction

    /**
     * Abfragen, ob ein bestimmter Button bettigt wurde . <br/>
     * Wird durch Abscannen aller Namen der Requestparameter ermittelt.<br/>
     * Tritt ein parameter mit dem Prefix "cnc_button_"+gew&uuml;nschter name auf,<br/>
     * so wird true zur&uuml;ckgegeben.
     *
     * @param session ServletSession
     * @param strPaButtonName Name des Buttons ohne Prefix
     * @return Request-URI der vorherigen JSP-Seite
     */
    public static final boolean hasCncButton(
        HttpServletRequest request,
        String strPaButtonName)
    {
        // Abfragen requestUri

        // noch konstante machen
        String buttonPrefix = "cnc_button_";

        Object reqObject;

        reqObject = request.getParameter(buttonPrefix + strPaButtonName);
        if (reqObject != null)
        {
            return true;
        } // end if

        // bei Image-Buttons auf angehaengtes .x oder .y pruefen
        reqObject = request.getParameter(buttonPrefix + strPaButtonName + ".x");
        if (reqObject != null)
        {
            return true;
        } // end if

        return false;
    } // end method isCnCButton

    /**
     * Abfragen, Name des Buttons . <br/>
     * Wird durch Abscannen aller Namen der Requestparameter ermittelt.<br/>
     * Tritt ein parameter mit dem Prefix "cnc_button_"+gew&uuml;nschter name auf,<br/>
     * so wird der Name ohne Prefix zur&uuml;ckgegeben.
     *
     */
    public static final String getCncButtonName(HttpServletRequest request)
    {
        // noch konstante machen
        final String buttonPrefix = "cnc_button_";

        String btnName = scanForNameStartsWith(request, buttonPrefix);
        btnName = StringUtil.cutLeft(btnName, buttonPrefix);

        if (btnName == null )
        {
            return null;
        }
        if (btnName.startsWith("_cnc_no_validate_"))
        {
            btnName = StringUtil.cutLeft(btnName, "_cnc_no_validate_");
        }

        //    if ( btnName.endsWith( "]" ) && btnName.indexOf( '[' ) >= 0 && btnName.indexOf( '[' ) < btnName.length() - 2 )
        //    {
        //        //Button-Name endet mit ']', es ist ein indizierter Button
        //        btnName = btnName.substring( 0 , btnName.indexOf( '[' ) );
        //    }
        if (btnName.indexOf("_cnc_index_") > 0)
        {
            //Button-Name endet mit ']', es ist ein indizierter Button
            btnName = btnName.substring(0, btnName.indexOf("_cnc_index_"));
        }

        if (btnName.endsWith(".x") || btnName.endsWith(".y"))
        {
            //es handelt sich um einen GIF-Button, am Ende des Namens wurde ein '.x' angehngt
            btnName = btnName.substring(0, btnName.length() - 2);
        }
        return btnName;
    } // end method getCncButtonName

    /**
     * Achtung anfang der Button-Namen d&uuml;rfen nicht gleich sein
     */
    public static final boolean isCnCButtonIndexed(//--
    final HttpServletRequest request, //--
    final String pStrButtonName)
    {
        String buttonPrefix = "cnc_button_"; //@todo: noch konstante machen
        String buttonNovalidateKey = "_cnc_no_validate_";

        Object reqObject;

        //@todo: tunen, nicht mehrmals scannen
        Log.debug("scanfor: " + buttonPrefix + pStrButtonName);
        reqObject =
            scanForNameStartsWith(request, buttonPrefix + pStrButtonName);
        if (reqObject != null)
        {
            return true;
        } // end if

        Log.debug(
            "scanfor: " + buttonPrefix + buttonNovalidateKey + pStrButtonName);
        reqObject =
            scanForNameStartsWith(
                request,
                buttonPrefix + buttonNovalidateKey + pStrButtonName);
        if (reqObject != null)
        {
            return true;
        } // end if

        return false;
    } // end method isCnCButtonIndexed

    /**
     * Abfrage, ob ein Button mit der Option validate=false" bettigt wurde
     * 
     * @param request
     * @return
     */
    public static boolean isNoValidate(//--
    final HttpServletRequest request) //--
    {
        // noch konstante machen
        final String buttonPrefix = "cnc_button_";

        String btnName = scanForNameStartsWith(request, buttonPrefix);
        btnName = StringUtil.cutLeft(btnName, buttonPrefix);

        if (btnName != null && btnName.startsWith("_cnc_no_validate_"))
        {
            return true;
        }
        return false;
    }

    /**
     * Abfragen des Cnc-Button-Index eines Buttons fr ein Listenelement.
     * 
     * -1 bedeutet undefiniert
     * spezieller Interfix f&uuml;r Button-Nummer _cnc_index_
     */
    public static final int getCnCButtonIndex(HttpServletRequest request)
    {
        String strSuch = "_cnc_index_"; // noch Konstante machen
        int retInt = -1; // undefiniert

        String strReqName;

        strReqName = (String) scanForNameContents(request, strSuch);
        if (strReqName != null)
        {
            // Log.debug( strReqName ) ;
            strReqName =
                strReqName.substring(
                    strReqName.indexOf(strSuch) + strSuch.length());
            // Log.debug( strReqName ) ;
            retInt = StringUtil.str2intTolerant(strReqName);
        } // end if

        return retInt;
    } // end method getCnCButtonIndex

    /**
     * Speichern des aktuellen Requests auf dem festen Session-Key
     * CNC_SINGLE_STEP_DEBUG_REQUEST . <br/>
     * F&uuml;r das Einzelschritt-Debug muss der jeweilige Request
     * zwischengespeichert und in /cncdebug/singlestepdebug.jsp
     * mit Hidden-Fields wieder hergestellt werden . <br/>
     */
    public static final void saveRequestInSession(HttpServletRequest request)
    {
        HashMap reqHasMap = new HashMap();

        Enumeration en = request.getParameterNames();

        while (en.hasMoreElements())
        {
            String s = (String) en.nextElement();
            Object o = request.getParameter(s);

            // debug("HttpServletRequest-Parameter " + s + " -> " + o);
            reqHasMap.put(s, o);
        } // end while

        // Request-Attribute werden erst mal nicht mit beachtet
        // en = req.getAttributeNames();
        // while (en.hasMoreElements()) {
        // String  s = (String) en.nextElement();
        // Object  o = req.getAttribute(s);
        //
        // debug("HttpServletRequest-Attribute " + s + " -> " + o);
        // }//end while

        HttpSession session = request.getSession(true);

        CncUtil.setAttribute(
            session,
            "CNC_SINGLE_STEP_DEBUG_REQUEST",
            reqHasMap);

    } // end method saveRequestInSession

    /**
     * Alle Objekte aus der Session L&ouml;schen f&uuml;r Logout. <br/>
     * Dabei wird auf jeden Fall auch das User-Objekt gel&ouml;scht<br/>
     * und so weitere Verwendung der Session verhindert.
     *
     * @param session ServletSession
     */
    public static void removeAll(HttpSession session)
    {
        while (true)
        {
            Enumeration en = session.getAttributeNames();

            if (!en.hasMoreElements())
            {
                break;
            }

            String s = (String) en.nextElement();

            session.removeAttribute(s);
        }
    } // end method

    /**
     * Zur&uuml;ckgeben der Message und des StackTrace einer Exception als String
     *
     * @param tr zu bearbeitende Exception
     * @return Message und StackTrace als String
     */
    public static String getMessageAndStackTrace(Throwable tr)
    {
        String retString = "";

        if (tr != null)
        {
            retString = tr.getMessage();

            StringWriter writer = new StringWriter();
            PrintWriter pwriter = new PrintWriter(writer);

            tr.printStackTrace(pwriter);

            retString += "\n\n\nStacktrace:\n\n " + writer.toString();
        }

        return retString;
    } // end method getMessageAndStackTrace

    /**
     * Spezial-Methode anstatt session.setAttribute,
     * weil Websphere das vorhandene SessionAttribute nicht berschreibt.
     *
     * @param session ServletSession
     * @param keyStr
     * @param sessionObject
     */
    public static void setAttribute(
        HttpSession session,
        String keyStr,
        Object sessionObject)
    {
        //der Scheiss-WebSphere kann keine Null auf die Session schreiben
        while (sessionObject == null && session.getAttribute(keyStr) != null)
        {
            session.removeAttribute(keyStr);
        }

        if (sessionObject != null)
        {
            session.setAttribute(keyStr, sessionObject);
        }
    }

    /**
     * Umwandlung von BigDecimal in Long nullsicher.
     * 
     * @param pDecimal
     * @return
     */
    public static Long bigDecimal2Long(BigDecimal pBigDecimal)
    {
        if (pBigDecimal == null)
        {
            return null;
        }
        return new Long(pBigDecimal.longValue());
    }

    /**
     * markierte Eintrge aus einer ArrayList entfernen . <br>
     * Achtung: Seiteneffekt auf die bergebene {@link List}.
     * 
     * @param pList
     * @return
     */
    public static List removeForMark(final List pList)
    {
        //Algorithmus zum Entfernen der markierten Eintrge aus der ArrayList
        for (Iterator iter = pList.iterator(); iter.hasNext();)
        {
            MarkableInterface markable = (MarkableInterface) iter.next();

            if (markable.getMark())
            {
                iter.remove();
            }
        }
        return pList;
    }

    /**
     * Rundung mit Angabe der Stellen hinter dem Komma.
     * 
     * @param pVal zu rundender Wert
     * @param pDigits Anzahl Stellen hinter dem Komma
     * @return gerundeter Wert
     */
    public static double round(double pVal, int pDigits)
    {
        try
        {
            // Runden durch Division durch eins und Ausnutzung Rundungs- und Skalierungsmechanik des BigDecimal
            BigDecimal bigDecValue = new BigDecimal(pVal);
            return bigDecValue
                .divide(new BigDecimal(1), pDigits, BigDecimal.ROUND_HALF_EVEN)
                .doubleValue();
        }
        catch (Exception e)
        {
            Log.fatal("" + pVal);
            Log.fatal(e);
            return pVal;
        }
    }

} // end class CncUtil
