package de.cnc.util;


/**
 * Hilfsmethoden zum Vergleich von Objekten . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public final class CompareUtil {

  /**
   * compareTo-Methode mit Abfangung null
   */
  private static final int compareTo(Comparable comPa1, Comparable comPa2) {
//Log.debug( "CompareUtil#compareTo >" + comPa1 + "< with >" + comPa2 + "<" );
    if (comPa1 == null && comPa2 == null) {
      // wenn comPa1 und comPa1 null ist, dann gilt gleich
      return 0;
    } else if (comPa1 == null) {
      // wenn comPa1 null ist, dann gilt kleiner als
      return -1;
    } else if (comPa2 == null) {
      // wenn comPa2 null ist, dann gilt groesser als
      return 1;
    } else {
      try {
          return comPa1.compareTo(comPa2);
      } catch (ClassCastException c) {
          String classStr1 = comPa1 == null ? "null" : comPa1.getClass().getName() ;
          String classStr2 = comPa2 == null ? "null" : comPa2.getClass().getName() ;
          throw new ClassCastException( "compare " + classStr1 + " with " + classStr2 );
      }
    }// end if
  }// end method compareTo

  public static final boolean equals(Comparable comPa1, Comparable comPa2) {
    return (compareTo(comPa1, comPa2) == 0);
  }// end method equals

  public static final boolean notEquals(Comparable comPa1, Comparable comPa2) {
    return !(compareTo(comPa1, comPa2) == 0);
  }// end method notEquals

  public static final boolean less(Comparable comPa1, Comparable comPa2) {
    return (compareTo(comPa1, comPa2) < 0);
  }// end method less

  public static final boolean lessEqual(Comparable comPa1, Comparable comPa2) {
    return (compareTo(comPa1, comPa2) <= 0);
  }// end method lessEqual

  public static final boolean greater(Comparable comPa1, Comparable comPa2) {
    return (compareTo(comPa1, comPa2) > 0);
  }// end method greater

  public static final boolean greaterEqual(Comparable comPa1, Comparable comPa2) {
    return (compareTo(comPa1, comPa2) >= 0);
  }// end method greaterEqual

}// end method CompareUtil
