package de.cnc.util;

import java.io.*;
import java.util.*;

import javax.servlet.ServletException;
import javax.servlet.http.*;

import de.cnc.*;
import de.cnc.util.config.*;

/**
 * Logging CnC-Applikation . <br/>
 * abgerstete Einfachstvariante . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id: $
 */
public class Log
{

   /*
    Log-Modi Hierarchie
   
    1   trace (the least serious)
    2   debug
    3   info
    4   warn
    5   error
    6   fatal (the most serious)
    */

   /**
    * Log-Mode-Konstante
    */
   public static final int TRACE = 1;

   /**
    * Log-Mode-Konstante
    */
   public static final int DEBUG = 2;

   /**
    * Log-Mode-Konstante
    */
   public static final int INFO = 3;

   /**
    * Log-Mode-Konstante
    */
   public static final int WARN = 4;

   /**
    * Log-Mode-Konstante
    */
   public static final int ERROR = 5;

   /**
    * Log-Mode-Konstante
    */
   public static final int FATAL = 6;

   /**
    * aktuell eingestellter Log-Mode, wer Lust hat,
    * kann diese Einstellung ber Config-Datei
    * setzbar machen.
    */
   // public static int currentLogMode = WARN ;  // Auslieferung
   public static int currentLogMode = DEBUG; // Entwicklung

   /**
    * fr Ausgabe in Log-File
    */
   private static FileWriter fileOut;

   /**
    * Constructor declaration<br/>
    * Instanziieren nicht erlaubt
    */
   private Log()
   {
      ;
   }

   /**
    * Init
    * 
    * @throws ServletException
    */
   public static void init() throws ServletException
   {
      CncProperties prop = ConfigManager.getProperties("log");

      // Initialisierung Log-File
      String outFileName = prop.getString("file");
      if (!StringUtil.empty(outFileName))
      {
         System.out.println("Cnc Log File: " + outFileName);

         boolean isFileAppend = prop.getBoolean("fileappend");

         System.out.println("Cnc Log File append mode: " + isFileAppend);

         if (!isFileAppend)
         {
            //kein Anhngmodus, Datei Lschen
            File file = new File(outFileName);
            if (file.exists())
            {
               file.delete();
            }
         }

         try
         {
            fileOut = new FileWriter(outFileName, true);
         }
         catch (Exception e)
         {
            System.err.println(e.getMessage());
            e.printStackTrace();
            throw new ServletException(e.getMessage());
         }
      }
      //@todo: Konfigurierbarer Log-Level      
      
   }

   /**
    * Ausgabe Meldung auf System.err und in Log (System.out, Log-File, Mail)
    * entsprechend Log-Modus ohne Angabe des werfenden Objektes.
    *
    * @param strPa auszugebende Meldung
    */
   public static void fatal(String strPa)
   {
      log(strPa, Log.FATAL);
   } // end method fatal

   /**
    * Ausgabe Exception auf System.err und in Log (System.out, Log-File, Mail) entsprechend Log-Modus ohne Angabe des werfenden Objektes.
    *
    * @param e auszugebende Exception
    */
   public static void fatal(Throwable e)
   {
      fatal(e, null);
   } // end method fatal

   /**
    * Ausgabe Exception auf System.err im FATAL-Log-Modus mit Angabe des werfenden Objektes . <br/>
    * Aufruf: fatal( Exception , this ) ;<br/>
    *
    * @param t auszugebende Exception
    * @param o ein Object (typisch this im Aufrufer)
    */
   public static void fatal(Throwable t, Object o)
   {
      StringBuffer msgStrBuff = new StringBuffer("" + t);

      if (o != null)
      {
         msgStrBuff.append(" in class " + o.getClass().getName());
      }
 
      //Ausgabe Datum Uhrzeit fr Log-File
      msgStrBuff.append( "\n" + new Date() + "\n" );

      msgStrBuff.append(getMessageAndStackTrace(t));

      log(msgStrBuff.toString(), Log.FATAL);
   } // end method fatal

   /**
    * Logging String mit Angabe Log-Mode . <br/>
    * Das Logging erfolgt nur, wenn der entsprechende Log-Mode in log.properties eingeschalten ist.<br/>
    *
    * @param msg zu loggender String
    * @param logmode
    */
   synchronized static public void log(String msg, int logMode)
   {
      // ob die Einstellung berhaupt Log erlaubt
      if (logMode < currentLogMode)
      {
         return;
      } // end if

      StringBuffer sb = new StringBuffer(1024);

      // sb.append(StringUtil.getTimeStamp() + " ");

      // switch (logmode) {
      // case Log.TRACE:
      // sb.append("[TRACE]: ");
      // break;
      // case Log.DEBUG:
      // sb.append("[DEBUG]: ");
      // break;
      // case Log.INFO:
      // sb.append("[INFO]: ");
      // break;
      // case Log.WARN:
      // sb.append("[WARN]: ");
      // break;
      // case Log.ERROR:
      // sb.append("[ERROR]: ");
      // break;
      // case Log.FATAL:
      // sb.append("[FATAL]: ");
      // break;
      // }

      sb.append(msg);

      System.out.println(sb.toString());
      final CncServlet cncServlet = CncServletAnchor.getCncServlet();
      if (cncServlet != null)
      {
          cncServlet.log(sb.toString());
      }

      // hier kann die Ausgabe in Log-Files usw. eingebaut werden
      if (fileOut != null)
      {
         try
         {
            fileOut.write(sb.toString() + "\n");
            fileOut.flush();
         }
         catch (IOException e)
         {
            System.err.println(e.getMessage());
            e.printStackTrace();
         }
      }

      // zustzliche Ausgabe auf System.err im Normalbetrieb
      if (logMode >= FATAL)
      {
         System.err.println(sb.toString());
      }

   } // end method

   /**
    * Methode fr hhere Performance bei ausgeschaltetem DEBUG-Level,
    * setzt den String, ein ': ' und das Paramter-Objekt zusammen.
    * Dadurch wird die toString()-Methode des Objektes nur bei
    * eingeschaltetem DEBUG-Level aufgerufen . <br>
    * 
    * Debug-Log als statische Methode von berall leicht aufzurufen.<br/>
    * Log-Mode, fest eingestellt auf Log.DEBUG, bestimmt, ob die Ausgabe erfolgt.<br/>
    *
    * @param strPaOut auszugebender Text
    */
   public static void debug(String pStr, Object pObj)
   {
      // ob die Einstellung berhaupt Log erlaubt
      if (DEBUG < currentLogMode)
      {
         return;
      } // end if

      debug(pStr + ": " + pObj);
   } // end method debug

   /**
    * Methode fr hhere Performance bei ausgeschaltetem DEBUG-Level,
    * setzt den String, ein ': ' und das Paramter-Objekt zusammen.
    * Dadurch wird die toString()-Methode des Objektes nur bei
    * eingeschaltetem DEBUG-Level aufgerufen . <br>
    * 
    * Debug-Log als statische Methode von berall leicht aufzurufen.<br/>
    * Log-Mode, fest eingestellt auf Log.DEBUG, bestimmt, ob die Ausgabe erfolgt.<br/>
    *
    * @param strPaOut auszugebender Text
    */
   public static void debug(String pStr, int pInt)
   {
      // ob die Einstellung berhaupt Log erlaubt
      if (DEBUG < currentLogMode)
      {
         return;
      } // end if

      debug(pStr + ": " + pInt);
   } // end method debug

   /**
    * Methode fr hhere Performance bei ausgeschaltetem DEBUG-Level,
    * setzt die Paramter-Objekte als Strings zusammen.
    * Dadurch wird die toString()-Methode der Objekte nur bei
    * eingeschaltetem DEBUG-Level aufgerufen . <br>
    * 
    * Debug-Log als statische Methode von berall leicht aufzurufen.<br/>
    * Log-Mode, fest eingestellt auf Log.DEBUG, bestimmt, ob die Ausgabe erfolgt.<br/>
    *
    * @param strPaOut auszugebender Text
    */
   public static void debug(Object pObj1, Object pObj2, Object pObj3)
   {
      // ob die Einstellung berhaupt Log erlaubt
      if (DEBUG < currentLogMode)
      {
         return;
      } // end if

      debug("" + pObj1 + " " + pObj2 + " " + pObj3);
   } // end method debug

   /**
    * Methode fr hhere Performance bei ausgeschaltetem DEBUG-Level,
    * setzt die Paramter-Objekte als Strings zusammen.
    * Dadurch wird die toString()-Methode der Objekte nur bei
    * eingeschaltetem DEBUG-Level aufgerufen . <br>
    * 
    * Debug-Log als statische Methode von berall leicht aufzurufen.<br/>
    * Log-Mode, fest eingestellt auf Log.DEBUG, bestimmt, ob die Ausgabe erfolgt.<br/>
    *
    * @param strPaOut auszugebender Text
    */
   public static void debug(Object pObj1, Object pObj2, Object pObj3, Object pObj4)
   {
      // ob die Einstellung berhaupt Log erlaubt
      if (DEBUG < currentLogMode)
      {
         return;
      } // end if

      debug("" + pObj1 + " " + pObj2 + " " + pObj3 + " " + pObj4 );
   } // end method debug

   /**
    * Debug-Log als statische Methode von berall leicht aufzurufen.<br/>
    * Log-Mode, fest eingestellt auf Log.DEBUG, bestimmt, ob die Ausgabe erfolgt.<br/>
    *
    * @ deprecated
    * @param strPaOut auszugebender Text
    */
   public static void debug(String strPaOut)
   {
      log(strPaOut, Log.DEBUG);
   } // end method debug

   /**
    * Ausgabe Request-Parameter mit Wert im Log
    *
    * @param req HttpServletRequest
    */
   public static void dump(HttpServletRequest req)
   {
      Enumeration en = req.getParameterNames();

      while (en.hasMoreElements())
      {
         String s = (String) en.nextElement();
         String o = req.getParameter(s);

         debug("HttpServletRequest-Parameter" , s , "->" , o);
      }

      en = req.getAttributeNames();

      while (en.hasMoreElements())
      {
         String s = (String) en.nextElement();
         Object o = req.getAttribute(s);

         debug("HttpServletRequest-Attribute" , s , "->" , o);
      }
   } // end method dump(HttpServletRequest)

   /**
    * Ausgabe Session-Attribute mit Wert im Log
    *
    * @param session HttpSession
    */
   public static void dump(HttpSession session)
   {
      Enumeration en = session.getAttributeNames();

      while (en.hasMoreElements())
      {
         String s = (String) en.nextElement();
         Object o = session.getAttribute(s);

         // dafr sorgen, da bei ArrayList nicht alle Daten augegenben werden
         if (o instanceof ArrayList)
         {
            String strTmp = "ArrayList size: " + ((ArrayList) o).size();

            if (((ArrayList) o).size() > 0)
            {
               // Klasse des ersten Elements angeben
               strTmp += " class " + ((ArrayList) o).get(0).getClass().getName();
            } // end if
            o = strTmp;
         } // end if

         debug("HttpSession-Attribute" , s , "->" , o);
      } // end while
   } // end method dump(HttpSession)

   /**
    * Ausgabe PageContext-Attribute mit Wert im Log
    *
    * @param req HttpServletRequest
    */

   /*
    public static void dump(PageContext pageContext)
    {
    Enumeration en;
    int         scope;
   
    scope = 0;
    en = pageContext.getAttributeNamesInScope(scope);
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = pageContext.getAttribute(s, scope);
   
    debug("PageContext-Attribute (scope " + scope + ") " + s + " -> " + o);
    }
   
    scope = PageContext.PAGE_SCOPE;
    en = pageContext.getAttributeNamesInScope(scope);
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = pageContext.getAttribute(s, scope);
   
    debug("PageContext-Attribute (scope PAGE_SCOPE " + scope + ") " + s + " -> " + o);
    }
   
    scope = PageContext.REQUEST_SCOPE;
    en = pageContext.getAttributeNamesInScope(scope);
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = pageContext.getAttribute(s, scope);
   
    debug("PageContext-Attribute (scope REQUEST_SCOPE " + scope + ") " + s + " -> " + o);
    }
   
    scope = PageContext.SESSION_SCOPE;
    en = pageContext.getAttributeNamesInScope(scope);
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = pageContext.getAttribute(s, scope);
   
    debug("PageContext-Attribute (scope SESSION_SCOPE " + scope + ") " + s + " -> " + o);
    }
   
    scope = PageContext.APPLICATION_SCOPE;
    en = pageContext.getAttributeNamesInScope(scope);
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = pageContext.getAttribute(s, scope);
   
    debug("PageContext-Attribute (scope APPLICATION_SCOPE " + scope + ") " + s + " -> " + o);
    }
    } // end method dump(PageContext)
    */

   /**
    * Ausgabe Request-Parameter mit Wert im Log
    *
    * @param req HttpServletRequest
    */

   /*
    public static String htmlDump(HttpServletRequest req)
    {
    String      outStr = "";
    Enumeration en = req.getParameterNames();
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = req.getParameter(s);
   
    outStr += ("HttpServletRequest-Parameter " + s + " -> " + o) + "<BR>";
    }
   
    en = req.getAttributeNames();
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = req.getAttribute(s);
   
    outStr += ("HttpServletRequest-Attribute " + s + " -> " + o) + "<BR>";
    }
   
    return outStr;
    } // end method htmlDump(HttpServletRequest)
    */

   /**
    * Ausgabe Session-Attribute mit Wert im Log
    *
    * @param session HttpSession
    */
   public static String htmlDump(HttpSession session)
   {
      StringBuffer outStrBuff = new StringBuffer(1024 * 32);
      Enumeration en = session.getAttributeNames();

      outStrBuff.append("<pre>\n");

      // outStrBuff.append( "HttpSession\n" ) ;
      // outStrBuff.append( "-----------\n" ) ;
      outStrBuff.append("<u>HttpSession</u>\n");

      // outStrBuff.append( "\n" ) ;
      outStrBuff.append("\n");

      //TreeMap fr sortierte Ausgabe
      TreeMap treeMap = new TreeMap();

      while (en.hasMoreElements())
      {
         String s = (String) en.nextElement();
         Object o = session.getAttribute(s);
         String strTmp = null;

         strTmp = objToStrListCompact(o);

         //outStrBuff.append(s + " -> " + ( o != null ? o.getClass().getName() + ": " : "null" ) + " " + strTmp + "\n");
         treeMap.put(s, (o != null ? o.getClass().getName() + ": " : "null") + " " + strTmp + "\n");
      } // end while

      for (Iterator iter = treeMap.entrySet().iterator(); iter.hasNext();)
      {
         Map.Entry element = (Map.Entry) iter.next();
         outStrBuff.append(element.getKey() + " -> " + element.getValue());
      }

      outStrBuff.append("</pre>\n");
      return outStrBuff.toString();
   }// end method htmlDump(HttpSession)

   /**
    * Objekt in String-Darstellung umwandeln,
    * aber bei Instanzen von List nicht alles
    * ausgeben . <br>
    * 
    * @param pObj umzuwandelndes Object
    * @return String-Darstellung
    */
   public static String objToStrListCompact(Object pObj)
   {
      String strTmp;
      // dafr sorgen, da bei List nicht alle Daten ausgegeben werden
      if (pObj instanceof List)
      {
         strTmp = "List size: " + ((List) pObj).size();
      
         if (((List) pObj).size() > 0)
         {
            // Klasse des ersten Elements angeben
            strTmp += " class " + ((List) pObj).get(0).getClass().getName();
         } // end if
      }
      else
      {
         strTmp = "" + pObj;
      } // end if
      return strTmp;
   }

   /**
    * Ausgabe Request-Parameter mit Wert im Log
    *
    * @param request HttpServletRequest
    */
   public static String htmlDump(HttpServletRequest request)
   {
      StringBuffer outStrBuff = new StringBuffer(1024 * 32);
      Enumeration en = request.getParameterNames();

      outStrBuff.append("<pre>\n");

      // outStrBuff.append( "HttpSession\n" ) ;
      // outStrBuff.append( "-----------\n" ) ;
      outStrBuff.append("<u>HttpServletRequest</u>\n");

      // outStrBuff.append( "\n" ) ;
      outStrBuff.append("\n");

      //TreeMap fr sortierte Ausgabe
      TreeMap treeMap = new TreeMap();

      while (en.hasMoreElements())
      {
         String s = (String) en.nextElement();
         Object o = request.getParameter(s);

         //outStrBuff.append( s + " -> " + o + "\n");
         treeMap.put("Parameter " + s, o + "\n");
      }

//Ausgabe der request-Attribute klappt nicht bei df-webhosting
//      en = request.getAttributeNames();
//
//      while (en.hasMoreElements())
//      {
//         String s = (String) en.nextElement();
//         Object o = request.getAttribute(s);
//
//         //outStrBuff.append("HttpServletRequest-Attribute " + s + " -> " + o + "\n");
//         treeMap.put("Attribute " + s, o + "\n");
//      }

      for (Iterator iter = treeMap.entrySet().iterator(); iter.hasNext();)
      {
         Map.Entry element = (Map.Entry) iter.next();
         outStrBuff.append(element.getKey() + " -> " + element.getValue());
      }

      outStrBuff.append("</pre>\n");
      return outStrBuff.toString();
   } // end method htmlDump(HttpServletRequest)

   /**
    * Ausgabe PageContext-Attribute mit Wert im Log
    *
    * @param req HttpServletRequest
    */

   /*
    public static String htmlDump(PageContext pageContext)
    {
    String      outStr = "";
    Enumeration en;
    int         scope;
   
    scope = 0;
    en = pageContext.getAttributeNamesInScope(scope);
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = pageContext.getAttribute(s, scope);
   
    outStr += ("PageContext-Attribute (scope " + scope + ") " + s + " -> " + o) + "<BR>";
    }
   
    scope = PageContext.PAGE_SCOPE;
    en = pageContext.getAttributeNamesInScope(scope);
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = pageContext.getAttribute(s, scope);
   
    outStr += ("PageContext-Attribute (scope PAGE_SCOPE " + scope + ") " + s + " -> " + o) + "<BR>";
    }
   
    scope = PageContext.REQUEST_SCOPE;
    en = pageContext.getAttributeNamesInScope(scope);
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = pageContext.getAttribute(s, scope);
   
    outStr += ("PageContext-Attribute (scope REQUEST_SCOPE " + scope + ") " + s + " -> " + o) + "<BR>";
    }
   
    scope = PageContext.SESSION_SCOPE;
    en = pageContext.getAttributeNamesInScope(scope);
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = pageContext.getAttribute(s, scope);
   
    outStr += ("PageContext-Attribute (scope SESSION_SCOPE " + scope + ") " + s + " -> " + o) + "<BR>";
    }
   
    scope = PageContext.APPLICATION_SCOPE;
    en = pageContext.getAttributeNamesInScope(scope);
   
    while (en.hasMoreElements())
    {
    String  s = (String) en.nextElement();
    Object  o = pageContext.getAttribute(s, scope);
   
    outStr += ("PageContext-Attribute (scope APPLICATION_SCOPE " + scope + ") " + s + " -> " + o) + "<BR>";
    }
   
    return outStr;
    } // end method htmlDump(PageContext)
    */

   /**
    * Logging String-Array
    *
    * @param strArr auszugebendes String-Array
    */

   /*
    public static void debug(String[] strArr)
    {
    debug("String[] " + strArr);
   
    if (strArr != null)
    {
    debug("length " + strArr.length);
    }
   
    for (int i = 0; i < strArr.length; i++)
    {
    debug("[" + i + "]" + strArr[i]);
    }
    }
    */

   /**
    * Logging zweidimensionales String-Array
    *
    * @param strArrArr auszugebendes String-Array
    */

   /*
    public static void debug(String[][] strArrArr)
    {
    debug("String[][] " + strArrArr);
   
    if (strArrArr != null)
    {
    debug("length " + strArrArr.length);
    }
   
    for (int i = 0; i < strArrArr.length; i++)
    {
    debug("[" + i + "]" + strArrArr[i]);
    }
    }
    */

   /**
    * Hilfsmethode zum Debugging in {@link ConfigManager}
    *
    * @param prop zu loggende Properties
    */

   /*
    public static void debug(Properties prop)
    {
    String  retString = "";
   
    if (prop != null)
    {
    retString = "Properties: " + prop + "\n";
   
    StringWriter  strWriter = new StringWriter();
    PrintWriter   pWriter = new PrintWriter(strWriter);
   
    prop.list(pWriter);
   
    retString += strWriter.toString();
    }
   
    debug(retString);
    }
    */

   /**
    * Log einer Exception
    *
    * @param tr Exception
    */
   public static void debug(Throwable tr)
   {
      String retString = "Exception: " + tr + "\n";

      if (tr != null)
      {
         retString += getMessageAndStackTrace(tr);
      }
      debug(retString);
   } // end method

   /**
    * Zurckgeben der Message und des StackTrace einer Exception als String
    *
    * @param tr zu bearbeitende Exception
    * @return Message und StackTrace als String
    */
   public static String getMessageAndStackTrace(Throwable tr)
   {
      String retString = "";

      if (tr != null)
      {
         retString = tr.getMessage();

         StringWriter writer = new StringWriter();
         PrintWriter pwriter = new PrintWriter(writer);

         tr.printStackTrace(pwriter);

         retString += "\n\n\nStacktrace:\n\n " + writer.toString();
      }
      return retString;
   } // end method getMessageAndStackTrace

   /**
    * ob DEBUG eingeschaltet ist
    * @return
    */
   public static boolean isDebug()
   {
      // ob die Einstellung berhaupt Log erlaubt
      return ( DEBUG >= currentLogMode );
   }

} // end class Log
