package de.cnc.util;

import java.lang.reflect.Method;
import java.util.HashMap;

/**
 * Diese Klasse dient als Cache fr Methoden fr Reflections-Aufrufe.
 * 
 * @author Heiner Kcker
 * @since 14.09.2004
 */
public class MethodCache
{
   /**
    * Konstruktor private, Instanziieren verboten
    */
   private MethodCache()
   {
      ;
   }
   
   /**
    * Cache mit den Klassen
    */
   private static final HashMap classMethodsMap = new HashMap();
   
   /**
    * Ermitteln der Methode fr das bergebene Objekt und
    * den bergebenen Methoden-Namen . <br>
    * 
    * Achtung: Funktioniert nicht korrekt, wenn es mehrere
    * Methoden mit dem gleichen Namen gibt !!!
    * 
    * @param pObject
    * @param pMethodName
    * @return
    * @throws Exception
    */
//   public static Method getMethod(Object pObject, String pMethodName) 
//      throws Exception
//   {
//        Method method = null;
//         Method[] methodArray = pObject.getClass().getMethods();
//      
//         for (int i = 0; i < methodArray.length; i++)
//         {
//            if (pMethodName.equals(methodArray[i].getName()))
//            { // Namensgleichheit
//               method = methodArray[i];
//               break;
//            } // end if
//         } // end for
//      
//         if ( method == null )
//         {
//            throw new Exception(
//               "method " + pMethodName + " in Class " + pObject.getClass().getName() + " not found");
//         } // end if
//      return method;
//   }

   /**
    * Ermitteln der Methode fr das bergebene Objekt und
    * den bergebenen Methoden-Namen . <br>
    * 
    * Achtung: Funktioniert nicht korrekt, wenn es mehrere
    * Methoden mit dem gleichen Namen gibt !!!
    * 
    * @param pObject
    * @param pMethodName
    * @return
    * @throws Exception
    */
   public static Method getMethod(Object pObject, String pMethodName) 
      throws Exception
   {
      Class methodClass = pObject.getClass();
      return getMethod(methodClass, pMethodName);
   }

   /**
    * Ermitteln der Methode fr die bergebene Klasse und
    * den bergebenen Methoden-Namen . <br>
    * 
    * Achtung: Funktioniert nicht korrekt, wenn es mehrere
    * Methoden mit dem gleichen Namen gibt !!!
    * 
    * @param pObject
    * @param pMethodName
    * @return
    * @throws Exception
    */
   public static Method getMethod(Class pClass, String pMethodName) 
      throws Exception
   {
//        Method method = null;
//         Method[] methodArray = pClass.getMethods();
//      
//         for (int i = 0; i < methodArray.length; i++)
//         {
//            if (pMethodName.equals(methodArray[i].getName()))
//            { // Namensgleichheit
//               method = methodArray[i];
//               break;
//            } // end if
//         } // end for
//      
//         if ( method == null )
//         {
//            throw new Exception(
//               "method " + pMethodName + " for Class " + pClass.getName() + " not found");
//         } // end if
//      return method;

      HashMap methodMap = getMethodMap(pClass);
      Method method = (Method) methodMap.get( pMethodName );
      if ( method == null )
      {
         throw new Exception(
            "method " + pMethodName + " for Class " + pClass.getName() + " not found");
      } // end if
      return method;
   }

   /**
    * Puts the methods for the specified class in
    * the returned Map.
    * 
    * @param pClass
    * @return
    */
   private static HashMap createMethodMap(Class pClass)
   {
      HashMap methodMap = new HashMap();
      
      Method[] methodArray = pClass.getMethods();
      
      for (int i = 0; i < methodArray.length; i++)
      {
         Object prevObj = methodMap.put( methodArray[i].getName() , methodArray[i] );
         if (prevObj != null && ! methodArray[i].getDeclaringClass().equals( Object.class ) )
         {
            throw new RuntimeException( "es gibt mehr als eine Methode mit dem Namen " + methodArray[i].getName() + " in der Klasse " + pClass.getName() + " deklariert in der Klasse " + methodArray[i].getDeclaringClass().getName() + ". Das klappt nicht mit diesem MethodCache" );
         }
      } // end for
      return methodMap;
   }
   
   /**
    * Returns the method map for the specified class.
    * 
    * @param pClass
    * @return
    */
   private static HashMap getMethodMap(Class pClass)
   {
      HashMap methodMap = (HashMap) classMethodsMap.get( pClass );
      
      if (methodMap == null)
      {
         methodMap = createMethodMap(pClass);
      }
      return methodMap;
   }

}
