package de.cnc.util;

import java.lang.reflect.Method;

/**
 * Diese Klasse hat Hilfsmethoden fr Reflection
 * 
 * @author Heiner Kcker
 * @since 23.09.2004
 */
public class MethodUtil
{
   /**
    * Konstruktor private
    * Instanziieren verboten
    */
   private MethodUtil()
   {
   }
   
   /**
    * Methoden-Aufruf ohne Parameter mit Exception-Umwandlung
    * in {@link RuntimeException}.
    * 
    * @param method
    * @param invokeObject
    * @param valueObject
    * @return
    */
   public static final Object invoke( //--
      final Method method, //--
      final Object invokeObject)
   {
      try
      {
         return method.invoke( invokeObject , null);
      }
      catch (Exception e)
      {
         throw new RuntimeException( "error on invoke " + method + " on object: " + invokeObject );
      }
   }
   
   /**
    * Methoden-Aufruf mit einem Parameter mit Exception-Umwandlung
    * in {@link RuntimeException}.
    * 
    * @param method
    * @param invokeObject
    * @param valueObject
    * @return
    */
   public static final Object invoke( //--
      final Method method, //--
      final Object invokeObject, //-
      final Object valueObject)
   {
      Object[] valueObjectArray = new Object[] { valueObject };

      try
      {
         return method.invoke( invokeObject , valueObjectArray);
      }
      catch (Exception e)
      {
         throw new RuntimeException( "error on invoke " + method + " on object: " + invokeObject + " with value: " 
         + valueObject + " " + ( valueObject != null ? valueObject.getClass().getName() : "" ) );
      }
   }

   /**
    * Methoden-Aufruf mit mehreren Parametern mit Exception-Umwandlung
    * in {@link RuntimeException}.
    * 
    * @param method
    * @param invokeObject
    * @param valueObject
    * @return
    */
   public static final Object invoke( //--
      final Method method, //--
      final Object invokeObject, //-
      final Object[] valueObjectArray)
   {
      try
      {
         return method.invoke( invokeObject , valueObjectArray);
      }
      catch (Exception e)
      {
         String prmStr = "";
         for (int i = 0; i < valueObjectArray.length; i++)
         {
            prmStr += "" + valueObjectArray[ i ];
            if (valueObjectArray[ i ] != null)
            {
               prmStr += " " + valueObjectArray[ i ].getClass().getName();
            }
            if ( i < valueObjectArray.length - 1 )
            {
               prmStr += ", ";
            }
         }
         throw new RuntimeException( 
            "error on invoke " + method 
            + " on object: " + invokeObject + "(" + invokeObject.getClass().getName() + ")" 
            + " with values: " 
            + prmStr );
      }
   }

   /**
    * Methoden-Aufruf mit einem Parameter mit Exception-Umwandlung
    * in {@link RuntimeException}.
    * 
    * @param method
    * @param invokeObject
    * @param valueObject
    * @return
    * @see de.cnc.util.BeanUtil#convertToClass(Object, Class)
    */
//@todo: mal fertig programmieren, versuchen in den richtigen ziel-typen(klasse) aus java.lang.* umzuwandeln
//gibt es schon: de.cnc.util.BeanUtil#convertToClass(Object, Class)
//   public static final Object invokeTypeCorrect( //--
//      final Method method, //--
//      final Object invokeObject, //-
//      final Object valueObject)
//   {
//      Class[] paramClassArr = method.getParameterTypes();
//      Class paramClass = paramClassArr[0];
//      if ( ! ( paramClass.isAssignableFrom( valueObject ) ) )
//      {
//         valueObject = paramClass.???
//      }
//      
//      Object[] valueObjectArray = new Object[] { valueObject };
//
//      try
//      {
//         return method.invoke( invokeObject , valueObjectArray);
//      }
//      catch (Exception e)
//      {
//         throw new RuntimeException( "error on invoke " + method + " on object: " + invokeObject + " with value: " 
//         + valueObject + " " + ( valueObject != null ? valueObject.getClass().getName() : "" ) );
//      }
//   }

}
