package de.cnc.util.config;

import java.io.*;
import java.util.Properties;

import de.cnc.util.*;

/**
 * Erweiterung der Klasse <b>Properties</b> um weitere Methoden . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id:$
 */
public class CncProperties extends Properties
{
  /**
   * Teil einer Aktualisierungslsung zum Neuziehen der Properties ohne Neustart des Servers
   */
  private long  lastModified = 0;

  /**
   * Laden Properties
   *
   * @param filename FileName
   *
   * @throws IOException wenn IO-Fehler
   */
  public void load(String filename)
    throws IOException
  {
    FileInputStream file = new FileInputStream(filename);
    super.load(file);
    file.close();
  }

  /**
   * Abfragen boolean-Wert
   *
   * @param s Property-Key
   * @return boolean-Wert
   */
  public boolean getBoolean(String s)
  {
    String  property = super.getProperty(s);

    property = StringUtil.trim( property.toLowerCase() );

    if ( "true".equals( property ) || "yes".equals( property ) )
    {
      return true;
    }
    else if ( "false".equals( property ) || "no".equals( property ) )
    {
      return false;
    }
    else
    {
      Log.fatal(new ConfigurationException(ConfigurationException.PropertyNotFound + s), this);
    }

    return false;
  }

  /**
   * Abfragen int-Wert
   *
   * @param s Property-Key
   * @return int-Wert
   */
  public int getInt(String s)
  {
    String  property = super.getProperty(s);

    try
    {
      return Integer.valueOf(property).intValue();
    }
    catch (Exception e)
    {
      Log.fatal(new ConfigurationException(ConfigurationException.PropertyNotValid + s), this);

      return 0;
    }
  }

  /**
   * Abfragen long-Wert
   *
   * @param s Property-Key
   * @return long-Wert
   */
  public long getLong(String s)
  {
    String  property = super.getProperty(s);

    try
    {
      return Long.valueOf(property).longValue();
    }
    catch (Exception e)
    {
      Log.fatal(new ConfigurationException(ConfigurationException.PropertyNotValid + s), this);

      return 0;
    }
  }

  /**
   * Abfragen String
   *
   * @param s Property-Key
   * @return String
   */
  public String getString(String s)
  {
    String  property = super.getProperty(s);

    if (property == null)
    {
      Log.fatal(new ConfigurationException(ConfigurationException.PropertyIsEmpty + s), this);
    }

    property = StringUtil.trim( property );

    return property;
  }

  /**
   * Setzen Merker letzte nderung {@link #lastModified}
   *
   * @param l time codiert als long
   */
  public void setLastModified(long l)
  {
    lastModified = l;
  }

  /**
   * Abfragen Merker letzte nderung {@link #lastModified}
   *
   * @return time codiert als long
   */
  public long getLastModified()
  {
    return lastModified;
  }

}//end class CncProperties
