package de.cncdemo.actions;


import java.util.ArrayList;

import javax.servlet.http.*;

import de.cnc.*;
import de.cnc.util.*;
import de.cncdemo.formbeans.*;


/**
 * Action-Methoden. <br/>
 * In dieser Klasse sind alle Action-Methoden hinterlegt, die mit dem Prefix <b>Bestellung#</b> aufgerufen werden. <br/>
 * siehe {@link de.cnc.ActionMethodClassCache} und <b>Konzept_Actions.txt</b>.<br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner K&uuml;cker</a>
 * <br/>
 *
 * @see de.cnc.ActionMethodClassCache
 * @see CncServlet#getActionPackagePrefix
 * @author Heiner K&uuml;cker
 * @version $Id$
 */
public final class Bestellung {

  // *** alle Bestellungen bearbeiten ********************************************************************************

  /**
   * getBestellungenListe-Action . <br/>
   * Aufbau Vorlage Bestellung aus Artikel-Liste.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="Bestellung#getArtikelListe"&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param iPaIndex   Index-Nummer (Basis 0) eines indizierten Buttons oder Links
   * @param request    HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response   HttpServletResponse des zentralen Servlets
   * @param CncContext Kontext-Klasse des CnC-Frameworks, zum Beispiel mit den User-Information, fr sptere Erweiterungen vorgesehen
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static Object getBestellungenListe( //-
      final int iPaIndex, //--
      final HttpServletRequest request, //--
      final HttpServletResponse response, // --
      final CncContext cncContext)
  {
    HttpSession session = request.getSession(true);
    // Artikel-ArrayList holen
    ArrayList artikelArrList = (ArrayList) CncUtil.getAttribute(session, "ARTIKEL_LISTE", ArrayList.class);

    // Liste Bestellungen
    ArrayList bestellungenList = new ArrayList();

    for (int i = 0; i < 10; i++) {
      BestellungFormBean bestellung = new BestellungFormBean();

      bestellung.setNr(StringUtil.strzero(i + 1, 3));
      bestellung.setBezeichnung("Bestellung " + StringUtil.strzero(i + 1, 3));

      // Bestell-Positionen-ArrayList
      ArrayList bestellPosList = genBestellList(i, artikelArrList);

      bestellung.setPositionen(bestellPosList);

      bestellungenList.add(bestellung);
    }// end for

    session.setAttribute(SessionKeys.CNC_LIST_KEY, bestellungenList);

    return "";
  }// end method getBestellungenListe

  /**
   * Erzeugen Bestell-Positionen f&uuml;r eine Bestellung
   */
  private static ArrayList genBestellList(int iPaBestellNr, ArrayList arrListPaArtikel) {
    ArrayList bestellList = new ArrayList();

    for (int i = iPaBestellNr; i < arrListPaArtikel.size(); i += 10) {
      BestellPositionFormBean bestellPosition = new BestellPositionFormBean();

      bestellPosition.setArtikel((ArtikelFormBean) arrListPaArtikel.get(i));
      bestellPosition.setPosMenge(1);
      bestellList.add(bestellPosition);
    }// end for
    return bestellList;
  }// end method

  /**
   * setBestellPositionenToListKey-Action . <br/>
   * Liste Bestell-Positionen auf CNC_LIST_KEY legen.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="Bestellung#getArtikelListe"&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param iPaIndex Index-Nummer (Basis 0) eines indizierten Buttons oder Links
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String setBestellPositionenToListKey( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    HttpSession session = request.getSession(true);

    // Bestellung
    BestellungFormBean bestellung = (BestellungFormBean) CncUtil.getForm(session, BestellungFormBean.class);

    // Bestell-Positionen-ArrayList
    ArrayList bestellPosList = bestellung.getPositionen();

    session.setAttribute(SessionKeys.CNC_LIST_KEY, bestellPosList);

    return "";
  }// end method setBestellPositionenToListKey

  /**
   * newBestellung-Action . <br/>
   * Neue Bestellung erzeugen und auf CNC_FORM_KEY legen.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="Bestellung#getArtikelListe"&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param iPaIndex Index-Nummer (Basis 0) eines indizierten Buttons oder Links
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String newBestellung( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    HttpSession session = request.getSession(true);

    // Bestellungs-Liste holen
    ArrayList bestellungenList = (ArrayList) CncUtil.getAttribute(session, "BESTELLUNGEN_LISTE", ArrayList.class);

    if (bestellungenList == null) {
      throw new RuntimeException("ArrayList on Session-Key BESTELLUNGEN_LISTE not found");
    }// end if

    int iNeu = bestellungenList.size();

    // Bestellung
    BestellungFormBean newBestellung = new BestellungFormBean();

    newBestellung.setNr(StringUtil.strzero(iNeu + 1, 3));
    newBestellung.setBezeichnung("Bestellung " + StringUtil.strzero(iNeu + 1, 3));

    CncUtil.setForm(session, newBestellung);
    return "";
  }// end method newBestellung

  // *** eine Bestellung bearbeiten ********************************************************************************

  /**
   * saveIndex-Action . <br/>
   * Vermerken Position f&uuml;r Insert Artikel in Bestellung.<br/>
   * Benutzt in <b>procdure ???</b>.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="Bestellung#saveInsIndex"&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param iPaIndex Index-Nummer (Basis 0) eines indizierten Buttons oder Links
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String saveIndex( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    HttpSession session = request.getSession(true);

    session.setAttribute("BESTELLUNG_INDEX", "" + iPaIndex);
    return "";
  }// end method saveIndex

  /**
   * addBestellPos-Action . <br/>
   * Einf&uuml;gen (Insert) Position (Artikel) in Bestellung.<br/>
   * Benutzt in <b>procdure ???</b>.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="Bestellung#saveInsIndex"&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param iPaIndex Index-Nummer (Basis 0) eines indizierten Buttons oder Links
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String addBestellPos( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    HttpSession session = request.getSession(true);
    // gewhlten Artikel holen
    ArtikelFormBean artikelFormBean = (ArtikelFormBean) CncUtil.getForm(session, ArtikelFormBean.class);

    // Bestellung holen
    ArrayList bestellArrList = (ArrayList) CncUtil.getList(session, ArrayList.class);

    if (artikelFormBean != null && bestellArrList != null) {
      BestellPositionFormBean bestellPosition = new BestellPositionFormBean();

      bestellPosition.setArtikel(artikelFormBean);
      bestellPosition.setPosMenge(1);
      bestellArrList.add(bestellPosition);
    }// end if

    return "";
  }// end method addBestellPos

  /**
   * insBestellPos-Action . <br/>
   * Einf&uuml;gen (Insert) Position (Artikel) in Bestellung.<br/>
   * Benutzt in <b>procdure ???</b>.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="Bestellung#saveInsIndex"&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param iPaIndex Index-Nummer (Basis 0) eines indizierten Buttons oder Links
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String insBestellPos( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    HttpSession session = request.getSession(true);
    String strIndex = (String) session.getAttribute("BESTELLUNG_INDEX");

    if (strIndex == null) {
      throw new RuntimeException("Bestellung#insBestellPos: Insert-Index on Session-Key BESTELLUNG_INDEX not found");
    }// end if

    int iIndex = StringUtil.str2int(strIndex);

    // gewhlten Artikel holen
    ArtikelFormBean artikelFormBean = (ArtikelFormBean) CncUtil.getForm(session, ArtikelFormBean.class);

    // Bestellung holen
    ArrayList bestellArrList = (ArrayList) CncUtil.getList(session, ArrayList.class);

    if (artikelFormBean != null && bestellArrList != null && iIndex >= 0 && iIndex < bestellArrList.size())
    {

//@todo: hier Kontrolle auf schon vorhandenen Artikel einbauen

      BestellPositionFormBean bestellPosition = new BestellPositionFormBean();
      bestellPosition.setArtikel(artikelFormBean);
      bestellPosition.setPosMenge(1);
      bestellArrList.add(iIndex, bestellPosition);
    }// end if

    return "";
  }// end method insBestellPos

  /**
   * getArtikelFromPosition-Action . <br/>
   * Setzen Position (Artikel) in Bestellung.<br/>
   * Benutzt in <b>procdure ???</b>.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="Bestellung#saveInsIndex"&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param iPaIndex Index-Nummer (Basis 0) eines indizierten Buttons oder Links
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String getArtikelFromPosition( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    HttpSession session = request.getSession(true);

    // gewhlten Artikel holen
    BestellPositionFormBean bestellPosition = (BestellPositionFormBean) CncUtil.getForm(session,
        BestellPositionFormBean.class);
    ArtikelFormBean artikelFormBean = bestellPosition.getArtikel();

    CncUtil.setForm(session, artikelFormBean);
    return "";
  }// end method getArtikelFromPosition

  /**
   * setArtikel-Action . <br/>
   * Setzen Position (Artikel) in Bestellung.<br/>
   * Benutzt in <b>procdure ???</b>.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="Bestellung#saveInsIndex"&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param iPaIndex Index-Nummer (Basis 0) eines indizierten Buttons oder Links
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String setPosArtikel( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    HttpSession session = request.getSession(true);
    String strIndex = (String) session.getAttribute("BESTELLUNG_INDEX");

    if (strIndex == null) {
      throw new RuntimeException("Bestellung#setPosArtikel: Insert-Index on Session-Key BESTELLUNG_INDEX not found");
    }// end if

    int iIndex = StringUtil.str2int(strIndex);

    // gewhlten Artikel holen
    ArtikelFormBean artikelFormBean = (ArtikelFormBean) CncUtil.getForm(session, ArtikelFormBean.class);

    // Bestellung holen
    ArrayList bestellArrList = (ArrayList) CncUtil.getList(session, ArrayList.class);

    if (artikelFormBean != null && bestellArrList != null && iIndex >= 0 && iIndex < bestellArrList.size()) {
      BestellPositionFormBean bestellPosition = (BestellPositionFormBean) bestellArrList.get(iIndex);

      bestellPosition.setArtikel(artikelFormBean);
    }// end if

    return "";
  }// end method setPosArtikel

  // *** Artikel ****************************************************************************************

  /**
   * getArtikelListe-Action . <br/>
   * Benutzt in <b>procdure artikelstamm</b>.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="Bestellung#getArtikelListe"&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param iPaIndex Index-Nummer (Basis 0) eines indizierten Buttons oder Links
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String getArtikelListe( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    String[] itemStrArr = { "Hut", "M&uuml;tze", "Jacke", "Mantel", "Pullover", "Hemd", "T-Shirt", "Sweat-Shirt", "Hose" }; // , "Shorts" , "Unterhemd" , "Slip" , "Paar Socken" } ;
    String[] colorStrArr = { "schwarz", "blau", "rot", "gr&uuml;n", "gelb" }; // , "braun" , "grau" , "beige" , "weiss"
    String[] sizeStrArr = { "S", "M", "L", "XL" }; // , "XXL" , "XXXL" } ;

    ArrayList arrList = new ArrayList();
    int iLfd = 0;

    for (int i1 = 0; i1 < itemStrArr.length; i1++) {
      for (int i2 = 0; i2 < colorStrArr.length; i2++) {
        for (int i3 = 0; i3 < sizeStrArr.length; i3++) {
          ArtikelFormBean artikelFormBean = new ArtikelFormBean();

          iLfd++;
          artikelFormBean.setArtNr(StringUtil.strzero(iLfd, 10));
          artikelFormBean.setBez(itemStrArr[ i1 ] + " " + colorStrArr[ i2 ] + " " + sizeStrArr[ i3 ]);
          artikelFormBean.setMenge(iLfd);
          artikelFormBean.setMe("St&uuml;ck");
          arrList.add(artikelFormBean);
        }// end for
      }// end for
    }// end for

    HttpSession session = request.getSession(true);

    session.setAttribute(SessionKeys.CNC_LIST_KEY, arrList);

    return "";
  }// end method getArtikelListe

  /**
   * validArtikelAdd-Action . <br/>
   * Pr&uuml;fen der Eingaben bei Anlegen Artikel.<br/>
   * Benutzt in <b>procdure artikelstamm</b>.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String validArtikelAdd( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    HttpSession session = request.getSession(true);
    ArtikelFormBean artikelFormBean = (ArtikelFormBean) CncUtil.getForm(session, ArtikelFormBean.class);

    // Pr&uuml;fen, ob Artikel-Nummer eingegeben
    if (StringUtil.empty(artikelFormBean.getArtNr())) {
      CncUtil.setWarnMessage(session, "Bitte Artikel-Nummer Eingeben");
    } // Pr&uuml;fen, ob Artikel-Nummer in num. Format
    else if (StringUtil.str2intTolerant(artikelFormBean.getArtNr()) < 1) {
      CncUtil.setWarnMessage(session, "Bitte Artikel-Nummer Korrigieren");
    } // Pr&uuml;fen, ob Artikel-Nummer schon vergeben
    else if (!isUnUsedArtikelnummmer(session)) {
      CncUtil.setWarnMessage(session, "Bitte Artikel-Nummer Korrigieren, Artikel-Nummer schon verwendet");
    } // Pr&uuml;fen, ob Artikel-Bezeichnung eingegeben
    else if (StringUtil.empty(artikelFormBean.getBez())) {
      CncUtil.setWarnMessage(session, "Bitte Bezeichnung Eingeben");
    } // Pr&uuml;fen, ob Mengen-Einheit eingegeben
    else if (StringUtil.empty(artikelFormBean.getMe())) {
      CncUtil.setWarnMessage(session, "Bitte Mengen-Einheit Eingeben");
    } else {
      // Alles ok
      session.setAttribute("FORM_VALID", "OK");
    }// end if

    return "";
  }// end method validArtikelAdd

  /**
   *
   */
  private static boolean isUnUsedArtikelnummmer(HttpSession session) {
    boolean bReturn = false;
    ArtikelFormBean artikelFormBean = (ArtikelFormBean) CncUtil.getForm(session, ArtikelFormBean.class);
    int iArtNr = StringUtil.str2intTolerant(artikelFormBean.getArtNr());

    // Log.debug( "iArtNr " + iArtNr );

    if (iArtNr < 1) {
      return bReturn;
    }// end if

    // Feststellen, ob Art-Nr schon vergeben
    ArrayList arrList = (ArrayList) CncUtil.getList(session, ArrayList.class);

    int iMax = 0;

    for (int i = 0; i < arrList.size(); i++) {
      ArtikelFormBean maxArtikelFormBean = (ArtikelFormBean) arrList.get(i);
      int iMaxLaufArtNr = StringUtil.str2intTolerant(maxArtikelFormBean.getArtNr());

      if (iMaxLaufArtNr > iMax) {
        iMax = iMaxLaufArtNr;
      }// end if

      // Log.debug( "iMax " + iMax ) ;
    }// end for

    // Log.debug( "iMax " + iMax ) ;
    // Log.debug( "iArtNr " + iArtNr ) ;
    if (iArtNr <= iMax) {
      return bReturn;
    }// end if

    artikelFormBean.setArtNr(StringUtil.strzero(iArtNr, 10));
    bReturn = true;
    return bReturn;
  }// end method

  /**
   * validArtikelEdit-Action . <br/>
   * Pr&uuml;fen der Eingaben bei Anlegen Artikel.<br/>
   * Benutzt in <b>procdure artikelstamm</b>.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String validArtikelEdit( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    HttpSession session = request.getSession(true);
    ArtikelFormBean artikelFormBean = (ArtikelFormBean) CncUtil.getForm(session, ArtikelFormBean.class);

    // Pr&uuml;fen, ob Artikel-Bezeichnung eingegeben
    if (StringUtil.empty(artikelFormBean.getBez())) {
      CncUtil.setWarnMessage(session, "Bitte Bezeichnung Eingeben");
    } // Pr&uuml;fen, ob Mengen-Einheit eingegeben
    else if (StringUtil.empty(artikelFormBean.getMe())) {
      CncUtil.setWarnMessage(session, "Bitte Mengen-Einheit Eingeben");
    } else {
      // Alles ok
      session.setAttribute("FORM_VALID", "OK");
    }// end if

    return "";
  }// end method validArtikelEdit

  /**
   * setArtikelFokus-Action . <br/>
   * Fokus auf vorher gewhlten Artikel setzen.<br/>
   * Das Artikel-Objekt des vorher gewhlten Artikels liegt auf dem Session-Key "artikel" (param artikel).<br/>
   * Benutzt in <b>procdure artikelstamm</b>.<br/>
   * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
   *
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String setArtikelFokus( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    HttpSession session = request.getSession(true);
    ArtikelFormBean wahlArtikelFormBean = (ArtikelFormBean) CncUtil.getAttribute(session, "artikel", ArtikelFormBean.class);

    if (wahlArtikelFormBean != null) {
      int iWahlArtNr = StringUtil.str2intTolerant(wahlArtikelFormBean.getArtNr());
      ArrayList arrList = (ArrayList) CncUtil.getList(session, ArrayList.class);

      for (int i = 0; i < arrList.size(); i++) {
        ArtikelFormBean maxArtikelFormBean = (ArtikelFormBean) arrList.get(i);
        int iLaufArtNr = StringUtil.str2intTolerant(maxArtikelFormBean.getArtNr());

        if (iLaufArtNr == iWahlArtNr) {
          BrowseControl.staticSetFocus(session, i);
          break;
        }// end if
      }// end for
    }// end if

    return "";
  }// end method setArtikelFokus

  /**
   * isWahlArtikel-Action . <br/>
   * Abfrage, ob der auf Session-Key CNC_FORM_KEY liegende Artikel mit dem auf Session-Key CNC identisch ist.<br/>
   * Das Artikel-Objekt des vorher gewhlten Artikels liegt auf dem Session-Key "artikel" (param artikel).<br/>
   * Benutzt in <b>procdure artikelstamm</b>.<br/>
   * Der Aufruf erfolgt vom <b>artikelliste.jsp</b> zur Anzeige des in der bearbeiteten St&uuml;ckliste ausgewhlten
   * Artikels durch einen Pfeil <b>&gt;&gt;&gt;</b>.<br/>
   *
   * @param request  HttpServletRequest  des zentralen Servlets f&uuml;r Zugriff auf Session und so weiter
   * @param response HttpServletResponse des zentralen Servlets
   *
   * @return String-R&uuml;ckgabewert f&uuml;r eventuelle Auswertung
   */
  public static String isWahlArtikel( //-
  final int iPaIndex, //--
  final HttpServletRequest request, //--
  final HttpServletResponse response, // --
  final CncContext cncContext) {
    String retStr = "false";
    HttpSession session = request.getSession(true);
    // ArtikelFormBean iterateArtikelFormBean = (ArtikelFormBean) CncUtil.getAttribute( session, SessionKeys.CNC_LIST_ELEMENT_KEY , ArtikelFormBean.class ) ;
    Object iterateArtikelFormBean = session.getAttribute(SessionKeys.CNC_LIST_ELEMENT_KEY);

    // bei Bestellposition den Artikel aus der Bestell-Position herausholen
    if (iterateArtikelFormBean instanceof BestellPositionFormBean) {
      iterateArtikelFormBean = ((BestellPositionFormBean) iterateArtikelFormBean).getArtikel();
    }

    Object formArtikelFormBean = session.getAttribute("CNC_FORM_KEY");

    // unmittelbarer Objekt-Vergleich
    if (formArtikelFormBean == iterateArtikelFormBean) {
      retStr = "true";
    }// end if

    return retStr;
  }// end method setArtikelFokus

}// end class Bestellung
