package de.cncdemo.actions;

import java.io.*;
import java.math.BigDecimal;
import java.util.*;

import javax.servlet.http.*;

import de.cnc.*;
import de.cnc.expression.*;
import de.cnc.expression.exceptions.ExpressionEvaluationException;
import de.cnc.implicit.TreeActions;
import de.cnc.treecontrol.TreeNode;
import de.cnc.util.*;
import de.cncdemo.formbeans.ListeActionFormBean;
import de.cncdemo.generic.Constants;

/**
 * Allgemeine Action-Methoden. <br/>
 * In dieser Klasse sind alle Action-Methoden hinterlegt, die ohne Prefix aufgerufen werden. <br/>
 * siehe {@link de.cnc.ActionMethodClassCache} und <b>Konzept_Actions.txt</b>.<br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @see de.cnc.ActionMethodClassCache
 * @see CncServlet#getActionPackagePrefix
 * @author Heiner Kcker
 * @version $Id$
 */
public final class CncDemoDefaultActions
{

    // *** generelle Actions ******************************************************************************

    /**
     * -Action . <br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String listElementToForm(//-
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        // HttpSession session = request.getSession( true ) ;
        Log.debug("listElementToForm iPaIndex " + iPaIndex);
        CncUtil.setListElementToForm(request, iPaIndex);
        return "";
    } // end method listElementToForm

    /**
     * -Action . <br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String addFormToList(//-
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        CncUtil.addFormToList(request);
        return "";
    } // end method addFormToList

    /**
     * -Action . <br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String goLastPage(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        BrowseControl.staticGoLast(request.getSession(true));
        return "";
    } // end method addFormToList

    /**
     * -Action . <br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static Object deleFormListElement(//--
        final int iPaIndex, //--
        final HttpServletRequest request, //--
        final HttpServletResponse response, // --
        final CncContext cncContext)
    {
        HttpSession session = request.getSession(true);

        ArrayList arrList =
            (ArrayList) session.getAttribute(SessionKeys.CNC_LIST_KEY);
        Object formObject = session.getAttribute(SessionKeys.CNC_FORM_KEY);
        boolean bOk = false;

        if (arrList != null)
        {
            int iIndex = arrList.indexOf(formObject);

            if (iIndex >= 0)
            { // -1 steht fr Fehler
                arrList.remove(iIndex);
                CncUtil.setInfoMessage(session, "Objekt gelscht");
                bOk = true;
            } // end if
        } // end if

        if (!bOk)
        {
            CncUtil.setErrMessage(session, "Lschen fehlgeschlagen");
        } // end if

        return "";
    } // end method addFormToList

    /**
     * ClearSession-Action . <br/>
     * Lschen der nicht mehr bentigten Variablen.<br>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String clearSession(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        HttpSession session = request.getSession(true);

        while (true)
        {
            Enumeration en = session.getAttributeNames();

            String s = null;

            // ber nicht zu lschende Elemente hinweg laufen
            while (en.hasMoreElements())
            {
                s = (String) en.nextElement();
                // System.out.println("s:" + s);
                // if ((!"USER".equals(s)) && (!s.startsWith("CNC_"))) {
                if ((!"USER".equals(s))
                    && (!"CNC_STATUS".equals(s))
                    && (!"CNC_SINGLE_STEP_DEBUG".equals(s))
                    && (!"CNC_FLOWSTACK_KEY".equals(s))
                    && (!"CNC_MESSAGE_KEY".equals(s)))
                {
                    break;
                }
                s = null;
            }
            if (s != null)
            {
                // System.out.println("remove s:" + s);
                session.removeAttribute(s);
            }
            if (!en.hasMoreElements())
            {
                break;
            }
        }

        return "";
    } // end method clearSession

    // Actions fr Logon ******************************************************************************

    /**
     * logon-Action . <br/>
     * Benutzt im logon-bereich der <b>procdure main</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String logon(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
//Log.debug( "JavaScript is on: " + StringUtil.str2bool( request.getParameter( RequestKeys.CNC_JAVASCRIPT ) ) );

        String strUserId = (String) request.getParameter("userId");
        String strPassword = (String) request.getParameter("password");
        HttpSession session = request.getSession(true);

        if (session.getAttribute(Constants.USER) instanceof String)
        {
            // User-Objekt ist schon in Session abgelegt
            return "";
        } // end if

        if (strUserId == null || strPassword == null)
        {
            CncUtil.setInfoMessage(session, "Bitte User-Id und Passwort eingeben");
        }
        else if (strUserId == null)
        {
            CncUtil.setWarnMessage(session, "User-Id unbekannt");
        }
        else if (StringUtil.empty(strPassword))
        {
            CncUtil.setWarnMessage(session, "Passwort falsch");
        }
        else
        { // User-ID und Passwort ok
            // USER-Objekt in Session legen
            Log.debug("USER-Objekt in Session legen");
            session.setAttribute(Constants.USER, "USER"); // user ) ;
            Log.debug("" + session.getAttribute(Constants.USER));

        }
        return "";
    } // end method logon

    // Actions fr Suchen ******************************************************************************

    /**
     * suchen-Action . <br/>
     * Benutzt in <b>procdure suchen</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String suchen(// --
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        // der Parameter wahl wird in eine int und wieder zurck in einen String (der ist jetzt unbedingt numerisch) umgewandelt
        // der numerische String (Trefferanzahl) wird in der Session gespeichert
        String suchStr = (String) request.getParameter("wahl");

        Log.debug("wahl=" + suchStr);
        int suchInt = StringUtil.str2int(suchStr);
        suchStr = "" + suchInt;
        HttpSession session = request.getSession(true);

        session.setAttribute("such_treffer", new Integer(suchInt));
        return "";
    } // end method

    // Actions fr Listen-Editor ***********************************************************************

    /**
     * setListeInSession-Action . <br/>
     * Benutzt in <b>procdure liste_edit</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String setListeInSession(//--
        final int iPaIndex, //--
        final HttpServletRequest request, //--
        final HttpServletResponse response, // --
        final CncContext cncContext)
    {
        ArrayList arrList = new ArrayList();

        for (int i = 0; i < 4; i++)
        {
            ListeActionFormBean listeActionFormBean = new ListeActionFormBean();

            listeActionFormBean.setTxt("" + (i + 1));
            arrList.add(listeActionFormBean);
        } // end for

        HttpSession session = request.getSession();

        session.setAttribute(SessionKeys.CNC_LIST_KEY, arrList);

        return "";
    } // end method set_liste_in_session

    /**
     * add-Action . <br/>
     * Benutzt in <b>procdure liste_edit</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String add(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        ArrayList arrList =
            (ArrayList) (request.getSession(true)).getAttribute(
                SessionKeys.CNC_LIST_KEY);

        ListeActionFormBean listeActionFormBean = new ListeActionFormBean();

        arrList.add(listeActionFormBean);

        return "";
    } // end method add

    /**
     * ins-Action . <br/>
     * Benutzt in <b>procdure liste_edit</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String ins(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        ArrayList arrList =
            (ArrayList) (request.getSession(true)).getAttribute(
                SessionKeys.CNC_LIST_KEY);

        ListeActionFormBean listeActionFormBean = new ListeActionFormBean();

        arrList.add(iPaIndex, listeActionFormBean);
        return "";
    } // end method ins

    /**
     * del-Action . <br/>
     * Benutzt in <b>procdure liste_edit</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String del(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        ArrayList arrList =
            (ArrayList) (request.getSession(true)).getAttribute(
                SessionKeys.CNC_LIST_KEY);

        arrList.remove(iPaIndex);
        return "";
    } // end method del

    /**
     * mvup-Action . <br/>
     * Benutzt in <b>procdure liste_edit</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String mvup(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        ArrayList arrList =
            (ArrayList) (request.getSession(true)).getAttribute(
                SessionKeys.CNC_LIST_KEY);

        ArrayListUtil.moveUp(arrList, iPaIndex);
        return "";
    } // end method mvup

    /**
     * mvdn-Action . <br/>
     * Benutzt in <b>procdure liste_edit</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String mvdn(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        ArrayList arrList =
            (ArrayList) (request.getSession(true)).getAttribute(
                SessionKeys.CNC_LIST_KEY);

        ArrayListUtil.moveDn(arrList, iPaIndex);
        return "";
    } // end method mvdn

    /**
     * save-Action . <br/>
     * Benutzt in <b>procdure liste_edit</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String save(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        // Abspeichern in Datenbank nicht ausprogrammiert
        HttpSession session = request.getSession(true);

        session.removeAttribute(SessionKeys.CNC_LIST_KEY);
        return "";
    } // end method save

    /**
     * Ablegen einer Test-Liste zum Sortieren mit impliziten Actions auf den Sessionkey CNC_LIST_KEY
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String getSortList(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        ArrayList arrList = new ArrayList();

        for (int i = 0; i <= 9; i++)
        {
            arrList.add(new Integer(i));
        }

        HttpSession session = request.getSession(true);

        session.setAttribute(SessionKeys.CNC_LIST_KEY, arrList);
        return "";
    } // end method getSortList

    // Actions fr Expression-Parser ***********************************************************************

    /**
     * Expression-Parser-Action . <br/>
     * Benutzt in <b>procdure expression</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String expression(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        HttpSession session = request.getSession(true);

        session.removeAttribute("expressionValue"); // Reinigen Ergebniswert

        try
        {
            String expressionStr = (String) request.getParameter("expression");

            if (expressionStr != null)
            {
                // Encoden der Expression, weil sonst aus set("xxx",yyy) nur set( wurde wegen Quotes (Anfhrungsstrichen)
                session.setAttribute(
                    "expression",
                    StringUtil.encodeHtml(expressionStr));
            }

            // Laufzeit-Umgebung mit Variablen vorbereiten
            StandaloneRuntimeEnvironment runEnv = getOrCreateRuntEnv(session);

            Object objEval = null;

            // Umlenkung der Funktionen print und println der EL auf session.message
            ByteArrayOutputStream barStream = new ByteArrayOutputStream();

            de.cnc.expression.util.OutPrintStream.setOut(
                new PrintStream(barStream));

            // TimeOut setzen, um Verklemmen der Webapp zu vermeiden
            de.cnc.expression.util.TimeOut.setStart((new Date()).getTime());
            de.cnc.expression.util.TimeOut.setMaxTime(30000); // 30 Sekunden

            objEval = Expression.parseStrict(expressionStr, 0, 0).eval(runEnv);

            de.cnc.expression.util.OutPrintStream.getOut().close();
            de.cnc.expression.util.OutPrintStream.setOut(null);

            CncUtil.setInfoMessage(
                session,
                "<pre>"
                    + StringUtil.encodeBreakToTag(
                        StringUtil.encodeHtml(barStream.toString()))
                    + "</pre>");

            CncUtil.setInfoMessage(
                session,
                StringUtil.encodeBreakToTag("Ergibt: " + objEval));

            // Ergebnis in Session legen
            if (objEval == null)
            {
                session.removeAttribute("expressionValue");
            }
            else
            {
                session.setAttribute("expressionValue", objEval);
            }

        }
        catch (Exception exc)
        {
            CncUtil.setErrMessage(
                session,
                "<pre>"
                    + StringUtil.encodeBreakToTag(exc.getMessage())
                    + "</pre>");
        }

        return null;
    }

    /**
     * Expression-Parser-Action fr Test Index-Operator. <br/>
     * Benutzt in <b>procdure expression</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static String setArrListMap(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        HttpSession session = request.getSession(true);

        try
        {
            // Laufzeit-Umgebung mit Variablen vorbereiten
            StandaloneRuntimeEnvironment runEnv = getOrCreateRuntEnv(session);

            // Test Array, List und Map geschachtelt
            Object[] objArr = new Object[1];

            ArrayList arrList = new ArrayList();

            objArr[0] = arrList;

            HashMap hashMap = new HashMap();

            arrList.add(hashMap);

            hashMap.put("A", "A");

            runEnv.setVariable("arrListMap", objArr);
        }
        catch (Exception exc)
        {
            CncUtil.setErrMessage(
                session,
                "<pre>"
                    + StringUtil.encodeBreakToTag(exc.getMessage())
                    + "</pre>");
        }

        return null;
    }

    /**
     * Laufzeit-Umgebung mit Variablen vorbereiten
     * 
     * @param session
     * @return
     * @throws ExpressionEvaluationException
     */
    private static StandaloneRuntimeEnvironment getOrCreateRuntEnv(HttpSession session)
        throws ExpressionEvaluationException
    {
        // Laufzeit-Umgebung mit Variablen vorbereiten
        StandaloneRuntimeEnvironment runEnv =
            (StandaloneRuntimeEnvironment) session.getAttribute(
                "expressionRuntimeEnvironment");

        if (runEnv == null)
        {
            runEnv = new StandaloneRuntimeEnvironment();
            // Setzen Anfangsvariable
            runEnv.setVariable("var1", new BigDecimal(1));
            runEnv.setVariable("var2", new BigDecimal(2));
            runEnv.setVariable(
                "arr1",
                new String[] { "str0", "str1", "str2", "str3", "str4" });

            session.setAttribute("expressionRuntimeEnvironment", runEnv);
        }
        return runEnv;
    } // end method

    // Actions fr Tree-Control ***********************************************************************

    /**
     * Tree Erzeugen. <br/>
     * Benutzt in <b>procdure tree</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static Object createTree(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        HttpSession session = request.getSession(true);

        TreeNode root = new TreeNode("Node /");
        for (int i1 = 0; i1 < 8; i1++)
        {
            TreeNode sub1Node = new TreeNode("Node " + i1);
            root.add( sub1Node );
            for (int i2 = 0; i2 < 8; i2++)
            {
                TreeNode sub2Node = new TreeNode("Node " + i1 + " " + i2);
                sub1Node.add( sub2Node );
                for (int i3 = 0; i3 < 8; i3++)
                {
                    TreeNode sub3Node = new TreeNode("Node " + i1 + " " + i2 + " " + i3);
                    sub2Node.add( sub3Node );
                    for (int i4 = 0; i4 < 8; i4++)
                    {
                        TreeNode sub4Node = new TreeNode("Node " + i1 + " " + i2 + " " + i3 + " " + i4);
                        sub3Node.add( sub4Node );
                    }
                }
            }
        }        
        
        //session.setAttribute( SessionKeys.CNC_FORM_KEY, root);
        return root;
    }

    /**
     * Kopieren der Sub-Nodes der adressierten Node des Tree
     * auf den {@link SessionKeys#CNC_LIST_KEY} . <br/>
     * 
     * Benutzt in <b>procdure tree</b>.<br/>
     * Der Aufruf erfolgt vom {@link de.cnc.FlowControl} beim Befehl <b>&lt:action name="..."&gt;</b> in <b>flow.cnc</b>.<br/>
     *
     * @param request  HttpServletRequest  des zentralen Servlets fr Zugriff auf Session und so weiter
     * @param response HttpServletResponse des zentralen Servlets
     *
     * @return String-Rckgabewert fr eventuelle Auswertung
     */
    public static Object copyNodes(//--
    final int iPaIndex, //--
    final HttpServletRequest request, //--
    final HttpServletResponse response, // --
    final CncContext cncContext)
    {
        final String pathStr = request.getParameter("elemPath"); //@todo: Konstante

        HttpSession session = request.getSession();

        final TreeNode root = (TreeNode) session.getAttribute( SessionKeys.CNC_TREE_KEY );

        final TreeNode node = TreeActions.getSubNode(root, pathStr);

        session.setAttribute( SessionKeys.CNC_FORM_KEY, node);
        
        ArrayList list = node.getNodeList();

        session.setAttribute( SessionKeys.CNC_LIST_KEY, list);
        return null;
    }

} // end class CncDemoDefaultActions
