package de.cncdemo.servlet;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;

import de.cnc.CncServlet;
import de.cnc.util.Log;
import de.cncdemo.generic.Constants;

/**
 * Zentrales und einziges Servlet der CnC-Demo-Applikation . <br/>
 * Dieses von {@link CncServlet} geerbte Servlet schleust die applikations-spezifischen Dinge ins CnC-System ein. <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a><br/>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class CncDemoServlet extends CncServlet
{

    /**
     * Initializes the Servlet. <br/>
     */
    public void init() throws ServletException
    {
        super.init();
    } // end method init

    /**
     * Diese Methode gibt das Package der Action-Methoden-Klassen zurck . <br/>
     * Sie muss vom zentralen Servlet der CnC-Applikation implementiert werden.<br/>
     * Siehe Konzept_Actions.txt .<br/>
     *
     * @return String mit Package-Angabe
     */
    public String getActionPackage()
    {
        return "de.cncdemo.actions.";
    } // end method getActionPackagePrefix

    /**
     * Diese Methode gibt den Namen der Default-Action-Methoden-Klasse zurck . <br/>
     * Sie muss vom zentralen Servlet der CnC-Applikation implementiert werden.<br/>
     * Siehe Konzept_Actions.txt .<br/>
     *
     * @return String mit Package-Angabe
     */
    public String getDefaultActionClassName()
    {
        return "CncDemoDefaultActions";
    } // end method getDefaultActionClassName

    /**
     * Diese Methode gibt den Prefix der impliziten Actions der CnC-Anwendung zur&uuml;ck . <br/>
     * Sie muss vom zentralen Servlet der CnC-Applikation implementiert werden.<br/>
     * Siehe Konzept_Actions.txt .<br/>
     *
     * @return String mit Package-Angabe
     */
    public String getAppImplActionPackage()
    {
        return "de.cncdemo.implicit.";
    } // end method getImplActionPackagePrefix

    /**
     * Diese Methode gibt den Package-Prefix der Action-Methoden-Klassen f&uuml;r implizite Actions zur&uuml;ck . <br/>
     * Sie muss vom zentralen Servlet der Cnc-Applikation implementiert werden.<br/>
     * Siehe Konzept_Actions.txt .<br/>
     *
     * @return String mit Prefix-Angabe
     */
    public String getAppImplActionPrefix()
    {
        return "CNCDEMO_IMPL_ACTION_";
    } // end method getAppImplActionPrefix

    /**
     * Prfen der Berechtigung des aktuellen Users zum Ausfhren der CnC-Prozedur . <br/>
     *
     * @param request hierber hat die Methode Zugriff auf die Session, aus der sich die Methode das User-Objekt holen kann
     * @param procName Name der flow.cnc-procedure, fr die das Zugriffsrecht geprft werden soll
     *
     * @return ob berechtigt
     */
    public boolean hasUserPermissionForProcedure(
        HttpServletRequest request,
        String procName)
    {
        return true;
        // Log.debug( "hasUserPermissionForProcedure " + procName ) ;
        // if ( ( request.getSession( true ) ).getAttribute( Constants.USER ) == null ) {
        // Log.debug( "no User-Object in Session" ) ;
        // return false ;
        // }//end if
        //
        // if ( "main".equals( procName ) || "blaettern".equals( procName ) ) {
        // Log.debug( "ok" ) ;
        // return true ;
        // }//end if
        // Log.debug( "not ok" ) ;
        // CncUtil.setMessage( request , "Sie habe keine Berechtigung fr diesen Bereich" ) ;
        // return false ;
    } // end method

    /**
     * Prfen der Berechtigung des aktuellen Users zum Ausf&uuml;hren der Action-Methode . <br/>
     *
     * @param request hierber hat die Methode Zugriff auf die Session, aus der sich die Methode das User-Objekt holen kann
     * @param procName Name der Action-Methode, fr die das Zugriffsrecht geprft werden soll
     *
     * @return ob berechtigt
     */
    public boolean hasUserPermissionForAction(
        HttpServletRequest request,
        String actionName)
    {
        return true;
        // Log.debug( "hasUserPermissionForAction " + actionName ) ;
        // if ( ( request.getSession( true ) ).getAttribute( Constants.USER ) == null ) {
        // Log.debug( "no User-Object in Session" ) ;
        // return false ;
        // }//end if
        // Log.debug( "ok" ) ;
        // return true ;
    } // end method hasUserPermissionForAction

    /**
     * Pr&uuml;fen der Berechtigung des aktuellen Users zum Ausf&uuml;hren der impliziten Action . <br/>
     *
     * @param request hier&uuml;ber hat die Methode Zugriff auf die Session, aus der sich die Methode das User-Objekt holen kann
     * @param procName Name der Action-Methode, f&uuml;r die das Zugriffsrecht gepr&uuml;ft werden soll
     *
     * @return ob berechtigt
     */
    public boolean hasUserPermissionForImplAction(
        HttpServletRequest request,
        String actionName)
    {
        return true;
        // Log.debug( "hasUserPermissionForAction " + actionName ) ;
        // if ( ( request.getSession( true ) ).getAttribute( Constants.USER ) == null ) {
        // Log.debug( "no User-Object in Session" ) ;
        // return false ;
        // }//end if
        // Log.debug( "ok" ) ;
        // return true ;
    } // end method hasUserPermissionForImplAction

    /**
     * Prfen der Berechtigung des aktuellen Users zum Ausfhren der JSP . <br/>
     *
     * @param request hierber hat die Methode Zugriff auf die Session, aus der sich die Methode das User-Objekt holen kann
     * @param procName Name der JSP, fr die das Zugriffsrecht geprft werden soll
     *
     * @return ob berechtigt
     */
    public boolean hasUserPermissionForJsp(
        HttpServletRequest request,
        String jspName)
    {
        return true;
        // Log.debug( "hasUserPermissionForJsp " + jspName ) ;
        // if ( ( request.getSession( true ) ).getAttribute( Constants.USER ) == null ) {
        // Log.debug( "no User-Object in Session" ) ;
        // return false ;
        // }//end if
        //
        // if ( "/mainmenu.jsp".equals( jspName ) || jspName.startsWith( "/blaettern" ) ) {
        // Log.debug( "ok" ) ;
        // return true ;
        // }//end if
        // Log.debug( "not ok" ) ;
        // CncUtil.setMessage( request , "Sie habe keine Berechtigung fr diesen Bereich" ) ;
        // return false ;
    } // end method

    /**
     * die Implementationsklasse von CncServlet liefert einen alternativen
     * forward, falls keine Berechtigung fr die aktuelle Prozedur, JSP oder Action-Methode vorliegt . <br/>
     * Dies sollte die login.jsp oder mainmenu.jsp sein.<br/>
     *
     * @param request hierber hat die Methode Zugriff auf die Session, aus der sich die Methode das User-Objekt holen kann
     * @param strPaSection String mit Namen der Prozedur, JSP oder Action-Methode, fr welche die Berechtigung verweigert wurde.<br/>
     *                     Durch Vergleich mit String.startsWith() kann festgestellt werden, um welchen Applikationsteil es sich handelt.
     *
     * @return String mit alternativem forward
     */
    public String getAlternateForward(
        HttpServletRequest request,
        String strPaSection)
    {
        Log.debug("getAlternateForward " + strPaSection);
        if ((request.getSession(true)).getAttribute(Constants.USER) == null)
        {
            Log.debug("no User-Object in Session");
            return "/logon.jsp";
        } // end if
        return "/mainmenu.jsp";
    } // end method getAlternateForward

    /**
     * Name der Applikation . 
     * 
     * @see de.cnc.CncServlet#getApplicationName()
     */
    public String getApplicationName()
    {
        return "CncDemo";
    }

} // end class CncDemoServlet
