package de.cncdemo.tags;


import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.jsp.JspException;
import javax.servlet.jsp.tagext.TagSupport;

import de.cnc.*;
import de.cnc.util.*;


/**
 * Tag zum Prfen der Berechtigung des Users zum Betreten der aktuellen JSP-Seite . <br/>
 * Dieses Tag mu auf allen JSP-Seiten des Projektes CnC eingesetzt werden,<br/>
 * auer <b>/logon.jsp</b> (sonst gibt es eine Endlosschleife) <br/>
 * und auer <b>/cncdebug/singlestepdebug.jsp</b> (Achtung Sicherheitsloch, wenn in Applikation verwendet). <br/>
 * Wenn gar keine Berechtigung vorliegt, wird zum Logon verzweigt.<br/>
 * Wenn fr den speziellen Bereich keine Berechtigung vorliegt(auer GENERIC fr bersichtseite/Hauptmen),<br/>
 * wird zum Hauptmen verzweigt.<br/>
 * Benutzt die Methoden<br/>
 * {@link CncServlet#hasUserPermissionForJsp} und<br/>
 * {@link CncServlet#getAlternateForward}.<br/>
 * <br/>
 * Verwendung: <b>&lt;cncdemo:checkuser forsection=""/&gt;</b> (mainmenu.jsp)<br/>
 * Verwendung: <b>&lt;cncdemo:checkuser forsection="name_applikationsbereich"/&gt;</b><br/><br/>
 * siehe Datei <b>cncdemo.tld</b> . <br/>
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class CheckUserTag extends TagSupport {

  /**
   * Variable zum Speichern des Applikations-Teils,
   * fr welchen der Benutzer zugelassen sein mu&szlig;,
   * um die JSP-Seite zu betreten.
   */
  private String forsection = "";

  /**
   * get forsection
   *
   * @return forsection
   */
  public String getForsection() {
    return forsection;
  }// end method getForsection

  /**
   * set forsection . <br/>
   * Setzen des Applikations-Teils, fr welchen der Benutzer zugelassen sein mu&szlig;.<br/>
   * (wird von Servletengine beim Parsen des Tags erledigt).<br/>
   * zwingend erforderlich
   *
   * @param forsection new Value
   */
  public void setForsection(String newForsection) {
    this.forsection = newForsection;
  }// end method setForsection

  /**
   * Defer our checking until the end of this tag is encountered.
   *
   * @exception JspException if a JSP exception has occurred
   */
  public int doStartTag()
      throws JspException {
    return (EVAL_PAGE);
  } // end method doStartTag

  /**
   * Prfung Berechtigung und entsprechende Weiterleitung . <br/>
   * Berechtigung ok: Abarbeitung der Seite fortsetzen<br/>
   * Berechtigung nicht ok: Weiterleitung zur {@link CncDemoServlet#getAlternateForward}<br/>
   *
   * @return Code fr Weiterbearbeiten Seite oder Abbruch
   *
   * @exception JspException if a JSP exception has occurred
   */
  public int doEndTag()
      throws JspException {
    int                 retValue = EVAL_PAGE; // Rckgabewert fr aktuelle Seite weiter abarbeiten
    HttpServletRequest  request = (HttpServletRequest) pageContext.getRequest();
    // HttpSession         session = pageContext.getSession();

    String jspName = CncUtil.getCurrentJspName(request);

    CncServlet cncServlet = CncServletAnchor.getCncServlet();
    String forwardAction = cncServlet.getAlternateForward(request, jspName); // Sprungziel falls gar keine Berechtigung

    // Log.debug( "CheckUserTag forwardAction default " + forwardAction ) ;

    if (!cncServlet.hasUserPermissionForJsp(request, jspName)) {
      // User hat leider keine Erlaubnis
      forwardAction = cncServlet.getAlternateForward(request, jspName);
      retValue = SKIP_PAGE; // Rckgabewert fr aktuelle Seite verlassen falls keine Berechtigung
      // Log.debug( "CheckUserTag forwardAction " + forwardAction ) ;
      try {
        // die Abarbeitung der aktuellen Seite wird abgebrochen und zum neuen Ziel geforwarded
        pageContext.forward(forwardAction);
      } catch (IOException e) {
        throw new JspException(Log.getMessageAndStackTrace(e));
      } catch (ServletException e) {
        throw new JspException(Log.getMessageAndStackTrace(e));
      }
    }// end if forward

    return retValue;
  }// end method doEndTag

}// end class CheckUserTag
